/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "base/singletoninitializer.h"
#include "actioncategory.h"

REGISTER_STATICINIT(sh::actions, ActionCategory, "base")


sh::actions::ActionCategory::ActionCategory(QString name, QString label, QString icon)
        : _name(name), _label(label), _icon(icon)
{

}

std::shared_ptr<sh::actions::ActionCategory> sh::actions::ActionCategory::getByName(QString name)
{
    for(std::shared_ptr<ActionCategory> category : _categories)
        if (category->name() == name)
            return category;
    return 0;
}

std::shared_ptr<sh::actions::ActionCategory> sh::actions::ActionCategory::add(QString name, QString label, QString icon)
{
    std::shared_ptr<ActionCategory> result = std::make_shared<ActionCategory>(name, label, icon);
    _categories.append(result);
    return result;
}

QList<std::shared_ptr<sh::actions::ActionCategory>> sh::actions::ActionCategory::categories()
{
    return _categories;
}

void sh::actions::ActionCategory::doInitialize()
{
    ActionCategory::add("create", QObject::tr("&Create"), "list-add");
    ActionCategory::add("open", QObject::tr("&Open"), "media-playback-start");
    ActionCategory::add("manage", QObject::tr("M&anage"), "adjustlevels");
}

void sh::actions::ActionCategory::doShutdown()
{
}

QList<std::shared_ptr<sh::actions::ActionCategory>> sh::actions::ActionCategory::_categories;

QString sh::actions::ActionCategory::name()
{
    return _name;
}

QString sh::actions::ActionCategory::icon()
{
    return _icon;
}

QString sh::actions::ActionCategory::label()
{
    return _label;
}
