/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONCATEGORY_H
#define ACTIONCATEGORY_H

#include <memory>
#include <QList>


namespace sh{
    namespace actions{

        /** \brief
         * Action categories are used for grouping actions in menus.
         *
         * They provide the primary grouping in the toolbar an the context menus.
         * Per default, the categories `"create"`, `"open"` and `"manage"` exist.
         */
        class ActionCategory
        {
        public:
            /** \brief Constructed only by the infrastructure and made available otherwise.
             */
            ActionCategory(QString name, QString label, QString icon);
            /** \brief
             * The category name.
             */
            QString name();
            /** \brief
             * The category label.
             */
            QString label();
            /** \brief
             * The category icon.
             */
            QString icon();
            /** \brief
             * Gets a category by name.
             */
            static std::shared_ptr<ActionCategory> getByName(QString name);
            /** \brief
             * Adds a new category.
             */
            static std::shared_ptr<ActionCategory> add(QString name, QString label, QString icon);
            /** \brief
             * Returns a list of all existing categories.
             */
            static QList<std::shared_ptr<ActionCategory>> categories();

            /** @a_infrastructure
             */
            static void doInitialize();

            /** @a_infrastructure
             */
            static void doShutdown();

        private:
            static QList<std::shared_ptr<ActionCategory>> _categories;
            QString _name;
            QString _label;
            QString _icon;
        };

    }
}

#endif // ACTIONCATEGORY_H
