/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <memory>
#include <QStack>
#include "actions/actionexecutioninfo.h"
#include "actions/actionexecutionuserfeedback.h"
#include "filesystem/eurl.h"
#include "filesystem/filesystemoperation.h"
#include "exceptions/argumentexception.h"
#include "exceptions/ioexception.h"
#include "filesystem/filesystemmodel.h"
#include "filesystem/filesystemhandler.h"
#include "filesystem/filesystemhandlerregister.h"
#include "filesystem/operation.h"
#include "base/threaddispatcher.h"
#include "actioncopytree.h"


sh::actions::common::ActionCopyTree::ActionCopyTree(QList< std::shared_ptr<const sh::filesystem::Eurl>> sources, std::shared_ptr<const sh::filesystem::Eurl> destination) :
    sh::actions::common::ActionAbstractTransferTree(sources, destination)
{
}

void sh::actions::common::ActionCopyTree::action(sh::actions::ActionExecutionInfo* info)
{
    info->setHead(tr("Copying..."));
    info->setVisualProcessFeedbackActive(true);
    info->operation()->filesystem()->copyItems(_sources, _destination, std::make_shared<MyFilesystemOperationProgressMonitor>(info));
}
