/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONNAVIGATION_H
#define ACTIONNAVIGATION_H

#include "actions/actionactionitem.h"
#include "actions/submenuactionitem.h"
#include "tools/historytracker.h"


namespace sh{
    namespace filesystem{ class Eurl; class FilesystemNode; }
    namespace ui { class FileView; }
    namespace actions {
        namespace common {

            /** \brief
             * Action for navigating to one particular place in the directory history stack.
             */
            class ActionNavigateInHistory : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                ActionNavigateInHistory(sh::ui::FileView *filelist, int idx, std::shared_ptr<const sh::filesystem::Eurl>eurl, bool isdefault);
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                sh::ui::FileView* _filelist;
                int _idx;
                std::shared_ptr<const sh::filesystem::Eurl> _eurl;
            };

            /** \brief
             * Abstract action for navigating in the directory history stack (stepwise or randomly).
             */
            class ActionHistoryNavigate : public sh::actions::SubmenuActionItem
            {
                Q_OBJECT
            public:
                ActionHistoryNavigate(QString text, std::function<QList<sh::tools::HistoryTracker<std::shared_ptr<const sh::filesystem::Eurl>>::HistoryEntry>()> _getentriesfct,
                                      QString icon);
            protected:
                void initialize();
                std::function<QList<sh::tools::HistoryTracker<std::shared_ptr<const sh::filesystem::Eurl>>::HistoryEntry>()> getentriesfct;
            };

            /** \brief
             * Action for navigating forward in the directory history stack.
             */
            class ActionHistoryNavigateForward : public ActionHistoryNavigate
            {
                Q_OBJECT
            public:
                ActionHistoryNavigateForward();
            };

            /** \brief
             * Action for navigating backward in the directory history stack.
             */
            class ActionHistoryNavigateBackward : public ActionHistoryNavigate
            {
                Q_OBJECT
            public:
                ActionHistoryNavigateBackward();
            };

        }
    }
}

#endif // ACTIONNAVIGATION_H
