/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QApplication>
#include "actions/actionexecutioninfo.h"
#include "actions/actionfactory.h"
#include "actions/actionsmanager.h"
#include "base/singletoninitializer.h"
#include "base/mainthread.h"
#include "filesystem/eurl.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/operation.h"
#include "actionopendirectoryinnewwindow.h"

REGISTER_STATICINIT(sh::actions::common, ActionOpenDirectoryInNewWindow, "base")


void sh::actions::common::ActionOpenDirectoryInNewWindow::doInitialize()
{
    sh::actions::ActionsManager::instance()->registerActionFactory(
                std::make_shared<sh::actions::ActionFactory<sh::actions::common::ActionOpenDirectoryInNewWindow>>(
                    sh::actions::ActionCategory::getByName("open"), QList<std::shared_ptr<Predicate>>({
                    std::make_shared<sh::actions::HideOnSelectionLevelPredicate>(),
                    std::make_shared<sh::actions::PositionIndexPredicate>(-300000),
                    std::make_shared<sh::actions::KeyShortcutPredicate>(QKeySequence(Qt::CTRL | Qt::Key_N), true),
    })));
}

void sh::actions::common::ActionOpenDirectoryInNewWindow::doShutdown()
{
}

sh::actions::common::ActionOpenDirectoryInNewWindow::ActionOpenDirectoryInNewWindow(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes)
    :sh::actions::ActionActionItem(tr("Open in new window..."))
{
    _nodes = nodes;
}

void sh::actions::common::ActionOpenDirectoryInNewWindow::action(sh::actions::ActionExecutionInfo*)
{
    foreach(std::shared_ptr<sh::filesystem::FilesystemNode> node, _nodes)
        QProcess::startDetached(QApplication::instance()->applicationFilePath(), {node->eurl()->asString()});
}

