/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONOPENFILE_H
#define ACTIONOPENFILE_H

#include "actions/actionactionitem.h"


namespace sh{
    namespace filesystem{class FilesystemNode;}
    namespace tools {
        namespace filetypes { class OpenMethod; }
    }

    namespace actions{
        namespace common{

            /** \brief
             * Action for opening a file with a automatically chosen program.
             */
            class ActionOpenFile : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionOpenFile(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);

                /** @a_infrastructure
                 */
                static void doInitialize();

                /** @a_infrastructure
                 */
                static void doShutdown();

            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                virtual void initialize();

                QList<std::shared_ptr<sh::filesystem::FilesystemNode>> _nodes;
                std::shared_ptr<sh::tools::filetypes::OpenMethod> _openmethod = 0;
            };

        }
    }
}

#endif // ACTIONOPENFILE_H
