/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONOPENFILEWITH_H
#define ACTIONOPENFILEWITH_H

#include "actions/actionactionitem.h"
#include "actions/submenuactionitem.h"


namespace sh{
    namespace filesystem { class Eurl; class FilesystemNode; }
    namespace tools {
        namespace filetypes { class OpenMethod;}
    }
    namespace actions {
        namespace common {

            /** \brief
             * Submenu action for opening a file with a manually chosen program.
             */
            class ActionOpenFileWith : public sh::actions::SubmenuActionItem
            {
                Q_OBJECT
                friend class ActionOpenFile;
            public:
                explicit ActionOpenFileWith(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);
                static void waitProgramClosedIfNeeded(sh::actions::ActionExecutionInfo* info, QStringList filelist);

                /** @a_infrastructure
                 */
                static void doInitialize();

                /** @a_infrastructure
                 */
                static void doShutdown();

            private:

                class _ActionOpenFileWithEntry : public sh::actions::ActionActionItem
                {
                public:
                    _ActionOpenFileWithEntry(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes, QString name, QString cmd, QStringList arguments, QStringList rememberformimetype);
                    virtual void action(sh::actions::ActionExecutionInfo* info);
                    QString _command;
                    QStringList _commandArguments;
                    QList<std::shared_ptr<sh::filesystem::FilesystemNode>> _nodes;
                protected:
                    QStringList _rememberformimetype;
                    QString node2cmdfile(sh::actions::ActionExecutionInfo* info, std::shared_ptr<sh::filesystem::FilesystemNode> node, bool urlmode);
                };

                class _ActionOpenFileWithUI : public sh::actions::ActionActionItem
                {
                public:
                    _ActionOpenFileWithUI(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes, QString mimetype);
                    virtual void action(sh::actions::ActionExecutionInfo* info);
                    QList<std::shared_ptr<sh::filesystem::FilesystemNode>> _nodes;
                    QString _mimetype;
                };

            protected:

                class RememberedOpenAction
                {
                public:
                    RememberedOpenAction(QString command, QStringList arguments);
                    QString command;
                    QStringList arguments;
                    bool operator==(const RememberedOpenAction& a2) const;
                };

                static void rememberOpenAction(QString mimetype, QString cmd, QStringList arguments);
                static void readOpenAction(QString path, QString* mimetype, QString* cmd, QStringList* arguments);
                static QList<RememberedOpenAction> getRememberedOpenActions(QString mimetype);
                QList<std::shared_ptr<sh::filesystem::FilesystemNode>> _nodes;
                virtual void initialize();
            };

        }
    }
}

#endif // ACTIONOPENFILEWITH_H
