/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "actions/actionfactory.h"
#include "actions/actionsmanager.h"
#include "filesystem/eurl.h"
#include "filesystem/operation.h"
#include "filesystem/filesystemoperation.h"
#include "filesystem/filesystemnode.h"
#include "base/mainthread.h"
#include "base/singletoninitializer.h"
#include "ui/mainwindow.h"
#include "actionshowproperties.h"

REGISTER_STATICINIT(sh::actions::common, ActionShowProperties, "base")


void sh::actions::common::ActionShowProperties::doInitialize()
{
    sh::actions::ActionsManager::instance()->registerActionFactory(
                std::make_shared<sh::actions::ActionFactory<sh::actions::common::ActionShowProperties>>(
                    sh::actions::ActionCategory::getByName("manage"), QList<std::shared_ptr<Predicate>>({
                    std::make_shared<sh::actions::DontResolveLinksPredicate>(),
                    std::make_shared<sh::actions::OnSingleEntrySelectionPredicate>(),
                    std::make_shared<sh::actions::PositionIndexPredicate>(-500000),
                    std::make_shared<sh::actions::KeyShortcutPredicate>(QKeySequence(Qt::CTRL|Qt::SHIFT|Qt::Key_P), false)
    })));
}

void sh::actions::common::ActionShowProperties::doShutdown()
{
}

sh::actions::common::ActionShowProperties::ActionShowProperties(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes)
    :sh::actions::ActionActionItem( tr("Properties..."), true, QString())
{
    _nodes = nodes;
}

void sh::actions::common::ActionShowProperties::initialize()
{
}

void sh::actions::common::ActionShowProperties::action(sh::actions::ActionExecutionInfo*)
{
    sh::ui::MainWindow::mainWindow()->showFilePropertyDialog(_nodes);
}
