/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONTRANSFERTO_H
#define ACTIONTRANSFERTO_H

#include "actions/actionactionitem.h"
#include "actions/submenuactionitem.h"


namespace sh{
    namespace filesystem { class FilesystemNode; class Eurl; }
    namespace actions{
        namespace common {

            /** \brief
             * Abstract action for transferring a selection to some other locations (e.g. to other file views, bookmarks).
             */
            class ActionAbstractTransferTo : public sh::actions::SubmenuActionItem
            {
                friend class ActionTransferToHelper;
                Q_OBJECT
            public:
                explicit ActionAbstractTransferTo(QString text, QString icon, QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);
            protected:
                virtual void initialize();
                virtual void doTransfer(sh::actions::ActionExecutionInfo* info, std::shared_ptr<const sh::filesystem::Eurl> to) = 0;
                QList<std::shared_ptr<sh::filesystem::FilesystemNode>> _nodes;
            };

            /** \brief
             * This action is used as subitems in ActionAbstractTransferTo.
             */
            class ActionTransferToHelper : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionTransferToHelper(QString text, QString icon, std::shared_ptr<const sh::filesystem::Eurl> transferto, std::shared_ptr<ActionAbstractTransferTo> root);
            protected:
                void action(sh::actions::ActionExecutionInfo* info);
                std::shared_ptr<const sh::filesystem::Eurl> _transferto;
                std::shared_ptr<ActionAbstractTransferTo> _root;
            };

            /** \brief
             * Action for copying a selection to some other locations.
             */
            class ActionTransferToCopy : public ActionAbstractTransferTo
            {
                Q_OBJECT
            public:
                explicit ActionTransferToCopy(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void doInitialize();
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void doShutdown();
            protected:
                virtual void doTransfer(sh::actions::ActionExecutionInfo* info, std::shared_ptr<const sh::filesystem::Eurl> to);
            };

            /** \brief
             * Action for moving a selection to some other locations.
             */
            class ActionTransferToMove : public ActionAbstractTransferTo
            {
                Q_OBJECT
            public:
                explicit ActionTransferToMove(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);

                /** @a_infrastructure
                 */
                static void doInitialize();

                /** @a_infrastructure
                 */
                static void doShutdown();

            protected:
                virtual void doTransfer(sh::actions::ActionExecutionInfo* info, std::shared_ptr<const sh::filesystem::Eurl> to);
            };

        }
    }
}

#endif // ACTIONTRANSFERTO_H
