/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "settings/settingsmanager.h"
#include "base/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/fileview.h"
#include "actionapplyprofile.h"


sh::actions::mainmenu::ActionApplyProfile::ActionApplyProfile()
    : sh::actions::SubmenuActionItem(tr("Switch to &profile"), QList<std::shared_ptr<sh::actions::AbstractActionItem>>())
{
}

void sh::actions::mainmenu::ActionApplyProfile::initialize()
{
    connect(sh::settings::SettingsManager::instance().get(), &sh::settings::SettingsManager::profilesChanged, this, [=](){
        configure();
    });
    sh::ui::MainWindow::mainWindow()->onCurrentProfileChanged([=](){
        configure();
    }, this);
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        configure();
    });
}

void sh::actions::mainmenu::ActionApplyProfile::configure()
{
    QList<std::shared_ptr<sh::actions::AbstractActionItem>> subitems;
    QString currentprofile = sh::ui::MainWindow::mainWindow()->currentProfile();
    foreach(QString profile, sh::settings::SettingsManager::instance()->getProfileList())
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new ActionApplyProfileX(profile, (profile==currentprofile))));
    setSubitems(subitems);
}

sh::actions::mainmenu::ActionApplyProfile::ActionApplyProfileX::ActionApplyProfileX(QString profilename, bool checked)
    : sh::actions::ActionActionItem(profilename, true, QString(), 0, true, checked), profilename(profilename)
{
}

void sh::actions::mainmenu::ActionApplyProfile::ActionApplyProfileX::action(sh::actions::ActionExecutionInfo*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->setCurrentProfile(profilename);
    });
}
