/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/fileview.h"
#include "actionfileviewchangeicondimension.h"


sh::actions::mainmenu::ActionFileViewChangeIconDimension::ActionFileViewChangeIconDimension(QString name, int direction, QString icon, QKeySequence shortcut)
    : sh::actions::ActionActionItem(name, true, icon), _direction(direction)
{
    setShortcuthint(shortcut);
    sh::ui::MainWindow::mainWindow()->onFileViewOptionsChanged([=](int){
        configure();
    }, this);
    sh::ui::MainWindow::mainWindow()->onCurrentFileViewChanged([=](){
        configure();
    }, this);
    configure();
}

void sh::actions::mainmenu::ActionFileViewChangeIconDimension::configure()
{
    setVisible(sh::ui::MainWindow::mainWindow()->currentFileView()->viewmode() == sh::ui::FileViewMode::IconView);
}

void sh::actions::mainmenu::ActionFileViewChangeIconDimension::action(sh::actions::ActionExecutionInfo*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        auto v = sh::ui::MainWindow::mainWindow()->currentFileView();
        v->setIconDimension(v->iconDimension() + _direction);
    });
}

sh::actions::mainmenu::ActionFileViewIncreaseIconDimension::ActionFileViewIncreaseIconDimension()
    : sh::actions::mainmenu::ActionFileViewChangeIconDimension(tr("&Zoom in"), 1, "zoom-in", QKeySequence(Qt::CTRL | Qt::Key_Plus))
{
}


sh::actions::mainmenu::ActionFileViewDecreaseIconDimension::ActionFileViewDecreaseIconDimension()
    : sh::actions::mainmenu::ActionFileViewChangeIconDimension(tr("Zoo&m out"), -1, "zoom-out", QKeySequence(Qt::CTRL | Qt::Key_Minus))
{
}
