/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/fileview.h"
#include "actionfileviewshowhiddenfiles.h"


sh::actions::mainmenu::ActionFileviewShowHiddenFiles::ActionFileviewShowHiddenFiles()
    : sh::actions::ActionActionItem(tr("Show &hidden files"), true, QString(), 0, true)
{
    setShortcuthint(QKeySequence(Qt::CTRL | Qt::Key_H));
    sh::ui::MainWindow::mainWindow()->onFileViewOptionsChanged([=](int){
        configure();
    }, this);
    sh::ui::MainWindow::mainWindow()->onCurrentFileViewChanged([=](){
        configure();
    }, this);
    configure();
}

void sh::actions::mainmenu::ActionFileviewShowHiddenFiles::configure()
{
    _set = !sh::ui::MainWindow::mainWindow()->currentFileView()->hiddenFilesVisible();
    setChecked(!_set);
}

void sh::actions::mainmenu::ActionFileviewShowHiddenFiles::action(sh::actions::ActionExecutionInfo*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->currentFileView()->setHiddenFilesVisible(_set);
    });
}
