/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/fileview.h"
#include "actionfileviewview.h"


sh::actions::mainmenu::ActionFileviewView::ActionFileviewView()
    : sh::actions::ActionActionItem(tr("Show as &icons"), true, QString(), 0, true)
{
    setShortcuthint(QKeySequence(Qt::CTRL|Qt::SHIFT|Qt::Key_L));
    sh::ui::MainWindow::mainWindow()->onFileViewOptionsChanged([=](int){
        configure();
    }, this);
    sh::ui::MainWindow::mainWindow()->onCurrentFileViewChanged([=](){
        configure();
    }, this);
    configure();
}

void sh::actions::mainmenu::ActionFileviewView::configure()
{
    _setAsIcons = sh::ui::MainWindow::mainWindow()->currentFileView()->viewmode() == sh::ui::FileViewMode::ListView;
    setChecked(!_setAsIcons);
}

void sh::actions::mainmenu::ActionFileviewView::action(sh::actions::ActionExecutionInfo*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->currentFileView()->setViewmode(_setAsIcons ? sh::ui::FileViewMode::IconView : sh::ui::FileViewMode::ListView);
    });
}
