/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionsmanager.h"
#include "actions/separatoractionitem.h"
#include "base/mainthread.h"
#include "ui/fileview.h"
#include "ui/mainwindow.h"
#include "actionnumberfileviews.h"

REGISTER_STATICINIT(sh::actions::mainmenu, ActionNumberFileviews_Add, "base")
REGISTER_STATICINIT(sh::actions::mainmenu, ActionNumberFileviews_Remove_Current, "base")


void sh::actions::mainmenu::ActionNumberFileviews_Add::doInitialize()
{  // we need to initialize it also this way, since due to technical reasons, the keyboard shortcuts wouldn`t work the normal way
    sh::actions::ActionsManager::instance()->registerActionFactory(
                std::make_shared<sh::actions::ActionFactory<sh::actions::mainmenu::ActionNumberFileviews_Add>>(
                    (std::shared_ptr<sh::actions::ActionCategory>)0, QList<std::shared_ptr<Predicate>>({
                    std::make_shared<sh::actions::KeyShortcutPredicate>(sh::actions::mainmenu::ActionNumberFileviews::keySequenceAdd, false),
    })));
}

void sh::actions::mainmenu::ActionNumberFileviews_Add::doShutdown()
{
}

sh::actions::mainmenu::ActionNumberFileviews::ActionNumberFileviews()
    : sh::actions::SubmenuActionItem(tr("File &lists"), QList<std::shared_ptr<sh::actions::AbstractActionItem>>())
{
}

void sh::actions::mainmenu::ActionNumberFileviews::initialize()
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->onFileViewCountChanged([=](){
            configure();
        }, this);
        sh::ui::MainWindow::mainWindow()->onCurrentFileViewChanged([=](){
            configure();
        }, this);
        configure();
    });
}

void sh::actions::mainmenu::ActionNumberFileviews::configure()
{
    QList<std::shared_ptr<sh::actions::AbstractActionItem>> subitems;
    subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new ActionNumberFileviews_Add()));
    if (sh::ui::MainWindow::mainWindow()->fileViewCount() > 1)
    {
        int iCurrentFileview = sh::ui::MainWindow::mainWindow()->currentFileView()->index();
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::SeparatorActionItem()));
        for(int i=0; i<sh::ui::MainWindow::mainWindow()->fileViewCount(); i++)
            subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new ActionNumberFileviews_Remove(i, i == iCurrentFileview)));
    }
    setSubitems(subitems);
}

sh::actions::mainmenu::ActionNumberFileviews_Add::ActionNumberFileviews_Add(QList<std::shared_ptr<sh::filesystem::FilesystemNode> >)
    : sh::actions::ActionActionItem(QObject::tr("Add new file list"))
{
    setShortcuthint(sh::actions::mainmenu::ActionNumberFileviews::keySequenceAdd);  // this is just a visualization hack; we need a helper action for that
}

void sh::actions::mainmenu::ActionNumberFileviews_Add::action(sh::actions::ActionExecutionInfo*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->addFileView(true);
    });
}

sh::actions::mainmenu::ActionNumberFileviews_Remove::ActionNumberFileviews_Remove(int i, bool deflt)
    : sh::actions::ActionActionItem(QObject::tr("Remove list #%1").arg(i+1), true, QString(), deflt?1:0)
{
    _i = i;
    if (deflt)
        setShortcuthint(sh::actions::mainmenu::ActionNumberFileviews::keySequenceRemove);  // this is just a visualization hack; we need a helper action for that
}

void sh::actions::mainmenu::ActionNumberFileviews_Remove::action(sh::actions::ActionExecutionInfo*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->removeFileView(_i);
    });
}

void sh::actions::mainmenu::ActionNumberFileviews_Remove_Current::doInitialize()
{  // we need to initialize it also this way, since due to technical reasons, the keyboard shortcuts wouldn`t work the normal way
    sh::actions::ActionsManager::instance()->registerActionFactory(
                std::make_shared<sh::actions::ActionFactory<sh::actions::mainmenu::ActionNumberFileviews_Remove_Current>>(
                    (std::shared_ptr<sh::actions::ActionCategory>)0, QList<std::shared_ptr<Predicate>>({
                    std::make_shared<sh::actions::KeyShortcutPredicate>(sh::actions::mainmenu::ActionNumberFileviews::keySequenceRemove, false),
    })));
}

void sh::actions::mainmenu::ActionNumberFileviews_Remove_Current::doShutdown()
{
}

sh::actions::mainmenu::ActionNumberFileviews_Remove_Current::ActionNumberFileviews_Remove_Current(QList<std::shared_ptr<sh::filesystem::FilesystemNode>>)
{
}

void sh::actions::mainmenu::ActionNumberFileviews_Remove_Current::action(sh::actions::ActionExecutionInfo*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        auto icurr = sh::ui::MainWindow::mainWindow()->currentFileView()->index();
        if (sh::ui::MainWindow::mainWindow()->fileViewCount() > 1)
            sh::ui::MainWindow::mainWindow()->removeFileView(icurr);
    });
}

QKeySequence sh::actions::mainmenu::ActionNumberFileviews::keySequenceAdd(Qt::CTRL|Qt::SHIFT|Qt::Key_Asterisk);

QKeySequence sh::actions::mainmenu::ActionNumberFileviews::keySequenceRemove(Qt::CTRL|Qt::Key_W);
