/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONNUMBERFILEVIEWS_H
#define ACTIONNUMBERFILEVIEWS_H

#include "actions/actionactionitem.h"
#include "actions/submenuactionitem.h"


namespace sh{
    namespace filesystem { class FilesystemNode; }
    namespace actions {
        namespace mainmenu {

            /** \brief
             * Action for adding a new fileview.
             */
            class ActionNumberFileviews_Add : public sh::actions::ActionActionItem
            {
            public:
                ActionNumberFileviews_Add(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes={});

                /** @a_infrastructure
                 */
                static void doInitialize();

                /** @a_infrastructure
                 */
                static void doShutdown();

            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
            };

            /** \brief
             * Action for removing a certain fileview.
             */
            class ActionNumberFileviews_Remove : public sh::actions::ActionActionItem
            {
            public:
                ActionNumberFileviews_Remove(int i, bool deflt);
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                int _i;
            };

            /** \brief
             * Submenu action for changing the number of fileviews.
             */
            class ActionNumberFileviews : public sh::actions::SubmenuActionItem
            {
                friend class ActionNumberFileviews_Add;
                friend class ActionNumberFileviews_Remove;
                friend class ActionNumberFileviews_Remove_Current;
                Q_OBJECT

            public:
                ActionNumberFileviews();
            protected:
                virtual void initialize();
                void configure();
                static QKeySequence keySequenceAdd;
                static QKeySequence keySequenceRemove;
            };

            /** \brief
             * Action for removing the current fileview (for keyboard shortcut).
             */
            class ActionNumberFileviews_Remove_Current : public sh::actions::ActionActionItem
            {
            public:
                ActionNumberFileviews_Remove_Current(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);

                /** @a_infrastructure
                 */
                static void doInitialize();

                /** @a_infrastructure
                 */
                static void doShutdown();

            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
            };

        }
    }
}

#endif // ACTIONNUMBERFILEVIEWS_H
