/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/eurl.h"
#include "settings/settingsmanager.h"
#include "settings/setting.h"
#include "base/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/fileview.h"
#include "ui/storeprofiledialog.h"
#include "actionsavesettings.h"


sh::actions::mainmenu::ActionSaveSettings::ActionSaveSettings()
    : sh::actions::ActionActionItem(tr("&Save settings..."))
{
}

void sh::actions::mainmenu::ActionSaveSettings::action(sh::actions::ActionExecutionInfo* info)
{
    info->setVisualProcessFeedbackActive(false);
    sh::ui::FileView* currentfilelist;
    std::shared_ptr<const sh::filesystem::Eurl> currenteurl;
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        currentfilelist = sh::ui::MainWindow::mainWindow()->currentFileView();
        currenteurl = currentfilelist->node()->eurl();
    });
    std::shared_ptr<sh::ui::StoreProfileDialog> dialog = sh::ui::MainWindow::mainWindow()->showStoreProfileDialog(currenteurl);
    dialog->waitClosed();
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        if (dialog->wasAccepted()) {
            sh::exceptions::Exception::executeGuarded([&](){
                QMap<QString, QString> vals;
                QMap<QString, int> onlyinfileviewindex;
                foreach (sh::ui::StoreProfileDialog::SettingEntry setting, dialog->checkedSettings()) {
                    QString sname = setting.setting()->name();
                    QString sval = setting.setting()->getValue(currentfilelist);
                    if (!sval.isNull()) {
                        vals[sname] = sval;
                        onlyinfileviewindex[sname] = setting.fileview(); // -1 for 'not per fileview'
                    }
                }
                sh::settings::SettingsManager::instance()->storeProfile(dialog->hasGlobal() ? 0 : currenteurl, dialog->profileName(), vals,
                                                                        onlyinfileviewindex, dialog->withSubDirectories(), dialog->inheritsFrom());
            }, sh::exceptions::ExecuteGuardFlag::RESUMEABLE_PROGRAM_ERROR_END_WITH_USER_FEEDBACK_HERE);
        }
    });
}
