/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filesystem/filesystemnode.h"
#include "filesystem/filesystemnodelist.h"
#include "ui/fileview.h"
#include "ui/mainwindow.h"
#include "base/mainthread.h"
#include "actionselect.h"



sh::actions::mainmenu::ActionAbstractSelectNodes::ActionAbstractSelectNodes(QString text)
    : sh::actions::ActionActionItem(text)
{
}

void sh::actions::mainmenu::ActionAbstractSelectNodes::action(sh::actions::ActionExecutionInfo*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        auto fv = sh::ui::MainWindow::mainWindow()->currentFileView();
        if (fv) {
            QList<std::shared_ptr<sh::filesystem::FilesystemNode>> allNodes = fv->getAllVisibleNodes();
            QList<std::shared_ptr<sh::filesystem::FilesystemNode>> selNodes;
            prepareDecidings();
            foreach(auto n, allNodes) {
                if (decideSelected(n))
                    selNodes.append(n);
            }
            fv->setSelection(selNodes);
        }
    });
}

void sh::actions::mainmenu::ActionAbstractSelectNodes::prepareDecidings()
{
}

sh::actions::mainmenu::ActionSelectAllNodes::ActionSelectAllNodes()
    : ActionAbstractSelectNodes(tr("Select all"))
{
    setShortcuthint(QKeySequence(Qt::CTRL|Qt::Key_A));
}

bool sh::actions::mainmenu::ActionSelectAllNodes::decideSelected(std::shared_ptr<sh::filesystem::FilesystemNode>)
{
    return true;
}

sh::actions::mainmenu::ActionInvertNodeSelection::ActionInvertNodeSelection()
    : ActionAbstractSelectNodes(tr("Invert selection"))
{
    setShortcuthint(QKeySequence(Qt::CTRL|Qt::SHIFT|Qt::Key_A));
}

void sh::actions::mainmenu::ActionInvertNodeSelection::prepareDecidings()
{
    auto fv = sh::ui::MainWindow::mainWindow()->currentFileView();
    if (fv)  // should not really be false
        wasselected = fv->selectedNodes();
}

bool sh::actions::mainmenu::ActionInvertNodeSelection::decideSelected(std::shared_ptr<sh::filesystem::FilesystemNode> node)
{
    return !wasselected.contains(node);
}
