/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONSELECT_H
#define ACTIONSELECT_H

#include "actions/actionactionitem.h"


namespace sh{
    namespace filesystem { class FilesystemNode; }
    namespace actions {
        namespace mainmenu {

            /** \brief
             * Abstract subclass for actions which select some nodes in the current file view.
             */
            class ActionAbstractSelectNodes : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                ActionAbstractSelectNodes(QString text);
            protected:
                virtual void prepareDecidings();
                virtual bool decideSelected(std::shared_ptr<sh::filesystem::FilesystemNode> node) = 0;
                virtual void action(sh::actions::ActionExecutionInfo* info);
            };

            /** \brief
             * Action for selecting all nodes in the current file view.
             */
            class ActionSelectAllNodes : public ActionAbstractSelectNodes
            {
                Q_OBJECT
            public:
                ActionSelectAllNodes();
            protected:
                virtual bool decideSelected(std::shared_ptr<sh::filesystem::FilesystemNode> node);
            };

            /** \brief
             * Action for inverting the node selection in the current file view.
             */
            class ActionInvertNodeSelection : public ActionAbstractSelectNodes
            {
                Q_OBJECT
            public:
                ActionInvertNodeSelection();
            protected:
                virtual void prepareDecidings();
                virtual bool decideSelected(std::shared_ptr<sh::filesystem::FilesystemNode> node);
            private:
                QList<std::shared_ptr<sh::filesystem::FilesystemNode>> wasselected;
            };

        }
    }
}

#endif // ACTIONSELECT_H
