/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MAINTHREAD_H
#define MAINTHREAD_H

#include <QApplication>
#include "threaddispatcher.h"


namespace sh{
    namespace base{

        /** \brief
         * The main thread.
         *
         * Lots of data structure may only accessed from the main thread. The UI and the sh::filesystem::FilesystemModel also live in this thread
         * (although some functions there may allow multi-threading in some ways).
         */
        class MainThread
        {
        public:
            /** \brief
             * The sh::base::ThreadDispatcher, which allows to execute code in the main thread.
             */
            static sh::base::ThreadDispatcher* dispatcher();
        };

    }
}

#ifdef QT_DEBUG
#define NOT_ALLOWED_TO_CALL_FROM_MAIN_THREAD if (QThread::currentThread()==QApplication::instance()->thread()) sh::exceptions::Exception::executeGuarded([&](){ throw sh::exceptions::ThreadingException(sh::exceptions::ExceptionData().details("A method is called from the main thread, which is forbidden by that method.")); });
#define ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD if (QApplication::instance() && QThread::currentThread()!=QApplication::instance()->thread()) sh::exceptions::Exception::executeGuarded([&](){ throw sh::exceptions::ThreadingException(sh::exceptions::ExceptionData().details("A method is not called from the main thread as required by that method.")); });
#else
#define NOT_ALLOWED_TO_CALL_FROM_MAIN_THREAD
#define ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
#endif

#endif // MAINTHREAD_H
