/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DETAILCOLUMNCUSTOMATTRIBUTES_H
#define DETAILCOLUMNCUSTOMATTRIBUTES_H

#include "base/singletoninitializer.h"
#include "filesystem/detailcolumn.h"


namespace sh{
    namespace detailcolumns {

        /** \brief
         * Detail column which determines the custom attributes.
         *
         * Custom attributes are filesystem handler specific values about files
         * (like permissions).
         */
        class DetailColumnCustomAttributes : public sh::filesystem::DetailColumn, public sh::base::Singleton
        {
            DECLARE_SINGLETON(DetailColumnCustomAttributes)
        public:

            void applyValueByEurl(std::shared_ptr<const sh::filesystem::Eurl> eurl, sh::filesystem::Operation* op, QVariant value) override;

            /** @brief Converts the QVariant(QString) from native detail column representation to a QString/QString map.
             */
            static QMap<QString, QString> getMapByValue(QVariant val);

        protected:
            QVariant determineValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, sh::filesystem::Operation* op) override;
            bool isVisible() override;
            void doInitialize() override;

        private:
            DetailColumnCustomAttributes();

        };

    }
}

#endif // DETAILCOLUMNCUSTOMATTRIBUTES_H
