/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/filesystemhandler.h"
#include "detailcolumndirectsymlinktarget.h"

REGISTER_SINGLETON(sh::detailcolumns, DetailColumnDirectSymlinkTarget, "base")


sh::detailcolumns::DetailColumnDirectSymlinkTarget::DetailColumnDirectSymlinkTarget()
    : sh::filesystem::DetailColumn("DIRECT_SYMLINK_TARGET", QObject::tr("Direct symlink target"))
{
}

QVariant sh::detailcolumns::DetailColumnDirectSymlinkTarget::determineValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, sh::filesystem::Operation* op)
{
    if (node->nodetype()==sh::filesystem::FilesystemNodeType::Link)
        return node->handler()->getLinkTarget(op, node->eurl());
    else
        return QVariant();
}

bool sh::detailcolumns::DetailColumnDirectSymlinkTarget::isVisible()
{
    return false;
}

void sh::detailcolumns::DetailColumnDirectSymlinkTarget::doInitialize()
{
    sh::filesystem::DetailColumn::registerKnownDetailColumn("directsymlinktarget", instance());
}
