/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/filesystemhandler.h"
#include "filesystem/filesystemoperation.h"
#include "filesystem/filesystemmodelfileviewproxy.h"
#include "detailcolumnextendedattributes.h"
#include "detailcolumnresolvedsymlink.h"

REGISTER_SINGLETON(sh::detailcolumns, DetailColumnExtendedAttributes, "base")


sh::detailcolumns::DetailColumnExtendedAttributes::DetailColumnExtendedAttributes()
    : sh::filesystem::DetailColumn("XATTR", QObject::tr("Extended attributes"))
{
}

QVariant sh::detailcolumns::DetailColumnExtendedAttributes::determineValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, sh::filesystem::Operation* op)
{
    QStringList result;
    foreach(QString attname, node->handler()->listExtendedAttributes(op, node->eurl()))
    {
        QByteArray ba = node->handler()->getExtendedAttribute(op, node->eurl(), attname);
        result << attname << QString::fromUtf8(ba.toBase64());
    }
    return result;
}

QMap<QString, QByteArray> sh::detailcolumns::DetailColumnExtendedAttributes::getMapByValue(QVariant val)
{
    QMap<QString, QByteArray> result;
    QString key;
    foreach(QString s, val.toStringList())
    {
        if (key.isNull())
            key = s;
        else
        {
            result[key] = QByteArray::fromBase64(s.toUtf8());
            key = QString();
        }
    }
    return result;
}

bool sh::detailcolumns::DetailColumnExtendedAttributes::isVisible()
{
    return false;
}

void sh::detailcolumns::DetailColumnExtendedAttributes::applyValueByEurl(std::shared_ptr<const sh::filesystem::Eurl> eurl, sh::filesystem::Operation* op, QVariant value)
{
    QStringList lst = value.toStringList();
    while(!lst.isEmpty())
    {
        QString attname = lst.takeFirst();
        QString attvalbase64 = lst.takeFirst();
        QByteArray attval = QByteArray::fromBase64(attvalbase64.toUtf8());
        sh::filesystem::FilesystemHandlerRegister::instance()->findHandler(eurl->scheme())->setExtendedAttribute(op, eurl, attname, attval);
    }
}

void sh::detailcolumns::DetailColumnExtendedAttributes::doInitialize()
{
    sh::filesystem::FilesystemOperation::addTransferrableDetailColumn(500000, instance());
    sh::filesystem::DetailColumn::registerKnownDetailColumn("extendedattributes", instance());

}
