/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/filesystemhandler.h"
#include "detailcolumnmimetype.h"
#include "detailcolumnresolvedsymlink.h"

REGISTER_SINGLETON(sh::detailcolumns, DetailColumnMimetype, "base")


sh::detailcolumns::DetailColumnMimetype::DetailColumnMimetype() :
    sh::filesystem::DetailColumn("MIMETYPE", QObject::tr("Type"), DISPLAYINDEX_CORE+300, true, 180)
{
}

QVariant sh::detailcolumns::DetailColumnMimetype::determineValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, sh::filesystem::Operation* op)
{
    std::shared_ptr<sh::filesystem::DetailColumn> ressymlink = sh::detailcolumns::DetailColumnResolvedSymlink::instance();
    QList<std::shared_ptr<sh::filesystem::FilesystemNode>> resnodes = ressymlink->requestValue(node, op).value<QList<std::shared_ptr<sh::filesystem::FilesystemNode>>>();
    std::shared_ptr<sh::filesystem::FilesystemNode> resnode = (resnodes.isEmpty()) ? 0 : resnodes.first();
    if (resnode && resnode->nodetype() == sh::filesystem::FilesystemNodeType::File)
        return resnode->handler()->getMimetype(op, resnode->eurl());
    else
        return "";
}

QString sh::detailcolumns::DetailColumnMimetype::displayValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, const sh::filesystem::FilesystemModelFileviewProxy*)
{
    if (node->nodetype() != sh::filesystem::FilesystemNodeType::Directory)
        return value(node).toString();
    else
        return "";
}

void sh::detailcolumns::DetailColumnMimetype::doInitialize()
{
    sh::filesystem::DetailColumn::registerKnownDetailColumn("mimetype", instance());
}
