/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/filesystemhandler.h"
#include "filesystem/filesystemhandlerregister.h"
#include "filesystem/filesystemoperation.h"
#include "detailcolumnmtime.h"
#include "detailcolumnresolvedsymlink.h"

REGISTER_SINGLETON(sh::detailcolumns, DetailColumnMtime, "base")


sh::detailcolumns::DetailColumnMtime::DetailColumnMtime() :
    sh::filesystem::DetailColumn("MTIME", QObject::tr("Last modified"), DISPLAYINDEX_CORE+200, false, 180)
{
}

QVariant sh::detailcolumns::DetailColumnMtime::determineValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, sh::filesystem::Operation* op)
{
    return node->handler()->getMtime(op, node->eurl());
}

QString sh::detailcolumns::DetailColumnMtime::displayValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, const sh::filesystem::FilesystemModelFileviewProxy*)
{
    return value(node).toDateTime().toString(Qt::DefaultLocaleShortDate);
}

void sh::detailcolumns::DetailColumnMtime::applyValueByEurl(std::shared_ptr<const sh::filesystem::Eurl> eurl, sh::filesystem::Operation* op, QVariant value)
{
    sh::filesystem::FilesystemHandlerRegister::instance()->findHandler(eurl->scheme())->setMtime(op, eurl, value.toDateTime());
}

void sh::detailcolumns::DetailColumnMtime::doInitialize()
{
    sh::filesystem::FilesystemOperation::addTransferrableDetailColumn(999999, instance());
    sh::filesystem::DetailColumn::registerKnownDetailColumn("mtime", instance());
}
