/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/filesystemhandler.h"
#include "filesystem/filesystemoperation.h"
#include "filesystem/operation.h"
#include "detailcolumnresolvedsymlink.h"

REGISTER_SINGLETON(sh::detailcolumns, DetailColumnResolvedSymlink, "base")


sh::detailcolumns::DetailColumnResolvedSymlink::DetailColumnResolvedSymlink()
    : sh::filesystem::DetailColumn("RESOLVED_SYMLINK_TARGET", QObject::tr("Resolved symlink target"))
{
}

QVariant sh::detailcolumns::DetailColumnResolvedSymlink::determineValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, sh::filesystem::Operation* operation)
{
    if (node->nodetype() == sh::filesystem::FilesystemNodeType::Link)
    {
        QList<std::shared_ptr<sh::filesystem::FilesystemNode>> hnodes = operation->filesystem()->resolveNodeLink(node, true, true);
        node->setLinkDestinationNodes(hnodes);
        return QVariant::fromValue(hnodes);
    }
    else
    {
        node->setLinkDestinationNodes(QList<std::shared_ptr<sh::filesystem::FilesystemNode>>());
        return QVariant::fromValue(QList<std::shared_ptr<sh::filesystem::FilesystemNode>>({node}));
    }
}

QString sh::detailcolumns::DetailColumnResolvedSymlink::displayValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, const sh::filesystem::FilesystemModelFileviewProxy*)
{
    QList<std::shared_ptr<sh::filesystem::FilesystemNode>> v = value(node).value<QList<std::shared_ptr<sh::filesystem::FilesystemNode>>>();
    if (v.isEmpty())
        return "";
    else
        return v.first()->eurl()->asString();
}

bool sh::detailcolumns::DetailColumnResolvedSymlink::isVisible()
{
    return false;
}

void sh::detailcolumns::DetailColumnResolvedSymlink::doInitialize()
{
    sh::filesystem::DetailColumn::registerKnownDetailColumn("resolvedsymlink", instance());
}
