/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DETAILCOLUMNRESOLVEDSYMLINK_H
#define DETAILCOLUMNRESOLVEDSYMLINK_H

#include "base/singletoninitializer.h"
#include "filesystem/detailcolumn.h"

// this allows us std::shared_ptr in lists in QVariant.
Q_DECLARE_SMART_POINTER_METATYPE(std::shared_ptr)


namespace sh{
    namespace detailcolumns {

        /** \brief
         * Detail column which determines the symlink target (by resolving links recursively).
         *
         * This detail column has some active behavior. It informs the filesystem model about
         * the 'buddy nodes'.
         */
        class DetailColumnResolvedSymlink : public sh::filesystem::DetailColumn, public sh::base::Singleton
        {
            DECLARE_SINGLETON(DetailColumnResolvedSymlink)
        protected:
            void doInitialize() override;
            QVariant determineValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, sh::filesystem::Operation* op) override;
            QString displayValue(std::shared_ptr<sh::filesystem::FilesystemNode> node, const sh::filesystem::FilesystemModelFileviewProxy *viewmodel) override;
            bool isVisible() override;

        private:
            DetailColumnResolvedSymlink();

        };

    }
}

#endif // DETAILCOLUMNRESOLVEDSYMLINK_H
