/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILEPROPERTYDIALOGTABEXTENDEDATTRIBUTES_H
#define FILEPROPERTYDIALOGTABEXTENDEDATTRIBUTES_H

#include "actions/actionactionitem.h"
#include "ui/filepropertydialog.h"


namespace sh{
    namespace filepropertydialogtabs {

        /** \brief Properties dialog tab for extended attributes.
         */
        class FilePropertyDialogTabExtendedAttributes : public sh::ui::FilePropertyDialogTab
        {
        public:

            FilePropertyDialogTabExtendedAttributes();
            QString title() override;
            QList<QString> properties() override;
            void populateWidget(int i, sh::ui::FilePropertyDialogTabActionsView* widget) override;
            void updateWidget(int i, sh::ui::FilePropertyDialogTabActionsView* widget, sh::filesystem::Operation *op) override;

            /** @a_infrastructure
             */
            static void doInitialize();

            /** @a_infrastructure
             */
            static void doShutdown();

        protected:
            void slot_buttontriggered(int i);

            class ActionAddAttribute : public sh::actions::ActionActionItem
            {
            public:
                ActionAddAttribute(std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabExtendedAttributes> tab);
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabExtendedAttributes> tab;
            };

            class ActionChangeAttribute : public sh::actions::ActionActionItem
            {
            public:
                ActionChangeAttribute(std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabExtendedAttributes> tab);
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabExtendedAttributes> tab;
            };

            class ActionRemoveAttribute : public sh::actions::ActionActionItem
            {
            public:
                ActionRemoveAttribute(std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabExtendedAttributes> tab);
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabExtendedAttributes> tab;
            };

        };

    }
}

#endif // FILEPROPERTYDIALOGTABEXTENDEDATTRIBUTES_H
