/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILEPROPERTYDIALOGTABUNIXPERMISSIONS_H
#define FILEPROPERTYDIALOGTABUNIXPERMISSIONS_H

#include "actions/actionactionitem.h"
#include "ui/filepropertydialog.h"


namespace sh{
    namespace filepropertydialogtabs {

        /** \brief Properties dialog tab for unix permissions.
         */
        class FilePropertyDialogTabUnixPermissions : public sh::ui::FilePropertyDialogTab
        {
        public:

            FilePropertyDialogTabUnixPermissions();
            QString title() override;
            QList<QString> properties() override;
            void populateWidget(int i, sh::ui::FilePropertyDialogTabActionsView* widget) override;
            void updateWidget(int i, sh::ui::FilePropertyDialogTabActionsView* widget, sh::filesystem::Operation *op) override;

            static QString getUserName(int uid);
            static QString getGroupName(int gid);

            static QMap<int, QString> getAllUsers();
            static QMap<int, QString> getAllGroups();

            /** @a_infrastructure
             */
            static void doInitialize();

            /** @a_infrastructure
             */
            static void doShutdown();

        private:
            void slot_buttontriggered(int i);
            QString userPermissionsDescription(int flags, int rflag, int wflag, int xflag);

            class ActionChange : public sh::actions::ActionActionItem
            {
            public:
                ActionChange(std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabUnixPermissions> tab);
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabUnixPermissions> tab;
            };

        };

    }
}

#endif // FILEPROPERTYDIALOGTABUNIXPERMISSIONS_H
