/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QObject>
#ifdef Q_OS_WIN
#include "actions/actionexecutioninfo.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/eurl.h"
#include "filesystem/operation.h"
#include "base/singletoninitializer.h"
#include "base/threadpool.h"
#include "base/mainthread.h"
#include "ui/filepropertydialogtabactionsview.h"
#include "filesystemhandlers/localfilesystemhandler.h"
#include "filepropertydialogtabwindows.h"
#define _WIN32_WINNT 0x0601
#include <SdkDdkVer.h>
#include <windows.h>
#include <shlobj.h>
#include <shlwapi.h>
#include <objbase.h>

REGISTER_STATICINIT(sh::filepropertydialogtabs, FilePropertyDialogTabWindows, "base")


sh::filepropertydialogtabs::FilePropertyDialogTabWindows::FilePropertyDialogTabWindows()
{
}

QString sh::filepropertydialogtabs::FilePropertyDialogTabWindows::title()
{
    return QObject::tr("&System");
}

QList<QString> sh::filepropertydialogtabs::FilePropertyDialogTabWindows::properties()
{
    QStringList result;
    result << "";
    return result;
}

void sh::filepropertydialogtabs::FilePropertyDialogTabWindows::populateWidget(int, sh::ui::FilePropertyDialogTabActionsView* widget)
{
    widget->setButtons(QStringList({QObject::tr("Open Windows Properties dialog")}));
    widget->onButtonTriggered([=](int i){slot_buttontriggered(i);});
}

void sh::filepropertydialogtabs::FilePropertyDialogTabWindows::slot_buttontriggered(int)
{
    std::shared_ptr<sh::filepropertydialogtabs::FilePropertyDialogTabWindows> me =
            std::static_pointer_cast<sh::filepropertydialogtabs::FilePropertyDialogTabWindows>(shared_from_this());
    sh::base::ThreadPool::enqueueForce([=](){
        std::shared_ptr<sh::actions::ActionActionItem> a = std::make_shared<ActionOpenWindowsPropertiesDialog>(me);
        a->initializeSync();
        a->execute();
    });
}

void sh::filepropertydialogtabs::FilePropertyDialogTabWindows::updateWidget(int, sh::ui::FilePropertyDialogTabActionsView* widget, sh::filesystem::Operation*)
{
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        widget->setVisible(nodes().length() == 1 && qobject_cast<sh::filesystemhandlers::LocalFilesystemHandler*>(nodes().first()->handler()));
    });
}

void sh::filepropertydialogtabs::FilePropertyDialogTabWindows::doInitialize()
{
    REGISTER_FILEPROPERTYDIALOGTAB(sh::ui::FilePropertyDialogTabFactory::REGISTER_FILEPROPERTYDIALOGTAB_INDEX_NORMAL-1, sh::filepropertydialogtabs::FilePropertyDialogTabWindows);
}

void sh::filepropertydialogtabs::FilePropertyDialogTabWindows::doShutdown()
{
}

sh::filepropertydialogtabs::FilePropertyDialogTabWindows::ActionOpenWindowsPropertiesDialog::ActionOpenWindowsPropertiesDialog(std::shared_ptr<FilePropertyDialogTabWindows> tab)
            : tab(tab)
{
}

void sh::filepropertydialogtabs::FilePropertyDialogTabWindows::ActionOpenWindowsPropertiesDialog::action(sh::actions::ActionExecutionInfo*)
{
    std::shared_ptr<sh::filesystem::FilesystemNode> n = tab->nodes().first();
    sh::filesystemhandlers::LocalFilesystemHandler* lh = qobject_cast<sh::filesystemhandlers::LocalFilesystemHandler*>(n->handler());
    QString local = lh->eurlToLocal(n->eurl());
    wchar_t filename[local.length()+1];
    filename[local.toWCharArray((wchar_t*)filename)] = 0;
    wchar_t verb[11];
    verb[QString("properties").toWCharArray((wchar_t*)verb)] = 0;
    SHELLEXECUTEINFO ShExecInfo = {0};
    ShExecInfo.cbSize = sizeof(SHELLEXECUTEINFO);
    ShExecInfo.fMask = SEE_MASK_INVOKEIDLIST;
    ShExecInfo.hwnd = NULL;
    ShExecInfo.lpVerb = verb;
    ShExecInfo.lpFile = filename;
    ShExecInfo.lpParameters = NULL;
    ShExecInfo.lpDirectory = NULL;
    ShExecInfo.nShow = SW_SHOW;
    ShExecInfo.hInstApp = NULL;
    ShellExecuteEx(&ShExecInfo);
    WaitForSingleObject(ShExecInfo.hProcess, INFINITE);
}

#endif
