/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QMutex>
#include "base/mainthread.h"
#include "exceptions/exception.h"
#include "filesystemhandlers/localfilesystemhandler.h"
#include "filesystemhandlers/sharcfilesystemhandler.h"
#include "filesystemhandler.h"
#include "filesystemhandlerregister.h"

REGISTER_SINGLETON(sh::filesystem, FilesystemHandlerRegister, "filesystem")


sh::filesystem::FilesystemHandlerRegister::FilesystemHandlerRegister()
{
}

void sh::filesystem::FilesystemHandlerRegister::addHandler(QString scheme, std::shared_ptr<sh::filesystem::FilesystemHandler> handler)
{
    QMutexLocker _l(&mutex);
    if (scheme.length() < 2)
        throw sh::exceptions::ArgumentException(sh::exceptions::ExceptionData().details("Scheme names must have a length of at least 2."));
    handlers.insert(scheme, handler);
}

std::shared_ptr<sh::filesystem::FilesystemHandler> sh::filesystem::FilesystemHandlerRegister::findHandler(QString scheme)
{
    QMutexLocker _l(&mutex);
    return handlers.value(scheme, 0);
}
