/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILESYSTEMMODELDIRECTORYTREEPROXY_H
#define FILESYSTEMMODELDIRECTORYTREEPROXY_H

#include "filesystemnode.h"
#include <memory>
#include <QSet>
#include <QSortFilterProxyModel>
#include "base/singletoninitializer.h"


namespace sh {
    namespace filesystem{

        /** \brief A filesystem proxy model for the directory tree.
         *
         * It has a special sorting and filtering behavior.
         *
         * Used internally, mostly for the user interface.
         */
        class FilesystemModelDirectoryTreeProxy : public QSortFilterProxyModel, public sh::base::Singleton
        {
            Q_OBJECT
            DECLARE_SINGLETON(FilesystemModelDirectoryTreeProxy)
        public:

            /** \brief Marks a node as visible in the directory tree, even if it is a hidden node.
             */
            void enforceVisibility(std::shared_ptr<FilesystemNode> node);

            /** \brief Unmarks a node for being visible even if hidden (reverses enforceVisibility()).
             */
            void deEnforceVisibility(std::shared_ptr<FilesystemNode> node);

        protected:
            void doInitialize() override;
            bool filterAcceptsRow (int row, const QModelIndex & parent) const;
            bool filterAcceptsColumn (int column, const QModelIndex & parent ) const;
            bool lessThan ( const QModelIndex & left, const QModelIndex & right ) const;
            mutable QList<std::weak_ptr<FilesystemNode>> forcedvisible;

        private:
            explicit FilesystemModelDirectoryTreeProxy();

        };

    }
}

#endif // FILESYSTEMMODELDIRECTORYTREEPROXY_H
