/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILESYSTEMMODELDIRECTORYTREEPROXYVISIBILITYENFORCEMENTS_H
#define FILESYSTEMMODELDIRECTORYTREEPROXYVISIBILITYENFORCEMENTS_H

#include "filesystemnode.h"
#include <memory>
#include <QMultiMap>
#include "base/singletoninitializer.h"


namespace sh {
    namespace filesystem{

        /** \brief
         * Maintains a list of currently visible directories (i.e. the current one in each view) and controls
         * enforced visibility of them in the directory tree.
         */
        class FilesystemModelDirectoryTreeProxyVisibilityEnforcements : public QObject, public sh::base::Singleton
        {
            DECLARE_SINGLETON(FilesystemModelDirectoryTreeProxyVisibilityEnforcements)
        public:

            /** \brief Called when a view enters the given directory node.
             */
            void nodeEnteredView(std::shared_ptr<FilesystemNode> node);

            /** \brief Called when a view leaves the given directory node.
             */
            void nodeLeftView(std::shared_ptr<FilesystemNode> node);

            /** \brief Called when a directory node is collapsed in the directory tree.
             */
            void nodeCollapsedInTree(std::shared_ptr<FilesystemNode> node);

        protected:
            void doInitialize() override;

        private:
            FilesystemModelDirectoryTreeProxyVisibilityEnforcements();
            QMultiMap<std::shared_ptr<FilesystemNode>, QObject*> _enterednodesrepresentatives;
            bool _addenforcement(std::shared_ptr<FilesystemNode> node, std::shared_ptr<FilesystemNode> hnode);
            QList<std::tuple<std::weak_ptr<FilesystemNode>, QList<std::shared_ptr<FilesystemNode>>>> hiddenForcedVisibles;
        };

    }
}

#endif // FILESYSTEMMODELDIRECTORYTREEPROXYVISIBILITYENFORCEMENTS_H
