/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILESYSTEMMODELFILEVIEWPROXY_H
#define FILESYSTEMMODELFILEVIEWPROXY_H

#include <QSortFilterProxyModel>


namespace sh {
    namespace filesystem{

        class DetailColumn;

        enum SizeFormatting {
            SizeFormattingModePrefixes,
            SizeFormattingModePlainBytes
        };

        /** \brief A filesystem proxy model for a fileview.
         *
         * It regards the sorting and filtering behavior set up for the connected fileview.
         *
         * Used internally, mostly for the user interface.
         */
        class FilesystemModelFileviewProxy : public QSortFilterProxyModel
        {
            Q_OBJECT
        public:
            /** \brief Constructed only indirectly.
             */
            FilesystemModelFileviewProxy(QModelIndex root, bool withtooltip, QObject *parent = 0);
            void setSizeFormattingMode(SizeFormatting mode);
            SizeFormatting getSizeFormattingMode() const;
            void setHiddenFilesVisible(bool v);
            bool getHiddenFilesVisible() const;
            QVariant data(const QModelIndex &index, int role=Qt::DisplayRole) const;
            void triggerReloadData();
            void setThumbnail(bool enabled, int size=32);
            bool getThumbnailEnabled();
            int getThumbnailSize();

        protected:
            bool filterAcceptsRow (int row, const QModelIndex & parent) const;
            bool lessThan ( const QModelIndex & left, const QModelIndex & right ) const;
            bool _hiddenFilesVisible = false;
            SizeFormatting _sizeFormattingMode;
            QPersistentModelIndex _root;
            bool _thumbnailmode = false;
            int _thumbnailsize = 0;
            bool _withtooltip;
        };

    }
}

#endif // FILESYSTEMMODELFILEVIEWPROXY_H
