/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILESYSTEMMODELSUBTREEPROXY_H
#define FILESYSTEMMODELSUBTREEPROXY_H

#include <memory>
#include <QAbstractItemModel>


namespace sh {
    namespace filesystem{

        class FilesystemModelFileviewProxy;
        class FilesystemModel;
        class FilesystemNode;

        /** \brief A filesystem proxy model for setting a new root node to an existing fileview proxy.
         *
         * Used internally, mostly for the user interface.
         */
        class FilesystemModelSubtreeProxy : public QAbstractItemModel
        {
            Q_OBJECT
        public:
            /** \brief Constructed only indirectly.
             */
            explicit FilesystemModelSubtreeProxy(QModelIndex root, sh::filesystem::FilesystemModelFileviewProxy *upperproxy);
            QVariant data(const QModelIndex &index, int role) const;
            Qt::ItemFlags flags(const QModelIndex &index) const;
            QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
            QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
            QModelIndex parent(const QModelIndex &index) const;
            int rowCount(const QModelIndex &parent = QModelIndex()) const;
            int columnCount(const QModelIndex &parent = QModelIndex()) const;
            QModelIndex mapFromSource(const QModelIndex mi) const;
            QModelIndex mapToSource(const QModelIndex mi) const;

        protected slots:
            void slot_rowsAboutToBeInserted(const QModelIndex & parent, int start, int end);
            void slot_rowsInserted(const QModelIndex & parent, int start, int end);
            void slot_columnsAboutToBeInserted(const QModelIndex & parent, int start, int end);
            void slot_columnsInserted(const QModelIndex & parent, int start, int end);
            void slot_rowsAboutToBeRemoved(const QModelIndex & parent, int start, int end);
            void slot_rowsRemoved(const QModelIndex & parent, int start, int end);
            void slot_columnsAboutToBeRemoved(const QModelIndex & parent, int start, int end);
            void slot_columnsRemoved(const QModelIndex & parent, int start, int end);
            void slot_dataChanged(const QModelIndex & topLeft, const QModelIndex & bottomRight);

        signals:
            void becomesInvalid(std::shared_ptr<sh::filesystem::FilesystemNode> bestValid);

        private:
            bool _inChangeTransaction;
            std::shared_ptr<sh::filesystem::FilesystemNode> _rootnode;
            QPersistentModelIndex _rootindex;
            sh::filesystem::FilesystemModel* mainmodel;
            sh::filesystem::FilesystemModelFileviewProxy* _upperproxy;
        };

    }
}

#endif // FILESYSTEMMODELSUBTREEPROXY_H
