/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GNOMEIODEVICESFILESYSTEMHANDLER_H
#define GNOMEIODEVICESFILESYSTEMHANDLER_H

#include "base/singletoninitializer.h"
#include "gnomeiofilesystemhandler.h"


#ifdef GIO

namespace sh{
    namespace filesystemhandlers {

        /** \brief
         * A filesystem handler for the GIO filesystem 'Devices' subtree.
         */
        class GnomeIODevicesFilesystemHandler : public sh::filesystemhandlers::GnomeIOFilesystemHandler, public sh::base::Singleton
        {
            friend class ActionMount;
            Q_OBJECT
            DECLARE_SINGLETON(GnomeIODevicesFilesystemHandler)
        public:
            ~GnomeIODevicesFilesystemHandler();

            void itemlist(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, sh::filesystem::FilesystemNodeType type, sh::filesystem::FilesystemNodeListEditor *list) override;
            sh::filesystem::FilesystemNodeType getType(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            QList<std::shared_ptr<sh::actions::AbstractActionItem> > getActions(const QList<std::shared_ptr<const sh::filesystem::Eurl>> eurls) override;

            /** \brief Refreshes the device list.
             */
            void refresh();

            /** \brief Requests a new device node.
             */
            void requestNewNode(std::shared_ptr<const sh::filesystem::Eurl> eurl, std::shared_ptr<sh::filesystem::FilesystemNode>* nnode);

        protected:
            void doInitialize() override;
            sh::filesystem::FilesystemNodeList* childlist;
            std::shared_ptr<sh::filesystem::FilesystemNode> rootnode;
            std::shared_ptr<sh::filesystem::FilesystemNode> _getMountedNode(QString gvolumeuuid, QString grooturl, std::shared_ptr<sh::filesystem::FilesystemNode> parentnode);
            std::shared_ptr<sh::filesystem::FilesystemNode> _getUnmountedNode(QString gvolumeuuid, std::shared_ptr<sh::filesystem::FilesystemNode> parentnode);
            QHash<QString, QString> devicenames;
            QMutex masternodeslock;
            void* volumemonitor;

            struct GnomeIODevice
            {
                GnomeIODevice(QString name, QString displayname, QString url, bool isMounted);
                QString name;
                QString displayname;
                QString url;
                bool isMounted;
            };

        private:
            explicit GnomeIODevicesFilesystemHandler();

        };

    }
}

#endif

#endif // GNOMEIODEVICESFILESYSTEMHANDLER_H
