/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LOCALFILESYSTEMHANDLER_H
#define LOCALFILESYSTEMHANDLER_H

#include "base/singletoninitializer.h"
#include "filesystem/filesystemhandler.h"


namespace sh{
    namespace filesystemhandlers {

        /** \brief
         * A filesystem handler for the local filesystem.
         */
        class LocalFilesystemHandler : public sh::filesystem::FilesystemHandler, public sh::base::Singleton
        {
            Q_OBJECT
            DECLARE_SINGLETON(LocalFilesystemHandler)
        public:
            void itemlist(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, sh::filesystem::FilesystemNodeType type, sh::filesystem::FilesystemNodeListEditor *list) override;
            void configureItems(sh::filesystem::Operation* op, QList<std::shared_ptr<sh::filesystem::FilesystemNode> > nodes) override;
            sh::filesystem::FilesystemNodeType getType(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            qint64 getSize(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            QDateTime getMtime(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            void setMtime(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, QDateTime mtime) override;
            QString getMimetype(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            QString getLinkTarget(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            bool canGetFileContent(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            std::shared_ptr<QIODevice> getFileContent(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            bool canCreateDirectory(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            void createDirectory(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            bool canCreateLink(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            void createLink(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, QString target) override;
            bool canCreateFile(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            void createFile(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, QIODevice* content, HandlerTransfer* handlertransfer) override;
            bool canDeleteItem(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            void deleteItem(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            bool canRenameItem(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> src) override;
            void renameItem(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> src, QString destpath) override;
            QList<std::shared_ptr<sh::actions::AbstractActionItem> > getActions(const QList<std::shared_ptr<const sh::filesystem::Eurl>> eurls) override;
            QList<QString> listExtendedAttributes(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            quint64 getExtendedAttributeSize(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, QString attribute) override;
            QByteArray getExtendedAttribute(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, QString attribute) override;
            void setExtendedAttribute(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, QString attribute, QByteArray value) override;
            void removeExtendedAttribute(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, QString attribute) override;
            QMap<QString, QString> getCustomAttributes(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) override;
            void setCustomAttribute(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, QString attribute, QString value) override;
            bool requiresResolvedLinks() override;
            bool isWellKnownScheme() override;
            QString eurlToLocal(std::shared_ptr<const sh::filesystem::Eurl> eurl);
            std::shared_ptr<const sh::filesystem::Eurl> localToEurl(const QString local);

        protected:
            LocalFilesystemHandler();
            void doInitialize() override;
            std::shared_ptr<sh::filesystem::FilesystemNode> rootnode;
            void createRootNode();

        };

    }
}

#endif // LOCALFILESYSTEMHANDLER_H
