/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WINDOWSNETWORKFILESYSTEMHANDLER_H
#define WINDOWSNETWORKFILESYSTEMHANDLER_H

#include "localfilesystemhandler.h"
#include "base/singletoninitializer.h"


namespace sh{
    namespace filesystemhandlers {

        class WindowsNetworkFilesystemHandler : public sh::filesystemhandlers::LocalFilesystemHandler
        {
            Q_OBJECT
#ifdef Q_OS_WIN
            DECLARE_SINGLETON(WindowsNetworkFilesystemHandler)
        public:

            virtual void itemlist(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl, sh::filesystem::FilesystemNodeType type, sh::filesystem::FilesystemNodeListEditor *list);
            virtual sh::filesystem::FilesystemNodeType getType(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl);

            virtual std::shared_ptr<const sh::filesystem::Eurl> localToEurl(const QString local);
            virtual QString eurlToLocal(std::shared_ptr<const sh::filesystem::Eurl> eurl);

        protected:
            void createRootNode();
            void doInitialize() override;
            void doShutdown() override;

        private:
            explicit WindowsNetworkFilesystemHandler();
            std::shared_ptr<sh::filesystem::FilesystemNode> rootnode;
#endif
        };

    }
}

#endif // WINDOWSNETWORKFILESYSTEMHANDLER_H
