/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "actions/actionexecutionuserfeedback.h"
#include "api/apipaneldetailfactory.h"
#include "api/apiglobalobject.h"
#include "api/apidetailcolumn.h"
#include "api/apifilesystemhandler.h"
#include "api/apireaddatadevice.h"
#include "api/apiaction.h"
#include "api/apisetting.h"
#include "api/apipredicatedactionfactory.h"
#include "api/apifilesystemoperationprogressmonitor.h"
#include "api/apifilepropertydialogtab.h"
#include "api/apisearchcriterion.h"
#include "api/apithread.h"
#include "api/apitimer.h"
#include "api/apithumbnailprovider.h"
#include "filesystem/eurl.h"
#include "filesystem/filesystemoperation.h"
#include "filesystem/operation.h"
#include "filesystem/detailcolumn.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/filesystemnodelist.h"
#include "configuration/configurationvalue.h"
#include "ui/mainwindow.h"
#include "ui/filepropertydialog.h"
#include "tools/bookmarkmanager.h"
#include "api.h"
#include "pythonscriptinterpreter.h"


sh::scripting::Api::Api()
{
    sh::scripting::ApiClass* clsGlobals;
    registerClass("_Globals", &clsGlobals, typeid(sh::scripting::api::ApiGlobalObject));
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "getEurlFromString", std::shared_ptr<const sh::filesystem::Eurl>, sh::scripting::api::ApiGlobalObject::getEurlFromString, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "log_debug", void, sh::scripting::api::ApiGlobalObject::logDebug, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "log_info", void, sh::scripting::api::ApiGlobalObject::logInfo, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "log_warning", void, sh::scripting::api::ApiGlobalObject::logWarning, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "log_error", void, sh::scripting::api::ApiGlobalObject::logError, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "is_debug_build", bool, sh::scripting::api::ApiGlobalObject::isDebugBuild)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "register_predicatedactionfactory", void, sh::scripting::api::ApiGlobalObject::register_predicatedactionfactory, std::shared_ptr<sh::scripting::api::ApiActionFactory>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_ActionActionItem", std::shared_ptr<sh::scripting::api::ApiActionActionItem>, sh::scripting::api::ApiGlobalObject::create_ActionActionItem, QString, bool, QString, int)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_SubmenuActionItem", std::shared_ptr<sh::scripting::api::ApiSubmenuActionItem>, sh::scripting::api::ApiGlobalObject::create_SubmenuActionItem, QString, bool, QString, int)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_FilesystemHandler", std::shared_ptr<sh::scripting::api::ApiFilesystemHandler>, sh::scripting::api::ApiGlobalObject::create_FilesystemHandler)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_DetailColumn", std::shared_ptr<sh::scripting::api::ApiDetailColumn>, sh::scripting::api::ApiGlobalObject::create_DetailColumn, QString, QString, int, bool, int, bool)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_FilePropertyDialogTab", std::shared_ptr<sh::scripting::api::ApiFilePropertyDialogTab>, sh::scripting::api::ApiGlobalObject::create_FilePropertyDialogTab, QString, QList<QString>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_FilePropertyDialogTabFactory", std::shared_ptr<sh::scripting::api::ApiFilePropertyDialogTabFactory>, sh::scripting::api::ApiGlobalObject::create_FilePropertyDialogTabFactory)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_ActionFactory", std::shared_ptr<sh::scripting::api::ApiActionFactory>, sh::scripting::api::ApiGlobalObject::create_ActionFactory, QString, QList<std::shared_ptr<sh::actions::Predicate>>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createSetting", std::shared_ptr<sh::scripting::api::ApiSetting>, sh::scripting::api::ApiGlobalObject::createSetting, QString, QString, int, bool, bool, bool)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_DetailFactorySingle", std::shared_ptr<sh::scripting::api::ApiPanelDetailFactorySingle>, sh::scripting::api::ApiGlobalObject::create_DetailFactorySingle, int, int)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_SearchCriterion", std::shared_ptr<sh::scripting::api::ApiSearchCriterion>, sh::scripting::api::ApiGlobalObject::create_SearchCriterion, std::shared_ptr<sh::scripting::api::ApiSearchCriterionFactory>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_SearchCriterionFactory", std::shared_ptr<sh::scripting::api::ApiSearchCriterionFactory>, sh::scripting::api::ApiGlobalObject::create_SearchCriterionFactory, QString, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_ThumbnailProvider", std::shared_ptr<sh::scripting::api::ApiThumbnailProvider>, sh::scripting::api::ApiGlobalObject::create_ThumbnailProvider)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_Thread", std::shared_ptr<sh::scripting::api::ApiThread>, sh::scripting::api::ApiGlobalObject::create_Thread)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_Timer", std::shared_ptr<sh::scripting::api::ApiTimer>, sh::scripting::api::ApiGlobalObject::create_Timer)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_DetailFactoryMulti", std::shared_ptr<sh::scripting::api::ApiPanelDetailFactoryMulti>, sh::scripting::api::ApiGlobalObject::create_DetailFactoryMulti, int, int)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "registerFilesystemHandler", void, sh::scripting::api::ApiGlobalObject::_register_FilesystemHandler, std::shared_ptr<sh::scripting::api::ApiFilesystemHandler>, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createFilesystemNode", std::shared_ptr<sh::filesystem::FilesystemNode>, sh::scripting::api::ApiGlobalObject::_createFilesystemNode, std::shared_ptr<sh::filesystem::Eurl>, sh::scripting::api::ApiFilesystemHandler*, int, std::shared_ptr<sh::filesystem::FilesystemNode>, bool, bool, bool)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_FilesystemOperationProgressMonitor", std::shared_ptr<sh::scripting::api::ApiFilesystemOperationProgressMonitor>, sh::scripting::api::ApiGlobalObject::create_FilesystemOperationProgressMonitor, sh::actions::ActionExecutionInfo*)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "getOrCreateFilesystemNode", std::shared_ptr<sh::filesystem::FilesystemNode>, sh::scripting::api::ApiGlobalObject::_getOrCreateFilesystemNode, std::shared_ptr<sh::filesystem::Eurl>, sh::scripting::api::ApiFilesystemHandler*, int, std::shared_ptr<sh::filesystem::FilesystemNode>, bool, bool, bool)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "modelRootNode", std::shared_ptr<sh::filesystem::FilesystemNode>, sh::scripting::api::ApiGlobalObject::modelRootNode)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "findFilesystemNodesForEurl", QList<std::shared_ptr<sh::filesystem::FilesystemNode>>, sh::scripting::api::ApiGlobalObject::_findFilesystemNodesForEurl, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "tryGetFilesystemNodesForEurl", QList<std::shared_ptr<sh::filesystem::FilesystemNode>>, sh::scripting::api::ApiGlobalObject::_tryGetFilesystemNodesForEurl, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "refreshData", void, sh::scripting::api::ApiGlobalObject::refreshData, std::shared_ptr<const sh::filesystem::Eurl>, bool, bool)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createEurl", std::shared_ptr<sh::filesystem::Eurl>, sh::scripting::api::ApiGlobalObject::createEurl, QString, QString, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createOperation", std::shared_ptr<sh::filesystem::Operation>, sh::scripting::api::ApiGlobalObject::createOperation)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createArgumentException", sh::scripting::StringMap<QString>, sh::scripting::api::ApiGlobalObject::createArgumentException)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createIOException", sh::scripting::StringMap<QString>, sh::scripting::api::ApiGlobalObject::createIOException)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createProgramException", sh::scripting::StringMap<QString>, sh::scripting::api::ApiGlobalObject::createProgramException)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createRuntimeException", sh::scripting::StringMap<QString>, sh::scripting::api::ApiGlobalObject::createRuntimeException)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "createException", sh::scripting::StringMap<QString>, sh::scripting::api::ApiGlobalObject::createException)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_ReadDataDevice", std::shared_ptr<sh::scripting::api::ApiReadDataDevice>, sh::scripting::api::ApiGlobalObject::_createReadDataDevice)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_file", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_file)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_directory", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_directory)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_link", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_link)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_unknown", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_unknown)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_firsttype", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_firsttype)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_lastphysicaltype", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_lastphysicaltype)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_none", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_none)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_specialtreeonlydirectory", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_specialtreeonlydirectory)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filesystemnodetype_lasttype", int, sh::scripting::api::ApiGlobalObject::filesystemnodetype_lasttype)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "actiondefaultprecedencevalues_openfile", int, sh::scripting::api::ApiGlobalObject::actiondefaultprecedencevalues_openfile)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "actiondefaultprecedencevalues_builtinspecialopen", int, sh::scripting::api::ApiGlobalObject::actiondefaultprecedencevalues_builtinspecialopen)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "messageboxbutton_ok", int, sh::scripting::api::ApiGlobalObject::messageboxbutton_ok)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "messageboxbutton_continue", int, sh::scripting::api::ApiGlobalObject::messageboxbutton_continue)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "messageboxbutton_cancel", int, sh::scripting::api::ApiGlobalObject::messageboxbutton_cancel)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "messageboxbutton_retry", int, sh::scripting::api::ApiGlobalObject::messageboxbutton_retry)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "messageboxbutton_yes", int, sh::scripting::api::ApiGlobalObject::messageboxbutton_yes)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "messageboxbutton_no", int, sh::scripting::api::ApiGlobalObject::messageboxbutton_no)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "displayindex_core", int, sh::scripting::api::ApiGlobalObject::displayindex_core)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "displayindex_interesting", int, sh::scripting::api::ApiGlobalObject::displayindex_interesting)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "displayindex_exotic", int, sh::scripting::api::ApiGlobalObject::displayindex_exotic)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "configurationcategory_gui", int, sh::scripting::api::ApiGlobalObject::configurationcategory_gui)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "configurationcategory_behavior", int, sh::scripting::api::ApiGlobalObject::configurationcategory_behavior)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "configurationcategory_externaltools", int, sh::scripting::api::ApiGlobalObject::configurationcategory_externaltools)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "settinggroup_global", int, sh::scripting::api::ApiGlobalObject::settinggroup_global)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "settinggroup_gui", int, sh::scripting::api::ApiGlobalObject::settinggroup_gui)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "settinggroup_filehandling", int, sh::scripting::api::ApiGlobalObject::settinggroup_filehandling)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "settinggroup_dataview", int, sh::scripting::api::ApiGlobalObject::settinggroup_dataview)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "settinggroup_behavior", int, sh::scripting::api::ApiGlobalObject::settinggroup_behavior)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "settinggroup_special", int, sh::scripting::api::ApiGlobalObject::settinggroup_special)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "paneldetail_positionindex_veryinteresting", int, sh::scripting::api::ApiGlobalObject::paneldetail_positionindex_veryinteresting)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "paneldetail_positionindex_interesting", int, sh::scripting::api::ApiGlobalObject::paneldetail_positionindex_interesting)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "paneldetail_positionindex_exotic", int, sh::scripting::api::ApiGlobalObject::paneldetail_positionindex_exotic)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "register_ConfigurationValueInt", std::shared_ptr<sh::configuration::ConfigurationValue>, sh::scripting::api::ApiGlobalObject::register_ConfigurationValueInt, QString, int, QString, int, QString, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "register_ConfigurationValueFloat", std::shared_ptr<sh::configuration::ConfigurationValue>, sh::scripting::api::ApiGlobalObject::register_ConfigurationValueFloat, QString, int, QString, int, QString, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "register_ConfigurationValueBool", std::shared_ptr<sh::configuration::ConfigurationValue>, sh::scripting::api::ApiGlobalObject::register_ConfigurationValueBool, QString, bool, QString, int, QString, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "register_ConfigurationValueString", std::shared_ptr<sh::configuration::ConfigurationValue>, sh::scripting::api::ApiGlobalObject::register_ConfigurationValueString, QString, QString, QString, int, QString, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "operationstep_conflictresolution_mergedirectories", int, sh::scripting::api::ApiGlobalObject::operationstep_conflictresolution_mergedirectories)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "operationstep_conflictresolution_overwritedestination", int, sh::scripting::api::ApiGlobalObject::operationstep_conflictresolution_overwritedestination)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "operationstep_conflictresolution_renamedestinationbefore", int, sh::scripting::api::ApiGlobalObject::operationstep_conflictresolution_renamedestinationbefore)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "operationstep_conflictresolution_skip", int, sh::scripting::api::ApiGlobalObject::operationstep_conflictresolution_skip)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "operationstep_conflictresolution_unresolved", int, sh::scripting::api::ApiGlobalObject::operationstep_conflictresolution_unresolved)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "operationstep_conflictresolution_indirect", int, sh::scripting::api::ApiGlobalObject::operationstep_conflictresolution_indirect)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "operationstep_conflictresolution_usedifferentdestinationname", int, sh::scripting::api::ApiGlobalObject::operationstep_conflictresolution_usedifferentdestinationname)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filepropertydialogtab_index_core", int, sh::scripting::api::ApiGlobalObject::filepropertydialogtab_index_core)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filepropertydialogtab_index_veryimportant", int, sh::scripting::api::ApiGlobalObject::filepropertydialogtab_index_veryimportant)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filepropertydialogtab_index_important", int, sh::scripting::api::ApiGlobalObject::filepropertydialogtab_index_important)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filepropertydialogtab_index_normal", int, sh::scripting::api::ApiGlobalObject::filepropertydialogtab_index_normal)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filepropertydialogtab_index_exotic", int, sh::scripting::api::ApiGlobalObject::filepropertydialogtab_index_exotic)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filepropertydialogtab_propertytype_string", int, sh::scripting::api::ApiGlobalObject::filepropertydialogtab_propertytype_string)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filepropertydialogtab_propertytype_stringmap", int, sh::scripting::api::ApiGlobalObject::filepropertydialogtab_propertytype_stringmap)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "filepropertydialogtab_propertytype_icontextbanner", int, sh::scripting::api::ApiGlobalObject::filepropertydialogtab_propertytype_icontextbanner)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "searchcriterionfactory_index_core", int, sh::scripting::api::ApiGlobalObject::searchcriterionfactory_index_core)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "searchcriterionfactory_index_normal", int, sh::scripting::api::ApiGlobalObject::searchcriterionfactory_index_normal)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "searchcriterionfactory_index_exotic", int, sh::scripting::api::ApiGlobalObject::searchcriterionfactory_index_exotic)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "thumbnailprovider_index_core", int, sh::scripting::api::ApiGlobalObject::thumbnailprovider_index_core)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "thumbnailprovider_index_normal", int, sh::scripting::api::ApiGlobalObject::thumbnailprovider_index_normal)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "thumbnailprovider_index_exotic", int, sh::scripting::api::ApiGlobalObject::thumbnailprovider_index_exotic)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "thumbnailprovider_index_fallback", int, sh::scripting::api::ApiGlobalObject::thumbnailprovider_index_fallback)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "shallotDataDir", QString, sh::scripting::api::ApiGlobalObject::shallotDataDir)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "shallotBuiltinPluginDir", QString, sh::scripting::api::ApiGlobalObject::shallotBuiltinPluginDir)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "shallotSystemPluginDir", QString, sh::scripting::api::ApiGlobalObject::shallotSystemPluginDir)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "shallotUserPluginDir", QString, sh::scripting::api::ApiGlobalObject::shallotUserPluginDir)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "shallotLanguage", QString, sh::scripting::api::ApiGlobalObject::shallotLanguage)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "registerSetting", void, sh::scripting::api::ApiGlobalObject::registerSetting, std::shared_ptr<sh::settings::Setting>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "findDetailColumnByName", std::shared_ptr<sh::filesystem::DetailColumn>, sh::scripting::api::ApiGlobalObject::findDetailColumnByName, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "registerPanelDetailFactorySingle", void, sh::scripting::api::ApiGlobalObject::registerPanelDetailFactorySingle, std::shared_ptr<sh::scripting::api::ApiPanelDetailFactorySingle>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "registerPanelDetailFactoryMulti", void, sh::scripting::api::ApiGlobalObject::registerPanelDetailFactoryMulti, std::shared_ptr<sh::scripting::api::ApiPanelDetailFactoryMulti>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "registerFilePropertyDialogTabFactory", void, sh::scripting::api::ApiGlobalObject::registerFilePropertyDialogTabFactory, int, std::shared_ptr<sh::scripting::api::ApiFilePropertyDialogTabFactory>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "registerTransferrableDetailColumn", void, sh::scripting::api::ApiGlobalObject::registerTransferrableDetailColumn, int, std::shared_ptr<sh::filesystem::DetailColumn>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "mainwindow", sh::ui::MainWindow*, sh::scripting::api::ApiGlobalObject::mainwindow)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "open_items", void, sh::scripting::api::ApiGlobalObject::openItems, QList<std::shared_ptr<const sh::filesystem::Eurl>>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "registerSearchCriterionFactory", void, sh::scripting::api::ApiGlobalObject::registerSearchCriterionFactory, int, std::shared_ptr<sh::scripting::api::ApiSearchCriterionFactory>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "registerThumbnailProvider", void, sh::scripting::api::ApiGlobalObject::registerThumbnailProvider, int, std::shared_ptr<sh::scripting::api::ApiThumbnailProvider>)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "bookmarkManager", sh::tools::BookmarkManager*, sh::scripting::api::ApiGlobalObject::bookmarkManager)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_OnDirectoriesPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_OnDirectoriesPredicate)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_OnFilesPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_OnFilesPredicate)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_OnLinksPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_OnLinksPredicate)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_OnSingleEntrySelectionPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_OnSingleEntrySelectionPredicate)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_DontResolveLinksPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_DontResolveLinksPredicate)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_HideOnCurrentDirectoryLevelPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_HideOnCurrentDirectoryLevelPredicate)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_HideOnSelectionLevelPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_HideOnSelectionLevelPredicate)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_ByRegExpPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_ByRegExpPredicate, QString)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_KeyShortcutPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_KeyShortcutPredicate, QString, bool)
    REGISTERMETHOD(clsGlobals, sh::scripting::api::ApiGlobalObject, "create_PositionIndexPredicate", std::shared_ptr<sh::actions::Predicate>, sh::scripting::api::ApiGlobalObject::create_PositionIndexPredicate, int)

    registerRootObjectMember("internalstuff", clsGlobals, sh::scripting::api::ApiGlobalObject::instance().get());

    sh::scripting::ApiClass* clsMainWindow;
    registerClass("MainWindow", &clsMainWindow, typeid(sh::ui::MainWindow));
    REGISTERHELPERMETHOD(clsMainWindow, sh::ui::MainWindow, "jump_to_eurl", void, sh::scripting::api::ApiHelperMethods::MainWindow_jumpToEurl, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERHELPERMETHOD(clsMainWindow, sh::ui::MainWindow, "get_current_directory_node", std::shared_ptr<sh::filesystem::FilesystemNode>, sh::scripting::api::ApiHelperMethods::MainWindow_getCurrentDirectoryNode)

    sh::scripting::ApiClass* clsEurl;
    registerClass("Eurl", &clsEurl, typeid(sh::filesystem::Eurl));
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "as_string", QString, sh::filesystem::Eurl::asString)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "basename", QString, sh::filesystem::Eurl::basename)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "parent_segment", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Eurl::parentSegment)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "with_appended_segment", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Eurl::withAppendedSegment, QString)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "with_appended_segments", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Eurl::withAppendedSegments, QString)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "enwrap_with_outer_url", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Eurl::enwrapWithOuterUrl, QString, QString, QString)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "path", QString, sh::filesystem::Eurl::path)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "root", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Eurl::root)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "hostname", QString, sh::filesystem::Eurl::hostname)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "scheme", QString, sh::filesystem::Eurl::scheme)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "outer_url", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Eurl::outerUrl)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "outermost_inner_eurl", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Eurl::outermostInnerEurl)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "has_inner_urls", bool, sh::filesystem::Eurl::hasInnerUrls)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "outer_url_is_root_directory", bool, sh::filesystem::Eurl::outerUrlIsRootDirectory)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "has_parent_segment", bool, sh::filesystem::Eurl::hasParentSegment)
    REGISTERMETHODCONST(clsEurl, sh::filesystem::Eurl, "is_prefix_of", bool, sh::filesystem::Eurl::isPrefixOf, std::shared_ptr<const sh::filesystem::Eurl>)

    sh::scripting::ApiClass* clsOperation;
    registerClass("Operation", &clsOperation, typeid(sh::filesystem::Operation));
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "get_free_cachespace", quint64, sh::filesystem::Operation::getFreeCacheSpace)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "fetch_container_file", QString, sh::filesystem::Operation::fetchContainerFile, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "fetch_file", QString, sh::filesystem::Operation::fetchFile, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "fetch_container_file2", QString, sh::filesystem::Operation::fetchContainerFile, std::shared_ptr<const sh::filesystem::Eurl>, QString)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "fetch_file2", QString, sh::filesystem::Operation::fetchFile, std::shared_ptr<const sh::filesystem::Eurl>, QString)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "abort", void, sh::filesystem::Operation::abort)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "commit", void, sh::filesystem::Operation::commit)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "enable_detailscache", void, sh::filesystem::Operation::enableDetailsCache)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "is_detailscache_enabled", bool, sh::filesystem::Operation::isDetailsCacheEnabled)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "store_detail_in_cache", void, sh::filesystem::Operation::storeDetailInCache, std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::DetailColumn*, QVariant)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "get_detail_from_cache", QVariant, sh::filesystem::Operation::getDetailFromCache, std::shared_ptr<const sh::filesystem::Eurl>, const sh::filesystem::DetailColumn*)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "set_custom_data", void, sh::filesystem::Operation::setCustomData, std::shared_ptr<const sh::filesystem::Eurl>, QString, QVariant)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "get_custom_data", QVariant, sh::filesystem::Operation::getCustomData, std::shared_ptr<const sh::filesystem::Eurl>, QString)
    REGISTERMETHOD(clsOperation, sh::filesystem::Operation, "filesystem", sh::filesystem::FilesystemOperation*, sh::filesystem::Operation::filesystem)

    sh::scripting::ApiClass* clsDetailColumn;
    registerClass("DetailColumn", &clsDetailColumn, typeid(sh::filesystem::DetailColumn));
    REGISTERMETHOD(clsDetailColumn, sh::filesystem::DetailColumn, "compute_value", QVariant, sh::filesystem::DetailColumn::computeValue, std::shared_ptr<sh::filesystem::FilesystemNode>, sh::filesystem::Operation*)
    REGISTERMETHOD(clsDetailColumn, sh::filesystem::DetailColumn, "apply_value", void, sh::filesystem::DetailColumn::applyValueByEurl, std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Operation*, QVariant)

    sh::scripting::ApiClass* cls_DetailColumn;
    registerClass("ApiDetailColumn", &cls_DetailColumn, typeid(sh::scripting::api::ApiDetailColumn));
    REGISTERSCRIPTMETHOD(cls_DetailColumn, sh::scripting::api::ApiDetailColumn, "set_determinevalue", QString, sh::scripting::api::ApiDetailColumn::_determineValue, std::shared_ptr<sh::filesystem::FilesystemNode>, sh::filesystem::Operation*)
    REGISTERSCRIPTMETHOD(cls_DetailColumn, sh::scripting::api::ApiDetailColumn, "set_applyvalue", void, sh::scripting::api::ApiDetailColumn::_applyValue, std::shared_ptr<sh::filesystem::Eurl>, sh::filesystem::Operation*, QString)
    REGISTERMETHOD(cls_DetailColumn, sh::filesystem::DetailColumn, "compute_value", QVariant, sh::filesystem::DetailColumn::computeValue, std::shared_ptr<sh::filesystem::FilesystemNode>, sh::filesystem::Operation*)
    REGISTERMETHOD(cls_DetailColumn, sh::filesystem::DetailColumn, "apply_value", void, sh::filesystem::DetailColumn::applyValueByEurl, std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::Operation*, QVariant)

    sh::scripting::ApiClass* cls_FilesystemHandler;
    registerClass("_FilesystemHandler", &cls_FilesystemHandler, typeid(sh::scripting::api::ApiFilesystemHandler));
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getmimetype", QString, sh::scripting::api::ApiFilesystemHandler::_getMimetype, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_gettype", int, sh::scripting::api::ApiFilesystemHandler::_getType, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getsize", qint64, sh::scripting::api::ApiFilesystemHandler::_getSize, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getmtime", double, sh::scripting::api::ApiFilesystemHandler::_getMtime, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_setmtime", void, sh::scripting::api::ApiFilesystemHandler::_setMtime, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, double)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getlinktarget", QString, sh::scripting::api::ApiFilesystemHandler::_getLinkTarget, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_listextendedattributes", QList<QString>, sh::scripting::api::ApiFilesystemHandler::_listExtendedAttributes, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getextendedattributesize", quint64, sh::scripting::api::ApiFilesystemHandler::_getExtendedAttributeSize, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, QString attribute)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getextendedattribute", QByteArray, sh::scripting::api::ApiFilesystemHandler::_getExtendedAttribute, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, QString attribute)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_setextendedattribute", void, sh::scripting::api::ApiFilesystemHandler::_setExtendedAttribute, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, QString attribute, QByteArray value)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_removeextendedattribute", void, sh::scripting::api::ApiFilesystemHandler::_removeExtendedAttribute, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, QString attribute)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getcustomattributes", QList<QString>, sh::scripting::api::ApiFilesystemHandler::_getCustomAttributes, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_setcustomattribute", void, sh::scripting::api::ApiFilesystemHandler::_setCustomAttribute, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, QString attribute, QString value)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_cancreatedirectory", bool, sh::scripting::api::ApiFilesystemHandler::_canCreateDirectory, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_createdirectory", void, sh::scripting::api::ApiFilesystemHandler::_createDirectory, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_cancreatelink", bool, sh::scripting::api::ApiFilesystemHandler::_canCreateLink, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_createlink", void, sh::scripting::api::ApiFilesystemHandler::_createLink, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, QString)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_cancreatefile", bool, sh::scripting::api::ApiFilesystemHandler::_canCreateFile, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_candeleteitem", bool, sh::scripting::api::ApiFilesystemHandler::_canDeleteItem, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_deleteitem", void, sh::scripting::api::ApiFilesystemHandler::_deleteItem, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_canrenameitem", bool, sh::scripting::api::ApiFilesystemHandler::_canRenameItem, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_renameitem", void, sh::scripting::api::ApiFilesystemHandler::_renameItem, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, QString)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_itemlist", void, sh::scripting::api::ApiFilesystemHandler::_itemlist, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, int, sh::filesystem::FilesystemNodeListEditor*)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_configureitems", void, sh::scripting::api::ApiFilesystemHandler::_configureItems, sh::filesystem::Operation*, QList<std::shared_ptr<sh::filesystem::FilesystemNode>>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_cangetfilecontent", bool, sh::scripting::api::ApiFilesystemHandler::_canGetFileContent, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getfilecontent", std::shared_ptr<QIODevice>, sh::scripting::api::ApiFilesystemHandler::_getFileContent, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_createfile", void, sh::scripting::api::ApiFilesystemHandler::_createFile, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>, QIODevice*, sh::filesystem::FilesystemHandler::HandlerTransfer*)
    REGISTERSCRIPTMETHOD(cls_FilesystemHandler, sh::scripting::api::ApiFilesystemHandler, "set_getactions", QList<std::shared_ptr<sh::actions::AbstractActionItem>>, sh::scripting::api::ApiFilesystemHandler::_getActions, QList<std::shared_ptr<sh::filesystem::Eurl>>)

    sh::scripting::ApiClass* cls_ActionActionItem;
    registerClass("_ActionActionItem", &cls_ActionActionItem, typeid(sh::scripting::api::ApiActionActionItem));
    REGISTERMETHOD(cls_ActionActionItem, sh::scripting::api::ApiActionActionItem, "_execute", void, sh::scripting::api::ApiActionActionItem::_execute)
    REGISTERMETHOD(cls_ActionActionItem, sh::scripting::api::ApiActionActionItem, "_initializeSync", void, sh::scripting::api::ApiActionActionItem::_initializeSync)
    REGISTERHELPERMETHOD(cls_ActionActionItem, sh::scripting::api::ApiActionActionItem, "set_enabled", void, sh::scripting::api::ApiHelperMethods::ApiActionActionItem_setEnabled, bool)
    REGISTERHELPERMETHOD(cls_ActionActionItem, sh::scripting::api::ApiActionActionItem, "enabled", bool, sh::scripting::api::ApiHelperMethods::ApiActionActionItem_enabled)
    REGISTERHELPERMETHOD(cls_ActionActionItem, sh::scripting::api::ApiActionActionItem, "set_visible", void, sh::scripting::api::ApiHelperMethods::ApiActionActionItem_setVisible, bool)
    REGISTERHELPERMETHOD(cls_ActionActionItem, sh::scripting::api::ApiActionActionItem, "visible", bool, sh::scripting::api::ApiHelperMethods::ApiActionActionItem_visible)
    REGISTERSCRIPTMETHOD(cls_ActionActionItem, sh::scripting::api::ApiActionActionItem, "set_initialize", void, sh::scripting::api::ApiActionActionItem::_initialize)
    REGISTERSCRIPTMETHOD(cls_ActionActionItem, sh::scripting::api::ApiActionActionItem, "set_action", void, sh::scripting::api::ApiActionActionItem::_action, sh::actions::ActionExecutionInfo*)

    sh::scripting::ApiClass* cls_SubmenuActionItem;
    registerClass("_SubmenuActionItem", &cls_SubmenuActionItem, typeid(sh::scripting::api::ApiSubmenuActionItem));
    REGISTERHELPERMETHOD(cls_SubmenuActionItem, sh::scripting::api::ApiSubmenuActionItem, "set_subitems", void, sh::scripting::api::ApiHelperMethods::ApiSubmenuActionItem_setSubitems, QList<std::shared_ptr<sh::actions::AbstractActionItem>>)
    REGISTERHELPERMETHOD(cls_SubmenuActionItem, sh::scripting::api::ApiSubmenuActionItem, "set_enabled", void, sh::scripting::api::ApiHelperMethods::ApiSubmenuActionItem_setEnabled, bool)
    REGISTERHELPERMETHOD(cls_SubmenuActionItem, sh::scripting::api::ApiSubmenuActionItem, "enabled", bool, sh::scripting::api::ApiHelperMethods::ApiSubmenuActionItem_enabled)
    REGISTERHELPERMETHOD(cls_SubmenuActionItem, sh::scripting::api::ApiSubmenuActionItem, "set_visible", void, sh::scripting::api::ApiHelperMethods::ApiSubmenuActionItem_setVisible, bool)
    REGISTERHELPERMETHOD(cls_SubmenuActionItem, sh::scripting::api::ApiSubmenuActionItem, "visible", bool, sh::scripting::api::ApiHelperMethods::ApiSubmenuActionItem_visible)
    REGISTERSCRIPTMETHOD(cls_SubmenuActionItem, sh::scripting::api::ApiSubmenuActionItem, "set_initialize", void, sh::scripting::api::ApiSubmenuActionItem::_initialize)

    sh::scripting::ApiClass* clsFilesystemNode;
    registerClass("FilesystemNode", &clsFilesystemNode, typeid(sh::filesystem::FilesystemNode));
    REGISTERMETHOD(clsFilesystemNode, sh::filesystem::FilesystemNode, "eurl", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::FilesystemNode::eurl)
    REGISTERMETHOD(clsFilesystemNode, sh::filesystem::FilesystemNode, "nodetype", int, sh::filesystem::FilesystemNode::nodetype)
    REGISTERHELPERMETHOD(clsFilesystemNode, sh::filesystem::FilesystemNode, "ishidden", bool, sh::scripting::api::ApiHelperMethods::FilesystemNode_isHidden)
    REGISTERHELPERMETHOD(clsFilesystemNode, sh::filesystem::FilesystemNode, "set_hidden", void, sh::scripting::api::ApiHelperMethods::FilesystemNode_setHidden, bool)
    REGISTERHELPERMETHOD(clsFilesystemNode, sh::filesystem::FilesystemNode, "add_detail", void, sh::scripting::api::ApiHelperMethods::FilesystemNode_addDetail, std::shared_ptr<sh::filesystem::DetailColumn>)
    REGISTERHELPERMETHOD(clsFilesystemNode, sh::filesystem::FilesystemNode, "set_icon", void, sh::scripting::api::ApiHelperMethods::FilesystemNode_setIcon, QString)
    REGISTERHELPERMETHOD(clsFilesystemNode, sh::filesystem::FilesystemNode, "set_displayname", void, sh::scripting::api::ApiHelperMethods::FilesystemNode_setDisplayName, QString)

    sh::scripting::ApiClass* clsFilesystemNodeListEditor;
    registerClass("FilesystemNodeListEditor", &clsFilesystemNodeListEditor, typeid(sh::filesystem::FilesystemNodeListEditor));
    REGISTERMETHOD(clsFilesystemNodeListEditor, sh::filesystem::FilesystemNodeListEditor, "set_items", void, sh::filesystem::FilesystemNodeListEditor::setItems, QList<QString>)
    REGISTERMETHOD(clsFilesystemNodeListEditor, sh::filesystem::FilesystemNodeListEditor, "begin_iterative_adding", void, sh::filesystem::FilesystemNodeListEditor::beginIterativeAdding)
    REGISTERMETHOD(clsFilesystemNodeListEditor, sh::filesystem::FilesystemNodeListEditor, "add_item", void, sh::filesystem::FilesystemNodeListEditor::addItem, QString)
    REGISTERMETHOD(clsFilesystemNodeListEditor, sh::filesystem::FilesystemNodeListEditor, "end_iterative_adding", void, sh::filesystem::FilesystemNodeListEditor::endIterativeAdding)

    sh::scripting::ApiClass* clsIODevice;
    registerClass("IODevice", &clsIODevice, typeid(QIODevice));
    REGISTERHELPERMETHOD(clsIODevice, QIODevice, "read", QByteArray, sh::scripting::api::ApiHelperMethods::QIODevice_read)
    REGISTERHELPERMETHOD(clsIODevice, QIODevice, "readall", QByteArray, sh::scripting::api::ApiHelperMethods::QIODevice_readall)

    sh::scripting::ApiClass* clsAbstractActionItem;
    registerClass("AbstractActionItem", &clsAbstractActionItem, typeid(sh::actions::AbstractActionItem));

    sh::scripting::ApiClass* clsReadDataDevice;
    registerClass("ReadDataDevice", &clsReadDataDevice, typeid(sh::scripting::api::ApiReadDataDevice));
    REGISTERMETHOD(clsReadDataDevice, sh::scripting::api::ApiReadDataDevice, "_ended", void, sh::scripting::api::ApiReadDataDevice::_ended)
    REGISTERSCRIPTMETHOD(clsReadDataDevice, sh::scripting::api::ApiReadDataDevice, "set_getdata", QByteArray, sh::scripting::api::ApiReadDataDevice::_getdata)
    REGISTERHELPERMETHOD(clsReadDataDevice, sh::scripting::api::ApiReadDataDevice, "read", QByteArray, sh::scripting::api::ApiHelperMethods::ApiReadDataDevice_read)
    REGISTERHELPERMETHOD(clsReadDataDevice, sh::scripting::api::ApiReadDataDevice, "readall", QByteArray, sh::scripting::api::ApiHelperMethods::ApiReadDataDevice_readall)

    sh::scripting::ApiClass* clsActionExecutionInfo;
    registerClass("ActionExecutionInfo", &clsActionExecutionInfo, typeid(sh::actions::ActionExecutionInfo));
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "set_visualprocessfeedback_active", void, sh::actions::ActionExecutionInfo::setVisualProcessFeedbackActive, bool)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "is_visualprocessfeedback_active", bool, sh::actions::ActionExecutionInfo::isVisualProcessFeedbackActive)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "set_manual_intervention_needed", void, sh::actions::ActionExecutionInfo::setManualInterventionNeeded, bool)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "is_manual_intervention_needed", bool, sh::actions::ActionExecutionInfo::isManualInterventionNeeded)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "set_details", void, sh::actions::ActionExecutionInfo::setDetails, QString, QString, QString, QString)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "from_verb", QString, sh::actions::ActionExecutionInfo::fromVerb)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "from_objectname", QString, sh::actions::ActionExecutionInfo::fromObjectName)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "to_verb", QString, sh::actions::ActionExecutionInfo::toVerb)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "to_objectname", QString, sh::actions::ActionExecutionInfo::toObjectName)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "set_head", void, sh::actions::ActionExecutionInfo::setHead, QString)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "head", QString, sh::actions::ActionExecutionInfo::head)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "set_progress", void, sh::actions::ActionExecutionInfo::setProgress, quint64, quint64, QString)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "set_progress_indeterminate", void, sh::actions::ActionExecutionInfo::setProgressIndeterminate, QString)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "progress_done", quint64, sh::actions::ActionExecutionInfo::progressDone)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "progress_all", quint64, sh::actions::ActionExecutionInfo::progressAll)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "progress_text", QString, sh::actions::ActionExecutionInfo::progressText)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "is_cancelled", bool, sh::actions::ActionExecutionInfo::isCancelled)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "cancel", void, sh::actions::ActionExecutionInfo::cancel)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "respect_cancel", void, sh::actions::ActionExecutionInfo::respectCancel)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "operation", sh::filesystem::Operation*, sh::actions::ActionExecutionInfo::operation)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "add_changed_eurl", void, sh::actions::ActionExecutionInfo::addChangedEurl, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsActionExecutionInfo, sh::actions::ActionExecutionInfo, "userfeedback", sh::actions::ActionExecutionUserFeedback*, sh::actions::ActionExecutionInfo::userfeedback)

    sh::scripting::ApiClass* clsActionExecutionUserFeedback;
    registerClass("ActionExecutionUserFeedback", &clsActionExecutionUserFeedback, typeid(sh::actions::ActionExecutionUserFeedback));
    REGISTERMETHOD(clsActionExecutionUserFeedback, sh::actions::ActionExecutionUserFeedback, "simple_messagebox", int, sh::actions::ActionExecutionUserFeedback::simpleMessageBox, QString, int, QString, int, int)
    REGISTERHELPERMETHOD(clsActionExecutionUserFeedback, sh::actions::ActionExecutionUserFeedback, "messagebox", int, sh::scripting::api::ApiHelperMethods::ActionExecutionUserFeedback_messageBox, QString, QList<QString>, QString, int, int)
    REGISTERMETHOD(clsActionExecutionUserFeedback, sh::actions::ActionExecutionUserFeedback, "messagebox_with_icons", int, sh::actions::ActionExecutionUserFeedback::messageBox, QString, QList<QString>, QString, int, int, QList<QString>)
    REGISTERHELPERMETHOD(clsActionExecutionUserFeedback, sh::actions::ActionExecutionUserFeedback, "simple_inputbox", QString, sh::scripting::api::ApiHelperMethods::ActionExecutionUserFeedback_simpleInputBox, QString, QString)

    sh::scripting::ApiClass* clsConfigurationValue;
    registerClass("ConfigurationValue", &clsConfigurationValue, typeid(sh::configuration::ConfigurationValue));
    REGISTERMETHOD(clsConfigurationValue, sh::configuration::ConfigurationValue, "get_configvalue_int", int, sh::configuration::ConfigurationValue::getConfigValueInt)
    REGISTERMETHOD(clsConfigurationValue, sh::configuration::ConfigurationValue, "get_configvalue_float", double, sh::configuration::ConfigurationValue::getConfigValueFloat)
    REGISTERMETHOD(clsConfigurationValue, sh::configuration::ConfigurationValue, "get_configvalue_bool", bool, sh::configuration::ConfigurationValue::getConfigValueBool)
    REGISTERMETHOD(clsConfigurationValue, sh::configuration::ConfigurationValue, "get_configvalue_string", QString, sh::configuration::ConfigurationValue::getConfigValueString)
    REGISTERHELPERMETHOD(clsConfigurationValue, sh::configuration::ConfigurationValue, "set_configvalue_int", void, sh::scripting::api::ApiHelperMethods::ConfigurationValue_setConfigValueInt, int)
    REGISTERHELPERMETHOD(clsConfigurationValue, sh::configuration::ConfigurationValue, "set_configvalue_float", void, sh::scripting::api::ApiHelperMethods::ConfigurationValue_setConfigValueFloat, double)
    REGISTERHELPERMETHOD(clsConfigurationValue, sh::configuration::ConfigurationValue, "set_configvalue_bool", void, sh::scripting::api::ApiHelperMethods::ConfigurationValue_setConfigValueBool, bool)
    REGISTERHELPERMETHOD(clsConfigurationValue, sh::configuration::ConfigurationValue, "set_configvalue_string", void, sh::scripting::api::ApiHelperMethods::ConfigurationValue_setConfigValueString, QString)

    sh::scripting::ApiClass* clsPredicate;
    registerClass("Predicate", &clsPredicate, typeid(sh::actions::Predicate));

    sh::scripting::ApiClass* clsActionFactory;
    registerClass("ActionFactory", &clsActionFactory, typeid(sh::scripting::api::ApiActionFactory));
    REGISTERSCRIPTMETHOD(clsActionFactory, sh::scripting::api::ApiActionFactory, "set_construct", std::shared_ptr<sh::actions::AbstractActionItem>, sh::scripting::api::ApiActionFactory::_construct, QList<std::shared_ptr<sh::filesystem::FilesystemNode>>);

    sh::scripting::ApiClass* clsSetting;
    registerClass("Setting", &clsSetting, typeid(sh::scripting::api::ApiSetting));
    REGISTERSCRIPTMETHOD(clsSetting, sh::scripting::api::ApiSetting, "set_setvalue1", void, sh::scripting::api::ApiSetting::_setValue1, QString);
    REGISTERSCRIPTMETHOD(clsSetting, sh::scripting::api::ApiSetting, "set_setvalue2", void, sh::scripting::api::ApiSetting::_setValue2, int, QString);
    REGISTERSCRIPTMETHOD(clsSetting, sh::scripting::api::ApiSetting, "set_getvalue", QString, sh::scripting::api::ApiSetting::_getValue, int);
    REGISTERSCRIPTMETHOD(clsSetting, sh::scripting::api::ApiSetting, "set_valuedescription", QString, sh::scripting::api::ApiSetting::_valueDescription, QString);

    sh::scripting::ApiClass* clsFilesystemOperationStep;
    registerClass("FilesystemOperationStep", &clsFilesystemOperationStep, typeid(sh::filesystem::FilesystemOperationTransfers::OperationStep));
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "conflict_description", QString, sh::filesystem::FilesystemOperationTransfers::OperationStep::conflictDescription)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "conflict_resolution_rename_destination_before_to", QString, sh::filesystem::FilesystemOperationTransfers::OperationStep::conflictResolution_renameDestinationBeforeTo)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "conflict_resolution_different_destination_name_to", QString, sh::filesystem::FilesystemOperationTransfers::OperationStep::conflictResolution_differentDestinationNameTo)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "set_conflict_resolve_skip", void, sh::filesystem::FilesystemOperationTransfers::OperationStep::setConflictResolve_Skip)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "set_conflict_resolve_overwrite_destination", void, sh::filesystem::FilesystemOperationTransfers::OperationStep::setConflictResolve_OverwriteDestination)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "set_conflict_resolve_rename_destination_before", void, sh::filesystem::FilesystemOperationTransfers::OperationStep::setConflictResolve_RenameDestinationBefore, QString)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "set_conflict_resolve_use_different_destination_name", void, sh::filesystem::FilesystemOperationTransfers::OperationStep::setConflictResolve_UseDifferentDestinationName, QString)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "set_conflict_resolve_merge_directories", void, sh::filesystem::FilesystemOperationTransfers::OperationStep::setConflictResolve_MergeDirectories)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "sourcetype", int, sh::filesystem::FilesystemOperationTransfers::OperationStep::sourcetype)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "source", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::FilesystemOperationTransfers::OperationStep::source)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "original_destination", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::FilesystemOperationTransfers::OperationStep::originalDestination)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "effective_destination", std::shared_ptr<const sh::filesystem::Eurl>, sh::filesystem::FilesystemOperationTransfers::OperationStep::effectiveDestination)
    REGISTERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "set_conflict_resolve_merge_directories_check_allowed", bool, sh::filesystem::FilesystemOperationTransfers::OperationStep::setConflictResolve_MergeDirectories_checkAllowed)
    REGISTERHELPERMETHOD(clsFilesystemOperationStep, sh::filesystem::FilesystemOperationTransfers::OperationStep, "conflict_resolution", int, sh::scripting::api::ApiHelperMethods::OperationStep_conflictResolution)

    sh::scripting::ApiClass* clsFilesystemOperationProgressMonitor;
    registerClass("FilesystemOperationProgressMonitor", &clsFilesystemOperationProgressMonitor, typeid(sh::scripting::api::ApiFilesystemOperationProgressMonitor));
    REGISTERSCRIPTMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "set_changed", void, sh::scripting::api::ApiFilesystemOperationProgressMonitor::_changed);
    REGISTERSCRIPTMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "set_resolveconflicts", bool, sh::scripting::api::ApiFilesystemOperationProgressMonitor::_resolveConflicts, QList<sh::filesystem::FilesystemOperationTransfers::OperationStep*>);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "has_item_info", bool, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_hasItemInfo);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "done_items", quint64, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_doneItems);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "all_items", quint64, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_allItems);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "has_bytes_info", bool, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_hasBytesInfo);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "done_bytes", quint64, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_doneBytes);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "all_bytes", quint64, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_allBytes);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "get_item_info_from", QString, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_getItemInfoFrom);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "get_item_info_to", QString, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_getItemInfoTo);
    REGISTERHELPERMETHOD(clsFilesystemOperationProgressMonitor, sh::scripting::api::ApiFilesystemOperationProgressMonitor, "estimation", QString, sh::scripting::api::ApiHelperMethods::FilesystemOperationProgressMonitor_estimation);

    sh::scripting::ApiClass* clsFilesystemOperation;
    registerClass("FilesystemOperation", &clsFilesystemOperation, typeid(sh::filesystem::FilesystemOperation));
    REGISTERHELPERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "itemlist", QList<std::shared_ptr<sh::filesystem::Eurl>>, sh::scripting::api::ApiHelperMethods::FilesystemOperation_itemlist, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERHELPERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "itemlist_by_type", QList<std::shared_ptr<sh::filesystem::Eurl>>, sh::scripting::api::ApiHelperMethods::FilesystemOperation_itemlistByType, std::shared_ptr<const sh::filesystem::Eurl>, int)
    REGISTERHELPERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "get_type", int, sh::scripting::api::ApiHelperMethods::FilesystemOperation_getType, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "get_filesize", qint64, sh::filesystem::FilesystemOperation::getFileSize, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "get_linktarget", QString, sh::filesystem::FilesystemOperation::getLinkTarget, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "can_get_filecontent", bool, sh::filesystem::FilesystemOperation::canGetFileContent, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "get_file_content", std::shared_ptr<QIODevice>, sh::filesystem::FilesystemOperation::getFileContent, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "can_create_directory", bool, sh::filesystem::FilesystemOperation::canCreateDirectory, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "create_directory", void, sh::filesystem::FilesystemOperation::createDirectory, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "can_create_link", bool, sh::filesystem::FilesystemOperation::canCreateLink, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "create_link", void, sh::filesystem::FilesystemOperation::createLink, std::shared_ptr<const sh::filesystem::Eurl>, QString, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "can_create_file", bool, sh::filesystem::FilesystemOperation::canCreateFile, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERHELPERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "create_file", void, sh::scripting::api::ApiHelperMethods::FilesystemOperation_createFile, std::shared_ptr<const sh::filesystem::Eurl>, sh::scripting::api::ApiReadDataDevice*, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "can_delete_item", bool, sh::filesystem::FilesystemOperation::canDeleteItem, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "delete_item", void, sh::filesystem::FilesystemOperation::deleteItem, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "delete_directory_if_empty", void, sh::filesystem::FilesystemOperation::deleteDirectoryIfEmpty, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "can_move_item", bool, sh::filesystem::FilesystemOperation::canMoveItem, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "move_items", void, sh::filesystem::FilesystemOperation::moveItems, QList<std::shared_ptr<const sh::filesystem::Eurl>>, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "move_item", void, sh::filesystem::FilesystemOperation::moveItem, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "can_copy_item", bool, sh::filesystem::FilesystemOperation::canCopyItem, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "copy_items", void, sh::filesystem::FilesystemOperation::copyItems, QList<std::shared_ptr<const sh::filesystem::Eurl>>, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "copy_item", void, sh::filesystem::FilesystemOperation::copyItem, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<sh::filesystem::FilesystemOperationProgressMonitor>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "list_extendedattributes", QList<QString>, sh::filesystem::FilesystemOperation::listExtendedAttributes, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "get_extendedattribute_size", quint64, sh::filesystem::FilesystemOperation::getExtendedAttributeSize, std::shared_ptr<const sh::filesystem::Eurl>, QString)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "get_extendedattribute", QByteArray, sh::filesystem::FilesystemOperation::getExtendedAttribute, std::shared_ptr<const sh::filesystem::Eurl>, QString)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "set_extendedattribute", void, sh::filesystem::FilesystemOperation::setExtendedAttribute, std::shared_ptr<const sh::filesystem::Eurl>, QString, QByteArray)
    REGISTERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "remove_extendedattribute", void, sh::filesystem::FilesystemOperation::removeExtendedAttribute, std::shared_ptr<const sh::filesystem::Eurl>, QString)
    REGISTERHELPERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "resolve_node_link", QList<std::shared_ptr<sh::filesystem::FilesystemNode>>, sh::scripting::api::ApiHelperMethods::FilesystemOperation_resolveNodeLink, std::shared_ptr<sh::filesystem::FilesystemNode>)
    REGISTERHELPERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "resolve_node_link_nonrecursive", QList<std::shared_ptr<sh::filesystem::FilesystemNode>>, sh::scripting::api::ApiHelperMethods::FilesystemOperation_resolveNodeLink_NonRecursive, std::shared_ptr<sh::filesystem::FilesystemNode>)
    REGISTERHELPERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "resolve_eurl_link", std::shared_ptr<sh::filesystem::Eurl>, sh::scripting::api::ApiHelperMethods::FilesystemOperation_resolveEurlLink, std::shared_ptr<const sh::filesystem::Eurl>)
    REGISTERHELPERMETHOD(clsFilesystemOperation, sh::filesystem::FilesystemOperation, "resolve_eurl_link_nonrecursive", std::shared_ptr<sh::filesystem::Eurl>, sh::scripting::api::ApiHelperMethods::FilesystemOperation_resolveEurlLink_NonRecursive, std::shared_ptr<const sh::filesystem::Eurl>)

    sh::scripting::ApiClass* clsHandlerTransfer;
    registerClass("HandlerTransfer", &clsHandlerTransfer, typeid(sh::filesystem::FilesystemHandler::HandlerTransfer));
    REGISTERMETHOD(clsHandlerTransfer, sh::filesystem::FilesystemHandler::HandlerTransfer, "increment_transferred_bytes", void, sh::filesystem::FilesystemHandler::HandlerTransfer::incrementTransferredBytes, qint64 donebytes)
    REGISTERMETHOD(clsHandlerTransfer, sh::filesystem::FilesystemHandler::HandlerTransfer, "respect_cancel", void, sh::filesystem::FilesystemHandler::HandlerTransfer::respectCancel)

    sh::scripting::ApiClass* clsPanelDetailFactorySingle;
    registerClass("PanelDetailFactorySingle", &clsPanelDetailFactorySingle, typeid(sh::scripting::api::ApiPanelDetailFactorySingle));
    REGISTERSCRIPTMETHOD(clsPanelDetailFactorySingle, sh::scripting::api::ApiPanelDetailFactorySingle, "set_setvalue", void, sh::scripting::api::ApiPanelDetailFactorySingle::_setvalue, sh::paneldetails::PanelDetail*, std::shared_ptr<sh::filesystem::FilesystemNode>, sh::filesystem::Operation*);
    REGISTERSCRIPTMETHOD(clsPanelDetailFactorySingle, sh::scripting::api::ApiPanelDetailFactorySingle, "set_linktriggered", void, sh::scripting::api::ApiPanelDetailFactorySingle::_linktriggered, std::shared_ptr<sh::filesystem::FilesystemNode>, QString);

    sh::scripting::ApiClass* clsPanelDetailFactoryMulti;
    registerClass("PanelDetailFactoryMulti", &clsPanelDetailFactoryMulti, typeid(sh::scripting::api::ApiPanelDetailFactoryMulti));
    REGISTERSCRIPTMETHOD(clsPanelDetailFactoryMulti, sh::scripting::api::ApiPanelDetailFactoryMulti, "set_setvalue", void, sh::scripting::api::ApiPanelDetailFactoryMulti::_setvalue, sh::paneldetails::PanelDetail*, QList<std::shared_ptr<sh::filesystem::FilesystemNode>>, sh::filesystem::Operation*);
    REGISTERSCRIPTMETHOD(clsPanelDetailFactoryMulti, sh::scripting::api::ApiPanelDetailFactoryMulti, "set_linktriggered", void, sh::scripting::api::ApiPanelDetailFactoryMulti::_linktriggered, QList<std::shared_ptr<sh::filesystem::FilesystemNode>>, QString);

    sh::scripting::ApiClass* clsPanelDetail;
    registerClass("PanelDetail", &clsPanelDetail, typeid(sh::paneldetails::PanelDetail));
    REGISTERHELPERMETHOD(clsPanelDetail, sh::paneldetails::PanelDetail, "set_row", void, sh::scripting::api::ApiHelperMethods::PanelDetail_setRow, QString, QList<QString>)

    sh::scripting::ApiClass* clsFilePropertyDialogTab;
    registerClass("FilePropertyDialogTab", &clsFilePropertyDialogTab, typeid(sh::scripting::api::ApiFilePropertyDialogTab));
    REGISTERSCRIPTMETHOD(clsFilePropertyDialogTab, sh::scripting::api::ApiFilePropertyDialogTab, "set_updatewidget", QList<QString>, sh::scripting::api::ApiFilePropertyDialogTab::_updateWidget, int, sh::filesystem::Operation*, qintptr);
    REGISTERSCRIPTMETHOD(clsFilePropertyDialogTab, sh::scripting::api::ApiFilePropertyDialogTab, "set_buttontriggered", void, sh::scripting::api::ApiFilePropertyDialogTab::_buttonTriggered, QString);
    REGISTERHELPERMETHOD(clsFilePropertyDialogTab, sh::scripting::api::ApiFilePropertyDialogTab, "nodes", QList<std::shared_ptr<sh::filesystem::FilesystemNode>>, sh::scripting::api::ApiHelperMethods::ApiFilePropertyDialogTab_nodes)
    REGISTERHELPERMETHOD(clsFilePropertyDialogTab, sh::scripting::api::ApiFilePropertyDialogTab, "selected_index_for_property", int, sh::scripting::api::ApiHelperMethods::FilePropertyDialogTab_selectedIndexForProperty, qintptr)
    REGISTERHELPERMETHOD(clsFilePropertyDialogTab, sh::scripting::api::ApiFilePropertyDialogTab, "refresh", void, sh::scripting::api::ApiHelperMethods::FilePropertyDialogTab_refresh);

    sh::scripting::ApiClass* clsFilePropertyDialogTabFactory;
    registerClass("FilePropertyDialogTabFactory", &clsFilePropertyDialogTabFactory, typeid(sh::scripting::api::ApiFilePropertyDialogTabFactory));
    REGISTERSCRIPTMETHOD(clsFilePropertyDialogTabFactory, sh::scripting::api::ApiFilePropertyDialogTabFactory, "set_construct", std::shared_ptr<sh::scripting::api::ApiFilePropertyDialogTab>, sh::scripting::api::ApiFilePropertyDialogTabFactory::_construct);

    sh::scripting::ApiClass* clsThread;
    registerClass("Thread", &clsThread, typeid(sh::scripting::api::ApiThread));
    REGISTERSCRIPTMETHOD(clsThread, sh::scripting::api::ApiThread, "set_run", void, sh::scripting::api::ApiThread::_run);
    REGISTERMETHOD(clsThread, sh::scripting::api::ApiThread, "start", void, sh::scripting::api::ApiThread::start)

    sh::scripting::ApiClass* clsTimer;
    registerClass("Timer", &clsTimer, typeid(sh::scripting::api::ApiTimer));
    REGISTERSCRIPTMETHOD(clsTimer, sh::scripting::api::ApiTimer, "set_run", void, sh::scripting::api::ApiTimer::_run);
    REGISTERMETHOD(clsTimer, sh::scripting::api::ApiTimer, "start", void, sh::scripting::api::ApiTimer::start, int)
    REGISTERMETHOD(clsTimer, sh::scripting::api::ApiTimer, "stop", void, sh::scripting::api::ApiTimer::stop)

    sh::scripting::ApiClass* clsSearchCriterion;
    registerClass("SearchCriterion", &clsSearchCriterion, typeid(sh::scripting::api::ApiSearchCriterion));
    REGISTERSCRIPTMETHOD(clsSearchCriterion, sh::scripting::api::ApiSearchCriterion, "set_configure", void, sh::scripting::api::ApiSearchCriterion::_configure, sh::actions::ActionExecutionInfo*);
    REGISTERSCRIPTMETHOD(clsSearchCriterion, sh::scripting::api::ApiSearchCriterion, "set_match", bool, sh::scripting::api::ApiSearchCriterion::_match, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::Eurl>);
    REGISTERMETHOD(clsSearchCriterion, sh::scripting::api::ApiSearchCriterion, "searchspec", QList<QString>, sh::scripting::api::ApiSearchCriterion::getsearchspec);

    sh::scripting::ApiClass* clsSearchCriterionFactory;
    registerClass("SearchCriterionFactory", &clsSearchCriterionFactory, typeid(sh::scripting::api::ApiSearchCriterionFactory));
    REGISTERSCRIPTMETHOD(clsSearchCriterionFactory, sh::scripting::api::ApiSearchCriterionFactory, "set_getsearchspecdesc", QString, sh::scripting::api::ApiSearchCriterionFactory::_getsearchspecdesc, QList<QString>);
    REGISTERSCRIPTMETHOD(clsSearchCriterionFactory, sh::scripting::api::ApiSearchCriterionFactory, "set_construct", std::shared_ptr<sh::search::SearchCriterion>, sh::scripting::api::ApiSearchCriterionFactory::_construct);
    REGISTERSCRIPTMETHOD(clsSearchCriterionFactory, sh::scripting::api::ApiSearchCriterionFactory, "set_isvisiblefor", bool, sh::scripting::api::ApiSearchCriterionFactory::_isVisibleFor, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::FilesystemNode>);

    sh::scripting::ApiClass* clsThumbnailProvider;
    registerClass("ThumbnailProvider", &clsThumbnailProvider, typeid(sh::scripting::api::ApiThumbnailProvider));
    REGISTERSCRIPTMETHOD(clsThumbnailProvider, sh::scripting::api::ApiThumbnailProvider, "set_getthumbnail", QByteArray, sh::scripting::api::ApiThumbnailProvider::_getThumbnail, sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::FilesystemNode>, QString, int, int);

    sh::scripting::ApiClass* clsBookmarkManager;
    registerClass("BookmarkManager", &clsBookmarkManager, typeid(sh::tools::BookmarkManager));
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "get_bookmarks", QList<std::shared_ptr<sh::tools::Bookmark>>, sh::tools::BookmarkManager::getBookmarks);
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "has_bookmarks", bool, sh::tools::BookmarkManager::hasBookmarks);
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "add_bookmark", QString, sh::tools::BookmarkManager::addBookmark, QList<QString>, QString, std::shared_ptr<const sh::filesystem::Eurl>, QString);
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "remove_bookmark", void, sh::tools::BookmarkManager::removeBookmark, QString);
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "change_bookmark", void, sh::tools::BookmarkManager::changeBookmark, QString, QString, std::shared_ptr<const sh::filesystem::Eurl>);
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "change_bookmark_tags", void, sh::tools::BookmarkManager::changeBookmarkTags, QString, QString);
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "move_bookmark_up", void, sh::tools::BookmarkManager::moveBookmarkUp, QString);
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "move_bookmark_down", void, sh::tools::BookmarkManager::moveBookmarkDown, QString);
    REGISTERMETHOD(clsBookmarkManager, sh::tools::BookmarkManager, "move_bookmark_to_folder", void, sh::tools::BookmarkManager::moveBookmarkToFolder, QString, QList<QString>);

    sh::scripting::ApiClass* clsBookmark;
    registerClass("Bookmark", &clsBookmark, typeid(sh::tools::Bookmark));
    REGISTERMETHOD(clsBookmark, sh::tools::Bookmark, "eurl", std::shared_ptr<const sh::filesystem::Eurl>, sh::tools::Bookmark::eurl);
    REGISTERMETHOD(clsBookmark, sh::tools::Bookmark, "folder", QList<QString>, sh::tools::Bookmark::folder);
    REGISTERMETHOD(clsBookmark, sh::tools::Bookmark, "id", QString, sh::tools::Bookmark::id);
    REGISTERMETHOD(clsBookmark, sh::tools::Bookmark, "label", QString, sh::tools::Bookmark::label);
    REGISTERMETHOD(clsBookmark, sh::tools::Bookmark, "tags", QString, sh::tools::Bookmark::tags);
}

QStringList sh::scripting::Api::classes()
{
    return _classes.keys();
}

QStringList sh::scripting::Api::rootMembers()
{
    return _rootMembers.keys();
}

sh::scripting::ApiClass* sh::scripting::Api::getClass(QString name)
{
    return _classes[name];
}

sh::scripting::ApiClass* sh::scripting::Api::getClassByTypeIdName(QString typeidName)
{
    return _classesByNativeType[typeidName];
}

void* sh::scripting::Api::rootMember(QString name)
{
    return _rootMembers[name];
}

sh::scripting::ApiClass* sh::scripting::Api::rootMemberClass(QString name)
{
    return _rootMembersClasses[name];
}

sh::scripting::ApiMethod::ApiMethod(QString name)
{
    _name = name;
}

sh::scripting::ApiClass::ApiClass(QString name)
{
    _name = name;
}

sh::scripting::ApiClass::~ApiClass()
{
    foreach (sh::scripting::ApiMethod* c, _methods.values())
        delete c;
}

QString sh::scripting::ApiMethod::name()
{
    return _name;
}

QString sh::scripting::ApiClass::name()
{
    return _name;
}

sh::scripting::ApiMethod* sh::scripting::ApiClass::getMethod(QString name)
{
    return _methods[name];
}

void sh::scripting::ApiClass::addMethod(sh::scripting::ApiMethod *m)
{
    _methods[m->name()] = m;
}

QStringList sh::scripting::ApiClass::methods()
{
    return QStringList(_methods.keys());
}

void sh::scripting::Api::registerClass(QString name, sh::scripting::ApiClass** apiclass, const std::type_info &nativeType)
{
    sh::scripting::ApiClass* r = new sh::scripting::ApiClass(name);
    *apiclass = r;
    _classes[name] = r;
    _classesByNativeType[nativeType.name()] = r;
}

void sh::scripting::Api::registerRootObjectMember(QString name, sh::scripting::ApiClass *cls, void* member)
{
    _rootMembers[name] = member;
    _rootMembersClasses[name] = cls;
}

void sh::scripting::Api::registerMethod(sh::scripting::ApiClass* apiclass, QString name)
{
    sh::scripting::ApiMethod* method = new sh::scripting::ApiMethod(name);
    _classes[apiclass->name()]->addMethod(method);
}

sh::scripting::Api::~Api()
{
    foreach (sh::scripting::ApiClass* c, _classes)
        delete c;
}
