/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef APISEARCHCRITERION_H
#define APISEARCHCRITERION_H

#include "search/criteria/abstractactiondrivensearchcriterion.h"
#include "search/searchcriterionfactory.h"


namespace sh{
    namespace ui { class SearchPanelConfiguration; }
    namespace scripting {
        namespace api {

            class ApiSearchCriterionFactory : public sh::search::SearchCriterionFactory
            {
            public:
                ApiSearchCriterionFactory(QString key, QString description);
                virtual QString key();
                virtual QString description();
                virtual void editor(std::shared_ptr<sh::search::SearchCriterion> c, sh::ui::SearchPanelConfiguration* panelcfg);
                virtual void editorupdateconfig(sh::ui::SearchPanelConfiguration* panelcfg, std::shared_ptr<sh::search::SearchCriterion> c);
                virtual std::shared_ptr<sh::search::SearchCriterion> construct();
                std::function<QString(QList<QString>)> _getsearchspecdesc;
                std::function<std::shared_ptr<sh::search::SearchCriterion>()> _construct;
                std::function<bool(sh::filesystem::Operation*, std::shared_ptr<sh::filesystem::FilesystemNode>)> _isVisibleFor;
                virtual bool isVisibleFor(sh::filesystem::Operation* op, std::shared_ptr<sh::filesystem::FilesystemNode> node);
            protected:
                QString _key;
                QString _description;
            };

            class ApiSearchCriterion : public sh::search::criteria::AbstractActionDrivenSearchCriterion
            {
            public:
                ApiSearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);
                virtual bool match2(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl);
                std::function<void(sh::actions::ActionExecutionInfo*)> _configure;
                std::function<bool(sh::filesystem::Operation* op, std::shared_ptr<sh::filesystem::Eurl> eurl)> _match;
                QList<QString> getsearchspec();

            protected:
                virtual void configure(sh::actions::ActionExecutionInfo* info);
                virtual QString getSearchSpecDescription();
            };

        }
    }
}

#endif // APISEARCHCRITERION_H
