import shallot
import time
import datetime

# This actions show some advanced operations in shallot.Operations.FilesystemOperation.
# The complicated case is shown :-P with additional progress monitors for reacting on the transfer progress.
# They are not required. See the scripting help for reference.


class MyActionMenu(shallot.Actions.SubmenuAction):

    def __init__(self, nodes):
        shallot.Actions.SubmenuAction.__init__(self, "FS ops examples 2", icon="dialog-warning")
        self.nodes = nodes

    def initialize(self):
        self.set_subitems([GetFileContentAction(self.nodes),
                          CreateFileAction(self.nodes),
                          CopyItemsAction(self.nodes)])


class GetFileContentAction(shallot.Actions.ActionAction):

    def __init__(self, nodes):
        shallot.Actions.ActionAction.__init__(self, "Fetch a file content")
        self.nodes = nodes

    def action(self, info):
        currentdir = self.nodes[0].eurl()
        answer = info.userfeedback().simple_inputbox("Request content for which file (should be plaintext and small for this example)?", "")
        if answer is not None:
            resstream = info.operation().filesystem().get_file_content(currentdir.with_appended_segment(answer))
            res = resstream.readall().decode(errors='ignore')
            if len(res) > 1000:
                res = res[:1000] + " ..."
            info.userfeedback().simple_messagebox("Content of {0}: {1}".format(answer, res), shallot.Actions.ExecutionUserFeedback.MessageBoxButton.OK, None, shallot.Actions.ExecutionUserFeedback.MessageBoxButton.OK, 0)


class CreateFileAction(shallot.Actions.ActionAction):

    class MyFilesystemOperationProgressMonitor(shallot.Operations.FilesystemOperationProgressMonitor):

        def __init__(self, info):
            shallot.Operations.FilesystemOperationProgressMonitor.__init__(self, info)

        def changed(self):
            shallot.Logging.log_info("Create file progress: {0} of {1} bytes".format(self.done_bytes(), self.all_bytes()))

    def __init__(self, nodes):
        shallot.Actions.ActionAction.__init__(self, "Create a file")
        self.nodes = nodes

    def action(self, info):
        currentdir = self.nodes[0].eurl()
        answersrc = info.userfeedback().simple_inputbox("Request to create a file with the content from which one (i.e. the source)?", "")
        if answersrc is not None:
            answerdst = info.userfeedback().simple_inputbox("Name for the new file?", "")
            if answerdst is not None:
                resstream = info.operation().filesystem().get_file_content(currentdir.with_appended_segment(answersrc))
                info.operation().filesystem().create_file(currentdir.with_appended_segment(answerdst), resstream, CreateFileAction.MyFilesystemOperationProgressMonitor(info))


class CopyItemsAction(shallot.Actions.ActionAction):

    class MyFilesystemOperationProgressMonitor(shallot.Operations.FilesystemOperationProgressMonitor):

        def __init__(self, info):
            shallot.Operations.FilesystemOperationProgressMonitor.__init__(self, info)

        def changed(self):
            shallot.Logging.log_info("Create file progress: {0} of {1} bytes".format(self.done_bytes(), self.all_bytes()))

        def resolve_conflicts(self, steps):
            for step in steps:
                if step.set_conflict_resolve_merge_directories_check_allowed():
                    step.set_conflict_resolve_merge_directories()
                elif step.source().basename() == "file1":
                    step.set_conflict_resolve_skip()
                elif step.source().basename() == "file2":
                    step.set_conflict_resolve_overwrite_destination()
                elif step.source().basename() == "file3":
                    step.set_conflict_resolve_rename_destination_before("file3.original")
                elif step.source().basename() == "file4":
                    step.set_conflict_resolve_use_different_destination_name("file4.new")
                else:
                    step.set_conflict_resolve_skip()

    def __init__(self, nodes):
        shallot.Actions.ActionAction.__init__(self, "Copy some items")
        self.nodes = nodes

    def action(self, info):
        currentdir = self.nodes[0].eurl()
        answer = info.userfeedback().simple_inputbox("Name for a new directory:", "")
        if answer is not None:
            # prepare some files
            rootdir = currentdir.with_appended_segment(answer)
            mydirb = rootdir.with_appended_segment("mydir")
            _mydirint = rootdir.with_appended_segment("_tmp")
            mydira = _mydirint.with_appended_segment("mydir")
            stream1 = lambda: shallot.Streaming.BlobReadDataDevice("aaaa".encode())
            stream2 = lambda: shallot.Streaming.BlobReadDataDevice("bbbbbbbb".encode())
            for x in (rootdir, _mydirint, mydira, mydirb):
                info.operation().filesystem().create_directory(x, None)
            for mydir, contstream in ((mydira, stream1), (mydirb, stream2)):
                for filename in "file1", "file2", "file3", "file4":
                    info.operation().filesystem().create_file(mydir.with_appended_segment(filename), contstream(), None)
            # copy items with a custom conflict resolution and progress monitoring
            info.operation().filesystem().copy_item(mydira, mydirb, CopyItemsAction.MyFilesystemOperationProgressMonitor(info))


shallot.Actions.register(MyActionMenu, predicates=[shallot.Actions.HideOnSelectionLevelPredicate()])

