/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTACTIONDRIVENSEARCHCRITERION_H
#define ABSTRACTACTIONDRIVENSEARCHCRITERION_H

#include <QSet>
#include "actions/actionactionitem.h"
#include "search/searchcriterion.h"
#include "search/searchcriterionfactory.h"


namespace sh{
    namespace filesystem{ class Eurl; class FilesystemNode; }
    namespace ui { class SearchPanelConfiguration; }

    namespace search{
        namespace criteria {

            /** \brief Abstract search criterion filtering by something which is configured in a wizard-like way by an action execution.
             *
             * This provides an easy yet powerful programming interface, e.g. for usage in scripting.
             */
            class AbstractActionDrivenSearchCriterion : public SearchCriterion
            {
                friend class ActionAbstractActionDrivenSearchCriterionConfigure;
            public:
                QStringList searchspec;

                AbstractActionDrivenSearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);

                virtual bool match(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl);
                virtual bool match2(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl) =0;
                virtual QString valuedescription();

                static void _createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, sh::ui::SearchPanelConfiguration* panelcfg);
                static void _editorUpdateConfiguration(sh::ui::SearchPanelConfiguration* panelcfg, std::shared_ptr<sh::search::SearchCriterion> c);

            protected:
                static void updatelabel(std::shared_ptr<sh::search::SearchCriterion> cfg, ui::SearchPanelConfiguration *panelcfg);
                virtual void _deserialize(QString s);
                virtual QString _serialize();
                virtual void configure(sh::actions::ActionExecutionInfo* info) =0;
                virtual QString getSearchSpecDescription();

                static QSet<sh::ui::SearchPanelConfiguration*> _aliveWidgets;
            };

            class ActionAbstractActionDrivenSearchCriterionConfigure : public sh::actions::ActionActionItem
            {
            public:
                ActionAbstractActionDrivenSearchCriterionConfigure(std::shared_ptr<AbstractActionDrivenSearchCriterion> cfg, QMutex *mutex);
                virtual void action(sh::actions::ActionExecutionInfo* info);
            private:
                std::shared_ptr<AbstractActionDrivenSearchCriterion> _cfg;
                QMutex* _mutex;
            };

        }
    }
}

#endif // ABSTRACTACTIONDRIVENSEARCHCRITERION_H
