/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTSTRINGSEARCHCRITERION_H
#define ABSTRACTSTRINGSEARCHCRITERION_H

#include <limits>
#include "search/searchcriterion.h"
#include "search/searchcriterionfactory.h"


namespace sh{
    namespace filesystem{ class Eurl; class FilesystemNode; }

    namespace search{
        namespace criteria {

            /** \brief Abstract search criterion filtering by some string value (providing different modes like exact, fuzzy, regexp, ...).
             */
            class AbstractStringSearchCriterion : public SearchCriterion
            {
            public:
                enum class Mode
                {
                    Simple,
                    Exact,
                    RegExp,
                    Fuzzy
                };

                static const QStringList ModeCodes;

                AbstractStringSearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);
                QString string;
                Mode mode = Mode::Simple;
                virtual bool match(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl);
                virtual QString valuedescription();

                static void _createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, ui::SearchPanelConfiguration *panelcfg);
                static void _editorUpdateConfiguration(sh::ui::SearchPanelConfiguration* panelcfg, std::shared_ptr<sh::search::SearchCriterion> c);

                static int levenshteinSubstringDistance(QString needle, QString haystack);
                static double relativeLevenshteinPartsSubstringDistance(QString needle, QString haystack);

            protected:
                virtual void _deserialize(QString s);
                virtual QString _serialize();

                virtual QString itemstring(std::shared_ptr<const sh::filesystem::Eurl> eurl, sh::filesystem::Operation* op) =0;
                virtual bool forceCaseInsensitive() =0;

                QRegularExpression reString;
                QString reStringString;

                QRegularExpression reGlobString;
                QString reGlobStringString;

            };

        }
    }
}

#endif // ABSTRACTSTRINGSEARCHCRITERION_H
