/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QRegularExpression>
#include "filesystem/eurl.h"
#include "base/mainthread.h"
#include "base/singletoninitializer.h"
#include "filesystem/operation.h"
#include "filesystem/filesystemoperation.h"
#include "search/searchcriterionfactoryfromfunction.h"
#include "search/searchmanager.h"
#include "filecontentsearchcriterion.h"

REGISTER_STATICINIT(sh::search::criteria, FileContentSearchCriterion, "base")


void sh::search::criteria::FileContentSearchCriterion::doInitialize()
{
    REGISTER_CRITERION_FACTORY(sh::search::SearchManager::REGISTER_SEARCHCRITERION_INDEX_CORE+14000, "content", QObject::tr("File content"), sh::search::criteria::FileContentSearchCriterion)
}

void sh::search::criteria::FileContentSearchCriterion::doShutdown()
{
}

sh::search::criteria::FileContentSearchCriterion::FileContentSearchCriterion(std::shared_ptr<SearchCriterionFactory> factory)
                : sh::search::criteria::AbstractStringSearchCriterion(factory)
{
}

void sh::search::criteria::FileContentSearchCriterion::createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, sh::ui::SearchPanelConfiguration* panelcfg)
{
    AbstractStringSearchCriterion::_createEditor(cfg, panelcfg);
}

void sh::search::criteria::FileContentSearchCriterion::editorUpdateConfiguration(ui::SearchPanelConfiguration *panelcfg, std::shared_ptr<sh::search::SearchCriterion> c)
{
    AbstractStringSearchCriterion::_editorUpdateConfiguration(panelcfg, c);
}

QString sh::search::criteria::FileContentSearchCriterion::itemstring(std::shared_ptr<const sh::filesystem::Eurl> eurl, filesystem::Operation *op)
{
    std::shared_ptr<QIODevice> econtent = op->filesystem()->getFileContent(eurl);
    QByteArray ba = econtent->readAll();
    return QString::fromUtf8(ba);
}

bool sh::search::criteria::FileContentSearchCriterion::forceCaseInsensitive()
{
    return false;
}

