/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILECONTENTSEARCHCRITERION_H
#define FILECONTENTSEARCHCRITERION_H

#include "search/searchcriterionfactory.h"
#include "abstractstringsearchcriterion.h"


namespace sh{
    namespace filesystem{ class Eurl; class FilesystemNode; }

    namespace search{
        namespace criteria {

            /** \brief Search criterion filtering by file contents.
             */
            class FileContentSearchCriterion : public sh::search::criteria::AbstractStringSearchCriterion
            {
            public:

                FileContentSearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);

                static void createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, sh::ui::SearchPanelConfiguration* panelcfg);
                static void editorUpdateConfiguration(sh::ui::SearchPanelConfiguration* panelcfg, std::shared_ptr<sh::search::SearchCriterion> c);

                /** @a_infrastructure
                 */
                static void doInitialize();

                /** @a_infrastructure
                 */
                static void doShutdown();

            protected:
                virtual QString itemstring(std::shared_ptr<const sh::filesystem::Eurl> eurl, sh::filesystem::Operation* op);
                virtual bool forceCaseInsensitive();

            };

        }
    }
}

#endif // FILECONTENTSEARCHCRITERION_H
