/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filesystem/eurl.h"
#include "base/mainthread.h"
#include "base/singletoninitializer.h"
#include "search/searchcriterionfactoryfromfunction.h"
#include "search/searchmanager.h"
#include "filenamesearchcriterion.h"

REGISTER_STATICINIT(sh::search::criteria, FilenameSearchCriterion, "base")


void sh::search::criteria::FilenameSearchCriterion::doInitialize()
{
    REGISTER_CRITERION_FACTORY(sh::search::SearchManager::REGISTER_SEARCHCRITERION_INDEX_CORE+5000, "name", QObject::tr("File name"), sh::search::criteria::FilenameSearchCriterion)
}

void sh::search::criteria::FilenameSearchCriterion::doShutdown()
{
}

sh::search::criteria::FilenameSearchCriterion::FilenameSearchCriterion(std::shared_ptr<SearchCriterionFactory> factory)
                : sh::search::criteria::AbstractStringSearchCriterion(factory)
{
}

void sh::search::criteria::FilenameSearchCriterion::createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, sh::ui::SearchPanelConfiguration* panelcfg)
{
    AbstractStringSearchCriterion::_createEditor(cfg, panelcfg);
}

void sh::search::criteria::FilenameSearchCriterion::editorUpdateConfiguration(ui::SearchPanelConfiguration *panelcfg, std::shared_ptr<sh::search::SearchCriterion> c)
{
    AbstractStringSearchCriterion::_editorUpdateConfiguration(panelcfg, c);
}

QString sh::search::criteria::FilenameSearchCriterion::itemstring(std::shared_ptr<const sh::filesystem::Eurl> eurl, filesystem::Operation*)
{
    if (mode == Mode::RegExp)
        return eurl->path();
    else
        return eurl->basename();
}

bool sh::search::criteria::FilenameSearchCriterion::forceCaseInsensitive()
{
    return true;
}
