/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MTIMESEARCHCRITERION_H
#define MTIMESEARCHCRITERION_H

#include <QDateTime>
#include "search/searchcriterion.h"
#include "search/searchcriterionfactory.h"


namespace sh{
    namespace filesystem{ class Eurl; class FilesystemNode; }

    namespace search{
        namespace criteria {

            /** \brief Search criterion filtering by file modification times.
             */
            class MtimeSearchCriterion : public sh::search::SearchCriterion
            {
            public:

                QDateTime from, to;

                MtimeSearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);

                virtual bool match(sh::filesystem::Operation* op, std::shared_ptr<const sh::filesystem::Eurl> eurl);
                virtual QString valuedescription();

                static void createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, sh::ui::SearchPanelConfiguration* panelcfg);
                static void editorUpdateConfiguration(sh::ui::SearchPanelConfiguration* panelcfg, std::shared_ptr<sh::search::SearchCriterion> c);

                /** @a_infrastructure
                 */
                static void doInitialize();

                /** @a_infrastructure
                 */
                static void doShutdown();

            protected:
                virtual void _deserialize(QString s);
                virtual QString _serialize();

            };

        }
    }
}

#endif // MTIMESEARCHCRITERION_H
