/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "exceptions/argumentexception.h"
#include "searchconfiguration.h"


sh::search::SearchConfiguration::SearchConfiguration()
{
}

sh::search::SearchConfiguration* sh::search::SearchConfiguration::deserialize(QString s)
{
    sh::search::SearchConfiguration* result = new sh::search::SearchConfiguration();
    QStringList items = s.split("|");
    foreach(QString _s, items)
    {
        if (!_s.isEmpty())
        {
            QString s = QUrl::fromPercentEncoding(_s.toUtf8());
            int icolon = s.indexOf(":");
            if (icolon == -1)
                throw sh::exceptions::ArgumentException(sh::exceptions::ExceptionData().detailsAreInteresting().details(QObject::tr("Invalid format in search configuration token: ':' missing: ") + s));
            QString key = s.mid(0, icolon);
            s = s.mid(icolon+1);
            if (key == "")
                result->criteria.append(sh::search::SearchCriterion::deserialize(s));
            else if (key == "tree")
                result->showAsTree = (s == "1");
        }
    }
    return result;
}

QString sh::search::SearchConfiguration::serialize()
{
    QStringList lst;
    foreach(std::shared_ptr<sh::search::SearchCriterion> criterion, criteria)
        lst << ":" + criterion->serialize();
    if (showAsTree)
        lst << "tree:1";
    QStringList elst;
    foreach(QString s, lst)
        elst << s.replace("%", "%25").replace("/", "%2F").replace("|", "%7C");
    return elst.join('|');
}
