/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SEARCHCONFIGURATION_H
#define SEARCHCONFIGURATION_H

#include <memory>
#include <QUrl>
#include <QObject>
#include "searchcriterion.h"


namespace sh{
    namespace search{

        /** \brief A search configuration.
         *
         * Search configurations contain a list of search criteria and some auxiliary fields.
         * Each search configuration describes what and how a user wants to search (just not where).
         */
        class SearchConfiguration
        {
        public:
            SearchConfiguration();
            QList<std::shared_ptr<sh::search::SearchCriterion>> criteria;
            bool showAsTree = false;
            /** \brief Serialize this search configuration to a string. See also SearchConfiguration::deserialize.
             */
            QString serialize();
            /** \brief Deserialize a search configuration from a string. See also SearchConfiguration::serialize.
             */
            static SearchConfiguration* deserialize(QString s);
        };

    }
}

#endif // SEARCHCONFIGURATION_H
