/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ui/searchpanelconfiguration.h"
#include "searchcriterionfactoryfromfunction.h"


sh::search::SearchCriterionFactoryFromFunction::SearchCriterionFactoryFromFunction(QString key,
                             std::function<QString()> descriptionfct,
                             std::function<void(std::shared_ptr<sh::search::SearchCriterion>, sh::ui::SearchPanelConfiguration*)> editorfct,
                             std::function<void(sh::ui::SearchPanelConfiguration*, std::shared_ptr<sh::search::SearchCriterion>)> editorupdateconfigfct,
                             std::function<std::shared_ptr<sh::search::SearchCriterion>(std::shared_ptr<sh::search::SearchCriterionFactory>)> constructfct)
            : SearchCriterionFactory(), _key(key), _descriptionfct(descriptionfct), _editorfct(editorfct), _editorupdateconfigfct(editorupdateconfigfct), _constructfct(constructfct)
{
}

QString sh::search::SearchCriterionFactoryFromFunction::key()
{
    return _key;
}

QString sh::search::SearchCriterionFactoryFromFunction::description()
{
    return _descriptionfct();
}

void sh::search::SearchCriterionFactoryFromFunction::editor(std::shared_ptr<sh::search::SearchCriterion> c, sh::ui::SearchPanelConfiguration* panelcfg)
{
    _editorfct(c, panelcfg);
}

void sh::search::SearchCriterionFactoryFromFunction::editorupdateconfig(sh::ui::SearchPanelConfiguration* panelcfg, std::shared_ptr<sh::search::SearchCriterion> c)
{
    _editorupdateconfigfct(panelcfg, c);
}

std::shared_ptr<sh::search::SearchCriterion> sh::search::SearchCriterionFactoryFromFunction::construct()
{
    return _constructfct(shared_from_this());
}
