/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filesystem/eurl.h"
#include "filesystem/filesystemmodel.h"
#include "base/threadpool.h"
#include "base/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/fileview.h"
#include "fileviewpath.h"

REGISTER_SETTING(sh::settings::common::FileViewPath)

sh::settings::common::FileViewPath::FileViewPath()
{
}

QString sh::settings::common::FileViewPath::name()
{
    return "filesystempanel-path";
}

QString sh::settings::common::FileViewPath::description()
{
    return QObject::tr("File view path");
}

sh::settings::SettingGroup sh::settings::common::FileViewPath::group()
{
    return sh::settings::SettingGroup::DATAVIEW;
}

bool sh::settings::common::FileViewPath::isAdvancedSetting()
{
    return true;
}

void sh::settings::common::FileViewPath::setValue(sh::ui::FileView *fileview, QString value)
{
    IGNORE_FORBIDDEN_ASYNC_CALLS
    std::shared_ptr<const sh::filesystem::Eurl> eurl = sh::filesystem::Eurl::fromString(value);
    int index = fileview->index();
    sh::base::ThreadPool::enqueueForce([=](){
        QList<QPersistentModelIndex> il = sh::filesystem::FilesystemModel::instance()->findIndexesForEurl(eurl);
        if (!il.isEmpty())
        {
            sh::base::MainThread::dispatcher()->invokeAsync([=](){
                QModelIndex i = il.first();
                if (i.isValid() && sh::ui::MainWindow::mainWindow()->fileViewCount() > index) {
                    std::shared_ptr<sh::filesystem::FilesystemNode> node = sh::filesystem::FilesystemModel::instance()->getNodeForIndex(i);
                    sh::ui::MainWindow::mainWindow()->getFileView(index)->gotoDir(node);
                }
            });
        }
    });
}

QString sh::settings::common::FileViewPath::getValue(sh::ui::FileView *fileview)
{
    return fileview->node()->eurl()->asString();
}

bool sh::settings::common::FileViewPath::isGlobal()
{
    return true;
}

bool sh::settings::common::FileViewPath::isPerFileview()
{
    return true;
}

