/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ui/mainwindow.h"
#include "ui/fileview.h"
#include "fileviewviewmode.h"


REGISTER_SETTING(sh::settings::common::FileViewViewmode)

sh::settings::common::FileViewViewmode::FileViewViewmode()
{
}

QString sh::settings::common::FileViewViewmode::name()
{
    return "filesystempanel-viewmode";
}

QString sh::settings::common::FileViewViewmode::description()
{
    return QObject::tr("File view mode");
}

sh::settings::SettingGroup sh::settings::common::FileViewViewmode::group()
{
    return sh::settings::SettingGroup::DATAVIEW;
}

bool sh::settings::common::FileViewViewmode::isAdvancedSetting()
{
    return false;
}

void sh::settings::common::FileViewViewmode::setValue(sh::ui::FileView *fileview, QString value)
{
    int fileviewmode = QVariant(value).toInt();
    fileview->setViewmode((sh::ui::FileViewMode)fileviewmode);
}

QString sh::settings::common::FileViewViewmode::getValue(sh::ui::FileView *fileview)
{
    sh::ui::FileViewMode viewmode = fileview->viewmode();
    return QVariant((int)viewmode).toString();
}

bool sh::settings::common::FileViewViewmode::isGlobal()
{
    return false;
}

bool sh::settings::common::FileViewViewmode::isPerFileview()
{
    return true;
}

QString sh::settings::common::FileViewViewmode::valueDescription(QString value)
{
    sh::ui::FileViewMode fileviewmode = (sh::ui::FileViewMode) QVariant(value).toInt();
    if (fileviewmode == sh::ui::FileViewMode::IconView)
        return QObject::tr("icon view");
    else if (fileviewmode == sh::ui::FileViewMode::ListView)
        return QObject::tr("list view");
    else
        return QObject::tr("unknown");
}

