/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ui/mainwindow.h"
#include "stickytreeview.h"


REGISTER_SETTING(sh::settings::common::StickyTreeview)

sh::settings::common::StickyTreeview::StickyTreeview()
{
}

QString sh::settings::common::StickyTreeview::name()
{
    return "sticky-treeview";
}

QString sh::settings::common::StickyTreeview::description()
{
    return QObject::tr("Sticky treeview");
}

sh::settings::SettingGroup sh::settings::common::StickyTreeview::group()
{
    return sh::settings::SettingGroup::BEHAVIOR;
}

bool sh::settings::common::StickyTreeview::isAdvancedSetting()
{
    return true;
}

void sh::settings::common::StickyTreeview::setValue(QString value )
{
    sh::ui::MainWindow::mainWindow()->setTreeSticky(value=="1");
}

QString sh::settings::common::StickyTreeview::getValue(sh::ui::FileView * )
{
    return ( sh::ui::MainWindow::mainWindow()->treeSticky() ?"1":"0");
}

QString sh::settings::common::StickyTreeview::valueDescription(QString value)
{
    return (value=="1" ?
                QObject::tr("Tree cursor follows the file view") :
                QObject::tr("Tree cursor does not follow the file view"));
}

bool sh::settings::common::StickyTreeview::isGlobal()
{
    return false;
}

bool sh::settings::common::StickyTreeview::isPerFileview()
{
    return false;
}
