/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "accountsmanager.h"

REGISTER_SINGLETON(sh::tools::accounts, AccountsManager, "base")


sh::tools::accounts::AccountsManager::AccountsManager()
{
}

void sh::tools::accounts::AccountsManager::registerProvider(std::shared_ptr<AbstractAccountsProvider> provider)
{
    QMutexLocker l1(&mutex);
    accountsproviders.append(provider);
}

QList<std::shared_ptr<sh::tools::accounts::Account>> sh::tools::accounts::AccountsManager::findAccounts(std::shared_ptr<Account> pattern)
{
    QMutexLocker l1(&mutex);
    QList<std::shared_ptr<sh::tools::accounts::Account>> result;
    foreach (std::shared_ptr<AbstractAccountsProvider> provider, accountsproviders)
        provider->findAccounts(pattern, result);
    return result;
}

void sh::tools::accounts::AccountsManager::storeAccount(std::shared_ptr<Account> account)
{
    QMutexLocker l1(&mutex);
    foreach (std::shared_ptr<AbstractAccountsProvider> provider, accountsproviders) {
        if (provider->storeAccount(account))
            break;
    }
}
