/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACCOUNTSMANAGER_H
#define ACCOUNTSMANAGER_H

#include <memory>
#include <QMutex>
#include "base/singletoninitializer.h"
#include "abstractaccountsprovider.h"


namespace sh{
    namespace tools{
        namespace accounts{

            /** \brief
             * Account data (for accessing network drives).
             */
            class Account
            {
            public:
                Account() = default;
                Account(const Account& other) = default;
                QString username;
                QString domain;
                QString server;
                int serverport = -1;
                QString path;
                QString protocol;
                QString authtype;
                QByteArray authinfo;
            };

            /** \brief
             * Storage for account data (for accessing network drives).
             */
            class AccountsManager : public sh::base::Singleton
            {
                DECLARE_SINGLETON(AccountsManager)
            public:

                /** \brief
                 * Finds account infos by a given pattern.
                 */
                QList<std::shared_ptr<Account>> findAccounts(std::shared_ptr<Account> pattern);

                /** \brief
                 * Stores account infos.
                 */
                void storeAccount(std::shared_ptr<Account> account);

                /** \brief
                 * Registers a new accounts provider.
                 */
                void registerProvider(std::shared_ptr<AbstractAccountsProvider> provider);

            private:
                AccountsManager();
                QList<std::shared_ptr<AbstractAccountsProvider>> accountsproviders;
                QMutex mutex;
            };

        }
    }
}

#endif // ACCOUNTSMANAGER_H
