/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <algorithm>
#include "exceptions/ioexception.h"
#include "datastream.h"


sh::tools::LocalFile::LocalFile(QString& file, bool isTemp) :
    QFile(file), _istemp(isTemp)
{

}

bool sh::tools::LocalFile::isTemp()
{
    return _istemp;
}

sh::tools::ReadDataDevice::ReadDataDevice() : QIODevice()
{
    if (!open(QFile::ReadOnly))
        throw sh::exceptions::IOException(sh::exceptions::ExceptionData().detailsAreInteresting().details(tr("Unable to open a data device for reading: ") + this->metaObject()->className()));
}

qint64 sh::tools::ReadDataDevice::readData (char * data, qint64 maxSize)
{
    while (currentconsumed == currentlen)
    {
        current = getdata();
        currentconsumed = 0;
        currentlen = current.length();
        if (current.isNull())
            break;  // this is when the stream ended (instead of just temporarily be stalled)
    }
    qint64 res = std::min((qint64)(currentlen-currentconsumed), maxSize);
    memcpy(data, current.data()+currentconsumed, res);
    currentconsumed += res;
    return res;
}

qint64 sh::tools::ReadDataDevice::writeData (const char*, qint64)
{
    return -1;  // this is solely a read device
}
