/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef EARLYMUTEX_H
#define EARLYMUTEX_H

#include <QMutex>
#include "base/mainthread.h"
#include "base/singletoninitializer.h"


/*
 * Use DECLARE_EARLYMUTEX inside class declaration and REGISTER_EARLYMUTEX in source file for defining a mutex,
 * which is available very early (even before all static stuff is initialized).
 */

#define DECLARE_EARLYMUTEX(NAME) \
    public: \
    /** \brief Internal helper class for the NAME() mutex. */ \
    class _ShEarlyMutex_ ## NAME : public sh::base::Singleton { \
        DECLARE_SINGLETON(_ShEarlyMutex_ ## NAME) \
    public: \
        QMutex * const m; \
        _ShEarlyMutex_ ## NAME(); \
        ~_ShEarlyMutex_ ## NAME(); \
    }; \
    static QMutex* NAME(); \
    private: \

#define REGISTER_EARLYMUTEX(NS, NAME) \
    REGISTER_SINGLETON(NS, _ShEarlyMutex_ ## NAME, "early") \
    NS::_ShEarlyMutex_ ## NAME::_ShEarlyMutex_ ## NAME() : m(new QMutex(QMutex::Recursive)) {} \
    NS::_ShEarlyMutex_ ## NAME::~_ShEarlyMutex_ ## NAME() {delete m;} \
    /** \brief Returns the NAME mutex. */ \
    QMutex* NS::NAME() {return NS::_ShEarlyMutex_ ## NAME::instance()->m;}


#endif // EARLYMUTEX_H
