/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FREEDESKTOPORGTOOLSOPENMETHODDETERMINATIONMETHOD_H
#define FREEDESKTOPORGTOOLSOPENMETHODDETERMINATIONMETHOD_H

#include <QtConcurrentRun>
#include <QDir>
#include <QQueue>
#include <QRegularExpression>
#include <QSettings>
#include <QWaitCondition>
#include "filetypemanager.h"


namespace sh {
    namespace tools {
        namespace filetypes {

            /** \brief Tries to determine a 'open method' for a file with the freedesktop.org tools.
             */
            class FreedesktopOrgToolsOpenMethodDeterminationStrategy : public sh::tools::filetypes::FileTypeManager::OpenMethodDeterminationStrategy
            {
            public:
                FreedesktopOrgToolsOpenMethodDeterminationStrategy();
                ~FreedesktopOrgToolsOpenMethodDeterminationStrategy();
                virtual QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> getOpenMethods(QString mimetype, QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);

            private:

                struct ApplicationEntry {
                    QString name;
                    QStringList mimetypes;
                    QString command;
                    QStringList commandargs;
                    QIcon icon;
                    bool hidden;
                };

                QMultiMap<QString, ApplicationEntry*> _mimetype2applicationEntry;
                QMutex _mutex;
                QWaitCondition _cond_inited;
                bool _inited = false;

                QString _parseValue(QStringList content, QString key);
                void _parseExecLine(QString execline, QString* command, QStringList* arguments);
                QString _escapeExecLineToken(QString s);

            };

        }
    }
}

#endif // FREEDESKTOPORGTOOLSOPENMETHODDETERMINATIONMETHOD_H
