/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef USERDEFINEDOPENMETHODDETERMINATIONSTRATEGY_H
#define USERDEFINEDOPENMETHODDETERMINATIONSTRATEGY_H

#include "filetypemanager.h"
#include "base/singletoninitializer.h"


namespace sh {
    namespace filesystem {
        class FilesystemNode;
    }

    namespace tools {
        namespace filetypes {

            /** \brief Tries to determine a 'open method' for a file by information the user stored before.
             */
            class UserDefinedOpenMethodDeterminationStrategy : public sh::tools::filetypes::FileTypeManager::OpenMethodDeterminationStrategy, public sh::base::Singleton
            {
                DECLARE_SINGLETON(UserDefinedOpenMethodDeterminationStrategy)
            public:

                QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> getOpenMethods(QString mimetype, QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes) override;

                /** @brief Stores a custom open method.
                 */
                void storeCustomOpenMethod(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes, QString mimetype, std::shared_ptr<sh::tools::filetypes::OpenMethod> m, bool rememberForMimetype, std::shared_ptr<const sh::filesystem::Eurl> rememberForDirectory, bool rememberForFile);

            private:
                QMutex _mutex;
                UserDefinedOpenMethodDeterminationStrategy();

            };

        }
    }
}

#endif // USERDEFINEDOPENMETHODDETERMINATIONSTRATEGY_H
