/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef HISTORYTRACKER_H
#define HISTORYTRACKER_H

#include <QList>


namespace sh{
    namespace tools{

        /** \brief
         * A data structure for tracking a history of data.
         *
         * It is a templated container It is typically used for tracking
         * the directory history (used by sh::actions::common::ActionHistoryNavigate, et al).
         */
        template<class T>
        class HistoryTracker
        {
        public:

            class HistoryEntry
            {
            public:
                HistoryEntry(int i, T v) : _index(i), _value(v) {}
                int index() { return _index; }
                T value() { return _value; }
            private:
                int _index;
                T _value;
            };

            const int HISTORY_SIZE = 10;

            HistoryTracker() {}

            void visitValue(T v) {
                if (_current >= 0 && v == _items[_current])
                    return;
                for(int i=_items.length()-_current-2; i>=0; i--)
                    _items.removeLast();
                int l = _items.length();
                if (l == HISTORY_SIZE)
                {
                    _items.removeFirst();
                    l--;
                }
                _current = l;
                _items.append(v);
            }

            int count() { return _items.length(); }

            QList<HistoryEntry> forwardList() {
                QList<HistoryEntry> res;
                for (int i=_current+1; i<_items.length(); i++)
                    res.append(HistoryEntry(i, _items[i]));
                return res;
            }

            QList<HistoryEntry> backwardList() {
                QList<HistoryEntry> res;
                for (int i=_current-1; i>=0; i--)
                    res.append(HistoryEntry(i, _items[i]));
                return res;
            }

            void revisitValue(int idx) {
                _current = idx;
            }

        private:
            QList<T> _items;
            int _current = -1;
        };

    }
}

#endif // HISTORYTRACKER_H
