/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LOCALFILESYSTEMWATCHERCONNECTOR_H
#define LOCALFILESYSTEMWATCHERCONNECTOR_H

#include <memory>
#include <QList>
#include <QObject>
#include "base/singletoninitializer.h"
#include "filesystem/eurl.h"


namespace sh{
    namespace filesystem {
        class FilesystemNode;
    }
    namespace filesystemhandlers {
        class LocalFilesystemHandler;
    }
    namespace tools{

        /** \brief
         * Observes the list of file views via VisibleViews and controls LocalFilesystemWatcher with that
         * for observing the filesystem for changes.
         */
        class LocalFilesystemWatcherConnector : public QObject, public sh::base::Singleton
        {
            Q_OBJECT
            DECLARE_SINGLETON(LocalFilesystemWatcherConnector)
        protected slots:
            void slot_watcher_elementchanged(QString name, QList<qint64> ids);

        private:
            LocalFilesystemWatcherConnector();
            QList<QPair<std::shared_ptr<sh::filesystem::FilesystemNode>, sh::filesystemhandlers::LocalFilesystemHandler*>> getAllLocalParts(std::shared_ptr<filesystem::FilesystemNode> node);
            QHash<std::shared_ptr<const sh::filesystem::Eurl>, QList<qint64>> dir2watcherids;
            QHash<qint64, std::shared_ptr<const sh::filesystem::Eurl>> watcherid2dir;
        };

    }
}

#endif // LOCALFILESYSTEMWATCHERCONNECTOR_H
