/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filesystem/filesystemnode.h"
#include "base/mainthread.h"
#include "filesystem/operation.h"
#include "filesystem/eurl.h"
#include "operationscache.h"

REGISTER_SINGLETON(sh::tools, OperationsCache, "base")


sh::tools::OperationsCache::OperationsCache() : QObject()
{
}

std::shared_ptr<sh::filesystem::Operation> sh::tools::OperationsCache::getOperationForContainer(std::shared_ptr<const sh::filesystem::Eurl> container)
{
    {
        QMutexLocker _l1(&cachemutex);
        auto itr = cache.find(container);
        if (itr != cache.end())
            return *itr;
        else
        {
            std::shared_ptr<sh::filesystem::Operation> result(new sh::filesystem::Operation());
            result->setMaxAllowedSizeRatioPerPart(0.0001);
            cache.insert(container, result);
            sh::base::MainThread::dispatcher()->invokeAsync([=](){
                QTimer* timer = new QTimer(sh::base::MainThread::dispatcher());
                QObject::connect(timer, &QTimer::timeout, this, [=](){
                    QMutexLocker _l1(&cachemutex);
                    cache.remove(container);
                    timer->deleteLater();
                });
                timer->setSingleShot(true);
                timer->start(10000);
            });
            return result;
        }
    }
}

