/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef OPERATIONSCACHE_H
#define OPERATIONSCACHE_H

#include <memory>
#include <QHash>
#include <QMutex>
#include <QTimer>
#include "base/singletoninitializer.h"


namespace sh{
    namespace filesystem{ class Eurl; class Operation; }
    namespace tools{

        /** \brief
         * A cached factory for readonly sh::filesystem::Operation instances.
         *
         * You can use it for fetching an sh::filesystem::Operation for a certain sh::filesystem::Eurl.
         * Two restrictions apply:
         * - Data might be slightly outdated.
         * - Just for read-only access.
         */
        class OperationsCache : public QObject, public sh::base::Singleton
        {
            Q_OBJECT
            DECLARE_SINGLETON(OperationsCache)
        public:

            /** @brief Returns a cached sh::filesystem::Operation for a container (by eurl).
             */
            std::shared_ptr<sh::filesystem::Operation> getOperationForContainer(std::shared_ptr<const sh::filesystem::Eurl> container);

        private:
            OperationsCache();
            QHash<std::shared_ptr<const sh::filesystem::Eurl>, std::shared_ptr<sh::filesystem::Operation>> cache;
            QMutex cachemutex;
        };

    }
}

#endif // OPERATIONSCACHE_H
