/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filesystem/filesystemnode.h"
#include "filesystem/operation.h"
#include "base/mainthread.h"
#include "base/singletoninitializer.h"
#include "defaultimagethumbnailprovider.h"

REGISTER_STATICINIT(sh::tools::thumbnailproviders, DefaultImageThumbnailProvider, "base")


void sh::tools::thumbnailproviders::DefaultImageThumbnailProvider::doInitialize()
{
    ThumbnailManager::instance()->addThumbnailProvider(ThumbnailManager::REGISTER_THUMBNAILPROVIDER_INDEX_CORE+1000, std::make_shared<DefaultImageThumbnailProvider>());
}

void sh::tools::thumbnailproviders::DefaultImageThumbnailProvider::doShutdown()
{
}

void sh::tools::thumbnailproviders::DefaultImageThumbnailProvider::getThumbnail(sh::filesystem::Operation *operation, std::shared_ptr<sh::filesystem::FilesystemNode> node, QString contentType, int width, int height, QIcon* icon)
{
    if (contentType.startsWith("image/")) {
        QString imgfile;
        sh::exceptions::Exception::executeGuarded([&](){
            imgfile = operation->fetchFile(node->eurl());
        }, sh::exceptions::ExecuteGuardFlag::AUTOMATIC_RETRY_ENABLED|sh::exceptions::ExecuteGuardFlag::IGNORE_ALL_RESUMEABLE_ERRORS_SILENTLY);
        if (!imgfile.isNull()) {
            QImage img(imgfile);
            *icon = QIcon(QPixmap::fromImage(img.scaled(width, height, Qt::KeepAspectRatio, Qt::SmoothTransformation)));
        }
    }
}
