/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef USERDIRLOCK_H
#define USERDIRLOCK_H

#include <QDateTime>
#include <QDir>
#include <QFile>
#include <QMutex>
#include <QThread>


namespace sh{
    namespace tools {

        /** \brief Locks the Shallot user directory during exclusive usage.
         *
         * This lock is rather heavy-weight and should be used only if necessary!
         */
        class UserDirLock
        {
        public:

            /** \brief Is intended to be directly constructed from everywhere.
             */
            UserDirLock();

            ~UserDirLock();

        private:
            static QThread* currentThread;
            static QMutex mutex;
            bool dounlock;
#ifdef Q_OS_UNIX
            int semset_id;
#else
            QString slockpref;
            QString slockfile;
#endif
        };

#ifdef Q_OS_UNIX

        union semun {
            int val; /* Value for SETVAL */
            struct semid_ds *buf; /* Buffer for IPC_STAT, IPC_SET */
            unsigned short *array; /* Array for GETALL, SETALL */
            struct seminfo *__buf; /* Buffer for IPC_INFO (Linux-specific) */
        };

#endif

    }
}

#endif // USERDIRLOCK_H
