/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "filesystem/filesystemnode.h"
#include "filesystem/filesystemhandler.h"
#include "visibleviews.h"

REGISTER_SINGLETON(sh::tools, VisibleViews, "base")


sh::tools::VisibleViews::VisibleViews()
{
}

std::shared_ptr<QObject> sh::tools::VisibleViews::viewEnteredDirectory(std::shared_ptr<sh::filesystem::FilesystemNode> dir)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    std::shared_ptr<QObject> result = std::make_shared<QObject>();
    sh::exceptions::Exception::executeGuarded([&](){
        if (dir->handler() && visibleDirs[dir]++ == 0) {
            foreach(auto handler, onenteredhandlers)
                handler(dir);
            dir->handler()->viewEnteredDirectory(dir->eurl());
        }
        QObject::connect(result.get(), &QObject::destroyed, [=](){
            if (dir->handler() && --visibleDirs[dir] == 0) {
                visibleDirs.remove(dir);
                dir->handler()->viewLeftDirectory(dir->eurl());
                foreach(auto handler, onlefthandlers)
                    handler(dir);
            }
        });
    }, sh::exceptions::ExecuteGuardFlag::ALL_ERRORS_KILL_SHALLOT);
    return result;
}

void sh::tools::VisibleViews::registerOnEnteredHandlers(std::function<void(std::shared_ptr<sh::filesystem::FilesystemNode> dir)> handler)
{
    onenteredhandlers.append(handler);
}

void sh::tools::VisibleViews::registerOnLeftHandlers(std::function<void(std::shared_ptr<sh::filesystem::FilesystemNode> dir)> handler)
{
    onlefthandlers.append(handler);
}
