/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef VISIBLEVIEWS_H
#define VISIBLEVIEWS_H

#include <functional>
#include <memory>
#include <QHash>
#include <QObject>
#include "base/singletoninitializer.h"


namespace sh{
    namespace filesystem{class FilesystemNode;}

    namespace tools {

        /** \brief
         * Used for keeping track of which fileviews show which directories.
         *
         * Whenever the view goes to other directories, this manager gets
         * notified and redirects the notification to some other parts of the
         * program.
         */
        class VisibleViews : public sh::base::Singleton
        {
            DECLARE_SINGLETON(VisibleViews)
        public:

            /** \brief
             * Called when a view enters the given directory node.
             * @a_infrastructure
             */
            std::shared_ptr<QObject> viewEnteredDirectory(std::shared_ptr<sh::filesystem::FilesystemNode> dir);

            /** \brief
             * Registers a function to be called whenever the view just entered the given directory node.
             */
            void registerOnEnteredHandlers(std::function<void(std::shared_ptr<sh::filesystem::FilesystemNode> dir)> handler);

            /** \brief
             * Registers a function to be called whenever the view just left the given directory node.
             */
            void registerOnLeftHandlers(std::function<void(std::shared_ptr<sh::filesystem::FilesystemNode> dir)> handler);

        private:
            VisibleViews();
            QHash<std::shared_ptr<sh::filesystem::FilesystemNode>, int> visibleDirs;
            QList<std::function<void(std::shared_ptr<sh::filesystem::FilesystemNode>)>> onenteredhandlers;
            QList<std::function<void(std::shared_ptr<sh::filesystem::FilesystemNode>)>> onlefthandlers;
        };

    }
}

#endif // VISIBLEVIEWS_H
