/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONEXECUTIONINFODIALOG_H
#define ACTIONEXECUTIONINFODIALOG_H

#include <functional>
#include "actions/actionexecutionuserfeedback.h"


namespace sh {
    namespace actions {class ActionActionItem;class ActionExecutionInfo;}
    namespace ui {

        /** \brief
         * Progress dialog for action executions.
         */
        class ActionExecutionInfoDialog : public sh::actions::ActionExecutionUserFeedback
        {
        public:
            /** \brief
             * @a_infrastructure
             */
            ActionExecutionInfoDialog(sh::actions::ActionExecutionInfo *info);

            /** \brief Sets the item details text ('from a/foo.jpg', 'to b/foo.jpg').
             * @a_dooverride
             */
            virtual void setDetails(QString fv, QString fob, QString tv, QString tob) =0;

            /** \brief Sets the header text.
             * @a_dooverride
             */
            virtual void setHead(QString txt) =0;

            /** \brief Sets the progress.
             * @a_dooverride
             */
            virtual void setProgress(bool isDeterminate, quint64 done, quint64 all, QString text) =0;

            /** \brief Returns if this dialog is logically visible (i.e. set visible by the action).
             */
            bool isLogicallyVisible();

            /** \brief Sets if this dialog is logically visible (i.e. set visible by the action).
             * @a_dooverride
             */
            virtual void setLogicallyVisible(bool v);

            /** \brief Returns if this dialog is currently in background mode (i.e. not visible).
             */
            bool isBackground();

            /** \brief Sets if this dialog is currently in background mode (i.e. not visible).
             * @a_dooverride
             */
            virtual void setBackground(bool v);

            /** \brief Sets if this dialog is currently forced to be visible in foreground.
             * @a_dooverride
             */
            virtual void setForceForeground(bool v);

        protected:
            sh::actions::ActionExecutionInfo* info();
            sh::actions::ActionExecutionInfo* _info;
            bool _logicallyVisible = false;
            bool _isbackground = false;
            bool _isforeground_forced = false;
        };

    }
}

#endif // ACTIONEXECUTIONINFODIALOG_H
