/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONEXECUTIONINFOPANEL_H
#define ACTIONEXECUTIONINFOPANEL_H

#include <functional>
#include <QColor>
#include <QObject>


namespace sh{
    namespace actions{class ActionExecutionInfo;}

    namespace ui {

        /** \brief A helper for some signal/slot stuff of in ActionExecutionInfoPanel.
         */
        class _ActionExecutionInfoPanel_HelperQObject : public QObject
        {
            Q_OBJECT
        public:
            void _emit_clicked();
            void _emit_visibilityChanged();
        signals:
            void clicked();
            void visibilityChanged();
        };

        /** \brief
         * Abstract class for a status bar info-panel for an action execution.
         */
        class ActionExecutionInfoPanel
        {
        public:

            /** \brief Sets the label text.
             * @a_dooverride
             */
            virtual void setLabel(QString s) =0;

            /** \brief Sets the progress.
             * @a_dooverride
             */
            virtual void setProgress(bool isProgressDeterminate, quint64 progressDone, quint64 progressAll) =0;

            /** \brief Sets if the associated action is currently forced to be visible in foreground.
             * @a_dooverride
             */
            virtual void setForceForeground(bool v) =0;

            /** \brief Sets if the panel is visible.
             * @a_dooverride
             */
            virtual void setPanelVisible(bool v) =0;

            /** \brief Sets the panel with (in pixels).
             * @a_dooverride
             */
            virtual void setWidth(int width) =0;

            /** \brief Sets a handler for a click on the button (optionally bound to an owner lifetime).
             */
            void onClicked(std::function<void()> fct, QObject* owner=0);

            /** \brief Sets a handler for panel removal (optionally bound to an owner lifetime).
             */
            void onDestroyed(std::function<void()> fct, QObject* owner=0);

            /** \brief Sets a handler for panel visibility changes (optionally bound to an owner lifetime).
             */
            void onVisibilityChanged(std::function<void()> fct, QObject* owner=0);

            virtual ~ActionExecutionInfoPanel();

        protected:

            _ActionExecutionInfoPanel_HelperQObject myqobject;

            /** \brief Called by an implementation when the panel was clicked.
             */
            void _emit_clicked();

            /** \brief Called by an implementation when the panel visibility was changed.
             */
            void _emit_visibilityChanged();

        };

    }
}

#endif // ACTIONEXECUTIONINFOPANEL_H
