/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "base/threadpool.h"
#include "filesystem/filesystemmodel.h"
#include "filesystem/filesystemnode.h"
#include "tools/visibleviews.h"
#include "fileview.h"


sh::ui::FileView::FileView() : QObject()
{
    _node = sh::filesystem::FilesystemModel::instance()->rootNode();
}

sh::ui::FileView::~FileView()
{
    if (_model)
        delete _model;
}

void sh::ui::FileView::reload(bool skipModel)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    if (!skipModel && filemodel())
        filemodel()->triggerReloadData();
    auto _node = node();
    sh::base::ThreadPool::enqueueForce([=](){
        _node->requestContainedItems();
        //this fetches all files in the view and their details in one for loop (on top of the armada of threads for the visible ones)
        sh::base::MainThread::dispatcher()->invokeSync([&](){
            int childnodecount = _node->childnodeCount();
            for(int i=0; i<childnodecount; i++)
                _node->childnode(i)->requestDetails();
        });
    });
}

sh::filesystem::FilesystemModelFileviewProxy* sh::ui::FileView::filemodel()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _model;
}

void sh::ui::FileView::setFilemodel(sh::filesystem::FilesystemModelFileviewProxy* model)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    _model = model;
}

void sh::ui::FileView::gotoDir(std::shared_ptr<sh::filesystem::FilesystemNode> node)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    if (_node != node) {
        _visibleviewslifetimecanary = nullptr;
        if (node)
            _visibleviewslifetimecanary = sh::tools::VisibleViews::instance()->viewEnteredDirectory(node);
        _node = node;
        setSelection(QList<std::shared_ptr<sh::filesystem::FilesystemNode>>());
    }
}

std::shared_ptr<sh::filesystem::FilesystemNode> sh::ui::FileView::node()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _node;
}
