/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LOGVIEWDIALOG_H
#define LOGVIEWDIALOG_H

#include "dialog.h"
#include "base/logger.h"


namespace sh{
    namespace ui {

        /** \brief The 'Log' dialog..
         */
        class LogViewDialog : public Dialog
        {
        public:

            /**
             * \param headtext: The header text.
             * \param subheadtxt: The 2nd header text.
             * \param minseverity: The minimum message severity this dialog shows initially (the user can typically change that later on).
             * @a_infrastructure
             */
            LogViewDialog(QString headtext, QString subheadtxt, sh::base::LogSeverity minseverity);

            /** \brief Returns the header text.
             */
            QString headtext();

            /** \brief Returns the 2nd header text.
             */
            QString subheadtxt();

            /** \brief Returns the minimum severity.
             */
            sh::base::LogSeverity minimumSeverity();

        protected:
            /** \brief Sets the minimum severity to a new value.
             *
             * This only updates a value in the data structure. It's up to the
             * implementation around to actually react on that.
             */
             void _setMinimumSeverity(sh::base::LogSeverity minseverity);

        private:
            QString _headtext;
            QString _subheadtxt;
            sh::base::LogSeverity _minseverity;
        };

    }
}

#endif // LOGVIEWDIALOG_H
