/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filesystem/eurl.h"
#include "tools/bookmarkmanager.h"
#include "managebookmarksdialog.h"


sh::ui::ManageBookmarksDialog::ManageBookmarksDialog()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
}

std::shared_ptr<sh::ui::ManageBookmarksDialog::BM_Folder> sh::ui::ManageBookmarksDialog::BM_Folder::asFolder()
{
    return std::dynamic_pointer_cast<sh::ui::ManageBookmarksDialog::BM_Folder>(this->shared_from_this());
}

std::shared_ptr<sh::ui::ManageBookmarksDialog::BM_Bookmark> sh::ui::ManageBookmarksDialog::BM_Folder::asBookmark()
{
    return nullptr;
}

bool sh::ui::ManageBookmarksDialog::BM_Folder::isFolder()
{
    return true;
}

sh::ui::ManageBookmarksDialog::BM_Bookmark::BM_Bookmark(std::shared_ptr<sh::tools::Bookmark> bookmark) : bookmark(bookmark)
{
}

std::shared_ptr<sh::ui::ManageBookmarksDialog::BM_Folder> sh::ui::ManageBookmarksDialog::BM_Bookmark::asFolder()
{
    return nullptr;
}

std::shared_ptr<sh::ui::ManageBookmarksDialog::BM_Bookmark> sh::ui::ManageBookmarksDialog::BM_Bookmark::asBookmark()
{
    return std::dynamic_pointer_cast<sh::ui::ManageBookmarksDialog::BM_Bookmark>(this->shared_from_this());
}

bool sh::ui::ManageBookmarksDialog::BM_Bookmark::isFolder()
{
    return false;
}

QList<std::shared_ptr<sh::ui::ManageBookmarksDialog::BM>> sh::ui::ManageBookmarksDialog::getBookmarksStructure()
{
    QList<std::shared_ptr<sh::ui::ManageBookmarksDialog::BM>> result;
    QList<std::shared_ptr<sh::tools::Bookmark>> bookmarks = sh::tools::BookmarkManager::instance()->getBookmarks();
    qStableSort(bookmarks.begin(), bookmarks.end(), [](std::shared_ptr<sh::tools::Bookmark> a, std::shared_ptr<sh::tools::Bookmark> b){
        QStringList afolder = a->folder();
        QStringList bfolder = b->folder();
        int i = 0;
        while (true) {
            if (i >= afolder.length() && i >= bfolder.length())
                break;
            else if (i >= afolder.length())
                return false;
            else if (i >= bfolder.length())
                return true;
            else if (afolder[i] != bfolder[i])
                return afolder[i] < bfolder[i];
            i++;
        }
        return false;
    });
    foreach (std::shared_ptr<sh::tools::Bookmark> bookmark, bookmarks) {
        QList<std::shared_ptr<sh::ui::ManageBookmarksDialog::BM>>* parent = &result;
        if (!bookmark->folder().isEmpty()) {
            foreach (QString sgm, bookmark->folder()) {
                std::shared_ptr<sh::ui::ManageBookmarksDialog::BM_Folder> existingfolder = nullptr;
                foreach (std::shared_ptr<sh::ui::ManageBookmarksDialog::BM> bm, *parent) {
                    if (bm->isFolder() && bm->asFolder()->name == sgm) {
                        existingfolder = bm->asFolder();
                        break;
                    }
                }
                if (existingfolder)
                    parent = &existingfolder->children;
                else {
                    auto newfolder = std::make_shared<sh::ui::ManageBookmarksDialog::BM_Folder>();
                    newfolder->name = sgm;
                    parent->append(newfolder);
                    parent = &newfolder->children;
                }
            }
        }
        parent->append(std::make_shared<sh::ui::ManageBookmarksDialog::BM_Bookmark>(bookmark));
    }
    return result;
}

