/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MANAGEBOOKMARKSDIALOG_H
#define MANAGEBOOKMARKSDIALOG_H

#include "tools/bookmarkmanager.h"
#include "dialog.h"


namespace sh{
    namespace ui {

        /** \brief The 'Manage Bookmarks' dialog.
         */
        class ManageBookmarksDialog : public Dialog
        {
        public:

            /** \brief
             * @a_infrastructure
             */
            ManageBookmarksDialog();

        protected:

            class BM_Folder;
            class BM_Bookmark;

            struct BM : enable_shared_from_this<BM> {
                virtual std::shared_ptr<BM_Folder> asFolder() =0;
                virtual std::shared_ptr<BM_Bookmark> asBookmark() =0;
                virtual bool isFolder() =0;
            };

            struct BM_Folder : public BM {
                QString name;
                QList<std::shared_ptr<BM>> children;
                std::shared_ptr<BM_Folder> asFolder() override;
                std::shared_ptr<BM_Bookmark> asBookmark() override;
                bool isFolder() override;
            };

            struct BM_Bookmark : public BM {
                BM_Bookmark(std::shared_ptr<sh::tools::Bookmark> bookmark);
                const std::shared_ptr<sh::tools::Bookmark> bookmark;
                std::shared_ptr<BM_Folder> asFolder() override;
                std::shared_ptr<BM_Bookmark> asBookmark() override;
                bool isFolder() override;
            };

            QList<std::shared_ptr<BM>> getBookmarksStructure();

        };

    }
}

#endif // MANAGEBOOKMARKSDIALOG_H
