/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filesystem/eurl.h"
#include "settings/profilenode.h"
#include "settings/settingsmanager.h"
#include "mainwindow.h"
#include "manageprofilesdialog.h"


sh::ui::ManageProfilesDialog::ManageProfilesDialog()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
}

void sh::ui::ManageProfilesDialog::getProfileList(QString currentProfile, QStringList& profiles, int& selected)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    profiles = sh::settings::SettingsManager::instance()->getProfileList();
    QString cprofile = (currentProfile.isEmpty() ? sh::ui::MainWindow::mainWindow()->currentProfile() : currentProfile);
    selected = profiles.indexOf(cprofile);
    if (selected == -1)
        selected = profiles.indexOf("main");
    if (selected == -1 && profiles.length() > 0)
        selected = 0;
}

QList<sh::ui::ManageProfilesDialog::Node> sh::ui::ManageProfilesDialog::getNodesList(QString profile)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QList<Node> result;
    foreach (sh::settings::ProfileNode* n, sh::settings::SettingsManager::instance()->getNodesForProfile(profile)) {
        if (n->eurl())
            result.append(Node{n->eurl()->asString(), "arrow-right", false, n->nodeId(), n});
        else
            result.append(Node{QObject::tr("global"), "arrow-right-double", true, n->nodeId(), n});
    }
    qSort(result.begin(), result.end(), [&](const Node& n1, const Node& n2) {
        if (n1.isGlobal != n2.isGlobal)
            return n1.isGlobal ? true : false;
        else if (!n1.isGlobal)
            return n1.text < n2.text;
        else
            return n1.nodeId < n2.nodeId;
    });
    return result;
}

QString sh::ui::ManageProfilesDialog::_generateProfileListString(QStringList profiles)
{
    int lprofiles = profiles.length();
    if (lprofiles == 0)
        return QObject::tr("(none)");
    else if (lprofiles == 1)
        return profiles.first();
    else
        return QStringList(profiles.mid(0, profiles.length()-1)).join(", ") + " " + QObject::tr("and") + " " + profiles.last();
}
