/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "openwithdialog.h"


sh::ui::OpenWithDialog::OpenWithDialog(std::shared_ptr<sh::filesystem::FilesystemNode> node, QList<std::shared_ptr<tools::filetypes::OpenMethod> > openMethods) : _node(node), _openMethods(openMethods)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
}

std::shared_ptr<sh::filesystem::FilesystemNode> sh::ui::OpenWithDialog::node()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _node;
}

QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> sh::ui::OpenWithDialog::openMethods()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _openMethods;
}

std::shared_ptr<sh::tools::filetypes::OpenMethod> sh::ui::OpenWithDialog::_openMethodForExecutable(QString fpath)
{
    QString name = QFileInfo(fpath).fileName();
    QString command = fpath;
    QStringList arguments({"%f"});
    return std::make_shared<sh::tools::filetypes::OpenMethod>(name, command, arguments);
}
