/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef OPENWITHDIALOG_H
#define OPENWITHDIALOG_H

#include <QFileInfo>
#include "tools/filetypes/filetypemanager.h"
#include "dialog.h"


namespace sh{
    namespace filesystem { class Eurl; class FilesystemNode; }
    namespace tools {
        namespace filetypes { class OpenMethod; }
    }
    namespace ui {

        /** \brief The 'Open With' dialog.
         */
        class OpenWithDialog : public Dialog
        {
        public:

            /**
             * \param node: The file which is later to be opened.
             * \param openMethods: The open-methods (programs for opening the file) to present for choice.
             * @a_infrastructure
             */
            OpenWithDialog(std::shared_ptr<sh::filesystem::FilesystemNode> node, QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> openMethods);

            /** \brief Returns the chosen open-method (program for opening the file).
             */
            virtual std::shared_ptr<sh::tools::filetypes::OpenMethod> chosenMethod() =0;

            /** \brief Returns if the chosen method is checked to be remembered for the same mime-type in the future.
             */
            virtual bool rememberForMimetype() =0;

            /** \brief If it's checked for the chosen method to be remembered for some subdirectory in the future, it returns the Eurl of this subdirectory, otherwise `nullptr`.
             */
            virtual std::shared_ptr<const sh::filesystem::Eurl> rememberForDirectory() =0;

            /** \brief Returns if the chosen method is checked to be remembered for the same file in the future.
             */
            virtual bool rememberForFile() =0;

            /** \brief Returns the file node which is later to be opened.
             */
            std::shared_ptr<sh::filesystem::FilesystemNode> node();

            /** \brief Returns the open-methods (programs for opening the file) to present for choice.
             */
            QList<std::shared_ptr<tools::filetypes::OpenMethod> > openMethods();

        protected:

            /** \brief Returns an open-method for a user-selected executable.
             */
            static std::shared_ptr<sh::tools::filetypes::OpenMethod> _openMethodForExecutable(QString fpath);

        private:
            std::shared_ptr<sh::filesystem::FilesystemNode> _node;
            QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> _openMethods;
        };

    }
}

#endif // OPENWITHDIALOG_H
