/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QKeyEvent>
#include <QPushButton>
#include <QButtonGroup>
#include "gridforminnersimplechooser.h"


sh::ui::qt::feedbackpanels::GridFormInnerSimpleChooser::GridFormInnerSimpleChooser(sh::actions::ActionExecutionUserFeedback::Choices* choices, QWidget *parent) : QWidget(parent)
{
    this->choices = choices;
    mylayout = new QVBoxLayout(this);
    setLayout(mylayout);
    QButtonGroup* buttongroup = new QButtonGroup(this);
    int i=0;
    lineedit = new QLineEdit(this);
    lineedit->hide();
    lineedit->installEventFilter(this);
    connect(buttongroup, SIGNAL(buttonClicked(int)), this, SLOT(slot_chosen(int)));
    connect(lineedit, SIGNAL(textEdited(QString)), this, SLOT(slot_textedited(QString)));
    foreach(sh::actions::ActionExecutionUserFeedback::Choice* choice, choices->list)
    {
        QPushButton* btn = new QPushButton(choice->label, this);
        btn->setCheckable(true);
        buttongroup->addButton(btn, i);
        mylayout->addWidget(btn);
        if (choices->selected == i)
            btn->click();
        btn->installEventFilter(this);
        i++;
    }
}

bool sh::ui::qt::feedbackpanels::GridFormInnerSimpleChooser::eventFilter(QObject* obj, QEvent *event)
{ // this is a workaround for broken tab behavior of QButtonGroup
    if (event->type() == QEvent::KeyPress)
    {
        QKeyEvent* keyevent = (QKeyEvent*) event;
        if (keyevent->key() == Qt::Key_Down || keyevent->key() == Qt::Key_Up)
        {
            QWidget* btn = (QWidget*) obj;
            int pos = mylayout->indexOf(btn);
            int direction = (keyevent->key() == Qt::Key_Down) ? 1 : -1;
            int newpos = pos + direction;
            while (newpos >= 0 && newpos < mylayout->count())
            {
                QWidget* nextwidget = mylayout->itemAt(newpos)->widget();
                if (nextwidget->isVisible())
                {
                    nextwidget->setFocus();
                    return true;
                }
                else
                    newpos += direction;
            }
        }
    }
    return false;
}

void sh::ui::qt::feedbackpanels::GridFormInnerSimpleChooser::slot_textedited(QString t)
{
    currentchoice->text = t;
}

void sh::ui::qt::feedbackpanels::GridFormInnerSimpleChooser::slot_chosen(int id)
{
    answer = id;
    currentchoice = choices->list[id];
    choices->selected = id;
    if (!currentchoice->text.isNull())
    {
        mylayout->insertWidget(id+1, lineedit);
        lineedit->setText(currentchoice->text);
        lineedit->show();
    }
    else
        lineedit->hide();
}


sh::ui::qt::feedbackpanels::GridFormInnerSimpleChooser::~GridFormInnerSimpleChooser()
{
}

