/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/iconmanager.h"
#include "msgbox.h"
#include "ui_msgbox.h"


sh::ui::qt::feedbackpanels::MsgBox::MsgBox(QString question, QList<QString> answers, QString icon, QString withInputBox, bool inputBoxMultiline, int defaultanswer, int cancelanswer,int valuePreselectFrom, int valuePreselectTo, QList<QString> answericons, QWidget *parent) :
    sh::ui::qt::feedbackpanels::FeedbackPanel(parent),
    ui(new Ui::MsgBox), _inputBoxMultiline(inputBoxMultiline), _valuePreselectFrom(valuePreselectFrom), _valuePreselectTo(valuePreselectTo)
{
    ui->setupUi(this);
    ui->labelquestion->setText(question);
    if (!icon.isEmpty()) {
        ui->label1->setPixmap(sh::base::IconManager::instance()->getPixmap(icon, 32));
    }
    int i=0;
    foreach(QString s, answers)
    {
        QPushButton* btn = new QPushButton(ui->panel2);
        QString icon = answericons.value(i);
        if (i==cancelanswer)
            _cancelBtn = btn;
        if (i==defaultanswer)
            _defaultBtn = btn;
        btn2index[btn] = i;
        btn->setText(s);
        if (!icon.isEmpty())
            btn->setIcon(sh::base::IconManager::instance()->getIconByFullname(icon));
        connect(btn, SIGNAL(clicked()), this, SLOT(slot_btnclicked()));
        ui->panel2->layout()->addWidget(btn);
        i++;
    }
    _initialFocusWidget = withInputBox.isNull() ? (QWidget*)_defaultBtn : (_inputBoxMultiline ? (QWidget*)ui->textEdit : (QWidget*)ui->lineEdit );
    ui->lineEdit->hide();
    ui->textEdit->hide();
    if (!withInputBox.isNull())
    {
        if (_inputBoxMultiline)
        {
            ui->textEdit->show();
            ui->textEdit->setText(withInputBox);
        }
        else
        {
            ui->lineEdit->show();
            ui->lineEdit->setText(withInputBox);
            if (_defaultBtn)
                connect(ui->lineEdit, &QLineEdit::returnPressed, this, [=](){
                    _defaultBtn->click();
                });
        }
    }
}

void sh::ui::qt::feedbackpanels::MsgBox::cancelRequested()
{
    if (_cancelBtn)
        _cancelBtn->click();
}

void sh::ui::qt::feedbackpanels::MsgBox::showEvent(QShowEvent* event)
{
    if (event->spontaneous())
        return;
    if (_initialFocusWidget)
        _initialFocusWidget->setFocus();
    if (_valuePreselectFrom > -1)
        ui->lineEdit->setSelection(_valuePreselectFrom, _valuePreselectTo-_valuePreselectFrom);
    else {
        ui->textEdit->selectAll();
        ui->lineEdit->selectAll();
    }
}

void sh::ui::qt::feedbackpanels::MsgBox::slot_btnclicked()
{
    QPushButton* me = dynamic_cast<QPushButton*>(sender());
    answer = btn2index[me];
    text = _inputBoxMultiline ? ui->textEdit->document()->toPlainText() : ui->lineEdit->text();
    setClosed();
}

sh::ui::qt::feedbackpanels::MsgBox::~MsgBox()
{
    delete ui;
}

int sh::ui::qt::feedbackpanels::MsgBox::preferredHeight()
{
    QWidget* textbox = _inputBoxMultiline ? (QWidget*)ui->textEdit : (QWidget*)ui->lineEdit;
    return ui->labelquestion->heightForWidth(ui->labelquestion->width()) + ui->panel2->sizeHint().height()
            + (textbox->isVisible() ? textbox->sizeHint().height() :0)
            + layout()->spacing()*3 + 30 /* 30 is the margin; i'm too lazy for crazy layout typecasting here */;
}
