/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MSGBOX_H
#define MSGBOX_H

#include <QList>
#include <QHash>
#include <QPushButton>
#include <QShowEvent>
#include "feedbackpanel.h"


namespace Ui {
class MsgBox;
}

namespace sh{
    namespace ui{
        namespace qt{
            namespace feedbackpanels {

                /** \brief FeedbackPanel for showing a message, some answer buttons, and optionally a text input box.
                 */
                class MsgBox : public sh::ui::qt::feedbackpanels::FeedbackPanel
                {
                    Q_OBJECT

                public:
                    MsgBox(QString question, QList<QString> answers, QString icon="", QString withInputBox=QString(), bool inputBoxMultiline=false,
                                    int defaultanswer=-1, int cancelanswer=-1, int valuePreselectFrom=-1, int valuePreselectTo=-1,
                                    QList<QString> answericons=QList<QString>(), QWidget *parent = 0);
                    ~MsgBox();
                    virtual void cancelRequested();
                    int answer = -1;
                    QString text;

                private slots:
                    void slot_btnclicked();

                private:
                    Ui::MsgBox *ui;
                    QHash<QPushButton*, int> btn2index;
                    QWidget* _initialFocusWidget = 0;
                    QPushButton* _cancelBtn = 0;
                    QPushButton* _defaultBtn = 0;
                    bool _inputBoxMultiline;
                    int _valuePreselectFrom;
                    int _valuePreselectTo;

                protected:
                    virtual int preferredHeight();
                    void showEvent(QShowEvent* event);
                };

            }
        }
    }
}

#endif // MSGBOX_H
