/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "unixpermissions.h"
#include "ui_unixpermissions.h"


sh::ui::qt::feedbackpanels::UnixPermissions::UnixPermissions(QWidget *parent) :
    sh::ui::qt::feedbackpanels::FeedbackPanel(parent),
    ui(new Ui::UnixPermissions)
{
    ui->setupUi(this);
}

sh::ui::qt::feedbackpanels::UnixPermissions::~UnixPermissions()
{
    delete ui;
}

void sh::ui::qt::feedbackpanels::UnixPermissions::cancelRequested()
{
}

void sh::ui::qt::feedbackpanels::UnixPermissions::on_pushButton_clicked()
{
    accepted = true;
    userMayRead = ui->chkUserRead->isChecked();
    userMayWrite = ui->chkUserWrite->isChecked();
    userMayExecute = ui->chkUserExec->isChecked();
    groupMayRead = ui->chkGroupRead->isChecked();
    groupMayWrite = ui->chkGroupWrite->isChecked();
    groupMayExecute = ui->chkGroupExec->isChecked();
    othersMayRead = ui->chkOthersRead->isChecked();
    othersMayWrite = ui->chkOthersWrite->isChecked();
    othersMayExecute = ui->chkOthersExec->isChecked();
    sticky = ui->chkSticky->isChecked();
    setuid = ui->chkSetUid->isChecked();
    setgid = ui->chkSetGid->isChecked();
    user = ui->lstUser->selectionModel()->selectedIndexes().isEmpty() ? QString()
                                          : ui->lstUser->model()->data(ui->lstUser->selectionModel()->selectedIndexes().first()).toString();
    group = ui->lstGroup->selectionModel()->selectedIndexes().isEmpty() ? QString()
                                          : ui->lstGroup->model()->data(ui->lstGroup->selectionModel()->selectedIndexes().first()).toString();
    setClosed();
}

void sh::ui::qt::feedbackpanels::UnixPermissions::on_pushButton_2_clicked()
{
    accepted = false;
    setClosed();
}

void sh::ui::qt::feedbackpanels::UnixPermissions::setflags(bool userMayRead, bool userMayWrite, bool userMayExecute, bool groupMayRead, bool groupMayWrite, bool groupMayExecute,
         bool othersMayRead, bool othersMayWrite, bool othersMayExecute, bool sticky, bool setuid, bool setgid)
{
    ui->chkUserRead->setChecked(userMayRead);
    ui->chkUserWrite->setChecked(userMayWrite);
    ui->chkUserExec->setChecked(userMayExecute);
    ui->chkGroupRead->setChecked(groupMayRead);
    ui->chkGroupWrite->setChecked(groupMayWrite);
    ui->chkGroupExec->setChecked(groupMayExecute);
    ui->chkOthersRead->setChecked(othersMayRead);
    ui->chkOthersWrite->setChecked(othersMayWrite);
    ui->chkOthersExec->setChecked(othersMayExecute);
    ui->chkSticky->setChecked(sticky);
    ui->chkSetUid->setChecked(setuid);
    ui->chkSetGid->setChecked(setgid);
}

void sh::ui::qt::feedbackpanels::UnixPermissions::setusers(QStringList users, QString selecteduser)
{
    QModelIndex curridx;
    QStandardItemModel* imodel = new QStandardItemModel(users.count(), 1, this);
    int i=0;
    foreach(QString u, users)
    {
        QStandardItem* itm = new QStandardItem(u);
        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
        imodel->setItem(i, 0, itm);
        if (u == selecteduser)
            curridx = imodel->index(i, 0);
        i++;
    }
    ui->lstUser->setModel(imodel);
    ui->lstUser->setCurrentIndex(curridx);
    ui->lstUser->scrollTo(curridx);
}

void sh::ui::qt::feedbackpanels::UnixPermissions::setgroups(QStringList groups, QString selectedgroup)
{
    QModelIndex curridx;
    QStandardItemModel* imodel = new QStandardItemModel(groups.count(), 1, this);
    int i=0;
    foreach(QString g, groups)
    {
        QStandardItem* itm = new QStandardItem(g);
        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
        imodel->setItem(i, 0, itm);
        if (g == selectedgroup)
            curridx = imodel->index(i, 0);
        i++;
    }
    ui->lstGroup->setModel(imodel);
    ui->lstGroup->setCurrentIndex(curridx);
    ui->lstGroup->scrollTo(curridx);
}
