/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QDateTime>
#include <QDesktopServices>
#include <QUrl>
#include "base/iconmanager.h"
#include "base/shallotprocess.h"
#include "qtmainwindow.h"
#include "qtuistyle.h"
#include "qtaboutdialog.h"
#include "ui_qtaboutdialog.h"


sh::ui::qt::QtAboutDialog::QtAboutDialog() : ui(new Ui::QtAboutDialog)
{
    ui->setupUi(this);

    ui->btnClose->setSizeByFactor(120);
    ui->btnClose->setIcon(sh::base::IconManager::instance()->getIcon("dialog-close"));

    auto uistyle = sh::ui::qt::QtMainWindow::mainWindow()->uiStyle();
    uistyle->createSheet().background_liketitlebar().applyTo(ui->panelHead);
    uistyle->createSheet().fontsize_byFactor(140).applyTo(ui->label);
    uistyle->createSheet().fontsize_byFactor(120).applyTo(ui->labelVersion);
    uistyle->createSheet().applyTo(ui->labelBuildtime);
    ui->labelVersion->setText(sh::base::ShallotProcess::revisionString());
    ui->labelBuildtime->setText(tr("built %1").arg(
                    sh::base::ShallotProcess::buildtime().toString(Qt::DefaultLocaleShortDate)
                ));

}

sh::ui::qt::QtAboutDialog::~QtAboutDialog()
{
    delete ui;
}

void sh::ui::qt::QtAboutDialog::on_btnClose_clicked()
{
    QtDialog::close();
}

void sh::ui::qt::QtAboutDialog::on_btnLicense_clicked()
{
    QDesktopServices::openUrl(QUrl(licenseUrl()));
}

void sh::ui::qt::QtAboutDialog::on_btnHomepage_clicked()
{
    QDesktopServices::openUrl(QUrl(homepageUrl()));
}
