/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTACTIONEXECUTIONINFOPANEL_H
#define QTACTIONEXECUTIONINFOPANEL_H

#include <QColor>
#include <QHideEvent>
#include <QPixmap>
#include <QShowEvent>
#include <QTimer>
#include <QWidget>
#include "ui/actionexecutioninfopanel.h"


namespace sh{
    namespace actions{class ActionExecutionInfo;}

    namespace ui {
        namespace qt {

            /** \brief
             * Qt status bar info-panel for an action execution.
             */
            class QtActionExecutionInfoPanel : public QWidget, public sh::ui::ActionExecutionInfoPanel
            {
                Q_OBJECT
            public:
                QtActionExecutionInfoPanel(sh::actions::ActionExecutionInfo *info, QColor color, QWidget *parent = 0);
                void setLabel(QString s);
                void setWidth(int width);
                QSize sizeHint() const;
                void setProgress(bool isProgressDeterminate, quint64 progressDone, quint64 progressAll);
                void setForceForeground(bool v);
                void setPanelVisible(bool v);

            protected:
                void mousePressEvent(QMouseEvent* e);
                void paintEvent( QPaintEvent* e );
                void showEvent(QShowEvent* e);
                void hideEvent(QHideEvent* e);

            protected slots:
                void slot_indeterminateanimationtimer();

            private:
                QString lbl1;
                bool _progressDeterminate = false;
                int _indeterminateAnimationCounter = 0;
                int _width;
                QTimer *_indeterminateAnimationTimer;
                quint64 _progressAll, _progressDone;
                bool _isforeground_forced = false;
                void _check_indeterminateanimationtimer_enable();
                sh::actions::ActionExecutionInfo *info;
                QPixmap gradient;
                QColor colorProgress2, colorProgress1, colorBase, colorFrame, colorText, colorTextDark;
            };

        }
    }
}

#endif // QTACTIONEXECUTIONINFOPANEL_H
