/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTACTIONMENU_H
#define QTACTIONMENU_H

#include <QLabel>
#include <QMenu>
#include <QPainter>
#include <QWidgetAction>
#include <QDesktopWidget>


namespace sh{
    namespace ui {
        namespace qt {

            /** \brief Qt based action menu used for context menus and in the toolbar.
             */
            class QtActionMenu : public QMenu
            {
                Q_OBJECT
            public:
                QtActionMenu(QWidget* parent=0);
                void setHeader(QString t);
                QList<QAction *> allActions();
                void clearAllActions();
                void removeActionAt(int i);
                QAction* addNewAction(int i=-1);
                QAction* addNewSubMenu(sh::ui::qt::QtActionMenu** qsubmenu, int i=-1);
                QAction* addNewSeparator(int i=-1);
                QAction* addNewHeader(int i=-1);
                bool actionIsHeader(QAction* a);

            protected:
                void paintEvent(QPaintEvent *e) override;
            private:
                void _observeAction(QAction* a);
                void _correctMenuPosition();
                QColor brandingcolor;
                QFont _boldfont;
                QFont _normalfont;
                int _origMenuPosX = -1, _origMenuPosY = -1;
                QString _header;
                int _cntInternalActions;
            };

        }
    }
}

#endif // QTACTIONMENU_H
