/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTACTIONMENUHANDLER_H
#define QTACTIONMENUHANDLER_H

#include <memory>
#include <QList>


namespace sh{
    namespace actions { class ActionInstantiation; class AbstractActionItem; }
    namespace ui {
        namespace qt {
            class QtActionMenu;

            /** \brief A handler which reflects the sh::actions objects to a graphical menu (and keeps that up-to-date).
             */
            class QtActionMenuHandler
            {
                friend class QtToolbarButtonHandler;
            public:
                QtActionMenuHandler(std::shared_ptr<sh::actions::ActionInstantiation> ai, std::shared_ptr<QtActionMenu> menu);
            private:
                static std::shared_ptr<sh::actions::ActionActionItem> _getDefaultAction(QList<std::shared_ptr<sh::actions::AbstractActionItem> > actionList);
                static void _markDefault(std::weak_ptr<sh::actions::SubmenuActionItem> itmSubmenu, sh::ui::qt::QtActionMenu *menu);
                static QAction *_createAndConnectAction(sh::actions::AbstractActionItem* itm, sh::ui::qt::QtActionMenu *menu, std::function<void()> onchanged, QObject* onchangedbuddy=0);
                static void _applyPropertiesToQAction(sh::actions::AbstractActionItem* itm, QAction* _widgetaction);
                static void _updateSubmenu(sh::actions::SubmenuActionItem* itm, QtActionMenu* menu, std::function<void()> onchanged=0);
                void _markDefault2();
                void _append_actions(sh::ui::qt::QtActionMenu *menu, QList<std::shared_ptr<sh::actions::ActionInstantiation> > acts, std::shared_ptr<sh::actions::ActionCategory> category);
                QList<std::shared_ptr<sh::actions::ActionInstantiation>> selacts;
                QList<std::shared_ptr<sh::actions::ActionInstantiation>> diracts;
                QHash<QAction*, std::shared_ptr<sh::actions::AbstractActionItem>> qaction2action;
                std::weak_ptr<QtActionMenu> menu;
            };

        }
    }
}

#endif // QTACTIONMENUHANDLER_H
