/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "qtdialog.h"
#include "qtmainwindow.h"


bool sh::ui::qt::QtDialogManager::wasAccepted(std::shared_ptr<Dialog> dialog)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QDialog* qdialog = dynamic_cast<QDialog*>(dialog.get());
    return qdialog->result() == QDialog::Accepted;
}

void sh::ui::qt::QtDialogManager::waitClosed(std::shared_ptr<Dialog> dialog)
{
    QDialog* qdialog = dynamic_cast<QDialog*>(dialog.get());
    sh::base::MainThread::dispatcher()->invokeSync([&](){
        qdialog->exec();
    });
}

void sh::ui::qt::QtDialogManager::stopDialog(std::shared_ptr<Dialog> dialog)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QDialog* qdialog = dynamic_cast<QDialog*>(dialog.get());
    qdialog->close();
}

void sh::ui::qt::QtDialogManager::showDialog(std::shared_ptr<Dialog> dialog)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    auto qdialog = dynamic_cast<QDialog*>(dialog.get());
    qdialog->show();
}

sh::ui::qt::QtDialog::QtDialog() : QDialog(QtMainWindow::mainWindow())
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    connect(this, &QtDialog::finished, this, [=](){
        auto dialog = dynamic_cast<Dialog*>(this);
        dialog->close();
    });
}
