/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTDIALOG_H
#define QTDIALOG_H

#include <QDialog>
#include "base/mainthread.h"
#include "ui/dialog.h"


namespace sh {
    namespace ui {
        namespace qt {

            /** @brief A DialogManager used in Qt ui.
             */
            class QtDialogManager : public DialogManager
            {
            protected:
                bool wasAccepted(std::shared_ptr<Dialog> dialog) override;
                void waitClosed(std::shared_ptr<Dialog> dialog) override;
                void stopDialog(std::shared_ptr<Dialog> dialog) override;
            private:
                void showDialog(std::shared_ptr<Dialog> dialog) override;
            };

            /** \brief Abstract base class for a qt based dialog. Typically used for also implementing some sh::ui::Dialog.
             */
            class QtDialog : public QDialog
            {
            public:
                QtDialog();
            };

        }
    }
}

#endif // QTDIALOG_H
