/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTFILEDETAILSPANEL_H
#define QTFILEDETAILSPANEL_H

#include <functional>
#include <memory>
#include <QHash>
#include <QPaintEvent>
#include <QPainter>
#include <QTimer>
#include <QWidget>


namespace sh{
    namespace filesystem{class FilesystemNode;}
    namespace paneldetails{class PanelDetail; class PanelDetailRowValueElement; class PanelDetailRow;}

    namespace ui {
        namespace qt {

            /** \brief
             * The details panel.
             *
             * Can be shown in the main window. It presents detail information
             * about the selected file.
             */
            class QtFileDetailsPanel : public QWidget
            {
                Q_OBJECT
            public:

                class DetailRowPlacement
                {
                public:
                    DetailRowPlacement(int h, int contentx, int contenty, QList<int> elementwidths);
                    DetailRowPlacement() = default;
                    DetailRowPlacement(const DetailRowPlacement&) = default;
                    int h;
                    int contentx;
                    int contenty;
                    QList<int> elementwidths;
                };

                class DetailPlacement
                {
                public:
                    DetailPlacement(int x, int y, int w, int h, QList<DetailRowPlacement> rowplacements, std::shared_ptr<sh::paneldetails::PanelDetail> detail);
                    std::shared_ptr<sh::paneldetails::PanelDetail> detail;
                    int x;
                    int y;
                    int w;
                    int h;
                    QList<DetailRowPlacement> rowplacements;
                };

                explicit QtFileDetailsPanel(QWidget *parent = 0);
                void setNodes(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);
                void setOrientation(Qt::Orientation orientation);
                QSize sizeHint() const override;
                ~QtFileDetailsPanel();

            protected:
                void showEvent(QShowEvent * event) override;
                void paintEvent(QPaintEvent* e) override;
                void resizeEvent(QResizeEvent*) override;
                void mouseMoveEvent(QMouseEvent *event) override;
                void leaveEvent(QEvent *event) override;
                void mouseReleaseEvent(QMouseEvent *event) override;
                void refresh();
                void refreshview();
                void _invalidate_widgetimagecache_ifrefreshdone(bool force=false);
                QList<int> computevalueelementgeometry(QList<sh::paneldetails::PanelDetailRowValueElement> value, int detailvaluewidth);

                void _iterate(
                        std::function<void(int x, int y, int w, int h, sh::paneldetails::PanelDetailRow* row, QString text)> _key,
                        std::function<void(int x, int y, int w, int h, sh::paneldetails::PanelDetailRow* row, sh::paneldetails::PanelDetailRowValueElement* element, std::shared_ptr<sh::paneldetails::PanelDetail>)> _element
                        );

            private:
                int PADDINGX;
                int PADDINGY;
                QList<std::shared_ptr<sh::filesystem::FilesystemNode>> _nodes;
                QList<std::shared_ptr<sh::paneldetails::PanelDetail> > _panelDetails;
                QList<DetailPlacement> _placements;
                QFont fontNormal;
                QFont fontBold;
                QPixmap _widgetimagecache;
                QTimer _widgetimagecachetimer;
                Qt::Orientation _orientation = Qt::Horizontal;
            };

        }
    }
}

#endif // QTFILEDETAILSPANEL_H
