/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTFILEICONVIEW_H
#define QTFILEICONVIEW_H

#include <memory>
#include <QListView>
#include <QPainter>
#include <QStyledItemDelegate>
#include "filesystem/filesystemmodelfileviewproxy.h"
#include "tools/historytracker.h"
#include "qtfileview.h"


namespace sh{
    namespace actions{class AbstractActionItem;}
    namespace ui{
        class ColumnDimensions;
        namespace qt {

            /** \brief
             * Icon view for the contents of one directory.
             */
            class QtFileIconview : public QListView, public sh::ui::qt::QtFileView
            {
                Q_OBJECT
            public:
                /** \brief Constructed only by the infrastructure and made available otherwise.
                 */
                explicit QtFileIconview(QWidget *parent = 0);
                /** \brief Sets the size of the thumbnail image.
                 */
                void setThumbDimension(double size);

            protected:
                class MyItemDelegate : public QStyledItemDelegate
                {
                public:
                    MyItemDelegate(QObject* parent);
                    virtual QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override;
                    virtual void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
                private:
                    int _getVisibleCharCount(QString text, int width, QFontMetrics* fm, bool forward) const;
                    const QString ELLIPSIS = "…";
                };

                void dragEnterEvent(QDragEnterEvent *event) override;
                void dragMoveEvent(QDragMoveEvent *event) override;
                void dragLeaveEvent(QDragLeaveEvent *event) override;
                void dropEvent(QDropEvent *event) override;
                void mousePressEvent(QMouseEvent *event) override;
                void mouseMoveEvent(QMouseEvent *event) override;
                void keyPressEvent(QKeyEvent *event) override;
                void onBackgroundColorSet() override;

            private:
                int _deleg_w = 0;
                int _deleg_h = 0;
                int _dsx = 0;
                int _dsy = 0;
                int _lineheight = 0;
            };

        }
    }
}

#endif // QTFILEICONVIEW_H
