/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTFILELIST_H
#define QTFILELIST_H

#include <memory>
#include <QTreeView>
#include "qtfileview.h"


namespace sh{
    namespace ui {
        namespace qt {

            /** \brief
             * List view for the contents of one directory.
             */
            class QtFileList : public QTreeView, public sh::ui::qt::QtFileView
            {
                Q_OBJECT
            public:
                /** \brief Constructed only by the infrastructure and made available otherwise.
                 */
                explicit QtFileList(QWidget *parent = 0);
                void setSort(int column, Qt::SortOrder order) override;

            private slots:
                void slot_sectionResized(int index, int oldsize, int newsize);

            protected:
                void dragEnterEvent(QDragEnterEvent *event) override;
                void dragMoveEvent(QDragMoveEvent *event) override;
                void dragLeaveEvent(QDragLeaveEvent *event) override;
                void dropEvent(QDropEvent *event) override;
                void mousePressEvent(QMouseEvent *event) override;
                void mouseMoveEvent(QMouseEvent *event) override;
                void keyPressEvent(QKeyEvent *event) override;
                void _goto_modelloaded() override;
                void onBackgroundColorSet() override;

            private:
                static std::shared_ptr<sh::configuration::ConfigurationValue> cfgvalFileListIconSize;
                QString getColumnNameForIndex(int index);
                void _adaptColumnWidth(int index);
                bool _skip_slot_sectionResized = false;
            };

        }
    }
}

#endif // QTFILELIST_H
