/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTFILEPROPERTYDIALOG_H
#define QTFILEPROPERTYDIALOG_H

#include <functional>
#include <memory>
#include <QDialog>
#include <QHeaderView>
#include <QVBoxLayout>
#include "ui/qt/qtdialog.h"
#include "ui/filepropertydialog.h"


namespace Ui {
class QtFilePropertyDialog;
}

namespace sh {
    namespace filesystem {
        class FilesystemNode;
        class Operation;
    }
    namespace ui {
        namespace qt {

            /**
             * \brief Qt based properties dialog
             */
            class QtFilePropertyDialog : public QtDialog, public sh::ui::FilePropertyDialog
            {
                friend class FilePropertyDialogTab;
                Q_OBJECT
            public:
                ~QtFilePropertyDialog();

                /** \brief Constructed only by the infrastructure and made available otherwise.
                 */
                QtFilePropertyDialog(QList<std::shared_ptr<sh::filesystem::FilesystemNode> > nodes);

                void init(std::shared_ptr<Dialog> dialog) override;
                virtual void refresh() override;
                virtual sh::ui::FilePropertyDialogTabTableView* createTabViewTable() override;
                virtual sh::ui::FilePropertyDialogTabTextView* createTabViewText() override;
                virtual sh::ui::FilePropertyDialogTabIconTextBannerView* createTabViewIconTextBanner() override;

            public slots:
                void on_btnClose_clicked();
                void on_btnRefresh_clicked();
                void on_tabWidget_currentChanged(int index);
                void selectTabByScrollPosition();

            protected:
                bool eventFilter(QObject *obj, QEvent *event);

            private:
                Ui::QtFilePropertyDialog *ui;
                QList<QWidget*> _internaltabwidgets;
                QList<QString> _tabheads;
                int _skip_on_tabWidget_currentChanged = 0;
                int _skip_selectTabByScrollPosition = 0;
                QVBoxLayout* scrollAreaLayout;
            };

        }
    }
}

#endif // QTFILEPROPERTYDIALOG_H
