/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "qtfilepropertydialogtabtableview.h"

sh::ui::qt::QtFilePropertyDialogTabTableView::QtFilePropertyDialogTabTableView(QWidget* parent)
            : QTableView(parent)
{
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
    horizontalHeader()->setVisible(false);
    verticalHeader()->setSectionResizeMode(QHeaderView::Fixed);
    this->installEventFilter(this);
}

void sh::ui::qt::QtFilePropertyDialogTabTableView::setContent(QList<QPair<QString, QString>> content)
{
    this->content = content;
    createAndSetModel();
    resizeRowsToContents();
    updateGeometry();
}

void sh::ui::qt::QtFilePropertyDialogTabTableView::createAndSetModel()
{
    QStandardItemModel* imodel = new QStandardItemModel(this->content.count(), 1, this);
    int i=0;
    foreach(StringPair line, this->content)
    {
        imodel->setHeaderData(i, Qt::Vertical, line.first);
        QStandardItem* itm = new QStandardItem(line.second);
        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
        imodel->setItem(i, 0, itm);
        i++;
    }
    auto smodl = selectionModel();
    auto modl = model();
    setModel(imodel);
    if (modl)
        delete modl;
    if (smodl)
        delete smodl;
}

bool sh::ui::qt::QtFilePropertyDialogTabTableView::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == QEvent::Wheel)
        return true;
    return QTableView::eventFilter(obj, event);
}

QSize sh::ui::qt::QtFilePropertyDialogTabTableView::sizeHint() const
{
    return QSize(0, rowViewportPosition(content.count()-1) + rowHeight(content.count()-1) + 2*frameWidth());
}

QSize sh::ui::qt::QtFilePropertyDialogTabTableView::minimumSizeHint() const
{
    return QSize(20,20);
}

void sh::ui::qt::QtFilePropertyDialogTabTableView::resizeEvent(QResizeEvent* event)
{
    if (event->oldSize().width() != event->size().width() || event->oldSize().height() != event->size().height())
    {
        horizontalHeader()->resizeSection(0, this->width()-verticalHeader()->width()-2*frameWidth());
        resizeRowsToContents();
        updateGeometries();
        createAndSetModel();  // this is all because otherwise it scrolls away sometimes
        resizeRowsToContents();
    }
}

QList<QPair<int, int>> sh::ui::qt::QtFilePropertyDialogTabTableView::getSelectedItems()
{
    QList<QPair<int, int>> result;
    foreach (auto idx, selectionModel()->selectedIndexes())
        result.append(QPair<int, int>(idx.row(), idx.column()));
    return result;
}

QVariant sh::ui::qt::QtFilePropertyDialogTabTableView::getItemValue(int r, int c)
{
    if (c == 0)
        return model()->headerData(r, Qt::Vertical);
    else if (c == 1)
        return model()->data(model()->index(r, 0));
    else
        return QVariant();
}
