/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTFILESYSTEMPANEL_H
#define QTFILESYSTEMPANEL_H

#include <memory>
#include <QTreeView>
#include "settings/setting.h"
#include "tools/historytracker.h"
#include "ui/columndimensions.h"


namespace Ui {
class QtFilesystemPanel;
}

namespace sh{
    namespace filesystem{class FilesystemNode;}
    namespace settings{class SettingsRegistration;}

    namespace ui {

        enum class FileViewMode;

        namespace qt {

            class QtMainWindow;
            class QtFileView;
            class QtFileViewControl;

            /** \brief
             * A splitted horizontal panel of file views.
             */
            class QtFilesystemPanel : public QWidget
            {
                friend class ui::qt::QtMainWindow;
                friend class ui::FileView;
                Q_OBJECT

            public:
                explicit QtFilesystemPanel(QWidget *parent = 0);
                ~QtFilesystemPanel();
                void addView(bool reinitialize=false);
                void removeView(int i);
                sh::ui::FileView *currentView();
                int currentViewIndex();
                int viewsCount();
                sh::ui::FileView* view(int i);
                void selectFolder(std::shared_ptr<sh::filesystem::FilesystemNode> folder, int index = -1);
                QList<std::shared_ptr<sh::filesystem::FilesystemNode>> selectedNodes();
                void _emit_viewmodeChanged(int i);
                void buildView(std::shared_ptr<sh::ui::qt::QtFileView> *list, sh::ui::qt::QtFileViewControl *viewctrl, bool isNew);
                void setCustomWidget(int i, std::shared_ptr<QWidget> w);
                std::shared_ptr<sh::ui::qt::QtFileView> viewwidget(int i);

            signals:
                void panelFolderActivated(std::shared_ptr<sh::filesystem::FilesystemNode> folder, bool realSwitch);
                void panelCurrentViewChanged();
                void panelViewmodeChanged(int i);
                void panelViewOptionChanged(int i);
                void panelViewCountChanged();
                void panelSelectionChanged();

            protected:
                bool eventFilter(QObject *object, QEvent *event);

            private:
                Ui::QtFilesystemPanel *ui;
                std::shared_ptr<sh::ui::qt::QtFileView> activeView;
                QString activeColor;
                QString inactiveColor;
                QList<std::shared_ptr<sh::ui::qt::QtFileView>> _views;
                QList<QWidget*> _mainviews;
                QHash<QWidget*, std::shared_ptr<QWidget>> _customwidgets;
                QList<sh::ui::qt::QtFileViewControl*> _viewcontrols;
                bool _skip_eventfilter = false;
            };

        }
    }
}

#endif // QTFILESYSTEMPANEL_H
