/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTFILEVIEWCONTROL_H
#define QTFILEVIEWCONTROL_H

#include <memory>
#include <QObject>
#include "filesystem/filesystemmodelfileviewproxy.h"
#include "tools/historytracker.h"
#include "ui/fileview.h"


namespace sh{
    namespace ui{

        namespace qt {
            class QtFilesystemPanel;
            class QtFileView;

            /** \brief Qt based file view.
             */
            class QtFileViewControl : public sh::ui::FileView
            {
                friend class sh::ui::qt::QtFilesystemPanel;
                Q_OBJECT
            public:

                /** @a_infrastructure
                 */
                QtFileViewControl(sh::ui::qt::QtFilesystemPanel* panel, int i);

                ~QtFileViewControl();
                sh::ui::ColumnDimensions* columndimensions() override;
                sh::tools::HistoryTracker<std::shared_ptr<const sh::filesystem::Eurl>>* historyTracker() override;
                FileViewMode viewmode() override;
                void setViewmode(FileViewMode m) override;
                int index() override;
                void gotoDir(std::shared_ptr<sh::filesystem::FilesystemNode> n) override;
                sh::filesystem::SizeFormatting getSizeFormattingMode() override;
                void setSizeFormattingMode(sh::filesystem::SizeFormatting mode) override;
                bool hiddenFilesVisible() override;
                void setHiddenFilesVisible(bool v) override;
                int iconDimension() override;
                void setIconDimension(int v) override;
                void setSort(int column, Qt::SortOrder order) override;
                int sortColumn() override;
                Qt::SortOrder sortOrder() override;
                QList<std::shared_ptr<sh::filesystem::FilesystemNode>> selectedNodes() override;
                void setSelection(const QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes) override;
                QList<std::shared_ptr<sh::filesystem::FilesystemNode>> getAllVisibleNodes() override;

                void emit_nodesActivated(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);
                void emit_selectionChanged();
            signals:
                void nodesActivated(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);

            private:
                sh::ui::qt::QtFilesystemPanel* _panel;
                int i;
                sh::ui::ColumnDimensions* _columndimensions;
                sh::tools::HistoryTracker<std::shared_ptr<const sh::filesystem::Eurl>>* _historyTracker;
                FileViewMode _viewmode = FileViewMode::ListView;
                bool _hiddenfilesvisible = false;
                int _icondimension = 0;
                int _sortColumn;
                Qt::SortOrder _sortOrder;
                sh::filesystem::SizeFormatting _sizeformattingmode = sh::filesystem::SizeFormatting::SizeFormattingModePrefixes;
                void setIndex(int i);
            };

        }
    }
}

#endif // QTFILEVIEWCONTROL_H
