/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTMANAGEPROFILESDIALOG_H
#define QTMANAGEPROFILESDIALOG_H

#include <QDialog>
#include <QFrame>
#include <QKeyEvent>
#include <QListWidget>
#include <QMessageBox>
#include <QStringListModel>
#include "ui/manageprofilesdialog.h"
#include "qtdialog.h"


namespace Ui {
class QtManageProfilesDialog;
}

namespace sh{
    namespace settings{class ProfileNode;}

    namespace ui {
        namespace qt {

            /** \brief
             * Qt based manage saved settings dialog.
             */
            class QtManageProfilesDialog : public QtDialog, public sh::ui::ManageProfilesDialog
            {
                Q_OBJECT

            public:
                explicit QtManageProfilesDialog();
                ~QtManageProfilesDialog();

            private slots:
                void on_comboBox_currentTextChanged(const QString &arg1);
                void on_btnClose_clicked();
                void on_btnDelNode_clicked();
                void on_btnDelProfile_clicked();
                void slot_listViewactivated(QListWidgetItem *, QListWidgetItem *);

            protected:
                void keyPressEvent(QKeyEvent *event);
                void resizeEvent(QResizeEvent* event);

            private:
                Ui::QtManageProfilesDialog *ui;
                QStringListModel* _model;
                sh::settings::ProfileNode* _currentNode = 0;
                QHash<QListWidgetItem*, QString> _nodes;
                void _createProfileList();
                void _createSettingsList();
                void _createNodesList();
            };

        }
    }
}

#endif // QTMANAGEPROFILESDIALOG_H
