/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTOPENWITHDIALOG_H
#define QTOPENWITHDIALOG_H

#include <memory>
#include <QAbstractItemModel>
#include <QDialog>
#include <QFileDialog>
#include <QKeyEvent>
#include <QStringListModel>
#include "actions/actionactionitem.h"
#include "ui/openwithdialog.h"
#include "qtdialog.h"


namespace Ui {
class QtOpenWithDialog;
}

namespace sh{
    namespace filesystem { class Eurl; }
    namespace ui {
        namespace qt {

            /** \brief
             * Qt based open with dialog.
             */
            class QtOpenWithDialog : public QtDialog, public sh::ui::OpenWithDialog
            {
                Q_OBJECT

            public:
                explicit QtOpenWithDialog(std::shared_ptr<sh::filesystem::FilesystemNode> node, QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> openMethods);
                ~QtOpenWithDialog();

                void setProgramList(QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> openMethods);

                std::shared_ptr<sh::tools::filetypes::OpenMethod> chosenMethod() override;
                bool rememberForMimetype() override;
                std::shared_ptr<const sh::filesystem::Eurl> rememberForDirectory() override;
                bool rememberForFile() override;

            private slots:
                void on_btnCancel_clicked();
                void on_btnOk_clicked();
                void on_btnSelectOther_clicked();
                void on_listView_activated(const QModelIndex &index);
                void on_btnAnotherAncestor_clicked();
                void on_chkRememberDir_toggled(bool checked);

            private:
                std::shared_ptr<sh::tools::filetypes::OpenMethod> _chosenMethod;
                std::shared_ptr<const sh::filesystem::Eurl> _rememberfordirectory;
                QAbstractItemModel* _listmodel;
                Ui::QtOpenWithDialog *ui;

            };

            /**
             * \brief Used internally in OpenWithDialogModel.
             */
            class QtOpenWithDialogModel : public QStringListModel
            {
                Q_OBJECT
            public:
                QtOpenWithDialogModel(QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> methods, QObject* parent = 0);
                QVariant data(const QModelIndex &index, int role) const;
            private:
                QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> _methods;
            };

        }
    }
}

#endif // QTOPENWITHDIALOG_H
