/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTSEARCHPANEL_H
#define QTSEARCHPANEL_H

#include <memory>
#include <QCheckBox>
#include <QWidget>


namespace sh{
    namespace filesystem { class Eurl; class Operation; class FilesystemNode; }
    namespace search { class SearchConfiguration; }
    namespace ui {
        namespace qt {
            class QtLinkButton;
            class QtSearchPanelConfiguration;

            /** \brief Search panel for usage in the Qt main window.
             */
            class QtSearchPanel : public QWidget
            {
                friend class QtSearchPanelConfiguration;
                Q_OBJECT
            public:
                explicit QtSearchPanel(std::shared_ptr<sh::filesystem::FilesystemNode> node, QWidget *parent = 0);
                ~QtSearchPanel();
            protected:
                std::shared_ptr<const sh::filesystem::Eurl> _eurl ;
                std::shared_ptr<sh::filesystem::FilesystemNode> _node;
                void populateUI();
                sh::search::SearchConfiguration* cfg;
                QWidget* critwidget;
                QWidget* auxbtnpanel;
                QCheckBox* chkTree;
                sh::ui::qt::QtLinkButton* btnSearch;
                void startSearch();
                void closeSearch();
                void modeChangeMenu();
                sh::ui::qt::QtLinkButton* btnMode;
                bool eventFilter(QObject* o, QEvent* e);
                void keyPressEvent(QKeyEvent* e);
                void setLoadingInfoVisible(bool v);
                std::shared_ptr<sh::ui::qt::QtSearchPanelConfiguration> panelcfg;
            };

        }
    }
}

#endif // QTSEARCHPANEL_H
