/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "qtsearchpanel.h"
#include "qtsearchpanelconfiguration.h"


sh::ui::qt::QtSearchPanelButton::QtSearchPanelButton(QWidget *parent) : QtLinkButton(parent), sh::ui::SearchPanelButton()
{
    setFocusPolicy(Qt::ClickFocus);
}

void sh::ui::qt::QtSearchPanelButton::setButtonText(QString txt)
{
    setText(txt);
}

void sh::ui::qt::QtSearchPanelButton::setMenuSelection(int i)
{
    setSelectedMenuItem(i);
}

sh::ui::qt::QtSearchPanelTextEditor::QtSearchPanelTextEditor(QWidget *parent) : QtSearchPanelAbstractEditor<QLineEdit, SearchPanelTextEditor>(parent)
{
}

QString sh::ui::qt::QtSearchPanelTextEditor::textContent()
{
    return text();
}

void sh::ui::qt::QtSearchPanelTextEditor::setTextContent(QString s)
{
    setText(s);
}

QString sh::ui::qt::QtSearchPanelTextEditor::placeholderDescription()
{
    return placeholderText();
}

void sh::ui::qt::QtSearchPanelTextEditor::setPlaceholderDescription(QString s)
{
    setPlaceholderText(s);
}

sh::ui::qt::QtSearchPanelDateTimeEditor::QtSearchPanelDateTimeEditor(QWidget* parent) : QtSearchPanelAbstractEditor<QDateTimeEdit, SearchPanelDateTimeEditor>(parent)
{
    QSize s = sizeHint();
    s.setWidth(s.width() + 50);
    setMinimumSize(s);
}

QDateTime sh::ui::qt::QtSearchPanelDateTimeEditor::datetime()
{
    return dateTime();
}

void sh::ui::qt::QtSearchPanelDateTimeEditor::setDatetime(QDateTime s)
{
    setDateTime(s);
}

sh::ui::qt::QtSearchPanelLabelEditor::QtSearchPanelLabelEditor(QWidget *parent) : QtSearchPanelAbstractEditor<QLabel, SearchPanelLabelEditor>(parent)
{
}

QString sh::ui::qt::QtSearchPanelLabelEditor::textContent()
{
    return text();
}

void sh::ui::qt::QtSearchPanelLabelEditor::setTextContent(QString s)
{
    setText(s);
}

sh::ui::SearchPanelAbstractEditor* sh::ui::qt::QtSearchPanelLabelEditor::focusProxyEditor()
{
    return dynamic_cast<sh::ui::SearchPanelAbstractEditor*>(focusProxy());
}

void sh::ui::qt::QtSearchPanelLabelEditor::setFocusProxyEditor(SearchPanelAbstractEditor* w)
{
    setFocusProxy(dynamic_cast<QWidget*>(w));
}

sh::ui::qt::QtSearchPanelSpacerEditor::QtSearchPanelSpacerEditor(QWidget *parent) : QtSearchPanelAbstractEditor<QLabel, SearchPanelSpacerEditor>(parent)
{
}

sh::ui::qt::QtSearchPanelConfiguration::QtSearchPanelConfiguration(QtSearchPanel* owner, QHBoxLayout *editors) : owner(owner), editors(editors)
{
}

sh::ui::SearchPanelButton* sh::ui::qt::QtSearchPanelConfiguration::addMenuButton(QString text, QStringList menu, std::function<void(int)> onchanged)
{
    sh::ui::qt::QtSearchPanelButton* res = new sh::ui::qt::QtSearchPanelButton(owner->auxbtnpanel);
    QObject::connect(res, &sh::ui::qt::QtLinkButton::menuItemSelected, res, [=](int i){
        onchanged(i);
    });
    res->setMenu(menu, text);
    owner->auxbtnpanel->layout()->addWidget(res);
    return res;
}

sh::ui::SearchPanelButton* sh::ui::qt::QtSearchPanelConfiguration::addActionButton(QString text, std::function<void()> action)
{
    sh::ui::qt::QtSearchPanelButton* res = new sh::ui::qt::QtSearchPanelButton(owner->auxbtnpanel);
    QObject::connect(res, &sh::ui::qt::QtLinkButton::clicked, res, [=](){
        action();
    });
    res->setText(text);
    owner->auxbtnpanel->layout()->addWidget(res);
    return res;
}

sh::ui::SearchPanelTextEditor* sh::ui::qt::QtSearchPanelConfiguration::addTextEditor()
{
    auto w = new sh::ui::qt::QtSearchPanelTextEditor(owner);
    editors->addWidget(w);
    return w;
}

sh::ui::SearchPanelDateTimeEditor* sh::ui::qt::QtSearchPanelConfiguration::addDateTimeEditor()
{
    auto w = new sh::ui::qt::QtSearchPanelDateTimeEditor(owner);
    editors->addWidget(w);
    w->setCalendarPopup(true);
    return w;
}

sh::ui::SearchPanelLabelEditor* sh::ui::qt::QtSearchPanelConfiguration::addLabel()
{
    auto w = new sh::ui::qt::QtSearchPanelLabelEditor(owner);
    editors->addWidget(w);
    return w;
}

sh::ui::SearchPanelSpacerEditor* sh::ui::qt::QtSearchPanelConfiguration::addSpacer()
{
    auto w = new sh::ui::qt::QtSearchPanelSpacerEditor(owner);
    editors->addWidget(w);
    editors->setStretchFactor(w, 1);
    return w;
}

sh::ui::SearchPanelAbstractEditor* sh::ui::qt::QtSearchPanelConfiguration::getEditorAt(int i)
{
    return dynamic_cast<sh::ui::SearchPanelAbstractEditor*>(editors->itemAt(i)->widget());
}
