/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTSEARCHPANELCONFIGURATION_H
#define QTSEARCHPANELCONFIGURATION_H

#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QDateTimeEdit>
#include "ui/searchpanelconfiguration.h"
#include "qtlinkbutton.h"


namespace sh {
    namespace ui {
        namespace qt {
            class QtSearchPanel;

            /** \brief Qt based SearchPanelButton.
             */
            class QtSearchPanelButton : public QtLinkButton, public sh::ui::SearchPanelButton
            {
            public:
                QtSearchPanelButton(QWidget *parent = 0);
                virtual void setButtonText(QString txt);
                virtual void setMenuSelection(int i);
            };

            /** \brief Helper for other Qt based search panel editors.
             */
            template<class T1, class T2>
            class QtSearchPanelAbstractEditor : public T1, public T2
            {
            public:
                QtSearchPanelAbstractEditor(QWidget* parent =0) : T1(parent), T2() {
                }
                bool isWidgetEnabled() {
                    return this->isEnabled();
                }
                void setWidgetEnabled(bool v) {
                    this->setEnabled(v);
                }
            };

            /** \brief Qt based SearchPanelTextEditor.
             */
            class QtSearchPanelTextEditor : public QtSearchPanelAbstractEditor<QLineEdit, sh::ui::SearchPanelTextEditor>
            {
            public:
                explicit QtSearchPanelTextEditor(QWidget* parent =0);
                virtual QString textContent();
                virtual void setTextContent(QString s);
                QString placeholderDescription();
                void setPlaceholderDescription(QString s);
            };

            /** \brief Qt based SearchPanelDateTimeEditor.
             */
            class QtSearchPanelDateTimeEditor : public QtSearchPanelAbstractEditor<QDateTimeEdit, SearchPanelDateTimeEditor>
            {
            public:
                explicit QtSearchPanelDateTimeEditor(QWidget* parent =0);
                virtual QDateTime datetime();
                virtual void setDatetime(QDateTime s);
            };

            /** \brief Qt based SearchPanelLabelEditor.
             */
            class QtSearchPanelLabelEditor : public QtSearchPanelAbstractEditor<QLabel, sh::ui::SearchPanelLabelEditor>
            {
            public:
                explicit QtSearchPanelLabelEditor(QWidget* parent =0);
                virtual QString textContent();
                virtual void setTextContent(QString s);
                virtual SearchPanelAbstractEditor* focusProxyEditor();
                virtual void setFocusProxyEditor(SearchPanelAbstractEditor* w);
            };

            /** \brief Qt based SearchPanelSpacerEditor.
             */
            class QtSearchPanelSpacerEditor : public QtSearchPanelAbstractEditor<QLabel, sh::ui::SearchPanelSpacerEditor>
            {
            public:
                explicit QtSearchPanelSpacerEditor(QWidget* parent =0);
            };

            /** \brief Search panel configuration for usage in a Qt ui.
             */
            class QtSearchPanelConfiguration : public sh::ui::SearchPanelConfiguration
            {
            public:
                QtSearchPanelConfiguration(QtSearchPanel* owner, QHBoxLayout* editors);
                virtual SearchPanelButton* addMenuButton(QString text, QStringList menu, std::function<void(int)> onchanged);
                virtual SearchPanelButton* addActionButton(QString text, std::function<void()> action);
                virtual SearchPanelTextEditor* addTextEditor();
                virtual SearchPanelDateTimeEditor* addDateTimeEditor();
                virtual SearchPanelLabelEditor* addLabel();
                virtual SearchPanelSpacerEditor* addSpacer();
                virtual SearchPanelAbstractEditor* getEditorAt(int i);
            private:
                QtSearchPanel* owner;
                QHBoxLayout* editors;
            };

        }
    }
}

#endif // QTSEARCHPANELCONFIGURATION_H
