/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "settings/setting.h"
#include "qtmainwindow.h"
#include "qtuistyle.h"
#include "qtsettinguiframe.h"


sh::ui::qt::QtSettingUIFrame::QtSettingUIFrame(sh::settings::Setting* setting, bool withcheckbox, QString displayvalue, QString additionalCheck, QVariant additionalCheckValue, QWidget *parent) :
    QFrame(parent), _additionalCheckValue(additionalCheckValue)
{
    QHBoxLayout* layout = new QHBoxLayout(this);
    setLayout(layout);
    layout->setMargin(6);
    layout->setSpacing(10);
    if (withcheckbox) {
        _checkbox = new QCheckBox(this);
        layout->addWidget(_checkbox);
        _checkbox->setText("");
        if (!additionalCheck.isEmpty()) {
            connect(_checkbox, &QCheckBox::toggled, this, [=](){
                if (!_checkbox->isChecked()) {
                    _additionalcheckbox->setChecked(false);
                    _additionalcheckbox->setEnabled(false);
                }
                else
                    _additionalcheckbox->setEnabled(true);
            });
        }
    }
    QVBoxLayout* lbllayout = new QVBoxLayout();
    QLabel* _lbl = new QLabel(this);
    _lbl->setWordWrap(true);
    lbllayout->addWidget(_lbl);
    lbllayout->setSpacing(0);
    _lbl->setText(tr("Store '%1'").arg( setting->description()));
    layout->addLayout(lbllayout);
    QLabel* lblcurr = new QLabel(this);
    lblcurr->setAlignment(Qt::AlignRight);
    lblcurr->setWordWrap(true);
    lblcurr->setStyleSheet(QString("color:%1;").arg(sh::ui::qt::QtMainWindow::mainWindow()->uiStyle()->foregroundColorLighter1().name()));
    QString vdesc = "???";
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        vdesc = setting->valueDescription(displayvalue);
    });
    layout->addWidget(lblcurr);
    lblcurr->setText(vdesc);
    if(withcheckbox) {
        layout->setStretch(0,0);
        layout->setStretch(1,1);
        layout->setStretch(2,1);
    }
    if (!additionalCheck.isEmpty()) {
        QHBoxLayout* _additionalchecklayout = new QHBoxLayout();
        if (withcheckbox) {
            _additionalcheckbox = new QCheckBox(this);
            _additionalcheckbox->setText("");
            _checkbox->setChecked(true);
            _checkbox->setChecked(false);
            _additionalchecklayout->addWidget(_additionalcheckbox);
        }
        _additionalcheckboxlabel = new QLabel(this);
        _additionalcheckboxlabel->setWordWrap(true);
        _additionalchecklayout->addWidget(_additionalcheckboxlabel);
        _additionalcheckboxlabel->setText(additionalCheck);
        if (withcheckbox) {
            _additionalchecklayout->setStretch(0,0);
            _additionalchecklayout->setStretch(1,1);
            _additionalchecklayout->setStretch(2,1);
        }
        _additionalcheckwidget = new QWidget();
        _additionalcheckwidget->setLayout(_additionalchecklayout);
        lbllayout->addWidget(_additionalcheckwidget);
    }
    setStyleSheet(QString("border-radius:10px; background:%1;").arg(sh::ui::qt::QtMainWindow::mainWindow()->uiStyle()->windowColorHalfHighlighted().name()));
}

void sh::ui::qt::QtSettingUIFrame::setAdditionalCheckVisible(bool v)
{
    if (_additionalcheckwidget) {
        _additionalcheckwidget->setVisible(v);
        if (!v && _additionalcheckbox)
            _additionalcheckbox->setChecked(false);
    }
}

void sh::ui::qt::QtSettingUIFrame::mouseReleaseEvent(QMouseEvent *event)
{
    QFrame::mouseReleaseEvent(event);
    if (_additionalcheckwidget && _additionalcheckwidget->geometry().contains(event->pos())) {
        if (_additionalcheckbox && _additionalcheckbox->isEnabled())
            _additionalcheckbox->toggle();
    }
    else {
        if (_checkbox && _checkbox->isEnabled())
            _checkbox->toggle();
    }
}

bool sh::ui::qt::QtSettingUIFrame::isChecked()
{
    if (!_checkbox)
        return false;
    else
        return _checkbox->isChecked();
}

void sh::ui::qt::QtSettingUIFrame::setChecked(bool val)
{
    if (_checkbox)
        _checkbox->setChecked(val);
}

QVariant sh::ui::qt::QtSettingUIFrame::additionalCheckValue()
{
    if (_additionalcheckbox && _additionalcheckbox->isChecked())
        return _additionalCheckValue;
    else
        return QVariant();
}
