/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTSTOREPROFILEDIALOG_H
#define QTSTOREPROFILEDIALOG_H

#include <memory>
#include <QAbstractButton>
#include <QCompleter>
#include <QDialog>
#include <QHash>
#include <QInputDialog>
#include <QLabel>
#include <QSpacerItem>
#include "ui/storeprofiledialog.h"
#include "qtdialog.h"


namespace Ui {
class QtStoreProfileDialog;
}

namespace sh {
    namespace settings {class Setting;}

    namespace ui {

        namespace qt {
            class QtSettingUIFrame;

            /** \brief
             * Qt based save settings dialog.
             */
            class QtStoreProfileDialog : public QtDialog, public StoreProfileDialog
            {
                Q_OBJECT

            public:
                explicit QtStoreProfileDialog(std::shared_ptr<const sh::filesystem::Eurl> eurl);
                ~QtStoreProfileDialog();
                QList<sh::ui::StoreProfileDialog::SettingEntry> checkedSettings() override;
                QString profileName() override;
                bool withSubDirectories() override;
                QStringList inheritsFrom() override;
                bool hasGlobal() override;

            private slots:
                void on_btnAddProf_clicked();
                void on_btnGlobal_clicked();
                void on_btn22_clicked();
                void on_toolButton_2_clicked();
                void on_toolButton_clicked();

            private:
                void _settingsLevel(int level, bool persist = true);
                void _setglobal(bool v);
                QHash<sh::ui::qt::QtSettingUIFrame*, std::shared_ptr<sh::settings::Setting>> _widgets;
                QHash<int, QLabel*> _headerlabels;
                QStringList _inheritProfileNameList;
                bool _global = false;
                Ui::QtStoreProfileDialog *ui;
                int _level;
            };

        }
    }
}

#endif // QTSTOREPROFILEDIALOG_H
