/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTTOOLBARBUTTON_H
#define QTTOOLBARBUTTON_H

#include <functional>
#include <QMenu>
#include <QToolButton>


namespace sh{
    namespace actions{class ActionsManager;}
    namespace ui {

        namespace qt {
            class QtActionMenu;

            /** \brief
             * A qt toolbar button implementation with optional submenu.
             */
            class QtToolbarButton : public QToolButton
            {
                friend class sh::actions::ActionsManager;
                Q_OBJECT
            public:
                enum class Position {
                    LEFT,
                    RIGHT,
                    TOP,
                    BOTTOM
                };

                explicit QtToolbarButton(QWidget *parent = 0);
                void setText(const QString &text);
                QString text() const;
                void setIcon(const QIcon &icon);
                QIcon icon();
                QSize sizeHint() const;

                void setButtonText(QString v);
                QString buttonText();
                void setButtonIcon(QIcon v);
                QIcon buttonIcon();
                void setButtonEnabled(bool v);
                bool isButtonEnabled();

                bool hasExpander();
                void setSubmenu(QtActionMenu* menu);
                QtActionMenu* submenu();
                void setLocation(Position location);
                void setClickAction( std::function<void ()> action );
                void unsetClickAction();
                void trigger();
                void openMenu();

            protected:
                void paintEvent(QPaintEvent* ev);
                void mouseMoveEvent(QMouseEvent* event);
                void resizeEvent(QResizeEvent* event);
                bool event(QEvent* e);
                bool eventFilter(QObject* obj, QEvent *event);

            private slots:
                void slot_clicked();

            private:
                QString _text;
                QString _text1;
                QString _textWithoutMnemonic1;
                QString _text2;
                QString _textWithoutMnemonic2;
                QIcon _icon;
                bool _drawIconOnly = false;
                bool _hovered = false;
                bool _arrowhovered = false;
                sh::ui::qt::QtActionMenu* _menu = 0;
                std::function<void ()> _clickAction;
                Position location;
                void _calcDims();
                int tw1, tworig1, tw2, tworig2, twmax, tworigmax, th, thorig, iw, ih, iedim, ew, eh;
                const int textpad = 20;
                const int opad = 3;
                void computeArrowAndDividerLineCoordinates();
                int expx1, expx2, expy1, expy2;
                int xt, yt1, yt2, xi, yi;
                QPainterPath arrow;
                bool _drawmnemonics = false;
            };

        }
    }
}

#endif // QTTOOLBARBUTTON_H
