/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionactionitem.h"
#include "actions/actionsmanager.h"
#include "actions/submenuactionitem.h"
#include "base/iconmanager.h"
#include "base/mainthread.h"
#include "qtactionmenu.h"
#include "qtactionmenuhandler.h"
#include "qttoolbarbutton.h"
#include "qttoolbarbuttonhandler.h"


void sh::ui::qt::QtToolbarButtonHandler::_applyPropertiesToButton(sh::actions::SubmenuActionItem* action, QtToolbarButton* button, QAction* qaction)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    qaction->setVisible(action->visible());
    button->setText(action->text());
    button->setIcon(sh::base::IconManager::instance()->getIconByFullname(action->icon()));
    button->setEnabled(action->enabled());
    button->setCheckable(action->isCheckable());
    button->setChecked(action->isChecked());
}

void sh::ui::qt::QtToolbarButtonHandler::_updateSubmenu()
{
    sh::ui::qt::QtActionMenuHandler::_updateSubmenu(action.get(), &menu, [=](){ sh::ui::qt::QtActionMenuHandler::_markDefault(action, &menu); });
}

sh::ui::qt::QtToolbarButtonHandler::QtToolbarButtonHandler(std::shared_ptr<sh::actions::SubmenuActionItem> action, bool preventDefaultAction, QtToolbarButton* button, QAction* qaction) : action(action), preventDefaultAction(preventDefaultAction), button(button)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    sh::actions::SubmenuActionItem* paction = action.get();
    paction->initializeAsync();
    QObject::connect(paction, &sh::actions::SubmenuActionItem::subitemsChanged, button, [=](){
        _updateSubmenu();
    });
    _updateSubmenu();
    QObject::connect(paction, &sh::actions::AbstractActionItem::changed, button, [=](){
        _applyPropertiesToButton(paction, button, qaction);
    });
    _applyPropertiesToButton(paction, button, qaction);
    button->setSubmenu(&menu);
    button->setClickAction([=](){
        std::shared_ptr<sh::actions::ActionActionItem> defaultAction = 0;
        if (!preventDefaultAction) {
            defaultAction = QtActionMenuHandler::_getDefaultAction(action->subitems());
        }
        if (defaultAction)
            defaultAction->execute();
        else
            button->openMenu();
    } );
}

sh::ui::qt::QtToolbarButtonHandler::~QtToolbarButtonHandler()
{
}
