/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTTUNINGDIALOG_H
#define QTTUNINGDIALOG_H

#include <memory>
#include <QDialog>
#include <QFileDialog>
#include <QInputDialog>
#include "ui/tuningdialog.h"
#include "qtdialog.h"


namespace Ui {
class QtTuningDialog;
}

namespace sh{
    namespace configuration { class ConfigurationValue; }

    namespace ui {
        namespace qt {

            /** \brief
             * Qt based tuning dialog.
             */
            class QtTuningDialog : public QtDialog, public sh::ui::TuningDialog
            {
                Q_OBJECT

            public:
                explicit QtTuningDialog();
                ~QtTuningDialog();

            private slots:
                void on_btnCancel_clicked();
                void on_lineEditSearchInt_textChanged(const QString &arg1);
                void on_lineEditSearchExt_textChanged(const QString &arg1);
                void on_lineEditSearchBehav_textChanged(const QString &arg1);
                void on_lineEditSearchAppear_textChanged(const QString &arg1);

                void on_tabWidget_currentChanged(int index);

            private:
                typedef QPair<QString, QWidget*> BoxWidgetByDesc;
                Ui::QtTuningDialog *ui;
                QString _changedlglabel(std::shared_ptr<sh::configuration::ConfigurationValue> cv);
                void do_change(std::shared_ptr<sh::configuration::ConfigurationValue> cv);
                void filterVisibility(QString s);
                QList<BoxWidgetByDesc> boxWidgetsByDesc;
            };

        }
    }
}

#endif // QTTUNINGDIALOG_H
