/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTUISTYLE_H
#define QTUISTYLE_H

#include <memory>
#include <QColor>
#include <QObject>


namespace sh{
    namespace ui {
        namespace qt {

            class QtMainWindow;

            /** \brief
             * Methods for applying the Shallot visible style.
             *
             * See sh::ui::qt::QtMainWindow about how to get an instance.
             */
            class QtUIStyle
            {
            public:
                /** \brief Constructed only by the infrastructure and made available otherwise.
                 */
                explicit QtUIStyle(sh::ui::qt::QtMainWindow *mainwnd);

                QColor windowColor();
                QColor backgroundColor();
                QColor inactiveBackgroundColor();
                QColor windowColorHighlighted();
                QColor windowColorHalfHighlighted();
                QColor windowColorDarkened();
                QColor windowColorLikeTitlebar();
                QColor linkColor();
                QColor titlebarColor();
                QColor foregroundColor();
                QColor foregroundColorLighter1();
                QColor foregroundColorLighter2();
                int fontSizeInPt(int r=100);
                QColor mix(float n1, QColor c1, QColor c2);
                class Sheet;
                Sheet createSheet();

                class Sheet
                {
                    friend class QtUIStyle;
                public:
                    QString sheet();
                    Sheet customcss(QString css);
                    Sheet fontsize_byFactor(int f);
                    Sheet fontsize_header();
                    Sheet fontsize_smaller();
                    Sheet textcolor(QColor color);
                    Sheet textcolor_lighter1();
                    Sheet textcolor_lighter2();
                    Sheet background(QColor color, bool restrictQWidget = true);
                    Sheet background_highlighted(bool restrictQWidget = true);
                    Sheet background_halfhighlighted(bool restrictQWidget = true);
                    Sheet background_darkened(bool restrictQWidget = true);
                    Sheet background_liketitlebar(bool restrictQWidget = true);
                    Sheet applyTo(QWidget* widget);
                    Sheet bold();
                private:
                    Sheet(QtUIStyle* style, QString sheet);
                    QString _sheet;
                    QtUIStyle* _style;
                };

            private:
                sh::ui::qt::QtMainWindow* _mainWindow;

            };

        }
    }
}

#endif // QTUISTYLE_H
