/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "configuration/configurationmanager.h"
#include "storeprofiledialog.h"


std::shared_ptr<sh::configuration::ConfigurationValue> sh::ui::StoreProfileDialog::cfgvalLevel =
        sh::configuration::ConfigurationManager::instance()->registerConfigValue("sh::ui::StoreProfileDialog.Level", 0,
                              sh::configuration::ConfigurationValue::valueTypeInteger());


sh::ui::StoreProfileDialog::StoreProfileDialog(std::shared_ptr<const sh::filesystem::Eurl> eurl) : _eurl(eurl)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
}

std::shared_ptr<const sh::filesystem::Eurl> sh::ui::StoreProfileDialog::eurl()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _eurl;
}

sh::ui::StoreProfileDialog::SettingEntry::SettingEntry(sh::settings::Setting* setting, int onlyInFileview)
    : _setting(setting), _fileview(onlyInFileview)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
}

sh::settings::Setting* sh::ui::StoreProfileDialog::SettingEntry::setting()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _setting;
}

int sh::ui::StoreProfileDialog::SettingEntry::fileview()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _fileview;
}

bool sh::ui::StoreProfileDialog::SettingEntry::isForFileview()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _fileview != -1;
}
