/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef POCOWEBSERVERENGINE_H
#define POCOWEBSERVERENGINE_H

#if WEBUI_ENGINE == 1
#include <Poco/Net/ServerSocket.h>
#include <Poco/Net/HTTPClientSession.h>
#include <Poco/Net/HTTPServer.h>
#include <Poco/Net/HTTPRequestHandler.h>
#include <Poco/Net/HTTPRequestHandlerFactory.h>
#include <Poco/Net/HTTPResponse.h>
#include <Poco/Net/HTTPServerRequest.h>
#include <Poco/Net/HTTPServerResponse.h>
#include <Poco/Util/ServerApplication.h>
#include <Poco/URI.h>
#include "ui/web/webserverengine.h"


namespace sh {
    namespace filesystem { class FilesystemNode; }
    namespace actions { class AbstractActionItem; }
    namespace ui {
        namespace web {

            /** \brief WebServerEngine implementation using Poco::Net.
             */
            class PocoNetWebServerEngine : public WebServerEngine
            {
            public:
                PocoNetWebServerEngine(bool withMainModule, int minport, int maxport, QString externalUrl, QString dispatcherUrl, QByteArray wtoken);
                ~PocoNetWebServerEngine();
                QUrl url() override;
            private:

                class PocoNetWebServerEngineRequest : public WebServerEngineRequest
                {
                public:
                    PocoNetWebServerEngineRequest(QString url, QString clientHost, Poco::Net::HTTPServerRequest* req, Poco::Net::HTTPServerResponse* resp);
                    QString headerData(QString key) override;
                    QString getCookie(QString key) override;
                    void setCookie(QString key, QString value, int maxAgeSecs=-1) override;
                private:
                    Poco::Net::HTTPServerRequest* _req;
                    Poco::Net::HTTPServerResponse* _resp;
                };

                class PocoNetWebServerEngineRequestHandler : public Poco::Net::HTTPRequestHandler
                {
                public:
                    PocoNetWebServerEngineRequestHandler(PocoNetWebServerEngine* engine);
                    void handleRequest(Poco::Net::HTTPServerRequest &req, Poco::Net::HTTPServerResponse &resp) override;
                private:
                    PocoNetWebServerEngine* engine;
                };

                class PocoNetWebServerEngineRequestHandlerFactory : public Poco::Net::HTTPRequestHandlerFactory
                {
                public:
                    PocoNetWebServerEngineRequestHandlerFactory(PocoNetWebServerEngine* engine);
                    Poco::Net::HTTPRequestHandler* createRequestHandler(const Poco::Net::HTTPServerRequest &) override;
                private:
                    PocoNetWebServerEngine* engine;
                };

                class PocoNetWebServerEngineThread : public QThread
                {
                public:
                    PocoNetWebServerEngineThread(PocoNetWebServerEngine* engine);
                    void stopServer();
                    void waitWhileRunning();
                protected:
                    PocoNetWebServerEngine* engine;
                    bool running = true;
                    QMutex mutexRunning;
                    QWaitCondition cndRunning;
                    void run();
                };

                class PocoNetWebServerEngineApplication : public Poco::Util::ServerApplication
                {
                public:
                    PocoNetWebServerEngineApplication(PocoNetWebServerEngine* engine, PocoNetWebServerEngineThread* thread);
                protected:
                    int main(const std::vector<std::string>&) override;
                    PocoNetWebServerEngine* engine;
                    PocoNetWebServerEngineThread* thread;
                };

                QUrl _url;
                sh::ui::web::PocoNetWebServerEngine::PocoNetWebServerEngineThread* thread =0;
                QMutex _initedmutex;
                QWaitCondition _initedcondition;
                bool _inited = false;
            };

        }
    }
}

#endif

#endif // POCOWEBSERVERENGINE_H
