/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WEBACTIONEXECUTIONINFODIALOG_H
#define WEBACTIONEXECUTIONINFODIALOG_H

#include <QMutex>
#include <QWaitCondition>
#include "ui/actionexecutioninfodialog.h"


namespace sh{
    namespace ui{
        namespace web{

            /** \brief Web based action execution dialog.
             */
            class WebActionExecutionInfoDialog : public ActionExecutionInfoDialog
            {
                friend class WebActionManager;
            public:

                class UserFeedback : public QMap<QString, QVariant> {
                public:
                    qint64 id = -1;
                    bool answered = false;
                };

                WebActionExecutionInfoDialog(sh::actions::ActionExecutionInfo *info);
                virtual void setDetails(QString fv, QString fob, QString tv, QString tob);
                virtual void setHead(QString txt);
                virtual void setProgress(bool isDeterminate, quint64 done, quint64 all, QString text);
                virtual int messageBox(QString text, QList<QString> answers, QString icon=QString(), int defaultanswer=-1, int cancelanswer=-1, QList<QString> answericons=QList<QString>());
                virtual int inputBox(QString text, QList<QString> answers, QString *value, QString icon=QString(), int defaultanswer=-1, int cancelanswer=-1, int valuePreselectFrom=-1, int valuePreselectTo=-1);
                virtual int multilineInputBox(QString text, QList<QString> answers, QString *value, QString icon=QString(), int defaultanswer=-1, int cancelanswer=-1);
                virtual int simpleChooserGridform(QString text, GridformEntries* entries, QStringList answers, int defaultanswer=-1, int cancelanswer=-1);
                virtual bool credentialsDialog(QString text, bool showDomain, bool showUsername, bool showPassword, bool showAnonymous, bool showRemember, QString* domain, QString* username, QString* password, bool* isAnonymous, bool* isRemember);
                virtual bool unixPermissionsDialog(bool* userMayRead, bool* userMayWrite, bool* userMayExecute, bool* groupMayRead, bool* groupMayWrite, bool* groupMayExecute, bool* othersMayRead, bool* othersMayWrite, bool* othersMayExecute, bool* sticky, bool* setuid, bool* setgid, QStringList users, QStringList groups, QString* ownerUser, QString* ownerGroup);
                qint64 id();
                qint64 webts_created();
                QString details_fromverb();
                QString details_fromobject();
                QString details_toverb();
                QString details_toobject();
                QString head();
                bool progress_isDeterminate();
                quint64 progress_done();
                quint64 progress_all();
                QString progress_text();
                QJsonValue userFeedbackAsJsonValue();
            private:
                static QByteArray iconToBase64Src(QString icon, int sizeInPt);
                static QList<QVariant> iconsToBase64Srcs(QStringList icons, int sizeInPt);
                void _handleUserFeedback(QString kind, std::shared_ptr<UserFeedback> userfeedback);
                void _triggerChanged();
                qint64 _id;
                QString _details_fv;
                QString _details_fob;
                QString _details_tv;
                QString _details_tob;
                QString _head;
                bool _progress_isDeterminate;
                quint64 _progress_done;
                quint64 _progress_all;
                QString _progress_text;
                std::shared_ptr<UserFeedback> _currentUserFeedback = 0;
                QMutex _currentUserFeedbackMutex;
                QWaitCondition _currentUserFeedbackChangedCondition;
                qint64 _webts_created;
            };

        }
    }
}

#endif // WEBACTIONEXECUTIONINFODIALOG_H
