/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "webactionexecutioninfopanel.h"
#include "webactionmanager.h"
#include "webmainwindow.h"
#include "webserverengine.h"


sh::ui::web::WebActionExecutionInfoPanel::WebActionExecutionInfoPanel(sh::actions::ActionExecutionInfo* info, QColor color) : _color(color), _info(info)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    static qint64 next_id = 0;
    _id = next_id++;
    setWidth(100);
}

qint64 sh::ui::web::WebActionExecutionInfoPanel::id()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _id;
}

sh::ui::web::WebActionExecutionInfoPanel::~WebActionExecutionInfoPanel()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
}

void sh::ui::web::WebActionExecutionInfoPanel::_triggerChanged() {
    sh::ui::web::WebMainWindow::mainWindow()->actionManager()->_triggerActionuiChanged();
}

void sh::ui::web::WebActionExecutionInfoPanel::setLabel(QString s)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    _label = s;
    _triggerChanged();
}

void sh::ui::web::WebActionExecutionInfoPanel::setProgress(bool isProgressDeterminate, quint64 progressDone, quint64 progressAll)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    _isProgressDeterminate = isProgressDeterminate;
    _progressDone = progressDone;
    _progressAll = progressAll;
    _triggerChanged();
}

void sh::ui::web::WebActionExecutionInfoPanel::setForceForeground(bool v)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    bool changed = _forceForeground != v;
    _forceForeground = v;
    _triggerChanged();
    if (changed && v)
        sh::ui::web::WebMainWindow::mainWindow()->webserver()->triggerEvent("action/force_foreground", id()); // this is for dialogs which were visible before
}

void sh::ui::web::WebActionExecutionInfoPanel::setPanelVisible(bool v)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    _panelVisible = v;
    _triggerChanged();
}

void sh::ui::web::WebActionExecutionInfoPanel::setWidth(int v)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    _width = v * 2;
    _triggerChanged();
}

QString sh::ui::web::WebActionExecutionInfoPanel::label()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _label;
}

bool sh::ui::web::WebActionExecutionInfoPanel::isProgressDeterminate()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _isProgressDeterminate;
}

quint64 sh::ui::web::WebActionExecutionInfoPanel::progressDone()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _progressDone;
}

quint64 sh::ui::web::WebActionExecutionInfoPanel::progressAll()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _progressAll;
}

bool sh::ui::web::WebActionExecutionInfoPanel::forceForeground()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _forceForeground;
}

bool sh::ui::web::WebActionExecutionInfoPanel::panelVisible()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _panelVisible;
}

int sh::ui::web::WebActionExecutionInfoPanel::width()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _width;
}

QColor sh::ui::web::WebActionExecutionInfoPanel::color()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _color;
}

sh::actions::ActionExecutionInfo* sh::ui::web::WebActionExecutionInfoPanel::info()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return _info;
}
