/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WEBACTIONMANAGER_H
#define WEBACTIONMANAGER_H

#include <QColor>
#include <QMap>
#include <QTimer>
#include "webmodule.h"


namespace sh {
    namespace actions { class AbstractActionItem; class ActionExecutionInfo; class ActionInstantiation; class SubmenuActionItem; }
    namespace ui {
        namespace web {
            class WebActionExecutionInfoPanel;
            class WebActionExecutionInfoDialog;

            /** @brief Manages `sh::actions::AbstractActionItem` handling, e.g. showing them in the toolbar,
             * executing them, and displaying their user interface.
             */
            class WebActionManager : public QObject, public WebModule
            {
                friend class WebActionExecutionInfoPanel;
                friend class WebActionExecutionInfoDialog;
                Q_OBJECT
            public:

                /** @a_infrastructure
                 */
                WebActionManager();

                /** @a_infrastructure
                 */
                void initialize();

                /** @brief Adds and returns a new action info panel.
                 */
                std::shared_ptr<WebActionExecutionInfoPanel> addInfoPanel(int position, sh::actions::ActionExecutionInfo* info, QColor color=QColor());

                /** @brief Adds and returns a new action info dialog.
                 */
                std::shared_ptr<WebActionExecutionInfoDialog> addActionExecutionInfoDialog(sh::actions::ActionExecutionInfo* info);

                /** @brief Refreshes the main toolbar.
                 */
                void refreshToolbar(std::shared_ptr<sh::actions::ActionInstantiation> instantiation);

            private slots:
                void cmd__answer_userfeedback__M(WebServerEngineRequest* request);
                void cmd__execute(WebServerEngineRequest* request);
                void cmd__get(WebServerEngineRequest* request);
                void cmd__get_ui__M(WebServerEngineRequest* request);
                void cmd__icon(WebServerEngineRequest* request);
                void cmd__panel_clicked__M(WebServerEngineRequest* request);

            private:
                bool executeCommand(WebServerEngineRequest* request, QString command) override;
                void _observeToolbarAction(sh::actions::AbstractActionItem* a);
                std::shared_ptr<sh::actions::AbstractActionItem> resolveActionpath(QString sactionpath, QStringList* actionpath, QList<QPair<QString, std::shared_ptr<sh::actions::AbstractActionItem>>> *subactions);
                QList<WebActionExecutionInfoPanel*> _infopanels;
                QList<WebActionExecutionInfoDialog*> _infodialogs;
                void _triggerActionuiChanged();
                QTimer _triggerActionuiChangedTimer;
                QTimer _triggerToolbarRefreshTimer;
                QMap<QString, std::shared_ptr<sh::actions::SubmenuActionItem>> _permanentToolbarActions;
                static QRegularExpression reActionTextAccel;
            };

        }
    }
}

#endif // WEBACTIONMANAGER_H
