/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WEBDETAILPANELMANAGER_H
#define WEBDETAILPANELMANAGER_H

#include <QJsonArray>
#include <QJsonObject>
#include <QMutex>
#include "webmodule.h"


namespace sh {
    namespace filesystem { class FilesystemNode; }
    namespace paneldetails { class PanelDetail; }
    namespace ui {
        namespace web {

            /** @brief Manages the detail panel (typically in bottom part of main window).
             */
            class WebDetailPanelManager : public QObject, public WebModule
            {
                Q_OBJECT
            public:
                WebDetailPanelManager();

                /** @brief Loads details for a given list of nodes (typically called after they get selected).
                 */
                void load(QList<std::shared_ptr<sh::filesystem::FilesystemNode> > nodes);

            private slots:
                void cmd__get_details(WebServerEngineRequest* request);
                void cmd__get_thumbnail(WebServerEngineRequest* request);
                void cmd__link_clicked(WebServerEngineRequest* request);
            private:
                bool executeCommand(WebServerEngineRequest* request, QString command) override;
                QMutex _mutex;
                QList<std::shared_ptr<sh::paneldetails::PanelDetail>> _paneldetails;
                qint64 _webts_lastupdate;
            };

        }
    }
}

#endif // WEBDETAILPANELMANAGER_H
