/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/iconmanager.h"
#include "tools/misc.h"
#include "webexceptiondialog.h"


sh::ui::web::WebExceptionDialog::WebExceptionDialog(QString error1, QString error2, QString details, QString icon, bool mayRetry, bool mayClose, bool mayCancel, bool showLoglabelAndDetails)
    : WebDialog("ExceptionDialog"), ExceptionDialog(error1, error2, details, icon, mayRetry, mayClose, mayCancel, showLoglabelAndDetails)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    setDialogProperty("closeable", false);
    setDialogProperty("headtext1", error1);
    setDialogProperty("error2", error2);
    setDialogProperty("details", details);
    setDialogProperty("mayRetry", mayRetry);
    setDialogProperty("mayClose", mayClose);
    setDialogProperty("mayCancel", mayCancel);
    setDialogProperty("showLoglabelAndDetails", showLoglabelAndDetails);
}

sh::ui::ExceptionDialogResult sh::ui::web::WebExceptionDialog::answer()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    if (dialogResult().value("cancel").toBool())
        return ExceptionDialogResult::Cancel;
    else if (dialogResult().value("retry").toBool())
        return ExceptionDialogResult::Retry;
    else
        return ExceptionDialogResult::Close;
}

void sh::ui::web::WebExceptionDialog::cmd__icon__M(WebServerEngineRequest* request)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QIcon qicon;
    if (!icon().isEmpty())
        qicon = sh::base::IconManager::instance()->getIcon(icon());
    QByteArray bArray = sh::tools::Misc::iconToPngByteArray(qicon, 48);
    request->setResponse(bArray, "image/png");
}

bool sh::ui::web::WebExceptionDialog::executeDialogCommand(WebServerEngineRequest* request, QString command)
{
    return WebModule::executeCommand_byQObjectReflection(this, request, command, "exception");
}
