/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WEBFILEPROPERTYDIALOG_H
#define WEBFILEPROPERTYDIALOG_H

#include <QTimer>
#include "tools/misc.h"
#include "ui/filepropertydialog.h"
#include "ui/filepropertydialogtabactionsview.h"
#include "webdialog.h"
#include "webmodule.h"
#include "webserverengine.h"


namespace sh{
    namespace ui{
        namespace web{
            class WebFilePropertyDialog;

            typedef QPair<QIcon, int> WebIconTextBannerItemIcon;
            typedef QPair<WebIconTextBannerItemIcon, QString> WebIconTextBannerItem;

            /** \brief Web based file property dialog.
             */
            class WebFilePropertyDialog : public QObject, public WebDialog, public FilePropertyDialog
            {
                Q_OBJECT
            public:
                void refresh() override;
                FilePropertyDialogTabTableView *createTabViewTable() override;
                FilePropertyDialogTabTextView *createTabViewText() override;
                FilePropertyDialogTabIconTextBannerView *createTabViewIconTextBanner() override;
                WebFilePropertyDialog(QList<std::shared_ptr<sh::filesystem::FilesystemNode>> nodes);
                ~WebFilePropertyDialog();
            protected:
                void updateWidgets() override;
                bool executeDialogCommand(WebServerEngineRequest* request, QString command) override;
                void init(std::shared_ptr<Dialog> dialog) override;
            private slots:
                void cmd__get_properties__M(WebServerEngineRequest* request);
                void cmd__refresh__M(WebServerEngineRequest* request);
                void cmd__trigger_action__M(WebServerEngineRequest* request);
                void cmd__tabwidget_table_select__M(WebServerEngineRequest* request);

            private:

                class WebFilePropertyDialogChild {
                public:
                    WebFilePropertyDialogChild(WebFilePropertyDialog* dlg);
                    virtual ~WebFilePropertyDialogChild();
                protected:
                    void requestClientRefresh();
                    WebFilePropertyDialog* _dlg;
                };

                class WebFilePropertyDialogTabActionsView : public sh::ui::FilePropertyDialogTabActionsView, public sh::tools::Jsonable, public WebFilePropertyDialogChild {
                    friend class WebFilePropertyDialog;
                public:
                    WebFilePropertyDialogTabActionsView(WebFilePropertyDialog* dlg);
                    void setVisible(bool v) override;
                    void setContent(FilePropertyDialogTabViewContent* cnt) override;
                    void setButtons(QList<QString> buttons) override;
                    void setLabelText(QString text);
                    QJsonObject toJson() override;
                private:
                    bool _visible = false;
                    QString _labelText;
                };

                class WebFilePropertyDialogTabTableView : public sh::ui::FilePropertyDialogTabTableView, public sh::tools::Jsonable, public WebFilePropertyDialogChild {
                public:
                    WebFilePropertyDialogTabTableView(WebFilePropertyDialog* dlg);
                    void setContent(QList<QPair<QString, QString>> content) override;
                    QList<ItemIndex> getSelectedItems() override;
                    QVariant getItemValue(int r, int c) override;
                    QJsonObject toJson() override;
                    void setSelectedItems(QList<int> selitms);
                private:
                    QList<QPair<QString, QString>> _content;
                    QList<int> _selectedItems;
                };

                class WebFilePropertyDialogTabTextView : public sh::ui::FilePropertyDialogTabTextView, public sh::tools::Jsonable, public WebFilePropertyDialogChild {
                public:
                    WebFilePropertyDialogTabTextView(WebFilePropertyDialog* dlg);
                    void setContent(QString content) override;
                    QJsonObject toJson() override;
                private:
                    QString _content;
                };

                class WebFilePropertyDialogTabIconTextBannerView : public sh::ui::FilePropertyDialogTabIconTextBannerView, public sh::tools::Jsonable, public WebFilePropertyDialogChild {
                public:
                    WebFilePropertyDialogTabIconTextBannerView(WebFilePropertyDialog* dlg);
                    void clear() override;
                    void insertIcon(QIcon icon, int sizePt = 24) override;
                    void insertText(QString text) override;
                    QJsonObject toJson() override;
                private:
                    QList<WebIconTextBannerItem> _content;
                };

                void tabwidgetTableSelect(int cookie, int tabidx, int widgetidx, QList<int> lsel);
                void triggerAction(int cookie, int tabidx, int widgetidx, int btnidx);
                int _cookie = 0;
                QTimer _refreshTimer;
                QList<WebFilePropertyDialogChild*> _childs;
                QJsonArray getProperties();

            };

        }
    }
}

#endif // WEBFILEPROPERTYDIALOG_H
