/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/shallotprocess.h"
#include "tools/misc.h"
#include "webi18n.h"
#include "webserverengine.h"


QString sh::ui::web::WebI18n::get()
{
    QJsonObject result;
    result["About"] = QObject::tr("About");
    result["Advanced"] = QObject::tr("advanced");
    result["AdvancedMode"] = QObject::tr("Advanced mode");
    result["Add"] = QObject::tr("Add");
    result["AllDirectories"] = QObject::tr("all directories");
    result["AlsoAppliesOnSubfolders"] = QObject::tr("also applies on subfolders:");
    result["AlsoApplyOnSubdirs"] = QObject::tr("Also apply on sub-directories");
    result["Anonymous"] = QObject::tr("Anonymous");
    result["AnotherAncestor"] = QObject::tr("another ancestor");
    result["BackendClosedUnexpectedly"] = QObject::tr("The server side unexpectedly disappeared, probably due to a technical problem. Please try to reload this page!");
    result["Bookmarks"] = QObject::tr("Bookmarks");
    result["Built"] = QObject::tr("built %1");
    result["Cancel"] = QObject::tr("Cancel");
    result["Change"] = QObject::tr("change");
    result["Close"] = QObject::tr("Close");
    result["ConfirmDeleteBookmark"] = QObject::tr("Do you really want to delete '%1'?");
    result["ConfirmDeleteNode"] = QObject::tr("Do you really want to delete the selected entry from the profile?");
    result["ConfirmDeleteProfile1"] = QObject::tr("Do you really want to delete the complete profile '%1' with its one entry?");
    result["ConfirmDeleteProfile2"] = QObject::tr("Do you really want to delete the complete profile '%2' with its %1 entries?");
    result["Continue"] = QObject::tr("Continue");
    result["Delete"] = QObject::tr("Delete");
    result["Domain"] = QObject::tr("Domain");
    result["Down"] = QObject::tr("Down");
    result["EnterNewCollectionName"] = QObject::tr("Please enter the name for the new collection.");
    result["EntriesInProfile"] = QObject::tr("Entries in profile");
    result["Error_ReloadPage"] = QObject::tr("This should not happen! If you notice any misbehavior in the following, it's recommended to reload this page!");
    result["ErrorBrowser"] = QObject::tr("A browser-side error occurred.");
    result["ErrorGeneric"] = QObject::tr("An error occurred.");
    result["ErrorServer"] = QObject::tr("A server error occurred.");
    result["ExceptionDialogAProblemOccurred"] = QObject::tr("A problem occurred");
    result["ExceptionDialogLookAtTheLog"] = QObject::tr("Please take a look at the logfile, if you are interested in all technical details.");
    result["ForProfile"] = QObject::tr("For profile");
    result["Group"] = QObject::tr("Group");
    result["Homepage"] = QObject::tr("Homepage");
    result["InheritFrom"] = QObject::tr("Inherit from");
    result["InheritsFromOtherProfiles"] = QObject::tr("inherits from other profiles:");
    result["LNo"] = QObject::tr("no");
    result["LYes"] = QObject::tr("yes");
    result["Label"] = QObject::tr("Label");
    result["License"] = QObject::tr("License");
    result["Location"] = QObject::tr("Location");
    result["LogMessages"] = QObject::tr("Log Messages");
    result["LogMessagesVerbosityChange"] = QObject::tr("%1 (change)");
    result["LogMessagesVerbosityVeryVerbose"] = QObject::tr("very verbose");
    result["LogMessagesVerbosityVeryVerboseS"] = QObject::tr("very verbose");
    result["LogMessagesVerbosityInfo"] = QObject::tr("show infos");
    result["LogMessagesVerbosityInfoS"] = QObject::tr("shows infos");
    result["LogMessagesVerbosityWarning"] = QObject::tr("show warnings");
    result["LogMessagesVerbosityWarningS"] = QObject::tr("shows warnings");
    result["LogMessagesVerbosityError"] = QObject::tr("show errors");
    result["LogMessagesVerbosityErrorS"] = QObject::tr("shows errors");
    result["ManageBookmarks"] = QObject::tr("Manage bookmarks");
    result["ManageYourBookmarks"] = QObject::tr("Manage your bookmarks.");
    result["ManageProfile"] = QObject::tr("Manage profile:");
    result["ManageProfiles"] = QObject::tr("Manage saved settings");
    result["ManageProfilesHead"] = QObject::tr("Manage the lists of stored settings...");
    result["MayRead"] = QObject::tr("may read");
    result["MayWrite"] = QObject::tr("may write");
    result["MayExecute"] = QObject::tr("may execute");
    result["Move"] = QObject::tr("Move");
    result["New"] = QObject::tr("New");
    result["NewBookmark"] = QObject::tr("New bookmark");
    result["NewCollectionOfBookmarks"] = QObject::tr("New collection of bookmarks");
    result["No"] = QObject::tr("No");
    result["None"] = QObject::tr("(none)");
    result["NothingSelected"] = QObject::tr("Nothing selected.");
    result["OK"] = QObject::tr("OK");
    result["OnlyForFileListN"] = QObject::tr("only for file list #%1");
    result["OnlyRemoveEmpty"] = QObject::tr("Collections can only be removed if empty.");
    result["OpenAsText"] = QObject::tr("Open as text");
    result["OpenWith"] = QObject::tr("Open with");
    result["OpenWithAnotherAncestorWasInvalid"] = QObject::tr("The entered directory path was invalid.");
    result["OpenWithCustomOpenerPathWasInvalid"] = QObject::tr("The entered path was either malformed or didn't point to an existing file.");
    result["OpenWithHead"] = QObject::tr("Please choose a program for opening the file.");
    result["OpenWithPleaseEnterAnotherAncestor"] = QObject::tr("Please specify which directory to store this application choice for.");
    result["OpenWithPleaseEnterCustomOpenerPath"] = QObject::tr("Please enter the path to the program you want to open that file with.");
    result["OpenWithRememberDir"] = QObject::tr("Remember for files in this directory");
    result["OpenWithRememberFile"] = QObject::tr("Remember for this file");
    result["OpenWithRememberType"] = QObject::tr("Remember for this type of files");
    result["OtherUsers"] = QObject::tr("Other users");
    result["OwnerUser"] = QObject::tr("Owner user");
    result["Password"] = QObject::tr("Password");
    result["PropertiesTitle"] = QObject::tr("Properties");
    result["PropertiesTitleN"] = QObject::tr("%1 items");
    result["Refresh"] = QObject::tr("Refresh");
    result["ReloadShallot"] = QObject::tr("Reload");
    result["Remember"] = QObject::tr("Remember");
    result["RemoveSelectedProfile"] = QObject::tr("Remove complete profile...");
    result["RemoveSelectedSetting"] = QObject::tr("Remove selected");
    result["RestrictToCurrentDir"] = QObject::tr("restrict to current directory");
    result["Retry"] = QObject::tr("Retry");
    result["Save"] = QObject::tr("Save");
    result["SaveSettings"] = QObject::tr("Save Settings");
    result["Search"] = QObject::tr("Search");
    result["SelectOther"] = QObject::tr("Select Other");
    result["SetUserOnExecute"] = QObject::tr("set user on execute");
    result["SetGroupOnExecute"] = QObject::tr("set group on execute");
    result["ShallotCopyright"] = "Copyright (C) 2014-2020 Josef Hahn";
    result["ShallotInfo"] = QObject::tr("Shallot is a file manager.\n\nIt is licensed under the terms of the GNU General Public License 3.");
    result["ShallotVersion"] = sh::base::ShallotProcess::revisionString();
    result["ShallotBuildtime"] = sh::base::ShallotProcess::buildtime().currentMSecsSinceEpoch();
    result["Shallot"] = QObject::tr("Shallot");
    result["ShowDetails"] = QObject::tr("Show details");
    result["ShowDirTree"] = QObject::tr("Show Directory Tree");
    result["ShowFiles"] = QObject::tr("Show Files");
    result["ShowLog"] = QObject::tr("Show the Log");
    result["ShowLogAndContinue"] = QObject::tr("Show the Log & Continue");
    result["Simple"] = QObject::tr("simple");
    result["SimpleMode"] = QObject::tr("Simple mode");
    result["SpecifyNewProfileEntry"] = QObject::tr("Please enter the name of the profile you want to inherit from.");
    result["SpecifyWhichSettingsToStoreFor"] = QObject::tr("Please specify what and how shall be stored for");
    result["Sticky"] = QObject::tr("sticky");
    result["Store"] = QObject::tr("Store");
    result["StoreGlobally"] = QObject::tr("store globally");
    result["ThoseSettingsAreInTheNode"] = QObject::tr("Those settings are stored in the selected node:");
    result["Tuning"] = QObject::tr("Tuning");
    result["TuningAppearance"] = QObject::tr("Appearance");
    result["TuningBehavior"] = QObject::tr("Behavior");
    result["TuningExternalTools"] = QObject::tr("External Tools");
    result["TuningHead1"] = QObject::tr("Tune some valves under the hood...");
    result["TuningHead3"] = QObject::tr("Changes here should be rare cases. Most changes will not apply before you reconnect to Shallot.");
    result["TuningInternal"] = QObject::tr("Internal");
    result["TuningInternalPanelFooterWarning"] = QObject::tr("Warning: This is very internal stuff. You really should not change things here.");
    result["Up"] = QObject::tr("Up");
    result["User"] = QObject::tr("User");
    result["Yes"] = QObject::tr("Yes");
    return "shwebui.strings = " + sh::tools::Misc::qjsonToJson(result);
}
