/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "weblogviewdialog.h"
#include "webmainwindow.h"


sh::ui::web::WebLogViewDialog::WebLogViewDialog(QString headtext, QString subheadtxt, sh::base::LogSeverity minseverity)
    : WebDialog("LogViewDialog"), LogViewDialog(headtext, subheadtxt, minseverity)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    if (!headtext.isNull())
        setDialogProperty("headtext1", headtext);
    else
        setDialogProperty("headtext1", QObject::tr("Inspect the internal log messages from this session..."));
    if (!subheadtxt.isNull())
        setDialogProperty("headtext2", subheadtxt);
    setDialogProperty("minseverity", (int)minseverity);
}

void sh::ui::web::WebLogViewDialog::cmd__get_messages(WebServerEngineRequest* request)
{
    int fromidx = request->data("fromidx").toInt();
    if (fromidx < 0)
        fromidx = 0;
    QList<sh::base::Logger::LogMessage*> messages = sh::base::Logger::instance()->logMessages();
    QJsonArray jresult;
    for (int i=fromidx; i<messages.count(); i++)
    {
        sh::base::Logger::LogMessage* logmsg = messages[i];
        QJsonObject jmsg;
        jmsg["message"] = logmsg->message;
        jmsg["severity"] = (int)logmsg->severity;
        jmsg["source"] = logmsg->source;
        jmsg["time"] = logmsg->time.toMSecsSinceEpoch();
        jmsg["idx"] = i;
        jresult.append(jmsg);
    }
    request->setResponse(jresult);
}

bool sh::ui::web::WebLogViewDialog::executeDialogCommand(WebServerEngineRequest* request, QString command)
{
    return WebModule::executeCommand_byQObjectReflection(this, request, command, "logview");
}
