/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/mainthread.h"
#include "webmainwindow.h"
#include "webmodule.h"
#include "webserverengine.h"


sh::ui::web::WebModule::WebModule()
{
}

sh::ui::web::WebModule::~WebModule()
{
    if (_webserver)
        _webserver->removeModule(this);
}

bool sh::ui::web::WebModule::executeCommand(WebServerEngineRequest*, QString)
{
    return false;
}

bool sh::ui::web::WebModule::rootCommand(WebServerEngineRequest*, QString)
{
    return false;
}

void sh::ui::web::WebModule::setEngine(WebServerEngine* webserver)
{
    _webserver = webserver;
}

sh::ui::web::WebServerEngine* sh::ui::web::WebModule::webserver()
{
    return _webserver;
}

bool sh::ui::web::WebModule::executeCommand_byQObjectReflection(QObject* o, WebServerEngineRequest* request, QString command, QString justLeftside)
{
    QString pcmd;
    if (justLeftside.isNull())
        pcmd = command;
    else {
        int isl = command.indexOf("/");
        bool rr = false;
        if (isl > -1) {
            QString leftside = command.mid(0, isl);
            if (justLeftside == leftside) {
                pcmd = "_" + command.mid(isl+1);
                rr = true;
            }
        }
        if (!rr)
            return false;
    }
    pcmd = pcmd.replace("/", "_");
    QString cmdname = QString("cmd_%1(WebServerEngineRequest*)").arg(pcmd);
    QByteArray cmn = cmdname.toLocal8Bit();
    int icmdmethod = o->metaObject()->indexOfMethod(QMetaObject::normalizedSignature(cmn.data()));
    if (icmdmethod > -1) {
        QMetaMethod cmdmethod = o->metaObject()->method(icmdmethod);
        cmdmethod.invoke(o, Qt::DirectConnection, Q_ARG(WebServerEngineRequest*, request));
        return true;
    }
    QString cmdnamem = QString("cmd_%1__M(WebServerEngineRequest*)").arg(pcmd);
    QByteArray cmnm = cmdnamem.toLocal8Bit();
    int icmdmethodm = o->metaObject()->indexOfMethod(QMetaObject::normalizedSignature(cmnm.data()));
    if (icmdmethodm > -1) {
        QMetaMethod cmdmethod = o->metaObject()->method(icmdmethodm);
        sh::base::MainThread::dispatcher()->invokeSync([&](){
            cmdmethod.invoke(o, Qt::DirectConnection, Q_ARG(WebServerEngineRequest*, request));
        });
        return true;
    }
    return false;
}
