/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "filesystem/eurl.h"
#include "filesystem/filesystemnode.h"
#include "tools/filetypes/filetypemanager.h"
#include "tools/misc.h"
#include "configuration/configurationmanager.h"
#include "webmainwindow.h"
#include "webopenwithdialog.h"


sh::ui::web::WebOpenWithDialog::WebOpenWithDialog(std::shared_ptr<filesystem::FilesystemNode> node, QList<std::shared_ptr<tools::filetypes::OpenMethod> > openMethods)
        : WebDialog("OpenWithDialog"), OpenWithDialog(node, openMethods)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    setDialogProperty("anotherAncestor", node->eurl()->parentSegment()->asString());
}

bool sh::ui::web::WebOpenWithDialog::executeDialogCommand(WebServerEngineRequest* request, QString command)
{
    return WebModule::executeCommand_byQObjectReflection(this, request, command, "openwith");
}

std::shared_ptr<sh::tools::filetypes::OpenMethod> sh::ui::web::WebOpenWithDialog::chosenMethod()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QString custop = dialogResult().value("chosenCustomOpener").toString();
    if (custop.isEmpty())
        return openMethods().value(dialogResult().value("chosenMethod", -1).toInt(), nullptr);
    else
        return _openMethodForExecutable(custop);
}

bool sh::ui::web::WebOpenWithDialog::rememberForMimetype()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return dialogResult().value("rememberForMimetype", false).toBool();
}

std::shared_ptr<const sh::filesystem::Eurl> sh::ui::web::WebOpenWithDialog::rememberForDirectory()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QString seurl = dialogResult().value("rememberForDirectory").toString();
    if (seurl.isEmpty())
        return nullptr;
    else
        return sh::filesystem::Eurl::fromString(seurl);
}

bool sh::ui::web::WebOpenWithDialog::rememberForFile()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    return dialogResult().value("rememberForFile", false).toBool();
}

void sh::ui::web::WebOpenWithDialog::cmd__check_another_ancestor__M(WebServerEngineRequest* request)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QString eurl = request->data("eurl");
    QString fulleurl;
    QString basename;
    std::shared_ptr<const sh::filesystem::Eurl> eeurl = nullptr;
    try {
        eeurl = sh::filesystem::Eurl::fromString(eurl);
        fulleurl = eeurl->asString();
        basename = eeurl->basename();
    }
    catch (sh::filesystem::EurlMisformattedException&) {
    }
    request->setResponse(QJsonObject({{"fulleurl", fulleurl}, {"basename", basename}}));
}

void sh::ui::web::WebOpenWithDialog::cmd__check_custom_opener__M(WebServerEngineRequest* request)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QString path = request->data("path");
    QString fullpath;
    QString basename;
    if (!path.isEmpty()) {
        QFileInfo fileinfo(path);
        if (fileinfo.isFile()) {
            fullpath = fileinfo.absoluteFilePath();
            basename = fileinfo.fileName();
        }
    }
    request->setResponse(QJsonObject({{"fullpath", fullpath}, {"basename", basename}}));
}

void sh::ui::web::WebOpenWithDialog::cmd__open_method_icon__M(WebServerEngineRequest* request)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    int idx = request->data("idx").toInt();
    request->setResponse(sh::tools::Misc::iconToPngByteArray(openMethods()[idx]->icon, 32), "image/png");
}

void sh::ui::web::WebOpenWithDialog::cmd__open_methods__M(WebServerEngineRequest* request)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QJsonArray jopenMethods;
    QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> omethods = openMethods();
    for (int i=0; i<omethods.length(); i++)
        jopenMethods.append(QJsonObject({{"name", omethods[i]->name}, {"iconidx", i}}));
    request->setResponse(jopenMethods);
}
