/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WEBOPENWITHDIALOG_H
#define WEBOPENWITHDIALOG_H

#include <QDesktopServices>
#include "ui/openwithdialog.h"
#include "webdialog.h"


namespace sh{
    namespace filesystem { class Eurl; }
    namespace tools {
        namespace filetypes { class OpenMethod; }
    }

    namespace ui{
        namespace web{

            /** \brief Web based open with dialog.
             */
            class WebOpenWithDialog : public QObject, public WebDialog, public OpenWithDialog
            {
                Q_OBJECT
            public:
                WebOpenWithDialog(std::shared_ptr<sh::filesystem::FilesystemNode> node, QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> openMethods);
                std::shared_ptr<sh::tools::filetypes::OpenMethod> chosenMethod() override;
                bool rememberForMimetype() override;
                std::shared_ptr<const sh::filesystem::Eurl> rememberForDirectory() override;
                bool rememberForFile() override;
            protected:
                virtual bool executeDialogCommand(WebServerEngineRequest* request, QString command) override;
            private slots:
                void cmd__check_another_ancestor__M(WebServerEngineRequest* request);
                void cmd__check_custom_opener__M(WebServerEngineRequest* request);
                void cmd__open_method_icon__M(WebServerEngineRequest* request);
                void cmd__open_methods__M(WebServerEngineRequest* request);
            };

        }
    }
}

#endif // WEBOPENWITHDIALOG_H
