/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2020 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WEBTUNINGDIALOG_H
#define WEBTUNINGDIALOG_H

#include <QDesktopServices>
#include <QMutex>
#include <QTimer>
#include "configuration/configurationvalue.h"
#include "ui/tuningdialog.h"
#include "webdialog.h"


namespace sh{
    namespace ui{
        namespace web{

            /** \brief Web based tuning dialog.
             */
            class WebTuningDialog : public QObject, public WebDialog, public TuningDialog
            {
                Q_OBJECT
            public:
                WebTuningDialog();
            protected:
                virtual bool executeDialogCommand(WebServerEngineRequest* request, QString command) override;
                virtual void init(std::shared_ptr<Dialog> dialog) override;
            private slots:
                void cmd__change_configurationvalue(WebServerEngineRequest* request);
                void cmd__get_configurationvalues(WebServerEngineRequest* request);
            private:
                QMutex _cfgvaluesmutex;
                QWaitCondition _cfgvaluescondition;
                bool _cfgvaluesinited = false;
                QList<std::shared_ptr<sh::configuration::ConfigurationValue>> _cfgvalues;
                static QString categoryToName(sh::configuration::ConfigurationCategory category);
                static QString valueTypeToName(sh::configuration::ConfigurationValueType* valueType);
                static QJsonObject configurationValueToJsonObject(std::shared_ptr<sh::configuration::ConfigurationValue> cfgval);
            };

        }
    }
}

#endif // WEBTUNINGDIALOG_H
