/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    clove._ajaxprovider = shwebui.ajax;

    class EventFetcher {

        constructor() {
            this._lastid = -1;
            this._events = {};
        }

        getEvent(eventname) {
            var res = this._events[eventname];
            if (!res)
                this._events[eventname] = res = new clove.Event();
            return res;
        }

        _processEvent(ee) {
            var event = this._events[ee.name];
            if (event)
                event.trigger({data: JSON.parse(ee.data)});
        }

        _fetch() {
            var self = this;
            var ajaxreq = this._ajaxreq = shwebui.ajax({url: "events/get_next", success: function(data){
                if (ajaxreq != self._ajaxreq)  // either stopped or started again
                    return;
                var nlastid = self._lastid;
                try {
                    for(let ee of data) {
                        nlastid = Math.max(nlastid, ee.id);
                        self._processEvent(ee);
                    }
                }
                finally {
                    self._lastid = nlastid;
                    self._fetch();
                }
            }, data: {lastid: this._lastid}});
        }

        skipIds(lastId) {
            if (this._ajaxreq)
                throw new Error("can only be called while stopped");
            this._lastid = Math.max(lastId, this._lastid);
        }

        start() {
            if (!this._ajaxreq)
                this._fetch();
        }

        stop() {
            if (this._ajaxreq) {
                try {
                    this._ajaxreq.abort();
                }
                catch (e) { }
                this._ajaxreq = undefined;
            }
        }

    }

    shwebui.eventfetcher = new EventFetcher();

    class ConfigurationManager {

        constructor() {
            var self = this;
            this._events = {};
            var oncfgchanged = shwebui.eventfetcher.getEvent("configuration/changed");
            oncfgchanged.addHandler(function(e){
                self._s(e.data.key, e.data.value);
            });
        }

        _s(key, value) {
            this[key] = value;
            var event = this._events[key];
            if (event)
                event.trigger({key: key, value: value});
        }

        getKeyEvent(key) {
            var res = this._events[key];
            if (!res)
                this._events[key] = res = new clove.Event();
            return res;
        }

        unregisterKeyEvent(key) {
            delete this._events[key];
        }

        consume(key, consumefct, owner) {
            var evt = this.getKeyEvent(key);
            evt.addHandler(function(e){ consumefct(e.value); }, owner);
            consumefct(this[key]);
        }

    }

    shwebui.config = new ConfigurationManager();

    $(document).ready(function(){
        var _lastuserinteraction;
        var _ajaxreq_lastuserinteraction;
        function _refreshuserfeedback() {
            _lastuserinteraction = Date.now();
        };
        _refreshuserfeedback();
        document.body.addEventListener('mousedown', _refreshuserfeedback, true);
        document.body.addEventListener('keydown', _refreshuserfeedback, true);
        document.body.addEventListener('touchstart', _refreshuserfeedback, true);
        setInterval(function(){
            if (_ajaxreq_lastuserinteraction)
                _ajaxreq_lastuserinteraction.abort();
            var luia = Math.floor((_lastuserinteraction - Date.now()) / 1000);
            _ajaxreq_lastuserinteraction = shwebui.ajax({url: "dispatcher/set_last_user_activity", data: {value: luia}});
        }, 1000*58/*just for avoiding typical 1m timeouts*/);
    });

})();
