/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class Logger {

        constructor() {
            this.DEBUG = parseInt(shwebui.config.logseverity_debug);
            this.INFO = parseInt(shwebui.config.logseverity_info);
            this.WARNING = parseInt(shwebui.config.logseverity_warning);
            this.ERROR = parseInt(shwebui.config.logseverity_error);
        }

        _log(severity, message, source, config) {
            config = config || {};
            var ajcfg = {url: 'log/log_message', data: {message: message, severity: severity, source: source}};
            if (config.ignoreErrors)
                ajcfg.error = function(){};
            shwebui.ajax(ajcfg);
        }

        log(message, source, config) {
            this._log(this.INFO, message, source, config);
        }

        logWarning(message, source, config) {
            this._log(this.WARNING, message, source, config);
        }

        logError(message, source, config) {
            this._log(this.ERROR, message, source, config);
        }

    }

    shwebui.logger = new Logger();

    function default_errorhandler(config) {
        var veryearly = !shwebui.app;
        config = config || {};
        if (config.ignoreErrors === undefined)
            config.ignoreErrors = true;
        var s = config.message || shwebui.strings.ErrorGeneric;
        var lrnot = [{label: s + "\n\n"}, {label: shwebui.strings.Error_ReloadPage + "\n\n"}];
        var notification;
        if (config.hlog && !veryearly)
            lrnot.push({view: 'Button', label: shwebui.strings.ShowDetails, OnClicked: function(){
                notification.remove();
                shwebui.app.showLogView();
            }});
        notification = clove.notifications.notify({rows: lrnot}, {closeable: !veryearly});
        var slog = "An error occurred on client side of the web application: " + s;
        if (config.hlog) {
            for (var ohlog of config.hlog)
                slog += "\n - " + ohlog[0] + ": " + ohlog[1];
        }
        shwebui.logger.logError(slog, config.source, config);
    }

    shwebui.errorhandler = default_errorhandler;

    window.onerror = function(msg, url, lineNo, columnNo, error) {
        clove.utils.runOnLoaded(function(){ // errors ...
            setTimeout(function(){ // ... can come ...
                setTimeout(function(){ // ... very early ;)
                    shwebui.errorhandler({message: shwebui.strings.ErrorBrowser,
                                          hlog: [['msg', msg], ['url', url], ['lineNo', lineNo], ['columnNo', columnNo], ['error', error]]});
                });
            });
        });
    }

})();
