/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class DetailPanelSection extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [],
            }), domnode);
            this.declareProperty('pieces', []);
            this.OnLinkClicked = new clove.Event();
            this._keylbls = [];
        }

        _create_linkfct(linktarget) {
            var self = this;
            return function(){
                self.OnLinkClicked.trigger({linktarget: linktarget, webts_lastupdate: self.config.pieces.webts_lastupdate});
            };
        }

        pieces_setter(v) {
            this._keylbls = [];
            this.contentnode.innerHTML = '';
            this.table = document.createElement('table');
            this.contentnode.appendChild(this.table);
            for(var row of v.rows) {
                var tr = document.createElement('tr');
                var tdlbl = document.createElement('td');
                var lbl = document.createElement('div');
                lbl.className = 'shwebui_detailpanel_keylabel';
                tdlbl.appendChild(lbl);
                this._keylbls.push(lbl);
                if (row.key)
                    lbl.textContent = row.key.replace(" ", String.fromCharCode(160)) + ":";
                var tdval = document.createElement('td');
                for(var valelement of row.value) {
                    if (valelement.icon) {
                        var delem = document.createElement('img');
                        delem.src = valelement.icon;
                        tdval.appendChild(delem);
                    }
                    else if (valelement.linktarget) {
                        var delem = document.createElement('a');
                        delem.textContent = valelement.text;
                        tdval.appendChild(delem);
                        delem.className = 'shwebui_detailpanel_link';
                        if (valelement.linkautohide)
                            delem.className += ' shwebui_detailpanel_link_autohide';
                        $(delem).click(this._create_linkfct(valelement.linktarget));
                        var delems = document.createElement('span');
                        delems.textContent = ' ';
                        tdval.appendChild(delems);
                    }
                    else {
                        var delem = document.createElement('span');
                        delem.textContent = valelement.text + ' ';
                        tdval.appendChild(delem);
                    }
                }
                tr.appendChild(tdlbl);
                tr.appendChild(tdval);
                this.table.appendChild(tr);
            }
        }

        computePreferredWidth() {
            var maxkeywidth = 0;
            for(var lbl of this._keylbls) {
                var oclss = lbl.className;
                lbl.className += " clove_widget_contentnode_measure";
                var xw = clove.utils.getInnerSize(lbl).width;
                lbl.className = oclss;
                maxkeywidth = Math.max(maxkeywidth, xw);
            }
            return maxkeywidth + clove.utils.cssLengthToPixels((this.config.pieces.valueWidthHint * 1.4) + 'cm', 'x');
        }

        computePreferredHeightForWidth(w) {  //TODO# this might return too high values (leading to much space between lines)
            var oclss = this.table.className;
            this.table.className += " clove_widget_contentnode_measure";
            var xh = clove.utils.getInnerSize(this.table).height;
            this.table.className = oclss;
            return xh;
        }

        computeMinimalWidth() {
            return this.computePreferredWidth();
        }

        computeMinimalHeightForWidth(w) {
            return this.computePreferredHeightForWidth();
        }

    }

    shwebui.DetailPanelSection = DetailPanelSection;

    class DetailPanel extends clove.NameScope(clove.ResizeSplitter) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                horizontalStretchAffinity: 1,
                height: '100pt',
                body: [
                    {view: 'ImageView', name: 'imgThumb', zoom: 'auto'/*TODO#why needed?all a bit broken in clove*/}, //TODO # hide for icon view (and when no thumbnail is available?)
                    {view: 'Wrap', name: 'wrpDetails'},
                ],
            }), domnode);
            var self = this;
            this.addStyleClass('shwebui_detailpanel');
            shwebui.eventfetcher.getEvent("detailpanel/changed").addHandler(function(){
                self.refresh();
            });
            this.OnLinkClicked = new clove.Event();
            this._thumbnail_dummy = 0;
            this._thumbnail_width = 0;
            this._thumbnail_height = 0;
        }

        _create_linkfct(idx) {
            var self = this;
            return function(e){
                self.OnLinkClicked.trigger({idx: idx, linktarget: e.linktarget, webts_lastupdate: e.webts_lastupdate});
            };
        }

        doinit() {
            var self = this;
            super.doinit();
            var imgThumb = this.getByName('imgThumb');
            var wrpDetails = this.getByName('wrpDetails');
            imgThumb.OnResized.addHandler(function(){
                var ds = imgThumb.innerSize();
                self._thumbnail_width = parseInt(ds.width);
                self._thumbnail_height = parseInt(ds.height);
                self._refresh_thumbnail();
            }, this);
            setTimeout(function(){
                shwebui.config.consume("detailPanelThumbnailWidth", function(val){
                    var mwidth = imgThumb.outerSize().width + wrpDetails.outerSize().width;
                    if (mwidth > 0) {
                        var thumbx = parseInt(val) / mwidth;
                        self.setSizeFractions([thumbx, 1-thumbx]);
                    }
                }, self);
            });
        }

        _refresh_thumbnail() {
            this.getByName('imgThumb').setSource(shwebui.actioncallUrl('detailpanel/get_thumbnail?dummy='+(this._thumbnail_dummy++)+'&width='+this._thumbnail_width+'&height='+this._thumbnail_height));
        }

        refresh() {
            var self = this;
            var wrpDetails = this.getByName('wrpDetails');
            wrpDetails.setChildren([]);
            if (this._ajaxreq) {
                try {
                    this._ajaxreq.abort();
                }
                catch (e) { }
            }
            var ajaxreq = this._ajaxreq = shwebui.ajax({url: "detailpanel/get_details", success: function(data){
                if (ajaxreq != self._ajaxreq)  // either stopped or started again
                    return;
                var chlds = [];
                for (var idxd in data) {
                    var d = data[idxd];
                    chlds.push({view: 'shwebui.DetailPanelSection', pieces: d, OnLinkClicked: self._create_linkfct(idxd)});
                }
                wrpDetails.setChildren(chlds);
            }});
            this._refresh_thumbnail();
        }

    }

    shwebui.DetailPanel = DetailPanel;

})();
