/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class Dialog extends clove.NameScope(clove.Dialog) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                body: {rows: [
                    {rows: [
                        {view: 'Label', styleClass: 'shwebui_dlg_headlabel1', name: 'lblHead1', width: '1px', horizontalStretchAffinity: 1},
                        {view: 'Label', styleClass: 'shwebui_dlg_headlabel2', name: 'lblHead2', width: '1px', horizontalStretchAffinity: 1},
                        {view: 'Label', styleClass: 'shwebui_dlg_headlabel3', name: 'lblHead3', width: '1px', horizontalStretchAffinity: 1},
                        {cols: [], name: 'pnlAboveHeadTextsControls', styleClass: 'shwebui_dlg_aboveheadcontrols', newNameScope: true},
                        {cols: [
                            {cols: [], name: 'pnlButtonsLeft', newNameScope: true},
                            {view: 'Widget', horizontalStretchAffinity: 1},
                            {cols: [], name: 'pnlButtonsRight', newNameScope: true},
                        ], styleClass: 'shwebui_dlg_buttons'},
                    ], styleClass: 'shwebui_dlg_head'},
                    {cols: [], name: 'pnlBody', styleClass: 'shwebui_dlg_innerbody', newNameScope: true},
                ], styleClass: 'shwebui_dlg_outer'},
            }), domnode);
            this.declareProperty('leftHeadButtons', []);
            this.declareProperty('rightHeadButtons', []);
            this.declareProperty('dialogId');
            this.declareProperty('headtext1');
            this.declareProperty('headtext2');
            this.declareProperty('headtext3');
            this.declareProperty('aboveHeadTextsControls');
            this.declareProperty('innerBody', []);
            this.addStyleClass('shwebui_dlg');
            this._properties_webts = {};
        }

        _setProperty(k, v, webts) {
            var owebts = this._properties_webts[k];
            if (owebts === undefined || owebts < webts) {
                this._properties_webts[k] = webts;
                this.setProperty(k, v);
            }
        }

        doinit() {
            super.doinit();
            for (var kconfig in this.config)
                this._properties_webts[kconfig] = this.config.webts;
            var cns = this.containingNameScope();
            var pnlButtonsLeft = cns.getByName("pnlButtonsLeft");
            var pnlButtonsRight = cns.getByName("pnlButtonsRight");
            var pnlAboveHeadTextsControls = cns.getByName("pnlAboveHeadTextsControls");
            var pnlBody = cns.getByName("pnlBody");
            this.setHeadtext1(this.config.headtext1);
            this.setHeadtext2(this.config.headtext2);
            this.setHeadtext3(this.config.headtext3);
            if (this.config.aboveHeadTextsControls && this.config.aboveHeadTextsControls.length > 0)
                pnlAboveHeadTextsControls.setChildren(this.config.aboveHeadTextsControls);
            else
                pnlAboveHeadTextsControls.setVisibility(clove.InvisibleCollapsed);
            pnlButtonsLeft.setChildren(this.config.leftHeadButtons);
            pnlButtonsRight.setChildren(this.config.rightHeadButtons);
            pnlBody.setChildren(this.config.innerBody);
            cns.addChildNameScope(pnlButtonsLeft);
            cns.addChildNameScope(pnlButtonsRight);
            cns.addChildNameScope(pnlAboveHeadTextsControls);
            cns.addChildNameScope(pnlBody);
        }

        _headtext_setter_helper(n, v) {
            var l = this.containingNameScope().getByName(n);
            if (!l)
                return;
            l.setVisibility(v ? clove.Visible : clove.InvisibleCollapsed);
            l.setLabel(v);
        }

        headtext1_setter(v) {
            this._headtext_setter_helper("lblHead1", v);
        }

        headtext2_setter(v) {
            this._headtext_setter_helper("lblHead2", v);
        }

        headtext3_setter(v) {
            this._headtext_setter_helper("lblHead3", v);
        }

        ajax(config) {
            config.data = config.data || {};
            config.data['dialogId'] = this.dialogId();
            config.url = "dialog/" + config.url;
            var csuccess = config.success;
            if (csuccess) {
                config.success = function(data) {
                    if (data._dialog_error && data._dialog_error._dialog_orphaned)
                        return; // orphaned dialog; we haven't got any real response; we just skip all further processing
                    csuccess(data);
                };
            }
            return shwebui.ajax(config);
        }

        dlgurl(cmd, data) {
            var result = "dialog/" + cmd + "?dialogId=" + this.config.dialogId;
            for (var kdata in data)
                result += "&" + kdata + "=" + encodeURIComponent(data[kdata]);
            return result;
        }

        getEvent(eventname) {
            return shwebui.dialogs.getEvent(this.dialogId(), eventname);
        }

        publishChangedProperty(k, onsuccess) {
            shwebui.ajax({url: 'dialog/change_dialog_property',
                          data: {dialogId: this.dialogId(), key: k, value: JSON.stringify(this.getProperty(k))},
                          success: onsuccess});
        }

        acceptDialog(result) {
            var self = this;
            if (result !== undefined) {
                this.setProperty('_dialog_result', result);
                this.publishChangedProperty('_dialog_result', function(){
                    self.remove();
                });
            }
            else
                this.remove();
        }

    }

    shwebui.Dialog = Dialog;

    class DialogManager {

        constructor() {
            var self = this;
            this._dlgs = {};
            this._events = {};
            this._ievents = {};
        }

        start() {
            var self = this;
            shwebui.ajax({url: 'dialog/list', success: function(data) {
                for (var dl of data)
                    self.openDialog(dl);
            }});
        }

        getDialogById(dialogId) {
            var r = this._dlgs[dialogId];
            if (r)
                return r.widget;
        }

        getOpenDialogIds() {
            var r = [];
            for (var dlgid in this._dlgs)
                r.push(dlgid);
            return r;
        }

        getEvent(dialogId, eventname) {
            var self = this;
            var demap = this._events[dialogId];
            if (!demap)
                this._events[dialogId] = demap = {};
            var ev = demap[eventname];
            if (!ev)
                demap[eventname] = ev = new clove.Event();
            var iev = this._ievents[eventname];
            if (!iev) {
                this._ievents[eventname] = iev = shwebui.eventfetcher.getEvent("dialog/"+eventname);
                iev.addHandler(function(e){
                    var odemap = self._events[e.data.dialogId];
                    if (odemap) {
                        var oev = odemap[eventname];
                        if (oev)
                            oev.trigger(e.data);
                    }
                });
            }
            return ev;
        }

        openDialog(cfg) {
            cfg.closeable = (cfg.closeable === undefined) ? true : cfg.closeable;
            var dlgid = cfg.dialogId;
            var dlg = this._dlgs[dlgid];
            if (!dlg) {
                var dlg = clove.Dialog.show(cfg, {modal: true, panelClickClosesPopup: cfg.closeable});
                dlg.widget.OnDestroyed.addHandler(function(){
                    shwebui.ajax({url: 'dialog/closed_in_browser', data: {dialogId: dlgid}});
                });
                this._dlgs[dlgid] = dlg;
            }
            return dlg;
        }

        closeDialog(idx) {
            var dlg = this._dlgs[idx];
            if (dlg) {
                delete this._dlgs[idx];
                try {
                    dlg.widget.remove();
                }
                catch(e) {
                }
            }
            // cleanup this._events and friends
            delete this._events[idx];
            var alleventnames = new Set();
            for (var ixevents in this._events) {
                var xevents = this._events[ixevents];
                for (var nevent in xevents)
                    alleventnames.add(nevent);
            }
            for (var kievent in this._ievents) {
                if (!alleventnames.has(kievent))
                    delete this._ievents[kievent];
            }
            return dlg;
        }

        updateDialog(data) {
            var webts = data.webts;
            var dl = this._dlgs[data.dialogId];
            if (dl)
                dl.widget._setProperty(data.key, data.value, webts);
        }

    }

    shwebui.dialogs = new DialogManager();

})();
