/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class ExceptionDialog extends shwebui.Dialog {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                title: shwebui.strings.ExceptionDialogAProblemOccurred,
                rightHeadButtons: [{view: 'Button', label: shwebui.strings.ShowLogAndContinue, name: 'btnShowLogAndContinue'},
                                   {view: 'Button', label: shwebui.strings.ShowDetails, name: 'btnShowDetails', checkable: true},
                                   {view: 'Button', label: shwebui.strings.Retry, name: 'btnRetry'},
                                   {view: 'Button', label: shwebui.strings.Continue, name: 'btnContinue'},
                                   {view: 'Button', label: shwebui.strings.Cancel, name: 'btnCancel'},
                                   {view: 'Button', label: shwebui.strings.ReloadShallot, visibility: clove.InvisibleCollapsed, name: 'btnReloadShallot'},
                                   {view: 'Button', label: shwebui.strings.ShowLog, name: 'btnShowLog'},],
                innerBody: [{rows: [
                                {view: 'Label', name: 'lblMessage', styleClass: 'shwebui_exceptiondlg_msg'},
                                {view: 'ScrollView', height: '100pt', body: {view: 'Label', name: 'lblDetails', styleClass: 'shwebui_exceptiondlg_details'}},
                                {view: 'Label', label: shwebui.strings.ExceptionDialogLookAtTheLog, name: 'lblFooter', styleClass: 'shwebui_exceptiondlg_footer'},
                            ], styleClass: 'shwebui_dlg_innerbody_withmargin'}],
            }), domnode);
            this.addStyleClass('shwebui_exceptiondlg');
            this.declareProperty('mayClose', false);
            this.declareProperty('mayRetry', false);
            this.declareProperty('mayCancel', false);
            this.declareProperty('showLoglabelAndDetails', false);
            this.declareProperty('error2');
            this.declareProperty('details');
            this.declareProperty('noIcon', false);
        }

        doinit() {
            var self = this;
            super.doinit();
            if (!this.config.noIcon)
                this.setTitleicon(clove.Icon.byUrl(this.dlgurl("exception/icon")));
            this.btnShowLogAndContinue = this.containingNameScope().getByName('btnShowLogAndContinue');
            this.btnShowDetails = this.containingNameScope().getByName('btnShowDetails');
            this.btnRetry = this.containingNameScope().getByName('btnRetry');
            this.btnContinue = this.containingNameScope().getByName('btnContinue');
            this.btnCancel = this.containingNameScope().getByName('btnCancel');
            this.btnReloadShallot = this.containingNameScope().getByName('btnReloadShallot');
            this.btnShowLog = this.containingNameScope().getByName('btnShowLog');
            this.lblMessage = this.containingNameScope().getByName('lblMessage');
            this.lblDetails = this.containingNameScope().getByName('lblDetails');
            this.lblFooter = this.containingNameScope().getByName('lblFooter');
            this.btnShowLogAndContinue.OnClicked.addHandler(function(){
                self.acceptDialog({continueExec: true});
                shwebui.app.showLogView(undefined, undefined, 0);
            });
            this.btnRetry.OnClicked.addHandler(function(){
                self.acceptDialog({retry: true});
            });
            this.btnContinue.OnClicked.addHandler(function(){
                self.acceptDialog({continueExec: true});
            });
            this.btnCancel.OnClicked.addHandler(function(){
                self.acceptDialog({cancel: true});
            });
            this.btnShowDetails.OnClicked.addHandler(function(){
                self.btnShowDetails.setChecked(!self.btnShowDetails.checked());
                self._refresh();
            });
            this.btnReloadShallot.OnClicked.addHandler(function(){
                location.reload();
            });
            this.btnShowLog.OnClicked.addHandler(function(){
                shwebui.app.showLogView(undefined, undefined, 0);
            });
            this.lblMessage.setLabel(this.config.error2);
            this.lblDetails.setLabel(this.config.details);
            this.btnContinue.setVisibility(this.config.mayClose ? clove.Visible : clove.InvisibleCollapsed);
            this.btnShowLog.setVisibility(!this.config.mayClose ? clove.Visible : clove.InvisibleCollapsed);
            this.btnRetry.setVisibility(this.config.mayRetry ? clove.Visible : clove.InvisibleCollapsed);
            this.btnCancel.setVisibility(this.config.mayCancel ? clove.Visible : clove.InvisibleCollapsed);
            this.btnShowDetails.setVisibility(this.config.showLoglabelAndDetails ? clove.Visible : clove.InvisibleCollapsed);
            this._refresh();
            if (!(this.config.mayClose || this.config.mayRetry || this.config.mayCancel)) {
                this.btnReloadShallot.setVisibility(clove.Visible);
                // admittedly, this is not exactly ui :)
                shwebui.ajax({url: 'dispatcher/drop_worker'});
                shwebui.app.stop();
            }
        }

        _refresh() {
            var checked = this.btnShowDetails.checked();
            this.btnShowLog.setVisibility((checked&&!this.config.mayClose) ? clove.Visible : clove.InvisibleCollapsed);
            this.btnShowLogAndContinue.setVisibility((checked&&this.config.mayClose) ? clove.Visible : clove.InvisibleCollapsed);
            this.lblFooter.setVisibility(checked ? clove.Visible : clove.Invisible);
            this.lblDetails.setVisibility(checked ? clove.Visible : clove.InvisibleCollapsed);
        }

    }

    shwebui.ExceptionDialog = ExceptionDialog;

})();
