/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class Jumpbar extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'horizontal',
                children: [
                    {cols: [
                        {view: 'EditBox', name: 'pnlTextEditbox', horizontalStretchAffinity: 1},
                        {view: 'Button', label: clove.symbols.CheckMark, name: 'btnOk'},
                        {view: 'Button', label: clove.symbols.CheckX, name: 'btnCancel'},
                    ], name: 'pnlText'},
                    {cols: [], name: 'pnlButtons', horizontalStretchAffinity: 1, styleClass: 'shwebui_jumpbar_buttonpanel'},
                ],
                horizontalStretchAffinity: 1,
            }), domnode);
            this.declareProperty('isTextMode', false);
            this.declareProperty('eurl');
            this.OnCurrentEurlChanged = new clove.Event();
        }

        doinit() {
            var self = this;
            super.doinit();
            var pnlButtons = this.getByName('pnlButtons');
            var btnOk = this.getByName('btnOk');
            var btnCancel = this.getByName('btnCancel');
            var pnlTextEditbox = this.getByName('pnlTextEditbox');
            $(pnlButtons.topnode).click(function(e){
                if (e.target === pnlButtons.contentnode)
                    self.setIsTextMode(true);
            });
            function _cancel() {
                self.setIsTextMode(false);
            };
            function _ok() {
                self.OnCurrentEurlChanged.trigger({eurl: pnlTextEditbox.text()});
                self.setIsTextMode(false);
            };
            btnCancel.OnClicked.addHandler(_cancel);
            btnOk.OnClicked.addHandler(_ok);
            $(pnlTextEditbox.contentnode).find('input').keydown(function(e){
                if (e.which === 13)
                    _ok();
                else if (e.which === 27)
                    _cancel();
            });
        }

        isTextMode_setter(v) {
            var pnlText = this.getByName('pnlText');
            var pnlTextEditbox = this.getByName('pnlTextEditbox');
            var pnlButtons = this.getByName('pnlButtons');
            if (v) {
                pnlText.setVisibility(clove.Visible);
                pnlText.setWidth(undefined);
                pnlButtons.setVisibility(clove.InvisibleCollapsed);
                pnlButtons.setWidth('0');
                function _focusjumpbar() {
                    pnlTextEditbox.focus();
                    pnlTextEditbox.setTextSelection(0, pnlTextEditbox.text().length);
                    if (!pnlTextEditbox.hasFocus())
                        setTimeout(_focusjumpbar, 50);
                }
                _focusjumpbar();
            }
            else {
                pnlText.setVisibility(clove.InvisibleCollapsed);
                pnlText.setWidth('0');
                pnlButtons.setVisibility(clove.Visible);
                pnlButtons.setWidth(undefined);
            }
            this.relayout();
        }

        eurl_getter() {
            if (this.config.isTextMode)
                return this.pnlTextEditbox.text();
            else
                return this.config.eurl || "";
        }

        createSegmentbuttonclickHandler(eurl) {
            var self = this;
            return function() {
                self.OnCurrentEurlChanged.trigger({eurl: eurl});
            };
        }

        eurl_setter(v) {
            var pnlTextEditbox = this.getByName('pnlTextEditbox');
            var pnlButtons = this.getByName('pnlButtons');
            if (v && v.length > 0){
                pnlTextEditbox.setText(v[0].eurl);
                var btns = [];
                for (var es of v)
                    btns.splice(0, 0, {view: 'Button', label: es.displayName, OnClicked: this.createSegmentbuttonclickHandler(es.eurl),
                                       styleClass: 'shwebui_jumpbar_segmentbutton'});
                pnlButtons.setChildren(btns);
            }
            else {
                pnlTextEditbox.setText("");
            }
        }

    }

    shwebui.Jumpbar = Jumpbar;

})();
