/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class LogViewDialog extends clove.NameScope(shwebui.Dialog) {

        constructor(config, domnode) {
            config = config || {};
            super(clove.utils.applyDefaultsToConfig(config, {
                title: shwebui.strings.LogMessages,
                leftHeadButtons: [{view: 'PopupMenuButton', name: 'btnVerbosity'}],
                rightHeadButtons: [{view: 'Button', label: shwebui.strings.OpenAsText, name: 'btnOpenAsText'},
                                   {view: 'Button', label: shwebui.strings.Close, name: 'btnClose'}],
                innerBody: [{view: 'ScrollView', body: {view: 'DataView', horizontalStretchAffinity: 1, name: 'datvLogMessages', hideExpanders: true, gridVisible: false, styleClass: 'shwebui_logviewdlg_logmessagesview'}, styleClass: 'shwebui_dlg_innerbody_withmargin'},],
            }), domnode);
            var self = this;
            this.lverbosities = [{label: shwebui.strings.LogMessagesVerbosityVeryVerbose, name: shwebui.logger.DEBUG, stext: shwebui.strings.LogMessagesVerbosityVeryVerboseS},
                                 {label: shwebui.strings.LogMessagesVerbosityInfo, name: shwebui.logger.INFO, stext: shwebui.strings.LogMessagesVerbosityInfoS},
                                 {label: shwebui.strings.LogMessagesVerbosityWarning, name: shwebui.logger.WARNING, stext: shwebui.strings.LogMessagesVerbosityWarningS},
                                 {label: shwebui.strings.LogMessagesVerbosityError, name: shwebui.logger.ERROR, stext: shwebui.strings.LogMessagesVerbosityErrorS}];
            this.declareProperty('minseverity', shwebui.logger.DEBUG);
            this.addStyleClass('shwebui_logviewdlg');
            this._msgs = [];
            this._ds = undefined;
            this._nextDsMessageIdx = undefined;
            this._fastfetchcoins = 0;
            setTimeout(function(){self.refresh();});
        }

        minseverity_setter(v) {
            var self = this;
            setTimeout(function(){
                self._ds = undefined;
                self._refreshui();
            }); // not called directly, because this is called early, and ._refreshui() uses some stuff not available then
        }

        doinit() {
            var self = this;
            super.doinit();
            this.setBusy(true);
            var btnVerbosity = this.containingNameScope().getByName('btnVerbosity');
            var btnOpenAsText = this.containingNameScope().getByName('btnOpenAsText');
            var btnClose = this.containingNameScope().getByName('btnClose');
            var datvLogMessages = this.containingNameScope().getByName('datvLogMessages');
            btnVerbosity.setActions(this.lverbosities);
            btnVerbosity.OnActionTriggered.addHandler(function(e){
                var nverb = parseInt(e.action);
                self.setMinseverity(nverb);
            });
            btnOpenAsText.OnClicked.addHandler(function(){
                window.open(shwebui.actioncallUrl('log/as_text'));
            });
            btnClose.OnClicked.addHandler(function(){
                self.close();
            });
            var _datvLogMessages_table = datvLogMessages.table;
            datvLogMessages.setCellGenerator(function(widg, ptr){
                if (ptr.icol == 1)
                    return {rows: [{view: 'Label', styleClass: 'shwebui_logviewdlg_message_source'}, {view: 'Label', styleClass: 'shwebui_logviewdlg_message_message'}], update: function(wcc, ptr, ccnt){
                        var chlds = wcc.childrenWidgets();
                        if (!ccnt) //happens early during creation of row
                            return;
                        chlds[0].setLabel(ccnt[0]); // source
                        chlds[1].setLabel(ccnt[1]); // message
                        var jrow = $(_datvLogMessages_table.rows[ptr.irow+1]);
                        if (jrow) // just be sure
                            jrow.addClass(ccnt[2]); // styleclass
                    }};
            });
        }

        _refreshui() {
            var btnVerbosity = this.containingNameScope().getByName('btnVerbosity');
            var lv = {stext: "?"};
            for (var olv of this.lverbosities) {
                if (String(olv.name) === String(this.config.minseverity)) {
                    lv = olv;
                    break;
                }
            }
            btnVerbosity.setLabel(shwebui.strings.LogMessagesVerbosityChange.replace("%1", lv.stext));
            var rds;
            var newds;
            if (this._ds)
                rds = this._ds.root();
            else {
                this._nextDsMessageIdx = 0;
                newds = this._ds = new clove.NativeDatasource();
                rds = newds.root();
                rds.addColumn([]);
                rds.addColumn([]);
                rds.addColumn([]);
            }
            var ms = this._msgs;
            for (var icd=this._nextDsMessageIdx; icd<ms.length; icd++) {
                var cd = ms[icd];
                if (cd.severity < this.config.minseverity)
                    continue;
                var siseverity;
                var msgClass;
                if (cd.severity <= shwebui.logger.DEBUG) {
                    siseverity = "";
                    msgClass = "shwebui_logviewdlg_message_severitydebug";
                }
                else if (cd.severity <= shwebui.logger.INFO) {
                    siseverity = "+";
                    msgClass = "shwebui_logviewdlg_message_severityinfo";
                }
                else if (cd.severity <= shwebui.logger.WARNING) {
                    siseverity = "++";
                    msgClass = "shwebui_logviewdlg_message_severitywarning";
                }
                else if (cd.severity <= shwebui.logger.ERROR) {
                    siseverity = "+++";
                    msgClass = "shwebui_logviewdlg_message_severityerror";
                }
                else {
                    siseverity = "?";
                    msgClass = "shwebui_logviewdlg_message_severityunknown";
                }
                rds.insertRow(0, [new Date(cd.time).toLocaleTimeString(), [cd.source, cd.message, msgClass], siseverity]);
            }
            this._nextDsMessageIdx = icd;
            if (newds)
                this.containingNameScope().getByName('datvLogMessages').setDatasource(newds);
        }

        refresh() {
            var self = this;
            var ms = this._msgs;
            this.ajax({url: 'logview/get_messages', data: {fromidx: this._msgs.length}, success: function(data){
                var newstuff = false;
                for (var mdata of data) {
                    if (mdata.idx === ms.length) {
                        ms.push(mdata);
                        newstuff = true;
                    }
                }
                var nxtimeout = 400;
                if (newstuff) {
                    self._fastfetchcoins = 15;
                    self._refreshui();
                }
                else if (self._fastfetchcoins > 0)
                    self._fastfetchcoins--;
                else
                    nxtimeout = 1500;
                self.setBusy(false);
                setTimeout(function(){ self.refresh(); }, nxtimeout); //this loop automatically breaks when the dialog disappears (and ajax doesnt success anymore)
            }});
        }

    }

    shwebui.LogViewDialog = LogViewDialog;

})();
