/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class MainWindow extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [
                    {view: 'ResizeSplitter', orientation: 'vertical', body: [
                        {view: 'MainView', head2: String.fromCharCode(160), headControl: {view: 'shwebui.Jumpbar'},
                            bodyLeftViewActionLabel: shwebui.strings.ShowDirTree, bodyRightViewActionLabel: shwebui.strings.ShowFiles, splitterPosition: 0.4, name: 'mainview',
                            bodyLeft: {view: 'ScrollView', body: {view: 'TreeView', name: 'dirtree', styleClass: 'shwebui_dirtree', allowSelection: true, granularity: 'row', showOnlyFirstColumn: true, horizontalStretchAffinity: 1, verticalStretchAffinity: 1}, horizontalStretchAffinity: 1, verticalStretchAffinity: 1},
                            bodyRight: {view: 'ResizeSplitter', name: 'filepanelsplitter'}},
                        {view: 'shwebui.DetailPanel', name: 'detailpanel'},
                    ], sizeFractions: [0.8, 0.2]},
                    {view: 'shwebui.StatusBar', name: 'statusbar'},
                ],
            }), domnode);
            this._currentfileview = undefined;
            this.OnCurrentFileviewChanged = new clove.Event();
            this.OnNodeActivated = new clove.Event();
            this.OnFileviewSelectionChanged = new clove.Event();
            this.OnFileviewCheckedNodesChanged = new clove.Event();
        }

        setWindowTitle(v) {
            document.title = v;
        }

        doinit() {
            super.doinit();
            this.mainview = this.getByName("mainview");
            this.detailpanel = this.getByName("detailpanel");
            this.statusbar = this.getByName("statusbar");
            this.dirtree = this.getByName("dirtree");
            this.jumpbar = this.mainview.headControlWidget();
            this.filepanelsplitter = this.getByName("filepanelsplitter");
        }

        fileviews() {
            var r = [];
            for(var x of this.filepanelsplitter.bodyWidgets())
                r.push(x.bodyWidget());
            return r;
        }

        _fileviewToIndex(fileview) {
            var fw = this.fileviews();
            for(var i=0; i<fw.length; i++) {
                if (fw[i] == fileview)
                    return i;
            }
            return -1;
        }

        fileviewIdToIndex(id) {
            var fw = this.fileviews();
            for(var i=0; i<fw.length; i++) {
                if (fw[i].config.list_id == id)
                    return i;
            }
            return -1;
        }

        insertFileview(i, fileviewconfig) {
            var self = this;
            var bsw = this.filepanelsplitter.insertBodyWidget(i, {view: 'ScrollView',
                body: clove.utils.applyDefaultsToConfig(/* reversed, off-label usage :) */{
                    view: 'shwebui.ListFileView',
                    styleClass: 'shwebui_fileview',
                    horizontalStretchAffinity: 1,
                    verticalStretchAffinity: 1,
                    list_id: fileviewconfig.id
            }, fileviewconfig)});
            var bw = bsw.bodyWidget();
            $(bw.topnode).on("focusin", function(e){
                if (self._currentfileview != bw) {
                    self._currentfileview = bw;
                    var idx = self._fileviewToIndex(bw);
                    self.OnCurrentFileviewChanged.trigger({widget: bw, index: idx});
                }
            });
            bw.OnNodeActivated.addHandler(function(e){
                var eurl = ShallotApplication.getFileItemForValuePointer(e.node).eurl;
                var idx = self._fileviewToIndex(bw);
                self.OnNodeActivated.trigger({widget: bw, index: idx, eurls: [eurl]});
            }, bw);
            bw.OnSelectionChanged.addHandler(function(e){
                var eurl;
                var selection = bw.selection();
                if (selection)
                    eurl = ShallotApplication.getFileItemForValuePointer(selection).eurl;
                var idx = self._fileviewToIndex(bw);
                self.OnFileviewSelectionChanged.trigger({widget: bw, index: idx, eurl: eurl});
            }, bw);
            bw.OnCheckedNodesChanged.addHandler(function(e){
                var eurls = [];
                for(var x of bw.checkedCells())
                    eurls.push(ShallotApplication.getFileItemForValuePointer(x).eurl);
                var idx = self._fileviewToIndex(bw);
                self.OnFileviewCheckedNodesChanged.trigger({widget: bw, index: idx, eurls: eurls});
            }, bw);
            bw.setDatasource(shwebui.data.fileViewDatasource(fileviewconfig.id));
        }

        removeFileview(i) {
            var bw = this.filepanelsplitter.bodyWidgets()[i];
            if (bw)
                bw.remove();
        }

        reloadFileview(i) {
            var bw = this.fileviews()[i];
            if (bw)
                bw.config.datasource.async_pull();
        }

        infoPanels() {
            return this.statusbar.infoPanels();
        }

        insertInfoPanel(i, infopanelconfig) {
            return this.statusbar.insertInfoPanel(i, infopanelconfig);
        }

        removeInfoPanel(i) {
            this.statusbar.removeInfoPanel(i);
        }

        refreshInfoPanel(i, infopanelconfig) {
            var bw = this.infoPanels()[i];
            for(var kcfg in infopanelconfig)
                bw.setProperty(kcfg, infopanelconfig[kcfg]);
        }

        actionDialogs() {
            return this.statusbar.actionDialogs();
        }

        insertActionDialog(i, actiondialogconfig) {
            return this.statusbar.insertActionDialog(i, actiondialogconfig);
        }

        removeActionDialog(i) {
            this.statusbar.removeActionDialog(i);
        }

        refreshActionDialog(i, actiondialogconfig) {
            this.statusbar.refreshActionDialog(i, actiondialogconfig);
        }

        showActionDialog(i) {
            this.statusbar.showActionDialog(i);
        }

    }

    shwebui.MainWindow = MainWindow;

    class ShallotApplication {

        static getFileItemForValuePointer(ptr) {
            var irow = ptr.irow;
            var parent = ptr.parent();
            var ds = ptr.datasource;
            var rptr = ds.valuePointer(irow, 0, parent);
            var displayName = ds.getValue(rptr);
            var nodetype = ds.getValue(ds.valuePointer(irow, 1, parent));
            var isHidden = ds.getValue(ds.valuePointer(irow, 2, parent));
            var eurl = ds.getValue(ds.valuePointer(irow, 3, parent));
            return {displayName: displayName, eurl: eurl, nodetype: nodetype, isHidden: isHidden};
        }

        constructor() {
            var self = this;
            this.mainwindow = clove.build({view: 'shwebui.MainWindow'});
            this.mainwindow.dirtree.setDatasource(this.dsdirtree = shwebui.data.dirTreeDatasource());
            this.mainwindow.dirtree.OnSelectionChanged.addHandler(function(e){
                var s = e.widget.selection();
                if (s) {
                    var eurl = ShallotApplication.getFileItemForValuePointer(s).eurl;
                    self.requestGotoDirectory(eurl);
                }
            });
            this.mainwindow.jumpbar.OnCurrentEurlChanged.addHandler(function(e){
                self.requestGotoDirectory(e.eurl);
            });
            shwebui.config.consume('windowTitle', function(v){
                self.mainwindow.setWindowTitle(v);
                document.title = v;
            });
            shwebui.config.consume('jumpbarMode', function(v){
                self.mainwindow.jumpbar.setIsTextMode(v == "1");
            });
            shwebui.config.consume('detailPanelVisible', function(v){
                self._detailPanelVisible = (v == "1");
                self.mainwindow.detailpanel.setVisibility(self._detailPanelVisible ? clove.Visible : clove.InvisibleCollapsed);
            });
            shwebui.config.consume('currentEurl', function(v){
                var ocurr = JSON.parse(v);
                var ocurreurls = ocurr.npath;
                var teurl = ocurreurls[0].eurl;
                if (teurl === self._lastCurrentEurl)
                    return;
                self._lastCurrentEurl = teurl;
                self._ocurreurls = ocurreurls;
                var dtds = self.mainwindow.dirtree.datasource();
                function _treeJumpTo(lrlst, lptr) { //TODO # respect "tree follows filelist"
                    if (self._ocurreurls != ocurreurls)
                        return;
                    var _done = false;
                    function _jumpnext(e) {
                        if (_done)
                            return;
                        if (e.parent === lptr) {
                            _done = true;
                            if (self._ocurreurls == ocurreurls) {
                                var nxt = lrlst.pop();
                                var ircnt = dtds.rowCount(lptr);
                                for (var ir=0; ir<ircnt; ir++) {
                                    var irptr = dtds.valuePointer(ir, 0, lptr);
                                    var irnode = shwebui.ShallotApplication.getFileItemForValuePointer(irptr);
                                    if (irnode.eurl == nxt.eurl) {
                                        if (lrlst.length) {
                                            self.mainwindow.dirtree.expandCell(irptr);
                                            _treeJumpTo(lrlst, irptr);
                                        }
                                        else
                                            self.mainwindow.dirtree.selectCell(irptr);
                                        break;
                                    }
                                }
                            }
                            dtds.OnDataArrived.removeHandler(_jumpnext);
                        }
                    }
                    dtds.OnDataArrived.addHandler(_jumpnext);
                    setTimeout(function(){
                        dtds.OnDataArrived.removeHandler(_jumpnext);
                    }, 120000);  // if expanding failed somewhere for some reason, this avoids leaks
                    dtds.async_pull(lptr);
                }
                var currentEurl = ocurreurls[0].eurl;
                self.mainwindow.jumpbar.setEurl(ocurr.epath);
                if (self._detailPanelVisible)
                    self.mainwindow.detailpanel.refresh();
                ocurreurls.pop();
                _treeJumpTo(ocurreurls);
            });
            this.mainwindow.detailpanel.OnLinkClicked.addHandler(function(e) {
                shwebui.ajax({url: 'detailpanel/link_clicked', data: {idx: e.idx, linktarget: e.linktarget, webts_lastupdate: e.webts_lastupdate}});
            });
            this.mainwindow.mainview.setActions(shwebui.data.toolbarActionsDatasource());
            this.mainwindow.mainview.OnActionTriggered.addHandler(function(e){
                shwebui.ajax({url: 'action/execute', data: {actionpath: e.action}});
            });
            this.mainwindow.statusbar.OnUserFeedbackAnswered.addHandler(function(e){
                shwebui.ajax({url: 'action/answer_userfeedback', data: {updates: JSON.stringify(e)}});
            });
            this.mainwindow.OnCurrentFileviewChanged.addHandler(function(e){
                shwebui.ajax({url: 'fileview/focus_current', data: {index: e.index}});
            });
            this.mainwindow.OnNodeActivated.addHandler(function(e){
                shwebui.ajax({url: 'fileview/node_activated', data: {index: e.index, eurls: JSON.stringify(e.eurls)}});
            });
            this.mainwindow.OnFileviewSelectionChanged.addHandler(function(e){
                shwebui.ajax({url: 'fileview/selection_changed', data: {index: e.index, eurl: e.eurl}});
            });
            this.mainwindow.OnFileviewCheckedNodesChanged.addHandler(function(e){//TODO# use POST instead (not only here?!); makes it possible to select many nodes
                shwebui.ajax({url: 'fileview/checkednodes_changed', data: {index: e.index, eurls: JSON.stringify(e.eurls)}});
            });
            function _refreshfileviews(e){
                shwebui.ajax({url: 'fileview/get', success: function(data){
                    var fvs = self.mainwindow.fileviews();
                    var ids2 = [];
                    for(let x of data)
                        ids2.push(x.id);
                    for(var i=fvs.length-1; i>=0; i--) {
                        var fv = fvs[i];
                        if (ids2.indexOf(fv.config.list_id) == -1)
                            self.mainwindow.removeFileview(i);
                    }
                    var ids1 = [];
                    for(let x of self.mainwindow.fileviews())
                        ids1.push(x.config.list_id);
                    for(var i=data.length-1; i>=0; i--) {
                        var dfw = data[i];
                        if (ids1.indexOf(dfw.id) == -1)
                            self.mainwindow.insertFileview(i, dfw);
                    }
                }});
            }
            function _refreshactionui_bydata(data){
                //info panels
                var pdata = data.infoPanels;
                var pfvs = self.mainwindow.infoPanels();
                var pids2 = [];
                var pid2data = {};
                for(let x of pdata) {
                    pids2.push(x.id);
                    pid2data[x.id] = x;
                } // all panels from fresh data are now in pids2 and pid2data
                for(var i=pfvs.length-1; i>=0; i--) {
                    var fv = pfvs[i];
                    if (pids2.indexOf(fv.config.id) == -1)
                        self.mainwindow.removeInfoPanel(i);
                } // orphaned panels removed from ui
                var pids1 = [];
                var panels = self.mainwindow.infoPanels();
                for(var ipanel=0; ipanel<panels.length; ipanel++) {
                    var panel = panels[ipanel];
                    pids1.push(panel.config.id);
                    self.mainwindow.refreshInfoPanel(ipanel, pid2data[panel.config.id]);
                } // all existing ui panels are now in pids1
                for(var i=pdata.length-1; i>=0; i--) {
                    var dfw = pdata[i];
                    if (pids1.indexOf(dfw.id) == -1)
                        self.mainwindow.insertInfoPanel(i, dfw);
                } // if fresh data had some new panels inbetween, it's now inserted in ui
                //actions dialogs
                var ddata = data.dialogs;
                var dfvs = self.mainwindow.actionDialogs();
                var dids2 = [];
                var did2data = {};
                for(let x of ddata) {
                    dids2.push(x.id);
                    did2data[x.id] = x;
                } // all dialogs from fresh data are now in dids2 and did2data
                for(var i=dfvs.length-1; i>=0; i--) {
                    var fv = dfvs[i];
                    if (dids2.indexOf(fv.id) == -1)
                        self.mainwindow.removeActionDialog(i);
                } // orphaned dialogs removed from ui
                var dids1 = [];
                var dialogs = self.mainwindow.actionDialogs();
                for(var ipanel=0; ipanel<dialogs.length; ipanel++) {
                    var panel = dialogs[ipanel];
                    dids1.push(panel.id);
                    self.mainwindow.refreshActionDialog(ipanel, did2data[panel.id]);
                } // all existing ui dialogs are now in dids1
                var opendlg = -1; // we store if we open a fresh dialog now
                var opendlg_prio = -1;
                for(var i=ddata.length-1; i>=0; i--) {
                    var dfw = ddata[i];
                    if (dids1.indexOf(dfw.id) == -1) {
                        self.mainwindow.insertActionDialog(i, dfw);
                        var prio = dfw.forceForeground ? 1 : 0;
                        if (prio > opendlg_prio) {
                            opendlg = i;
                            opendlg_prio = prio;
                        }
                    }
                } // if fresh data had some new dialogs inbetween, it's now inserted in ui
                if (opendlg > -1)
                    self.mainwindow.showActionDialog(opendlg);
            }
            function _refreshactionui(e){
                shwebui.ajax({url: 'action/get_ui', success: function(data){
                    _refreshactionui_bydata(data);
                }});
            }
            shwebui.eventfetcher.getEvent("fileview/items_changed").addHandler(function(e){
                var lfw = self.mainwindow.fileviews();
                var id = Number(e.data);
                for(var ifw=0;ifw<lfw.length;ifw++) {
                    var fw = lfw[ifw];
                    if (fw.config.id == id) {
                        if (self._fileviewsitemschanged_idxs === undefined) {
                            self._fileviewsitemschanged_idxs = new Set([ifw]);
                            setTimeout(function(){
                                for(var fwidx of self._fileviewsitemschanged_idxs.values())
                                    self.mainwindow.reloadFileview(fwidx);
                                self._fileviewsitemschanged_idxs = undefined;
                            });
                        }
                        else
                            self._fileviewsitemschanged_idxs.add(ifw);
                        break;
                    }
                }
            }, this.mainwindow);
            shwebui.eventfetcher.getEvent("fileview/number_changed").addHandler(_refreshfileviews);
            _refreshfileviews();
            shwebui.eventfetcher.getEvent("fileview/selection_changed").addHandler(function(e){
                var fileviewidx = self.mainwindow.fileviewIdToIndex(e.data.id);
                if (fileviewidx > -1) {
                    var fileview = self.mainwindow.fileviews()[fileviewidx];
                    fileview.setSelectedNode(e.data.selectedNode);
                    fileview.setCheckedNodes(e.data.checkedNodes);
                }
            });
            shwebui.eventfetcher.getEvent("action/ui_changed").addHandler(_refreshactionui);
            shwebui.eventfetcher.getEvent("action/force_foreground").addHandler(function(e){
                var actiondlgid = parseInt(e.data);
                var dlgi = self.mainwindow.statusbar.getActionDialogIndexForActionDialogId(actiondlgid);
                if (dlgi !== undefined)
                    self.mainwindow.showActionDialog(dlgi);
            });
            shwebui.eventfetcher.getEvent("dialog/opened").addHandler(function(e){
                shwebui.dialogs.openDialog(e.data);
            });
            shwebui.eventfetcher.getEvent("dialog/closed").addHandler(function(e){
                shwebui.dialogs.closeDialog(Number(e.data));
            });
            shwebui.eventfetcher.getEvent("dialog/changed").addHandler(function(e){
                shwebui.dialogs.updateDialog(e.data);
            });
            window.addEventListener('beforeunload', function (e) {
                if (shwebui.config.preventCloseMessage) {
                    e.preventDefault();
                    return e.returnValue = shwebui.config.preventCloseMessage;
                }
            });
            _refreshactionui();
            shwebui.eventfetcher.start();
            shwebui.dialogs.start();
        }

        requestGotoDirectory(eurl) {
            shwebui.ajax({url: 'mainwindow/set_current_directory', data: {eurl: eurl}});
        }

        showLogView(headtext, subheadtxt, minseverity) {
            shwebui.ajax({url: 'mainwindow/show_logview', data: {headtext: headtext, subheadtxt: subheadtxt, minseverity: minseverity}});
        }

        stop() {
            shwebui.errorhandler = function() {};
            shwebui.ajax = function() {};
            shwebui.config.preventCloseMessage = undefined;
        }

    }

    shwebui.ShallotApplication = ShallotApplication;

    clove.populateUI(function(){
        shwebui.app = new ShallotApplication();
    });

})();

//TODO #zz menus: sometimes the submenu expander arrow is missing
//TODO # menus: they jump to top/left corner sometimes on open
//TODO # action keyboard shortcuts
//TODO # show action accelerators in menus (and also in main menu on Alt)
//TODO # poetschhttp2: integrate recoll
//TODO # from poetschhttp: how to mass-set parsley metadata (e.g. tags)?
//TODO #zz clipboard operations in web should be per session
//TODO ### .h: TO#DO spaces_between_methods missing_docu fix_docu_formatting sort_includes check_methods_proper_name check_membervars_proper_name | .cpp: TO#DO sort_includes basic_code_formatting head_defines_distances foreach
//TODO # allow (via commandline?!) to use another finetuning-configuration file (the writeable one)
//TODO # closing a shallot tab and undoing close (in firefox) brings a ui crash
