/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class ManageBookmarksDialog extends shwebui.Dialog {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                title: shwebui.strings.ManageBookmarks,
                headtext1: shwebui.strings.ManageYourBookmarks,
                leftHeadButtons: [{view: 'PopupMenuButton', label: shwebui.strings.New, name: 'btnNew', styleClass: 'shwebui_dlgbutton_small'},
                                  {view: 'Button', label: shwebui.strings.Delete, name: 'btnDelete', styleClass: 'shwebui_dlgbutton_small'},
                                  {view: 'Button', label: shwebui.strings.Up, name: 'btnUp', styleClass: 'shwebui_dlgbutton_small'},
                                  {view: 'Button', label: shwebui.strings.Down, name: 'btnDown', styleClass: 'shwebui_dlgbutton_small'},
                                  {view: 'PopupMenuButton', label: shwebui.strings.Move, name: 'btnMove', styleClass: 'shwebui_dlgbutton_small'},],
                rightHeadButtons: [{view: 'Button', label: shwebui.strings.Close, name: 'btnClose'},],
                innerBody: [
                    {rows: [
                        {view: 'ScrollView', body: {view: 'DataView', name: 'datvBookmarks', width: '370pt', hstretch: 1, showOnlyFirstColumn: true, allowSelection: true, gridVisible: false, granularity: 'row', styleClass: 'shwebui_managebookmarksdlg_bookmarkstree'}, height: '250pt'},
                        {view: 'Form', name: 'formEdit', sections: [
                            {view: 'EditBox', name: 'edtLabel', formSectionLabel: shwebui.strings.Label, hstretch: 1},
                            {view: 'EditBox', name: 'edtLocation', formSectionLabel: shwebui.strings.Location, hstretch: 1},
                            {view: 'Button', label: shwebui.strings.Store, name: 'btnStore', strictHorizontalSizing: true, styleClass: 'shwebui_dlgbutton shwebui_dlgbutton_small shwebui_managebookmarksdlg_storebtn'},
                        ], styleClass: 'shwebui_managebookmarksdlg_editform'},
                    ]},
                ],
            }), domnode);
            this.addStyleClass('shwebui_managebookmarksdlg');
            this._bookmarks = [];
            this._selectedId = this._lastSelectedId = undefined;
            this._dummyFolders = {};
        }

        selectedFolder() {
            var datvBookmarks = this.containingNameScope().getByName('datvBookmarks');
            var ptr = datvBookmarks.selection();
            var folder = [];
            var foldernode;
            while (ptr) {
                var ds = ptr.datasource;
                var ptrparent = ptr.parent();
                var label = ds.getValue(ds.valuePointer(ptr.irow, 1, ptrparent));
                var level = ds.getValue(ds.valuePointer(ptr.irow, 3, ptrparent));
                if (level == 1) {
                    folder.splice(0, 0, label);
                    if (!foldernode)
                        foldernode = ptr;
                }
                ptr = ptrparent;
            }
            return {folder: folder, foldernode: foldernode};
        }

        doinit() {
            var self = this;
            super.doinit();
            var btnClose = this.containingNameScope().getByName('btnClose');
            var btnNew = this.containingNameScope().getByName('btnNew');
            var btnDelete = this.containingNameScope().getByName('btnDelete');
            var btnUp = this.containingNameScope().getByName('btnUp');
            var btnDown = this.containingNameScope().getByName('btnDown');
            var btnMove = this.containingNameScope().getByName('btnMove');
            var btnStore = this.containingNameScope().getByName('btnStore');
            var datvBookmarks = this.containingNameScope().getByName('datvBookmarks');
            var formEdit = this.containingNameScope().getByName('formEdit');
            var edtLabel = this.containingNameScope().getByName('edtLabel');
            var edtLocation = this.containingNameScope().getByName('edtLocation');
            var dsnew = new clove.NativeDatasource();
            dsnew.root().addRow([{label: shwebui.strings.NewBookmark, name: 'bookmark'}]);
            dsnew.root().addRow([{label: shwebui.strings.NewCollectionOfBookmarks, name: 'folder'}]);
            btnNew.setActions(dsnew);
            btnClose.OnClicked.addHandler(function(){
                self.close();
            });
            btnNew.OnActionTriggered.addHandler(function(e){
                if (e.action == 'bookmark') {
                    var busytoken = self.registerBusy();
                    var ajaxreq = self._ajaxreq_reload = self.ajax({url: 'managebookmarks/add_bookmark', data: {folder: JSON.stringify(self.selectedFolder().folder)}, success: function(data) {
                        // don't add more stuff here (or change _ajaxreq_reload to sth new)
                        if (ajaxreq == self._ajaxreq_reload)
                            self._reload();
                        self.unregisterBusy(busytoken);
                    }});
                }
                else /* if (e.action == 'folder') */ {
                    clove.inputDialog({question: shwebui.strings.EnterNewCollectionName}).addHandler(function(e) {
                        var fname = e.answer;
                        if (fname) {
                            var parentfolder = self.selectedFolder().folder;
                            var dfs = self._dummyFolders;
                            while (parentfolder.length > 0) {
                                var pf = parentfolder.splice(0, 1)[0];
                                if (!dfs[pf])
                                    dfs[pf] = {};
                                dfs = dfs[pf];
                            }
                            dfs[fname] = {};
                            self._reload();
                        }
                    });
                }
            });
            btnDelete.OnClicked.addHandler(function(){
                var ptr = datvBookmarks.selection();
                var ds = datvBookmarks.datasource();
                var ptrparent = ptr.parent();
                var label = ds.getValue(ds.valuePointer(ptr.irow, 1, ptrparent));
                var level = ds.getValue(ds.valuePointer(ptr.irow, 3, ptrparent));
                if (self._selectedId) {
                    // bookmark selected
                    clove.messageDialog({message: shwebui.strings.ConfirmDeleteBookmark.replace('%1', label), buttons: [shwebui.strings.Yes, shwebui.strings.No]}).addHandler(function(e){
                        if (e.button === 0) {
                            var busytoken = self.registerBusy();
                            var ajaxreq = self._ajaxreq_reload = self.ajax({url: 'managebookmarks/delete_bookmark', data: {id: self._selectedId}, success: function(data) {
                                // don't add more stuff here (or change _ajaxreq_reload to sth new)
                                if (ajaxreq == self._ajaxreq_reload)
                                    self._reload();
                                self.unregisterBusy(busytoken);
                            }});
                        }
                    });
                }
                else {
                    var sel = self.selectedFolder();
                    if (level == 1) {
                        // folder selected
                        var rcnt = ds.rowCount(ptr);
                        if (rcnt > 0)
                            clove.messageDialog({message: shwebui.strings.OnlyRemoveEmpty});
                        else {
                            clove.messageDialog({message: shwebui.strings.ConfirmDeleteBookmark.replace('%1', label), buttons: [shwebui.strings.Yes, shwebui.strings.No]}).addHandler(function(e){
                                if (e.button === 0) {
                                    var sfolder = sel.folder;
                                    var dfs = self._dummyFolders;
                                    while (dfs && sfolder.length > 1) {
                                        var pf = sfolder.splice(0, 1)[0];
                                        dfs = dfs[pf];
                                    }
                                    if (dfs)
                                        delete dfs[sfolder[0]];
                                    self._reload();
                                }
                            });
                        }
                    }
                }
            });
            btnDown.OnClicked.addHandler(function(){
                var busytoken = self.registerBusy();
                var ajaxreq = self._ajaxreq_reload = self.ajax({url: 'managebookmarks/move_bookmark_down', data: {id: self._selectedId}, success: function(data) {
                    // don't add more stuff here (or change _ajaxreq_reload to sth new)
                    if (ajaxreq == self._ajaxreq_reload)
                        self._reload();
                    self.unregisterBusy(busytoken);
                }});
            });
            btnUp.OnClicked.addHandler(function(){
                var busytoken = self.registerBusy();
                var ajaxreq = self._ajaxreq_reload = self.ajax({url: 'managebookmarks/move_bookmark_up', data: {id: self._selectedId}, success: function(data) {
                    // don't add more stuff here (or change _ajaxreq_reload to sth new)
                    if (ajaxreq == self._ajaxreq_reload)
                        self._reload();
                    self.unregisterBusy(busytoken);
                }});
            });
            btnMove.OnActionTriggered.addHandler(function(e){
                var busytoken = self.registerBusy();
                var ajaxreq = self._ajaxreq_reload = self.ajax({url: 'managebookmarks/move_bookmark_to_folder', data: {id: self._selectedId, destination: e.action}, success: function(data) {
                    // don't add more stuff here (or change _ajaxreq_reload to sth new)
                    if (ajaxreq == self._ajaxreq_reload)
                        self._reload();
                    self.unregisterBusy(busytoken);
                }});
            });
            btnStore.OnClicked.addHandler(function(){
                var busytoken = self.registerBusy();
                var label = edtLabel.text();
                var location = edtLocation.text();
                var ajaxreq = self._ajaxreq_reload = self.ajax({url: 'managebookmarks/store', data: {label: label, location: location, id: self._selectedId}, success: function(data) {
                    // don't add more stuff here (or change _ajaxreq_reload to sth new)
                    if (ajaxreq == self._ajaxreq_reload)
                        self._reload();
                    self.unregisterBusy(busytoken);
                }});
            });
            datvBookmarks.OnSelectionChanged.addHandler(function(e){
                if (!self.isAlive())
                    return; // dead
                var sel = datvBookmarks.selection();
                var bsel = sel !== undefined;
                var label = "";
                var location = "";
                var level = 0;
                if (sel) {
                    var ds = sel.datasource;
                    var selparent = sel.parent();
                    label = ds.getValue(ds.valuePointer(sel.irow, 1, selparent));
                    location = ds.getValue(ds.valuePointer(sel.irow, 2, selparent));
                    level = ds.getValue(ds.valuePointer(sel.irow, 3, selparent));
                    self._selectedId = self._lastSelectedId = ds.getValue(ds.valuePointer(sel.irow, 4, selparent));
                }
                else
                    self._selectedId = undefined;
                formEdit.setEnabled(level >= 2);
                btnDelete.setEnabled(level >= 1);
                btnUp.setEnabled(level >= 2);
                btnDown.setEnabled(level >= 2);
                btnMove.setEnabled(level >= 2);
                edtLabel.setText(label);
                edtLocation.setText(location);
            });
            datvBookmarks.OnSelectionChanged.trigger({});
            this._reload();
        }

        _reload() {
            var self = this;
            var busytoken = this.registerBusy();
            var btnMove = this.containingNameScope().getByName('btnMove');
            var datvBookmarks = this.containingNameScope().getByName('datvBookmarks');
            var ajaxreq = this._ajaxreq_get = this.ajax({url: 'managebookmarks/get', data: {}, success: function(data) {
                if (ajaxreq == self._ajaxreq_get) {
                    self._bookmarks = data;
                    var ds = new clove.NativeDatasource();
                    datvBookmarks.setDatasource(ds); //TODO# move below _reload_helper doesnt work due to some clove bug (temporarily move down and comment-out some expandCell calls, fix clove bug)
                    var rnode = self._bookmarksrootnode = ds.root().addRow([shwebui.strings.Bookmarks, shwebui.strings.Bookmarks, "", 0, undefined]);
                    rnode.setMetadata('icon', clove.Icon.byUrl(self.config.folderIcon));
                    var folderlist = [];
                    self._reload_helper(datvBookmarks, "/", rnode, data, self._dummyFolders, folderlist);
                    var afolderlist = [];
                    for (var f of folderlist)
                        afolderlist.push({name: f, label: f});
                    btnMove.setActions(clove.utils.arrayToDatasource(afolderlist));
                }
                self.unregisterBusy(busytoken);
            }});
        }

        _reload_helper(datvBookmarks, currpath, dsnode, data, dummyfolders, folderlist) {
            if (folderlist.indexOf(currpath) == -1)
                folderlist.push(currpath);
            dummyfolders = dummyfolders || {};
            var cfolders = [];
            for (var dn of data) {
                var xnode = dsnode.addRow([dn.treelabel, dn.label, dn.location, dn.isFolder?1:2, dn.id]);
                if (dn.isFolder) {
                    xnode.setMetadata('icon', clove.Icon.byUrl(this.config.folderIcon));
                    var ccurrpath = currpath + (currpath.endsWith("/") ? "" : "/") + dn.label;
                    this._reload_helper(datvBookmarks, ccurrpath, xnode, dn.children, dummyfolders[dn.label], folderlist);
                    cfolders.push(dn.label);
                }
                if (this._lastSelectedId !== undefined && this._lastSelectedId === dn.id)
                    datvBookmarks.selectCell(xnode.valuePointer());
            }
            for (var ddn in dummyfolders) {
                if (cfolders.indexOf(ddn) == -1) {
                    var xnode = dsnode.addRow([ddn, ddn, "", 1, undefined]);
                    xnode.setMetadata('icon', clove.Icon.byUrl(this.config.folderIcon));
                    var ccurrpath = currpath + (currpath.endsWith("/") ? "" : "/") + ddn;
                    this._reload_helper(datvBookmarks, ccurrpath, xnode, [], dummyfolders[ddn], folderlist);
                }
            }
            datvBookmarks.expandCell(dsnode.valuePointer());
        }

    }

    shwebui.ManageBookmarksDialog = ManageBookmarksDialog;

})();
