/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class ManageProfilesDialog extends shwebui.Dialog {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                title: shwebui.strings.ManageProfiles,
                headtext1: shwebui.strings.ManageProfilesHead,
                leftHeadButtons: [{label: shwebui.strings.ManageProfile, styleClass: 'shwebui_manageprofilesdlg_lblmanageprofile'}, {view: 'DropdownBox', name: 'lstProfiles'}],
                rightHeadButtons: [{view: 'Button', label: shwebui.strings.Close, name: 'btnClose'},],
                innerBody: [{view: 'ResizeSplitter', width: '450pt', height: '400pt', body: [
                    {rows: [
                        {label: shwebui.strings.EntriesInProfile, styleClass: 'shwebui_manageprofilesdlg_lblentries'},
                        {view: 'ScrollView', hstretch: 1, body: {view: 'DataView', hstretch: 1, name: 'lstNodes', hideExpanders: true, allowSelection: true, granularity: 'row', gridVisible: false}},
                        {view: 'Button', label: shwebui.strings.RemoveSelectedSetting, name: 'btnRemoveSelectedNode', styleClass: 'shwebui_dlgbutton', strictHorizontalSizing: true},
                        {view: 'Button', label: shwebui.strings.RemoveSelectedProfile, name: 'btnRemoveSelectedProfile', styleClass: 'shwebui_dlgbutton', strictHorizontalSizing: true},
                    ]},
                    {rows: [], name: 'pnlNode'},
                ]}],
            }), domnode);
            this.addStyleClass('shwebui_manageprofilesdlg');
            this._currentProfile = undefined;
            this._currentNodeIdx = undefined;
            this._profiles = [];
        }

        doinit() {
            var self = this;
            super.doinit();
            var btnClose = this.containingNameScope().getByName('btnClose');
            var btnRemoveSelectedNode = this.containingNameScope().getByName('btnRemoveSelectedNode');
            var btnRemoveSelectedProfile = this.containingNameScope().getByName('btnRemoveSelectedProfile');
            var lstProfiles = this.containingNameScope().getByName('lstProfiles');
            var lstNodes = this.containingNameScope().getByName('lstNodes');
            btnClose.OnClicked.addHandler(function(){
                self.close();
            });
            btnRemoveSelectedNode.OnClicked.addHandler(function(){
                if (self._currentNodeIdx > -1) {
                    clove.messageDialog({message: shwebui.strings.ConfirmDeleteNode, buttons: [shwebui.strings.Yes, shwebui.strings.No]}).addHandler(function(e){
                        if (e.button === 0) {
                            var profile = self._getCurrentProfile();
                            var nodeId = profile.nodes[self._currentNodeIdx].nodeId;
                            var busytoken = self.registerBusy();
                            self.ajax({url: 'manageprofiles/remove_node', data: {nodeId: nodeId}, success: function(){
                                self._refreshProfileList();
                                self.unregisterBusy(busytoken);
                            }});
                        }
                    });
                }
            });
            btnRemoveSelectedProfile.OnClicked.addHandler(function(){
                var profile = self._getCurrentProfile();
                if (profile) {
                    var nodecount = profile.nodes.length;
                    var msg;
                    if (nodecount != 1)
                        msg = shwebui.strings.ConfirmDeleteProfile2.replace("%1", nodecount).replace("%2", profile.name);
                    else
                        msg = shwebui.strings.ConfirmDeleteProfile1.replace("%1", profile.name);
                    clove.messageDialog({message: msg, buttons: [shwebui.strings.Yes, shwebui.strings.No]}).addHandler(function(e){
                        if (e.button === 0) {
                            var busytoken = self.registerBusy();
                            self.ajax({url: 'manageprofiles/remove_profile', data: {profile: profile.name}, success: function(){
                                self._refreshProfileList();
                                self.unregisterBusy(busytoken);
                            }});
                        }
                    });
                }
            });
            lstProfiles.OnPropertyChanged.addHandler(function(e){
                if (e.propertyName == 'text') {
                    self._currentProfile = lstProfiles.text();
                    self._showProfile();
                }
            });
            lstNodes.OnSelectionChanged.addHandler(function(){
                if (!self.isAlive())
                    return; // dead
                var sel = lstNodes.selection();
                self._currentNodeIdx = sel ? (sel.irow) : -1;
                self._showNode();
            });
            this._refreshProfileList();
            this._showNode();
        }

        _showNode() {
            var pnlNode = this.containingNameScope().getByName('pnlNode');
            var chlds = [];
            if (this._currentNodeIdx > -1) {
                var profile = this._getCurrentProfile();
                var node = profile.nodes[this._currentNodeIdx];
                chlds.push({label: shwebui.strings.ThoseSettingsAreInTheNode, styleClass: 'shwebui_manageprofilesdlg_lblnodehead'});
                var empty = true;
                for (var setting of node.settings) {
                    setting['view'] = 'shwebui.SettingUIFrame';
                    if (setting.onlyInFileviewIndex > -1)
                        setting['additionalCheck'] = shwebui.strings.OnlyForFileListN.replace("%1", setting.onlyInFileviewIndex+1);
                    chlds.push(setting);
                    empty = false;
                }
                if (empty)
                    chlds.push({label: shwebui.strings.None, styleClass: 'shwebui_manageprofilesdlg_lblnodetext'});
                chlds.push({label: shwebui.strings.AlsoAppliesOnSubfolders, styleClass: 'shwebui_manageprofilesdlg_lblnodehead'});
                var sAlsoAppliesOnSubfolders = node.withSubfolders ? shwebui.strings.LYes : shwebui.strings.LNo;
                chlds.push({label: sAlsoAppliesOnSubfolders, styleClass: 'shwebui_manageprofilesdlg_lblnodetext'});
                chlds.push({label: shwebui.strings.InheritsFromOtherProfiles, styleClass: 'shwebui_manageprofilesdlg_lblnodehead'});
                var sInheritsFromOtherProfiles = node.inheritsFromDescription;
                chlds.push({label: sInheritsFromOtherProfiles, styleClass: 'shwebui_manageprofilesdlg_lblnodetext'});
            }
            else
                chlds.push({label: shwebui.strings.NothingSelected, styleClass: 'shwebui_manageprofilesdlg_lblnodehead'});
            pnlNode.setChildren(chlds);
        }

        _getCurrentProfile() {
            for (var d of this._profiles) {
                if (d.name == this._currentProfile)
                    return d;
            }
        }

        _showProfile() {
            var lstNodes = this.containingNameScope().getByName('lstNodes');
            var profile = this._getCurrentProfile();
            var nodesds = new clove.NativeDatasource();
            nodesds.root().addColumn([]);
            if (profile) {
                for (var node of profile.nodes)
                    nodesds.root().addRow([node.text]).setMetadata('icon', clove.Icon.byUrl(this.dlgurl('manageprofiles/icon', {name: node.icon})));
            }
            lstNodes.setDatasource(nodesds);
        }

        _refreshProfileList() {
            var self = this;
            var lstProfiles = this.containingNameScope().getByName('lstProfiles');
            var busytoken = this.registerBusy();
            var ajaxreq = this._ajaxreq = this.ajax({url: 'manageprofiles/get', data: {currentprofile: this._currentProfile}, success: function(data) {
                if (ajaxreq == self._ajaxreq) {
                    self._profiles = data.profiles;
                    var profiles = [];
                    for (var profile of data.profiles)
                        profiles.push(profile.name);
                    lstProfiles.setPopupItems(clove.utils.arrayToDatasource(profiles));
                    if (data.selected > -1)
                        lstProfiles.setText(profiles[data.selected]);
                }
                self.unregisterBusy(busytoken);
            }});
        }

    }

    shwebui.ManageProfilesDialog = ManageProfilesDialog;

})();
