/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class OpenWithDialog extends shwebui.Dialog {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                title: shwebui.strings.OpenWith,
                headtext1: shwebui.strings.OpenWithHead,
                rightHeadButtons: [{view: 'Button', label: shwebui.strings.SelectOther, name: 'btnSelectOther'},
                                   {view: 'Button', label: shwebui.strings.Cancel, name: 'btnClose'},
                                   {view: 'Button', label: shwebui.strings.OK, name: 'btnOK', enabled: false},],
                innerBody: [{rows: [
                                {view: 'Label', name: 'lblCustomOpener', styleClass: 'shwebui_openwithdlg_customopener'},
                                {cols: [
                                    {label:'', hstretch: 1},
                                    {view: 'ScrollView', body: {view: 'DataView', styleClass: 'shwebui_openwithdlg_openmethods', name: 'lstOpenMethods', hideExpanders: true, granularity: 'row', allowSelection: true, nodeActivationNeedsDoubleClick: true, hstretch: 1}},
                                    {label:'', hstretch: 1}
                                ]},
                                {view: 'CheckButton', label: shwebui.strings.OpenWithRememberType, name: 'chkRememberForMimetype'},
                                {cols: [
                                    {view: 'CheckButton', label: shwebui.strings.OpenWithRememberDir, name: 'chkRememberForDirectory'},
                                    {view: 'Button', name: 'btnAnotherAncestor', styleClass: 'shwebui_openwithdlg_btnanotherancestor shwebui_dlgbutton', label: shwebui.strings.AnotherAncestor},
                                ]},
                                {view: 'CheckButton', label: shwebui.strings.OpenWithRememberFile, name: 'chkRememberForFile'},
                            ], styleClass: 'shwebui_dlg_innerbody_withmargin'}],
            }), domnode);
            this.addStyleClass('shwebui_openwithdlg');
            this._customOpener = undefined;
            this._anotherAncestor = undefined;
        }

        doinit() {
            var self = this;
            super.doinit();
            this.btnSelectOther = this.containingNameScope().getByName('btnSelectOther');
            this.btnClose = this.containingNameScope().getByName('btnClose');
            this.btnOK = this.containingNameScope().getByName('btnOK');
            this.lblCustomOpener = this.containingNameScope().getByName('lblCustomOpener');
            this.lstOpenMethods = this.containingNameScope().getByName('lstOpenMethods');
            this.chkRememberForMimetype = this.containingNameScope().getByName('chkRememberForMimetype');
            this.chkRememberForDirectory = this.containingNameScope().getByName('chkRememberForDirectory');
            this.btnAnotherAncestor = this.containingNameScope().getByName('btnAnotherAncestor');
            this.chkRememberForFile = this.containingNameScope().getByName('chkRememberForFile');
            this.btnSelectOther.OnClicked.addHandler(function(){
                clove.inputDialog({question: shwebui.strings.OpenWithPleaseEnterCustomOpenerPath}).addHandler(function(e){
                    if (e.answer) {
                        self.setBusy(true);
                        var _coajaxreq = self._coajaxreq = self.ajax({url: "openwith/check_custom_opener", data: {path: e.answer}, success: function(data){
                            if (_coajaxreq != self._coajaxreq)
                                return;
                            self.setBusy(false);
                            if (data.fullpath) {
                                self._customOpener = data;
                                self.lstOpenMethods.selectCell();
                            }
                            else
                                clove.messageDialog({message: shwebui.strings.OpenWithCustomOpenerPathWasInvalid});
                            self._refresh();
                        }});
                    }
                });
            });
            this.btnClose.OnClicked.addHandler(function(){
                self.close();
            });
            this.btnOK.OnClicked.addHandler(function(){
                var sel = self.lstOpenMethods.selection();
                var chosenMethod = sel ? sel.irow : undefined;
                var chosenCustomOpener = self._customOpener ? self._customOpener.fullpath : undefined;
                var rememberForMimetype = self.chkRememberForMimetype.checked();
                var rememberForDirectory = self.chkRememberForDirectory.checked() ? self._rememberForDirectoryEurl() : undefined;
                var rememberForFile = self.chkRememberForFile.checked();
                self.acceptDialog({chosenMethod: chosenMethod, chosenCustomOpener: chosenCustomOpener, rememberForMimetype: rememberForMimetype,
                                   rememberForDirectory: rememberForDirectory, rememberForFile: rememberForFile});
            });
            this.chkRememberForDirectory.OnClicked.addHandler(function(){
                self._refresh();
            });
            this.btnAnotherAncestor.OnClicked.addHandler(function(){
                clove.inputDialog({question: shwebui.strings.OpenWithPleaseEnterAnotherAncestor, defaultAnswer: self._rememberForDirectoryEurl()}).addHandler(function(e){
                    if (e.answer) {
                        self.setBusy(true);
                        var _caajaxreq = self._caajaxreq = self.ajax({url: "openwith/check_another_ancestor", data: {eurl: e.answer}, success: function(data){
                            if (_caajaxreq != self._caajaxreq)
                                return;
                            self.setBusy(false);
                            if (data.fulleurl)
                                self._anotherAncestor = data;
                            else
                                clove.messageDialog({message: shwebui.strings.OpenWithAnotherAncestorWasInvalid});
                            self._refresh();
                        }});
                    }
                    else if (e.answer === "") {
                        self._anotherAncestor = undefined;
                        self._refresh();
                    }
                });
            });
            this.lstOpenMethods.OnSelectionChanged.addHandler(function(e) {
                if (self.lstOpenMethods.selection()) {
                    self._customOpener = undefined;
                    self._refresh();
                }
            });
            this.lstOpenMethods.OnNodeActivated.addHandler(function(e) {
                self.btnOK.OnClicked.trigger({});
            });
            this.setBusy(true);
            this.ajax({url: 'openwith/open_methods', success: function(data) {
                var dsom = new clove.NativeDatasource();
                var rdsom = dsom.root();
                rdsom.addColumn([]);
                for (var oom of (data))
                    rdsom.addRow([oom.name]).setMetadata('icon', clove.Icon.byUrl(self.dlgurl('openwith/open_method_icon', {idx: oom.iconidx})));
                self.lstOpenMethods.setDatasource(dsom);
                self.setBusy(false);
            }});
            this._refresh();
        }

        _rememberForDirectoryEurl() {
            return this._anotherAncestor ? this._anotherAncestor.fulleurl : this.config.anotherAncestor;
        }

        _refresh() {
            this.btnOK.setEnabled(this._customOpener || this.lstOpenMethods.selection());
            this.lblCustomOpener.setLabel(this._customOpener ? this._customOpener.basename : "");
            this.lblCustomOpener.setVisibility(this._customOpener ? clove.Visible : clove.InvisibleCollapsed);
            if (this.chkRememberForDirectory.checked()) {
                this.btnAnotherAncestor.setVisibility(clove.Visible);
                this.btnAnotherAncestor.setLabel(this._anotherAncestor ? this._anotherAncestor.basename : shwebui.strings.AnotherAncestor);
            }
            else
                this.btnAnotherAncestor.setVisibility(clove.Invisible);
        }

    }

    shwebui.OpenWithDialog = OpenWithDialog;

})();
