/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class PropertiesDialogWidgetText extends clove.Label {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("content");
        }

        content_setter(s) {
            this.setLabel(s);
        }

    }

    shwebui.PropertiesDialogWidgetText = PropertiesDialogWidgetText;

    class PropertiesDialogWidgetTable extends clove.DataView {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                hideExpanders: true,
                allowSelection: true,
                allowChecking: true,
                granularity: 'row',
            }), domnode);
            var self = this;
            this.addStyleClass("shwebui_propertiesdlg_tabwidget_table");
            this.declareProperty("content");
            this._sel_sel = undefined;
            this._sel_checked = [];
            this.OnSelectionChanged.addHandler(function(e){
                var ss = self.selection();
                self._sel_sel = ss ? ss.irow : undefined;
                if (self._sel_checked.length == 0)
                    self._selChanged();
            });
            this.OnCheckedNodesChanged.addHandler(function(e){
                var scl = [];
                for(var psc of self.checkedCells())
                    scl.push(psc.irow);
                self._sel_checked = scl;
                self._selChanged();
            });
        }

        _selChanged() {
            var selected = [];
            if (this._sel_checked.length > 0)
                selected = this._sel_checked;
            else if (this._sel_sel !== undefined)
                selected.push(this._sel_sel);
            this.config.dialog.ajax({url: 'properties/tabwidget_table_select',
                                     data: {cookie: this.config.cookie, tabidx: this.config.tabidx,
                                            widgetidx: this.config.widgetidx, selected: JSON.stringify(selected)}});
        }

        content_setter(cnt) {
            var ds = new clove.NativeDatasource();
            var dsroot = ds.root();
            dsroot.addColumn([]);
            dsroot.addColumn([]);
            for(var i=0; i<cnt.length; i++) {
                var cnti = cnt[i];
                dsroot.addRow([cnti[0], cnti[1]]);
            }
            this.setDatasource(ds);
        }

    }

    shwebui.PropertiesDialogWidgetTable = PropertiesDialogWidgetTable;

    class PropertiesDialogWidgetIconTextBanner extends clove.HorizontalStack {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("content");
        }

        content_setter(cnt) {
            var chlds = [];
            for(var ccnt of cnt) {
                if (ccnt.text !== undefined)
                    chlds.push({view: 'Label', label: ccnt.text, styleClass: 'shwebui_propertiesdlg_tabwidget_icontextbanner_text'});
                else /* if (ccnt.icon....) */
                    chlds.push({view: 'IconView', icon: clove.Icon.byUrl(shwebui.actioncallUrl(ccnt.icon.src)), size: ccnt.icon.sizeInPt+"pt"});
            }
            this.setChildren(chlds);
        }

    }

    shwebui.PropertiesDialogWidgetIconTextBanner = PropertiesDialogWidgetIconTextBanner;

    class FilePropertyDialog extends clove.NameScope(shwebui.Dialog) {

        constructor(config, domnode) {
            config = config || {};
            var cnodes = config.nodes || [];
            if (cnodes.length == 1)
                config.headtext1 = cnodes[0];
            else
                config.headtext1 = shwebui.strings.PropertiesTitleN.replace("%1", cnodes.length);
            super(clove.utils.applyDefaultsToConfig(config, {
                rightHeadButtons: [{view: 'Button', label: shwebui.strings.Refresh, name: 'btnRefresh'},
                                   {view: 'Button', label: shwebui.strings.Close, name: 'btnClose'}],
                innerBody: [{view: 'ScrollView', body: {rows: [], name: 'pnlPropertiesWidgets'}, styleClass: 'shwebui_dlg_innerbody_withmargin'},],
                title: shwebui.strings.PropertiesTitle,
            }), domnode);
            this.declareProperty('nodes', []);
            this.addStyleClass('shwebui_propertiesdlg');
        }

        _createButtonOnClickedFct(cookie, tabidx, widgetidx, btnidx) {
            var self = this;
            return function() {
                self.ajax({url: 'properties/trigger_action', data: {cookie: cookie, tabidx: tabidx, widgetidx: widgetidx, btnidx: btnidx}});
            };
        }

        _buildProperties() {
            var self = this;
            var pnlPropertiesWidgets = this.containingNameScope().getByName('pnlPropertiesWidgets');
            var ajaxreq = this._ajaxreq = this.ajax({url: 'properties/get_properties', success: function(data) {
                if (ajaxreq != self._ajaxreq)
                    return;
                var rch = [];
                for (var idata in data) {
                    var datai = data[idata];
                    var cookie = datai.cookie;
                    var crcl = [];
                    for (var iwidget in datai.widgets) {
                        var widgeti = datai.widgets[iwidget];
                        if (widgeti.visible) {
                            var crcrcl = [];
                            crcrcl.push({label: widgeti.labelText, styleClass: 'shwebui_propertiesdlg_tabwidgettitle'});
                            var wicnt = widgeti.content;
                            wicnt.view = 'shwebui.PropertiesDialogWidget' + wicnt.type;
                            wicnt.cookie = cookie;
                            wicnt.tabidx = idata;
                            wicnt.widgetidx = widgeti.idx;
                            wicnt.dialog = self;
                            crcrcl.push(wicnt);
                            var btnscrcrcl = [];
                            for (var ibtn in widgeti.buttons) {
                                var btn = widgeti.buttons[ibtn];
                                btnscrcrcl.push({view: 'Button', label: btn, styleClass: 'shwebui_dlgbutton', OnClicked: self._createButtonOnClickedFct(cookie, idata, widgeti.idx, ibtn)});
                            }
                            crcrcl.push({rows: btnscrcrcl});
                            crcl.push({rows: crcrcl, styleClass: 'shwebui_propertiesdlg_tabwidget'});
                        }
                    }
                    if (crcl.length > 0)
                        rch.push({rows: [{label: datai.titleWithoutMnemonic, styleClass: 'shwebui_propertiesdlg_tabtitle'}, {rows: crcl, horizontalStretchAffinity: 1}], styleClass: 'shwebui_propertiesdlg_tab'});
                }
                pnlPropertiesWidgets.setChildren(rch);
            }});
        }

        refresh() {
            this.ajax({url: 'properties/refresh'});
        }

        doinit() {
            var self = this;
            super.doinit();
            var btnRefresh = this.containingNameScope().getByName('btnRefresh');
            var btnClose = this.containingNameScope().getByName('btnClose');
            var lblHead1 = this.containingNameScope().getByName('lblHead1');
            btnRefresh.OnClicked.addHandler(function(){
                self.refresh();
            });
            btnClose.OnClicked.addHandler(function(){
                self.close();
            });
            this.getEvent("properties/changed").addHandler(function(e){
                self._buildProperties();
            }, this);
            this._buildProperties();
        }

    }

    shwebui.FilePropertyDialog = FilePropertyDialog;

})();
