/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class StatusBarPanelDialogUserFeedback__Abstract extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [
                    {cols: [
                        {view: 'IconView', name: 'iconMain', strictVerticalSizing: true},
                        {view: 'Label', name: 'lbltext', styleClass: 'shwebui_userfeedback_base_lbltext'},
                    ]},
                    {view: 'ScrollView', body: {rows: [], horizontalStretchAffinity: 1, name: 'panelinner', styleClass: 'shwebui_userfeedback_base_panelinner'}},
                    {view: 'ScrollView', body: {rows: [], horizontalStretchAffinity: 1, name: 'panelbuttons', styleClass: 'shwebui_userfeedback_base_panelbuttons'}},
                ],
            }), domnode);
            this.declareProperty('id');
            this.declareProperty('icon');
            this.declareProperty('text');
            this.declareProperty('answericons');
            this.declareProperty('answers');
            this.declareProperty('cancelanswer');
            this.declareProperty('defaultanswer');
            this.OnAnswered = new clove.Event();
        }

        text_setter(s) {
            var lbltext = this.getByName('lbltext');
            if (lbltext)
                lbltext.setLabel(s);
        }

        icon_setter(s) {
            var iconMain = this.getByName('iconMain');
            if (iconMain) {
                iconMain.setIcon(clove.Icon.byUrl(s));
                iconMain.setVisibility(s ? clove.Visible : clove.InvisibleCollapsed);
            }
        }

        _createpanelinner() {
        }

        _answer(id, data) {
            this.OnAnswered.trigger(clove.utils.applyDefaultsToConfig({id: id}, data));
        }

        _answerclicked(id, answeridx) {
        }

        _createanswersbuttons(answers) {
            var self = this;
            var onclickedfct = function(e) {
                self._answerclicked(e.widget.config.id, e.widget.config.answer);
            };
            var btns = [];
            for(var isa=0; isa<answers.length; isa++) {
                var bicon = (this.config.answericons||[])[isa];
                var cicon = bicon ? clove.Icon.byUrl(bicon) : undefined;
                btns.push({view: 'Button', label: answers[isa], icon: cicon, hstretch: 1,
                           answer: isa, id: this.config.id, OnClicked: onclickedfct});
            }
            this.panelbuttons.setChildren(btns);
        }

        children_setter(c) {
            var self = this;
            super.children_setter(c);
            this.panelinner = this.getByName('panelinner');
            this.panelbuttons = this.getByName('panelbuttons');
            //TODO # use defaultanswer
            //TODO # use cancelanswer
            this._startFocus = undefined;
            this._createanswersbuttons(this.config.answers);
            if (!this._startFocus)
                this._startFocus = this.panelbuttons.childrenWidgets()[0];
            this._createpanelinner();
            if (this._startFocus)
                setTimeout(function(){
                    try {
                        self._startFocus.focus();
                    } catch(e) {}
                });
            this.icon_setter(this.config.icon);
            this.text_setter(this.config.text);
        }

    }

    class StatusBarPanelDialogUserFeedback_MessageBox extends StatusBarPanelDialogUserFeedback__Abstract {

        _answerclicked(id, answeridx) {
            this._answer(id, {answer: answeridx});
        }

    }

    shwebui.StatusBarPanelDialogUserFeedback_MessageBox = StatusBarPanelDialogUserFeedback_MessageBox;

    class StatusBarPanelDialogUserFeedback_InputBox extends StatusBarPanelDialogUserFeedback__Abstract {

        _answerclicked(id, answeridx) {
            this._answer(id, {answer: answeridx, value: this.editbox.text()});
        }

        _createpanelinner() {
            this.editbox = this._startFocus = this.panelinner.addChild({view: 'EditBox', text: this.config.value, horizontalStretchAffinity: 1});
            this.editbox.setTextSelection(this.config.valuePreselectFrom, this.config.valuePreselectTo);
        }

    }

    shwebui.StatusBarPanelDialogUserFeedback_InputBox = StatusBarPanelDialogUserFeedback_InputBox;

    class StatusBarPanelDialogUserFeedback_MultilineInputBox extends StatusBarPanelDialogUserFeedback__Abstract {

        _answerclicked(id, answeridx) {
            this._answer(id, {answer: answeridx, value: this.editbox.text()});
        }

        _createpanelinner() {
            this.editbox = this._startFocus = this.panelinner.addChild({view: 'MultilineEditBox', text: this.config.value, horizontalStretchAffinity: 1});
        }

    }

    shwebui.StatusBarPanelDialogUserFeedback_MultilineInputBox = StatusBarPanelDialogUserFeedback_MultilineInputBox;

    class StatusBarPanelDialogUserFeedback_SimpleChooserGridform extends StatusBarPanelDialogUserFeedback__Abstract {

        _answerclicked(id, answeridx) {
            var entries = this.config.entries;
            for(var wes of this.panelinner.childrenWidgets()) {
                if (wes.config.col == 1) {
                    var ie = wes.config.row;
                    var centry = entries[ie];
                    var selected_choice = -1;
                    for(var wcs of wes.childrenWidgets()) {
                        var cidx = wcs.config.idx;
                        if (wcs.config.ischeck) {
                            if (wcs.isStyleClass("shwebui_userfeedback_gridform_selectedchoice"))
                                selected_choice = cidx;
                        }
                        else if (wcs.config.istext)
                            centry.choices[cidx].text = wcs.text();
                    }
                    centry.selected_choice=selected_choice;
                }
            }
            this._answer(id, {answer: answeridx, entries: entries});
        }

        _check(ie, ic) {
            var wentry;
            for(var wes of this.panelinner.childrenWidgets()) {
                if (wes.config.row == ie && wes.config.col == 1) {
                    wentry = wes;
                    break;
                }
            }
            for(var wcs of wentry.childrenWidgets()) {
                if (wcs.config.ischeck)
                    wcs.setStyleClassAssigned("shwebui_userfeedback_gridform_selectedchoice", ic == wcs.config.idx);
                else if (wcs.config.istext)
                    wcs.setVisibility(ic == wcs.config.idx ? clove.Visible : clove.InvisibleCollapsed);
            }
        }

        _createpanelinner() {
            var self = this;
            var ventries = [];
            var selections = {};
            for (var ie=0; ie<this.config.entries.length; ie++) {
                var centry = this.config.entries[ie];
                var vchoices = [];
                for (var ic=0; ic<centry.choices.length; ic++) {
                    var cchoice = centry.choices[ic];
                    vchoices.push({view: 'Button', label: cchoice.label, idx: ic, entry_idx: ie, ischeck: true, OnClicked: function(e){ self._check(e.widget.config.entry_idx, e.widget.config.idx); }});
                    if (cchoice.withText)
                        vchoices.push({view: 'EditBox', text: cchoice.text, idx: ic, istext: true});
                }
                ventries.push({label: centry.label, row: ie, col: 0});
                ventries.push({rows: vchoices, row: ie, col: 1});
                selections[ie] = centry.selected_choice;
            }
            this.panelinner.setChildren(ventries);
            for (var ie in selections)
                this._check(ie, selections[ie]);
        }

    }

    shwebui.StatusBarPanelDialogUserFeedback_SimpleChooserGridform = StatusBarPanelDialogUserFeedback_SimpleChooserGridform;

    class StatusBarPanelDialogUserFeedback_CredentialsDialog extends StatusBarPanelDialogUserFeedback__Abstract {

        _createanswersbuttons(a) {
            super._createanswersbuttons([shwebui.strings.OK, shwebui.strings.Cancel]);
        }

        _answerclicked(id, answeridx) {
            var edtdomain = this.foo.getByName('edtdomain');
            var edtusername = this.foo.getByName('edtusername');
            var edtpassword = this.foo.getByName('edtpassword');
            var chkanonymous = this.foo.getByName('chkanonymous');
            var chkremember = this.foo.getByName('chkremember');
            this._answer(id, {answer: answeridx == 0, domain: edtdomain.text(), username: edtusername.text(),
                              password: edtpassword.text(), isAnonymous: chkanonymous.checked(), isRemember: chkremember.checked()});
        }

        _createpanelinner() {
            function b2v(b) { return b?clove.Visible:clove.InvisibleCollapsed; }
            this.foo = this.panelinner.addChild({rows: [
                {view: 'CheckButton', label: shwebui.strings.Anonymous, name: 'chkanonymous', checked: this.config.isAnonymous, visibility: b2v(this.config.showAnonymous)},
                {view: 'Form', styleClass: 'shwebui_userfeedback_credentials_form', sections: [ //TODO# in cress: Form should also hide labels when some widgets inside the form get hidden
                    {view: 'EditBox', formSectionLabel: shwebui.strings.Domain, name: 'edtdomain', text: this.config.domain, visibility: b2v(this.config.showDomain)},
                    {view: 'EditBox', formSectionLabel: shwebui.strings.User, name: 'edtusername', text: this.config.username, visibility: b2v(this.config.showUsername)},
                    {view: 'PasswordEditBox', formSectionLabel: shwebui.strings.Password, name: 'edtpassword', text: this.config.password, visibility: b2v(this.config.showPassword)},
                ]},
                {view: 'CheckButton', label: shwebui.strings.Remember, name: 'chkremember', checked: this.config.isRemember, visibility: b2v(this.config.showRemember)},
            ], newNameScope: true});
            this._startFocus = this.foo.getByName('edtusername');
        }

    }

    shwebui.StatusBarPanelDialogUserFeedback_CredentialsDialog = StatusBarPanelDialogUserFeedback_CredentialsDialog;

    class StatusBarPanelDialogUserFeedback_UnixPermissionsDialog extends StatusBarPanelDialogUserFeedback__Abstract {

        _createanswersbuttons(a) {
            super._createanswersbuttons([shwebui.strings.Save, shwebui.strings.Cancel]);
        }

        _answerclicked(id, answeridx) {
            var userMayRead = this.chkur.checked();
            var userMayWrite = this.chkuw.checked();
            var userMayExecute = this.chkux.checked();
            var groupMayRead = this.chkgr.checked();
            var groupMayWrite = this.chkgw.checked();
            var groupMayExecute = this.chkgx.checked();
            var othersMayRead = this.chkor.checked();
            var othersMayWrite = this.chkow.checked();
            var othersMayExecute = this.chkox.checked();
            var sticky = this.chksticky.checked();
            var setuid = this.chksetuid.checked();
            var setgid = this.chksetgid.checked();
            var pownerUser = this.lstusers.selection();
            var ownerUser = pownerUser ? pownerUser.getValue() : undefined;
            var pownerGroup = this.lstgroups.selection();
            var ownerGroup = pownerGroup ? pownerGroup.getValue() : undefined;
            this._answer(id, {answer: answeridx == 0, userMayRead: userMayRead, userMayWrite: userMayWrite, userMayExecute: userMayExecute,
                              groupMayRead: groupMayRead, groupMayWrite: groupMayWrite, groupMayExecute: groupMayExecute,
                              othersMayRead: othersMayRead, othersMayWrite: othersMayWrite, othersMayExecute: othersMayExecute,
                              sticky: sticky, setuid: setuid, setgid: setgid, ownerUser: ownerUser, ownerGroup: ownerGroup});
        }

        _createpanelinner() {
            this.dsusers = clove.utils.arrayToDatasource(this.config.users);
            this.dsgroups = clove.utils.arrayToDatasource(this.config.groups);
            this.foo = this.panelinner.addChild({cols: [
                {rows: [ /* user column */
                    {label: shwebui.strings.OwnerUser},
                    {view: 'ScrollView', height: '200pt', body: {view: 'DataView', name: 'lstusers', datasource: this.dsusers, hideExpanders: true, allowSelection: true, gridVisible: false}},
                    {view: 'CheckButton', label: shwebui.strings.MayRead, name: 'chkur', checked: this.config.userMayRead},
                    {view: 'CheckButton', label: shwebui.strings.MayWrite, name: 'chkuw', checked: this.config.userMayWrite},
                    {view: 'CheckButton', label: shwebui.strings.MayExecute, name: 'chkux', checked: this.config.userMayExecute},
                ], verticalStretchAffinity: 1, horizontalStretchAffinity: 1},
                {rows: [ /* group column */
                    {label: shwebui.strings.Group},
                    {view: 'ScrollView', height: '200pt', body: {view: 'DataView', name: 'lstgroups', datasource: this.dsgroups, hideExpanders: true, allowSelection: true, gridVisible: false}},
                    {view: 'CheckButton', label: shwebui.strings.MayRead, name: 'chkgr', checked: this.config.groupMayRead},
                    {view: 'CheckButton', label: shwebui.strings.MayWrite, name: 'chkgw', checked: this.config.groupMayWrite},
                    {view: 'CheckButton', label: shwebui.strings.MayExecute, name: 'chkgx', checked: this.config.groupMayExecute},
                ], verticalStretchAffinity: 1, horizontalStretchAffinity: 1},
                {rows: [ /* settings column */
                    {view: 'CheckButton', label: shwebui.strings.Sticky, name: 'chksticky', checked: this.config.sticky},
                    {view: 'CheckButton', label: shwebui.strings.SetUserOnExecute, name: 'chksetuid', checked: this.config.setuid},
                    {view: 'CheckButton', label: shwebui.strings.SetGroupOnExecute, name: 'chksetgid', checked: this.config.setgid},
                    {label: shwebui.strings.OtherUsers},
                    {view: 'CheckButton', label: shwebui.strings.MayRead, name: 'chkor', checked: this.config.othersMayRead},
                    {view: 'CheckButton', label: shwebui.strings.MayWrite, name: 'chkow', checked: this.config.othersMayWrite},
                    {view: 'CheckButton', label: shwebui.strings.MayExecute, name: 'chkox', checked: this.config.othersMayExecute},
                ], verticalStretchAffinity: 1, horizontalStretchAffinity: 1},
            ], newNameScope: true});
            this.lstusers = this.foo.getByName("lstusers");
            this.chkur = this._startFocus = this.foo.getByName("chkur");
            this.chkuw = this.foo.getByName("chkuw");
            this.chkux = this.foo.getByName("chkux");
            this.lstgroups = this.foo.getByName("lstgroups");
            this.chkgr = this.foo.getByName("chkgr");
            this.chkgw = this.foo.getByName("chkgw");
            this.chkgx = this.foo.getByName("chkgx");
            this.chkor = this.foo.getByName("chkor");
            this.chkow = this.foo.getByName("chkow");
            this.chkox = this.foo.getByName("chkox");
            this.chksticky = this.foo.getByName("chksticky");
            this.chksetuid = this.foo.getByName("chksetuid");
            this.chksetgid = this.foo.getByName("chksetgid");
            for(var iru=0; iru<this.dsusers.rowCount(); iru++) {
                var vpi = this.dsusers.valuePointer(iru, 0);
                var nami = this.dsusers.getValue(vpi);
                if (nami == this.config.ownerUser) {
                    this.lstusers.selectCell(vpi);
                    break;
                }
            }
            for(var irg=0; irg<this.dsgroups.rowCount(); irg++) {
                var vpi = this.dsgroups.valuePointer(irg, 0);
                var nami = this.dsgroups.getValue(vpi);
                if (nami == this.config.ownerGroup) {
                    this.lstgroups.selectCell(vpi);
                    break;
                }
            }
        }

    }

    shwebui.StatusBarPanelDialogUserFeedback_UnixPermissionsDialog = StatusBarPanelDialogUserFeedback_UnixPermissionsDialog;

    class StatusBarPanelDialog extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [
                    {view: 'Label', name: 'lblhead', styleClass: 'shwebui_statusbarpaneldialog_head'},
                    {view: 'Label', name: 'lbldetails_fromverb', styleClass: 'shwebui_statusbarpaneldialog_lbldetailverb'},
                    {view: 'Label', name: 'lbldetails_fromobject', styleClass: 'shwebui_statusbarpaneldialog_lbldetailobject'},
                    {view: 'Label', name: 'lbldetails_toverb', styleClass: 'shwebui_statusbarpaneldialog_lbldetailverb'},
                    {view: 'Label', name: 'lbldetails_toobject', styleClass: 'shwebui_statusbarpaneldialog_lbldetailobject'},
                    {view: 'ProgressBar', name: 'progressbar', styleClass: 'shwebui_statusbarpaneldialog_progressbar', labelFunction: function(v,w){return w.config.label;}},
                    {rows: [], name: 'paneluserfeedback', styleClass: 'shwebui_statusbarpaneldialog_paneluserfeedback'},
                ],
            }), domnode);
            this.addStyleClass('shwebui_statusbarpaneldialog');
            this.declareProperty('id');
            this.declareProperty('details_fromverb');
            this.declareProperty('details_fromobject');
            this.declareProperty('details_toverb');
            this.declareProperty('details_toobject');
            this.declareProperty('head');
            this.declareProperty('progress_isDeterminate');
            this.declareProperty('progress_done');
            this.declareProperty('progress_all');
            this.declareProperty('progress_text');
            this.declareProperty('userfeedback');
            this.OnUserFeedbackAnswered = new clove.Event();
        }

        details_fromverb_setter(s) {
            var lbl = this.getByName("lbldetails_fromverb");
            if (lbl) {
                lbl.setLabel(s);
                lbl.setVisibility(s ? clove.Visible : clove.InvisibleCollapsed);
            }
        }

        details_fromobject_setter(s) {
            var lbl = this.getByName("lbldetails_fromobject");
            if (lbl) {
                lbl.setLabel(s);
                lbl.setVisibility(s ? clove.Visible : clove.InvisibleCollapsed);
            }
        }

        details_toverb_setter(s) {
            var lbl = this.getByName("lbldetails_toverb");
            if (lbl) {
                lbl.setLabel(s);
                lbl.setVisibility(s ? clove.Visible : clove.InvisibleCollapsed);
            }
        }

        details_toobject_setter(s) {
            var lbl = this.getByName("lbldetails_toobject");
            if (lbl) {
                lbl.setLabel(s);
                lbl.setVisibility(s ? clove.Visible : clove.InvisibleCollapsed);
            }
        }

        head_setter(s) {
            var lbl = this.getByName("lblhead");
            if (lbl)
                lbl.setLabel(s);
        }

        progress_isDeterminate_setter(s) {
            var progressbar = this.getByName("progressbar");
            if (progressbar) {
                if (this.config.progress_isDeterminate)
                    progressbar.setValue(this.config.progress_done / this.config.progress_all);
                else
                    progressbar.setValue(undefined);
            }
        }

        progress_done_setter(s) {
            this.progress_isDeterminate_setter();
        }

        progress_all_setter(s) {
            this.progress_isDeterminate_setter();
        }

        progress_text_setter(s) {
            var progressbar = this.getByName("progressbar");
            if (progressbar)
                progressbar.setLabel(s);
        }

        userfeedback_setter(uf) {
            var self = this;
            var paneluserfeedback = this.getByName("paneluserfeedback");
            var progressbar = this.getByName("progressbar");
            if (paneluserfeedback) {
                var oldpanel = paneluserfeedback.childrenWidgets()[0];
                var oldpanelid = oldpanel ? oldpanel.config.id : -1;
                var ufid = uf ? uf.id : -1;
                if (oldpanelid != ufid) {
                    progressbar.setVisibility(uf ? clove.InvisibleCollapsed : clove.Visible);
                    paneluserfeedback.setChildren([]);
                    if (uf) {
                        var panel = paneluserfeedback.addChild(uf);
                        panel.OnAnswered.addHandler(function(e) {
                            self.OnUserFeedbackAnswered.trigger(e);
                        });
                    }
                }
            }
        }

        children_setter(c) {
            super.children_setter(c);
            this.details_fromverb_setter(this.config.details_fromverb);
            this.details_fromobject_setter(this.config.details_fromobject);
            this.details_toverb_setter(this.config.details_toverb);
            this.details_toobject_setter(this.config.details_toobject);
            this.head_setter(this.config.head);
            this.progress_isDeterminate_setter(this.config.progress_isDeterminate);
            this.progress_done_setter(this.config.progress_done);
            this.progress_all_setter(this.config.progress_all);
            this.progress_text_setter(this.config.progress_text);
            this.userfeedback_setter(this.config.userfeedback);
        }

    }

    shwebui.StatusBarPanelDialog = StatusBarPanelDialog;

    class StatusBarPanel extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'horizontal',
                children: [ {view: 'ProgressBar', name: 'bar', horizontalStretchAffinity: 1, verticalStretchAffinity: 1, labelFunction: function(v,w){return w.config.label;}} ],
                horizontalStretchAffinity: 0,
            }), domnode);
            var self = this;
            this.addStyleClass('shwebui_statusbarpanel');
            this.declareProperty('id');
            this.declareProperty('label');
            this.declareProperty('panelWidth');
            this.declareProperty('isProgressDeterminate');
            this.declareProperty('progressDone');
            this.declareProperty('progressAll');
            this.declareProperty('forceForeground');
            this.declareProperty('color');
            this.OnClicked = new clove.Event();
            $(this.contentnode).click(function(e){
                e.preventDefault();
                self.OnClicked.trigger({widget: self});
            });
        }

        notify() {
            clove.utils.animateNode(this.topnode, "shwebui_actionui_panelfeedback_a", 2000);
            clove.utils.animateNode(this.bar.topnode, "shwebui_actionui_panelfeedback_b", 2000);
        }

        color_setter(col) {
            if (col)
                this.topnode.style.background = col;
        }

        panelWidth_setter(pw) {
            this.bar.setWidth(pw + "px");
        }

        label_setter(s) {
            this.bar.setLabel(s);
        }

        isProgressDeterminate_setter(s) {
            this.bar.setValue(this.config.progressDone / this.config.progressAll);
        }

        progressDone_setter(s) {
            this.isProgressDeterminate_setter();
        }

        progressAll_setter(s) {
            this.isProgressDeterminate_setter();
        }

        children_setter(c) {
            super.children_setter(c);
            this.bar = this.getByName("bar");
            this.label_setter(this.config.label);
            this.isProgressDeterminate_setter(this.config.isProgressDeterminate);
            this.panelWidth_setter(this.config.panelWidth);
        }

    }

    shwebui.StatusBarPanel = StatusBarPanel;

    class StatusBar extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'horizontal',
                children: [ ],
                horizontalStretchAffinity: 1,
            }), domnode);
            var self = this;
            this.addStyleClass('shwebui_statusbar');
            this.OnUserFeedbackAnswered = new clove.Event();
            this._actionDialogConfigs = [];
            this._openActionDialogs = [];
            var _intvl_checkAndNotifyPanelsWithUserFeedback = setInterval(function(){ self._checkAndNotifyPanelsWithUserFeedback(); }, 20*1000);
            this.OnDestroyed.addHandler(function(){ clearInterval(_intvl_checkAndNotifyPanelsWithUserFeedback); });
        }

        _checkAndNotifyPanelsWithUserFeedback() {
            var opendialogids = [];
            for (var odlg of this._openActionDialogs)
                opendialogids.push(odlg.config.id);
            for(var panel of this.infoPanels()) {
                if (panel.forceForeground) {
                    var dialog = this.getActionDialogForInfoPanel(panel);
                    if (dialog && opendialogids.indexOf(dialog.id) == -1)
                        panel.notify();
                }
            }
        }

        getActionDialogForId(id) {
            for(var x of this.actionDialogs()) {
                if (x.id == id)
                    return x;
            }
        }

        getInfoPanelForId(id) {
            for(var x of this.infoPanels()) {
                if (x.id == id)
                    return x;
            }
        }

        _getActionDialogForInfoPanel_Helper(infopanel) {
            if (infopanel) {
                var actiondialogs = this.actionDialogs();
                for(var iad=0; iad<actiondialogs.length; iad++) {
                    var d = actiondialogs[iad];
                    if (d.panel_id === infopanel.config.id)
                        return [iad, d];
                }
            }
            return [-1, undefined];
        }

        _getInfoPanelForActionDialog_Helper(actiondialogconfig) {
            if (actiondialogconfig) {
                var infopanels = this.infoPanels();
                for(var ipp=0; ipp<infopanels.length; ipp++) {
                    var p = infopanels[ipp];
                    if (p.config.id === actiondialogconfig.panel_id)
                        return [ipp, p];
                }
            }
            return [-1, undefined];
        }

        getActionDialogIndexForInfoPanelIndex(idx) {
            return this.getActionDialogIndexForInfoPanel(this.infoPanels()[idx]);
        }

        getInfoPanelIndexForActionDialogIndex(idx) {
            return this.getInfoPanelIndexForActionDialog(this.actionDialogs()[idx]);
        }

        getActionDialogIndexForInfoPanelId(id) {
            return this.getActionDialogIndexForInfoPanel(this.getInfoPanelForId(id));
        }

        getInfoPanelIndexForActionDialogId(id) {
            return this.getInfoPanelIndexForActionDialog(this.getActionDialogForId(id));
        }

        getActionDialogIndexForInfoPanel(infopanel) {
            return this._getActionDialogForInfoPanel_Helper(infopanel)[0];
        }

        getInfoPanelIndexForActionDialog(actiondialog) {
            return this._getInfoPanelForActionDialog_Helper(actiondialog)[0];
        }

        getActionDialogForInfoPanelIndex(idx) {
            return this.getActionDialogForInfoPanel(this.infoPanels()[idx]);
        }

        getInfoPanelForActionDialogIndex(idx) {
            return this.getInfoPanelForActionDialog(this.actionDialogs()[idx]);
        }

        getActionDialogForInfoPanelId(id) {
            return this.getActionDialogForInfoPanel(this.getInfoPanelForId(id));
        }

        getInfoPanelForActionDialogId(id) {
            return this.getInfoPanelForActionDialog(this.getActionDialogForId(id));
        }

        getActionDialogForInfoPanel(infopanel) {
            return this._getActionDialogForInfoPanel_Helper(infopanel)[1];
        }

        getInfoPanelForActionDialog(actiondialog) {
            return this._getInfoPanelForActionDialog_Helper(actiondialog)[1];
        }

        getActionDialogIndexForActionDialogId(id) {
            var actiondialogs = this.actionDialogs();
            for(var ipp=0; ipp<actiondialogs.length; ipp++) {
                var d = actiondialogs[ipp];
                if (d.id === id)
                    return ipp;
            }
        }

        getInfoPanelIndexForInfoPanelId(id) {
            var infopanels = this.infoPanels();
            for(var ipp=0; ipp<infopanels.length; ipp++) {
                var p = infopanels[ipp];
                if (p.config.id === id)
                    return ipp;
            }
        }

        getActionDialogForActionDialogIndex(idx) {
            return this.actionDialogs()[idx];
        }

        getInfoPanelForInfoPanelIndex(idx) {
            return this.infoPanels()[idx];
        }

        infoPanels() {
            var r = [];
            for(var x of this.childrenWidgets())
                r.push(x);
            r.sort(function(a, b){return a.config.col - b.config.col;});
            return r;
        }

        insertInfoPanel(i, infopanelconfig) {
            var self = this;
            this.insertColumn(i);
            var panel = this.addChild(clove.utils.applyDefaultsToConfig({view: 'shwebui.StatusBarPanel', col: i}, infopanelconfig));
            panel.OnClicked.addHandler(function(e){
                if (panel.config.handlesClick)
                    shwebui.ajax({url: "action/panel_clicked", data: {id: panel.config.id}});
                else {
                    var idlg = self.getActionDialogIndexForInfoPanel(panel);
                    if (idlg !== undefined)
                        self.showActionDialog(idlg);
                }
            });
            return panel;
        }

        removeInfoPanel(i) {
            this.removeColumn(i);
        }

        actionDialogs() {
            return this._actionDialogConfigs;
        }

        insertActionDialog(i, actiondialogconfig) {
            this._actionDialogConfigs.splice(i, 0, actiondialogconfig);
            return actiondialogconfig;
        }

        removeActionDialog(i) {
            var oactiondialogconfig = this._actionDialogConfigs.splice(i, 1)[0];
            for (var cdlg of this._openActionDialogs) {
                if (cdlg.config.id == oactiondialogconfig.id)
                    cdlg.remove(); // break afterwards is bad, because due to async deletion, there can be a zombie instance also
            }
        }

        refreshActionDialog(i, actiondialogconfig) {
            this._actionDialogConfigs[i] = actiondialogconfig;
            for (var cdlg of this._openActionDialogs) {
                if (cdlg.config.id == actiondialogconfig.id) {
                    for (var kcfg in actiondialogconfig)
                        cdlg.setProperty(kcfg, actiondialogconfig[kcfg]);
                    break;
                }
            }
        }

        showActionDialog(i, force) {
            var self = this;
            var acfg = this._actionDialogConfigs[i];
            var inFrontOfDialogs = false;
            // if this action dialog is newer than the latest dialog, we should show it in front of the dialog
            // (otherwise it's hidden when dialogs run actions)
            var dlgids = shwebui.dialogs.getOpenDialogIds();
            if (dlgids.length > 0) {
                var lastdlg = shwebui.dialogs.getDialogById(dlgids[dlgids.length-1]);
                inFrontOfDialogs = lastdlg.config.webts_created < acfg.webts_created;
            }
            // we should do the same, if the last open one is also this way
            if (!inFrontOfDialogs && this._openActionDialogs.length > 0)
                inFrontOfDialogs = this._openActionDialogs[this._openActionDialogs.length-1].config.inFrontOfDialogs;
            if (this._openActionDialogs.length > 0 && !inFrontOfDialogs) {
                if (force) {
                    for (var odlg of this._openActionDialogs)
                        odlg.remove();
                    this._openActionDialogs = [];
                }
                else
                    return;
            }
            for (var odlg of this._openActionDialogs) {
                if (odlg.config.id == acfg.id)
                    return;
            }
            var domparentnode = inFrontOfDialogs ? undefined : /*this way normal dialogs are always above statusbar panels*/shwebui.app.mainwindow.topnode;
            var panelClickClosesPopup = !inFrontOfDialogs;
            var fullyTransparent = !inFrontOfDialogs;
            var infpanel = this.getInfoPanelForActionDialogIndex(i);
            var r = clove.utils.popup(clove.utils.applyDefaultsToConfig({view: 'shwebui.StatusBarPanelDialog', inFrontOfDialogs: inFrontOfDialogs}, acfg),
                                      {alignTo: infpanel||this, modal: true, panelClickClosesPopup: panelClickClosesPopup,
                                       fullyTransparent: fullyTransparent, domparentnode: domparentnode});
            var wdlg = r.widget;
            this._openActionDialogs.push(wdlg);
            wdlg.OnDestroyed.addHandler(function(){
                var idx = self._openActionDialogs.indexOf(wdlg);
                if (idx > -1)
                    self._openActionDialogs.splice(idx, 1);
            });
            wdlg.OnUserFeedbackAnswered.addHandler(function(e){
                self.OnUserFeedbackAnswered.trigger(e);
            });
            return wdlg;
        }

    }

    shwebui.StatusBar = StatusBar;

})();
