/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class SettingUIFrame extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                children: [
                    {rows: [
                        {view: 'CheckButton', name: 'chkDescription', styleClass: 'shwebui_settinguiframe_description'},
                        {view: 'CheckButton', name: 'chkAdditionalCheck', styleClass: 'shwebui_settinguiframe_additionalcheck', strictHorizontalSizing: true},
                    ]},
                    {view: 'Label', name: 'lblValueDescription', hstretch: 1, styleClass: 'shwebui_settinguiframe_valuedescription'},
                ],
                orientation: 'horizontal',
                hstretch: 1,
            }), domnode);
            this.declareProperty('name');
            this.declareProperty('description');
            this.declareProperty('valueDescription');
            this.declareProperty('additionalCheck');
            this.declareProperty('additionalCheckValue');
            this.declareProperty('withCheckbox', false);
            this.declareProperty('checked', false);
            this.declareProperty('additionalChecked', false);
            this.addStyleClass('shwebui_settinguiframe');
        }

        checked_setter(v) {
            var chkDescription = this.getByName('chkDescription');
            chkDescription.setChecked(v);
        }

        additionalChecked_setter(v) {
            var chkAdditionalCheck = this.getByName('chkAdditionalCheck');
            chkAdditionalCheck.setChecked(v);
        }

        additionalCheckValue_getter() {
            if (this.additionalChecked())
                return this.config.additionalCheckValue;
        }

        checked_getter() {
            return this.getByName('chkDescription').checked();
        }

        additionalChecked_getter() {
            return this.checked() && this.getByName('chkAdditionalCheck').checked();
        }

        doinit() {
            var self = this;
            super.doinit();
            if (!this.config.withCheckbox)
                this.addStyleClass('shwebui_settinguiframe_withoutcheckbox');
            var chkDescription = this.getByName('chkDescription');
            var chkAdditionalCheck = this.getByName('chkAdditionalCheck');
            var lblValueDescription = this.getByName('lblValueDescription');
            chkDescription.setLabel(this.config.description);
            lblValueDescription.setLabel(this.config.valueDescription);
            if (this.config.additionalCheck)
                chkAdditionalCheck.setLabel(this.config.additionalCheck);
            else
                chkAdditionalCheck.setVisibility(clove.InvisibleCollapsed);
            chkDescription.OnClicked.addHandler(function(){
                if (self.config.withCheckbox) {
                    chkAdditionalCheck.setEnabled(chkDescription.checked());
                    if (!chkDescription.checked())
                        chkAdditionalCheck.setChecked(false);
                }
            });
            chkDescription.OnClicked.trigger({});
        }

    }

    shwebui.SettingUIFrame = SettingUIFrame;

    class StoreProfileDialog extends shwebui.Dialog {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                title: shwebui.strings.SaveSettings,
                headtext1: shwebui.strings.SpecifyWhichSettingsToStoreFor,
                leftHeadButtons: [{view: 'Button', name: 'btnScope', styleClass: 'shwebui_dlgbutton_small'}],
                rightHeadButtons: [{view: 'PopupMenuButton', name: 'btnAdvMode', styleClass: 'shwebui_dlgbutton_small'},
                                   {view: 'Button', label: shwebui.strings.Cancel, name: 'btnCancel'},
                                   {view: 'Button', label: shwebui.strings.Save, name: 'btnSave'},],
                innerBody: [{rows: [
                    {view: 'ScrollView', body: {rows: [], name: 'pnlSettings', hstretch: 1}, width: '350pt', height: '250pt', vstretch: 1},
                    {rows: [
                        {view: 'Form', sections: [
                            {view: 'EditComboBox', name: 'txtForProfile', formSectionLabel: shwebui.strings.ForProfile},
                            {cols: [
                                {view: 'Wrap', name: 'pnlInheritFrom', hstretch: 1},
                                {view: 'Button', label: shwebui.strings.Add, name: 'btnAddInheritFrom', strictVerticalSizing: true, styleClass: 'shwebui_dlgbutton shwebui_dlgbutton_small'},
                            ], formSectionLabel: shwebui.strings.InheritFrom},
                        ], hstretch: 1, styleClass: 'shwebui_storeprofiledlg_cfgform'},
                        {view: 'CheckButton', name: 'chkApplySubdirs', checked: true, label: shwebui.strings.AlsoApplyOnSubdirs, styleClass: 'shwebui_storeprofiledlg_cfgchk'},
                    ], name: 'pnlAdvanced'},
                ], hstretch: 1}],
            }), domnode);
            this.addStyleClass('shwebui_storeprofiledlg');
            this.declareProperty('eurl');
            this.declareProperty('advancedMode', false);
            this.declareProperty('restrictToDirectory', false);
            this.settingsdata = [];
            this.profilesdata = [];
        }

        advancedMode_setter(v) {
            var pnlAdvanced = this.containingNameScope().getByName('pnlAdvanced');
            var btnAdvMode = this.containingNameScope().getByName('btnAdvMode');
            var btnScope = this.containingNameScope().getByName('btnScope');
            if (!btnAdvMode)
                return;
            btnAdvMode.setLabel(v ? shwebui.strings.Advanced : shwebui.strings.Simple);
            pnlAdvanced.setVisibility(v ? clove.Visible : clove.InvisibleCollapsed);
            btnScope.setVisibility(v ? clove.Visible : clove.InvisibleCollapsed);
            if (!v)
                this.setRestrictToDirectory(false);
            this._generateSettingsList();
        }

        restrictToDirectory_setter(v) {
            var btnScope = this.containingNameScope().getByName('btnScope');
            var chkApplySubdirs = this.containingNameScope().getByName('chkApplySubdirs');
            if (!btnScope)
                return;
            btnScope.setLabel(v ? shwebui.strings.StoreGlobally : shwebui.strings.RestrictToCurrentDir);
            this.setHeadtext2(v ? this.config.eurl : shwebui.strings.AllDirectories);
            chkApplySubdirs.setEnabled(v);
            if (!v)
                chkApplySubdirs.setChecked(true);
            this._generateSettingsList();
        }

        doinit() {
            var self = this;
            super.doinit();
            var btnScope = this.containingNameScope().getByName('btnScope');
            var btnAdvMode = this.containingNameScope().getByName('btnAdvMode');
            var btnCancel = this.containingNameScope().getByName('btnCancel');
            var btnSave = this.containingNameScope().getByName('btnSave');
            var btnAddInheritFrom = this.containingNameScope().getByName('btnAddInheritFrom');
            var chkApplySubdirs = this.containingNameScope().getByName('chkApplySubdirs');
            var txtForProfile = this.containingNameScope().getByName('txtForProfile');
            var pnlInheritFrom = this.containingNameScope().getByName('pnlInheritFrom');
            var pnlSettings = this.containingNameScope().getByName('pnlSettings');
            var modesds = new clove.NativeDatasource();
            modesds.root().addColumn([]);
            modesds.root().addRow([{name: 'simple', label: shwebui.strings.SimpleMode}]);
            modesds.root().addRow([{name: 'advanced', label: shwebui.strings.AdvancedMode}]);
            btnAdvMode.setActions(modesds);
            btnScope.OnClicked.addHandler(function(){
                self.setRestrictToDirectory(!self.restrictToDirectory());
            });
            btnAdvMode.OnActionTriggered.addHandler(function(e){
                var advanced = e.action == 'advanced';
                self.setAdvancedMode(advanced);
                self.ajax({url: 'storeprofile/store_mode', data: {level: advanced?1:0}});
            });
            btnCancel.OnClicked.addHandler(function(){
                self.close();
            });
            btnSave.OnClicked.addHandler(function(){
                var settings = [];
                var withSubDirectories = chkApplySubdirs.checked();
                var profileName = txtForProfile.text();
                var hasGlobal = !self.restrictToDirectory();
                var pnlInheritFrom = self.containingNameScope().getByName('pnlInheritFrom');
                var inheritsFrom = [];
                for (var chk of pnlInheritFrom.childrenWidgets()) {
                    if (chk.checked())
                        inheritsFrom.push(chk.label());
                }
                for (var wset of pnlSettings.childrenWidgets()) {
                    if (wset instanceof shwebui.SettingUIFrame) {
                        if (wset.checked())
                            settings.push({name: wset.name(), additionalCheckValue: wset.additionalCheckValue()});
                    }
                }
                self.acceptDialog({withSubDirectories: withSubDirectories, profileName: profileName, hasGlobal: hasGlobal, settings: JSON.stringify(settings), inheritsFrom: JSON.stringify(inheritsFrom)});
            });
            btnAddInheritFrom.OnClicked.addHandler(function(){
                clove.inputDialog({question: shwebui.strings.SpecifyNewProfileEntry}).addHandler(function(e){
                    if (e.answer) {
                        var found = false;
                        for (var chk of pnlInheritFrom.childrenWidgets()) {
                            if (chk.label() == e.answer) {
                                found = true;
                                break;
                            }
                        }
                        if (!found)
                            pnlInheritFrom.addChild({view: 'CheckButton', label: e.answer, checked: true});
                    }
                });
            });
            var busytoken = this.registerBusy();
            this.ajax({url: 'storeprofile/get', success: function(data) {
                self.settingsdata = data.settings;
                txtForProfile.setPopupItems(clove.utils.arrayToDatasource(data.profiles));
                txtForProfile.setText(data.currentProfile);
                var chldsinheritfrom = [];
                for (var profile of data.profiles)
                    chldsinheritfrom.push({view: 'CheckButton', label: profile});
                pnlInheritFrom.setChildren(chldsinheritfrom);
                self._generateSettingsList();
                self.unregisterBusy(busytoken);
            }});
            this.setRestrictToDirectory(this.config.restrictToDirectory);
            this.setAdvancedMode(this.config.advancedMode);
        }

        _generateSettingsList() {
            var pnlSettings = this.containingNameScope().getByName('pnlSettings');
            var chlds = [];
            var lastgroup;
            var adv = this.advancedMode();
            var global = !this.restrictToDirectory();
            for (var setting of this.settingsdata) {
                var chld = Object.assign({}, setting);
                if (adv || !setting.isAdvanced) {
                    if (global || !setting.isGlobal) {
                        if (lastgroup != setting.groupDescription) {
                            lastgroup = setting.groupDescription;
                            chlds.push({label: setting.groupDescription, styleClass: 'shwebui_storeprofiledlg_settinggrouphead'});
                        }
                        if (adv && setting._additionalCheck) {
                            chld['additionalCheck'] = setting['_additionalCheck'];
                            chld['additionalCheckValue'] = setting['_additionalCheckValue'];
                        }
                        chld['view'] = 'shwebui.SettingUIFrame';
                        chld['withCheckbox'] = true;
                        chld['checked'] = !adv;
                        chlds.push(chld);
                    }
                }
            }
            pnlSettings.setChildren(chlds);
        }

    }

    shwebui.StoreProfileDialog = StoreProfileDialog;

})();
