/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of Shallot.

 Shallot is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Shallot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Shallot.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


(function(){

    class TuningDialogConfigElement extends clove.NameScope(clove.VerticalStack) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                rows: [
                    {rows: [
                        {view: 'Label', name: 'lblDescription', styleClass: 'shwebui_tuningdlg_cfgelem_desc'},
                        {view: 'Label', name: 'lblLongDescription', styleClass: 'shwebui_tuningdlg_cfgelem_longdesc'},
                        {view: 'Label', name: 'lblChangeHint', styleClass: 'shwebui_tuningdlg_cfgelem_changehint'},
                        {cols: [
                            {view: 'Label', name: 'lblValue', styleClass: 'shwebui_tuningdlg_cfgelem_value', hstretch: 1},
                            {view: 'Button', label: shwebui.strings.Change, name: 'btnChange', styleClass: 'shwebui_dlgbutton'},
                        ]},
                    ], styleClass: 'shwebui_tuningdlg_cfgelem_inner', hstretch: 1}
                ],
                hstretch: 1,
            }), domnode);
            this.declareProperty("cfgval");
            this.addStyleClass('shwebui_tuningdlg_cfgelem');
            this.OnChange = new clove.Event();
        }

        doinit() {
            var self = this;
            super.doinit();
            var btnChange = this.getByName("btnChange");
            btnChange.OnClicked.addHandler(function(){
                self.OnChange.trigger({widget: self, cfgval: self.cfgval()});
            });
        }

        cfgval_setter(c) {
            var btnChange = this.getByName("btnChange");
            var lblDescription = this.getByName('lblDescription');
            var lblValue = this.getByName('lblValue');
            var lblChangeHint = this.getByName('lblChangeHint');
            var lblLongDescription = this.getByName('lblLongDescription');
            btnChange.setVisibility(c.mayChange ? clove.Visible : clove.Invisible);
            lblDescription.setLabel(c.description);
            lblValue.setLabel(c.valueDescription);
            lblLongDescription.setLabel(c.longDescription);
            lblLongDescription.setVisibility(c.longDescription ? clove.Visible : clove.InvisibleCollapsed);
            lblChangeHint.setLabel(c.changeHint);
            lblChangeHint.setVisibility((c.changeHint && ['Boolean', 'LocalFilePath'].indexOf(this.config.valueType)!=-1) ? clove.Visible : clove.InvisibleCollapsed);
        }

    }

    shwebui.TuningDialogConfigElement = TuningDialogConfigElement;

    class TuningDialogPanel extends clove.NameScope(clove.VerticalStack) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                rows: [
                    {view: 'EditBox', name: 'edtSearch', hstretch: 1, hintText: shwebui.strings.Search, styleClass: 'shwebui_tuningdlg_panel_searchbox'}, //TODO# use EditBox clear button once Clove offers it
                    {view: 'ScrollView', body: {rows: [], name: 'pnl', hstretch: 1}},
                    {view: 'Label', name: 'lblFooter', styleClass: 'shwebui_tuningdlg_panel_footer'},
                ],
            }), domnode);
            this.declareProperty("footer");
            this.declareProperty("elements");
            this._desc2elems = {};
            this.OnChange = new clove.Event();
        }

        footer_setter(t) {
            var lblFooter = this.getByName('lblFooter');
            lblFooter.setLabel(t);
            lblFooter.setVisibility(t ? clove.Visible : clove.InvisibleCollapsed);
        }

        elements_setter(l) {
            var self = this;
            var pnl = this.getByName("pnl");
            var welems = [];
            var fctOnChange = function(e){ self.OnChange.trigger({widget: self, cfgval: e.cfgval}); };
            for (var w of (l || []))
                welems.push({view: 'shwebui.TuningDialogConfigElement', OnChange: fctOnChange, cfgval: w});
            pnl.setChildren(welems);
        }

        doinit() {
            var self = this;
            super.doinit();
            this.edtSearch = this.getByName("edtSearch");
            this.edtSearch.OnChanged.addHandler(function(){
                self._filter();
            });
        }

        _filter() {
            var ls = this.edtSearch.text().toLowerCase().replace('.', ' ').replace(':', ' ').split(' ');
            var pnl = this.getByName("pnl");
            for (var w of pnl.childrenWidgets()) {
                var vis = clove.Visible;
                var klwo = w.cfgval().description.toLowerCase();
                for (var ssp of ls) {
                    if (ssp && klwo.indexOf(ssp) == -1) {
                        vis = clove.InvisibleCollapsed;
                        break;
                    }
                }
                w.setVisibility(vis);
            }
        }

        updateConfigurationValue(cfgval) {
            var pnl = this.getByName("pnl");
            var w;
            for (var _w of pnl.childrenWidgets()) {
                if (_w.cfgval().name === cfgval.name) {
                    w = _w;
                    break;
                }
            }
            if (!w)
                throw new Error("no config ui element found for: " + cfgval.name);
            w.setCfgval(cfgval);
        }

    }

    shwebui.TuningDialogPanel = TuningDialogPanel;

    class TuningDialog extends shwebui.Dialog {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                title: shwebui.strings.Tuning,
                headtext1: shwebui.strings.TuningHead1,
                headtext3: shwebui.strings.TuningHead3,
                rightHeadButtons: [{view: 'Button', label: shwebui.strings.Close, name: 'btnClose'},],
                innerBody: [{view: 'TabView', width: '450pt', height: '250pt', tabs: [
                               {view: 'shwebui.TuningDialogPanel', name: 'pnlAppear', tabLabel: shwebui.strings.TuningAppearance},
                               {view: 'shwebui.TuningDialogPanel', name: 'pnlBehavior', tabLabel: shwebui.strings.TuningBehavior},
                               {view: 'shwebui.TuningDialogPanel', name: 'pnlExternal', tabLabel: shwebui.strings.TuningExternalTools},
                               {view: 'shwebui.TuningDialogPanel', name: 'pnlInternal', footer: shwebui.strings.TuningInternalPanelFooterWarning, tabLabel: shwebui.strings.TuningInternal},
                           ]}],
            }), domnode);
            this.addStyleClass('shwebui_tuningdlg');
        }

        doinit() {
            var self = this;
            super.doinit();
            this.btnClose = this.containingNameScope().getByName('btnClose');
            this.panels = [{name: 'GUI', widget: this.containingNameScope().getByName('pnlAppear')},
                           {name: 'Behavior', widget: this.containingNameScope().getByName('pnlBehavior')},
                           {name: 'ExternalTools', widget: this.containingNameScope().getByName('pnlExternal')},
                           {name: 'None', widget: this.containingNameScope().getByName('pnlInternal')}];
            this.mpanels = {};
            for (var kp of this.panels)
                this.mpanels[kp.name] = kp;
            this.btnClose.OnClicked.addHandler(function(){
                self.close();
            });
            this.setBusy(true);
            this.ajax({url: "tuning/get_configurationvalues", success: function(data){
                var dByPanel = {};
                for (var d of data) {
                    var panel = self.mpanels[d.category];
                    if (!panel)
                        throw new Error("unknown category: " + d.category);
                    var ll = dByPanel[panel.name];
                    if (!ll)
                        ll = dByPanel[panel.name] = [];
                    ll.push(d);
                }
                for (var panel of self.panels) {
                    var ll = dByPanel[panel.name];
                    if (ll)
                        panel.widget.setElements(ll);
                }
                self.getEvent("tuning/configurationvalue_changed").addHandler(function(e){ // do it that late in order to drop some redundant early events
                    var panel;
                    for (var p of self.panels) {
                        if (p.name === e.category) {
                            panel = p.widget;
                            break;
                        }
                    }
                    if (!panel)
                        throw new Error("no panel found for category: " + e.category);
                    panel.updateConfigurationValue(e);
                }, self);
                self.setBusy(false);
            }});
            for (var panel of self.panels)
                panel.widget.OnChange.addHandler(self._createOnChangeHandler());
        }

        _createOnChangeHandler() {
            var self = this;
            return function(e) {
                var ec = e.cfgval;
                var doChange = function(data) {
                    data = data || {};
                    data.name = ec.name;
                    data.category = ec.category;
                    self.ajax({url: 'tuning/change_configurationvalue', data: data});
                };
                var changeTextual = function(checkfct) {
                    var sarguments = arguments;
                    var question = ec.description;
                    if (ec.changeHint)
                        question += "\n\n" + ec.changeHint;
                    else if (ec.longDescription)
                        question += "\n\n" + ec.longDescription;
                    clove.inputDialog({question: question, defaultAnswer: ec.valueDescription}).addHandler(function(ei){
                        if (ei.answer !== undefined) {
                            var a = checkfct ? checkfct(ei.answer) : ei.answer;
                            if (a !== undefined)
                                doChange({textual: ei.answer});
                            else
                                changeTextual(...sarguments);
                        }
                    });
                };
                var vt = ec.valueType;
                if (vt == 'LocalFilePath')
                    changeTextual();
                else if (vt == 'String')
                    changeTextual();
                else if (vt == 'Integer')
                    changeTextual(function(s){ return parseInt(s)||undefined; });
                else if (vt == 'Float')
                    changeTextual(function(s){ return parseFloat(s)||undefined; });
                else if (vt == 'Boolean')
                    doChange();
                else
                    throw new Error("unknown valueType: " + vt);
            };
        }

    }

    shwebui.TuningDialog = TuningDialog;

})();
