/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2016-2020, Josef Hahn

 This file is part of Clove.

 Clove is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Clove is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Clove.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/

"use strict";

window.clove = new Object();

(function (){

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          CONSTANTS

    clove.Visible = "clove_visible";
    clove.Invisible = "clove_invisible";
    clove.InvisibleCollapsed = "clove_invisiblecollapsed";

    clove.symbols = {
        Actions: "☰",
        Add: "➕",
        ArrowDown: "↓",
        ArrowLeft: "←",
        ArrowLeftRight: "↔",
        ArrowRight: "→",
        ArrowUp: "↑",
        ArrowUpDown: "↕",
        Change: "✐",
        CheckMark: "✓",
        CheckX: "✗",
        Circle: "⚪",
        CircleFilled: "⚫",
        DocumentWithText: "🖹",
        DocumentWithTextAndPicture: "🖺",
        Error: "❎",
        FolderClosed: "📁",
        FolderOpen: "📂",
        Forbidden: "⛔",
        Information: "🛈",
        LockClosed: "🔒",
        LockOpen: "🔓",
        Question: "❔",
        Remove: "✕",
        RotateCCW: "⭯",
        RotateCW: "⭮",
        Search: "🔎",
        SmileyBad: "☹",
        SmileyGood: "☺",
        Star: "☆",
        StarFilled: "★",
        Timer: "⏲",
        TriangleDown: "▼",
        TriangleLeft: "◀",
        TriangleRight: "▶",
        TriangleUp: "▲",
        Waiting: "⌛",
        Warning: "⚠",
    };

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          I18N

    class I18N {

        static parseLangCode(lang) {
            lang = lang.replace("_", "-");
            var i = lang.indexOf("-");
            if (i>-1)
                return lang.substring(0, i).toLowerCase() + "-" + lang.substring(i+1).toUpperCase();
            else
                return lang.toLowerCase() + "-";
        }

        constructor() {
            this.strings = {};
            this.setLanguage(navigator.language);
        }

        setLanguage(langcode) {
            this.lang = I18N.parseLangCode(langcode);
            for(var k in this.strings)
                this.addString(k, this.strings[k]);
        }

        addString(id, texts) {
            var _c = {};
            var _m1 = -1;
            var _m2 = -1;
            var _m3 = -1;
            var _m4 = -1;
            for(var k in texts) {
                var x = I18N.parseLangCode(k);
                if (x == this.lang)
                    _m1 = k;
                else if (this.lang.indexOf(x)==0)
                    _m2 = k;
                else if (x == "en-" || x == "en-US")
                    _m3 = k;
                else if (_m4 == -1)
                    _m4 = k;
            }
            var _m;
            if (_m1 != -1)
                _m = _m1;
            else if (_m2 != -1)
                _m = _m2;
            else if (_m3 != -1)
                _m = _m3;
            else if (_m4 != -1)
                _m = _m4;
            this[id] = texts[_m];
            this.strings[id] = texts;
        }

    }

    clove.i18n = new I18N();

    // >> You should translate those strings.
    clove.i18n.addString('OK', {en: "OK", de: "OK"});
    clove.i18n.addString('Cancel', {en: "Cancel", de: "Abbrechen"});
    clove.i18n.addString('SwitchView', {en: "Switch view", de: "Ansicht wechseln"});
    clove.i18n.addString('Yes', {en: "Yes", de: "Ja"});
    clove.i18n.addString('No', {en: "No", de: "Nein"});

    // >> You should translate those strings for some optional features of clove.DataView.
    clove.i18n.addString('DataView_InsertRow', {en: "Insert row", de: "Zeile einfügen"});
    clove.i18n.addString('DataView_InsertRowBefore', {en: "Insert row before current", de: "Zeile vor Aktueller einfügen"});
    clove.i18n.addString('DataView_InsertColumn', {en: "Insert column", de: "Spalte einfügen"});
    clove.i18n.addString('DataView_InsertColumnBefore', {en: "Insert column before current", de: "Spalte vor Aktueller einfügen"});
    clove.i18n.addString('DataView_InsertChild', {en: "Insert child", de: "Kindknoten einfügen"});
    clove.i18n.addString('DataView_Edit', {en: "Edit", de: "Bearbeiten"});
    clove.i18n.addString('DataView_RemoveRow', {en: "Remove row", de: "Zeile entfernen"});
    clove.i18n.addString('DataView_RemoveColumn', {en: "Remove column", de: "Spalte entfernen"});
    clove.i18n.addString('DataView_ConfirmRemoveRows', {en: "Do you really want to remove rows {items}?", de: "Möchten Sie wirklich die Zeilen {items} löschen?"});
    clove.i18n.addString('DataView_ConfirmRemoveRow', {en: "Do you really want to remove row {items}?", de: "Möchten Sie wirklich die Zeile {items} löschen?"});
    clove.i18n.addString('DataView_ConfirmRemoveColumns', {en: "Do you really want to remove columns {items}?", de: "Möchten Sie wirklich die Spalten {items} löschen?"});
    clove.i18n.addString('DataView_ConfirmRemoveColumn', {en: "Do you really want to remove column {items}?", de: "Möchten Sie wirklich die Spalte {items} löschen?"});

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          EVENT

    class EventArgs {

        constructor(params) {
            this._skipExecution = false;
            for(var i in params)
                this[i] = params[i];
        }

        skipExecution()
        {
            this._skipExecution = true;
        }

    }

    clove.EventArgs = EventArgs;

    class Event {

        constructor() {
            this._handlers = [];
        }

        trigger(params) {
            var xxx = clove.utils.suspendResizing();
            try {
                var eventargs = new clove.EventArgs(params);
                var handlers = this._handlers.slice();
                for(var i=0; i<handlers.length; i++) {
                    var h = handlers[i];
                    h(eventargs);
                    if (eventargs._skipExecution)
                        break;
                }
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
            return eventargs;
        }

        hasHandlers() {
            return this._handlers.length > 0;
        }

        addHandler(fct, owner) {
            var self = this;
            if (owner) {
                owner.OnDestroyed.addHandler(function(){  //TODO #496
                    self.removeHandler(fct);
                });
            }
            this._handlers.push(fct);
            return fct;
        }

        addHandlerOnce(fct, owner) {
            var self = this;
            var ffct = function(e) {
                self.removeHandler(ffct);
                fct(e);
            }
            this.addHandler(ffct, owner);
        }

        removeHandler(fct) {
            var i = this._handlers.indexOf(fct);
            if (i > -1)
                this._handlers.splice(i, 1);
        }

    }

    clove.Event = Event;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          NAMESCOPE

    let NameScope = (superclass) => class extends superclass {

        constructor(config, domnode) {
            super(config, domnode);
            this._childnamescopes = [];
            if (this.topnode)
                this.topnode.dataset.clove_namescope = "1";
        }

        static _getByName(name, node, childnamescopes) {
            if (!name)
                return;
            if (!node)
                node = document.body;
            var result;
            for(let cnode of node.childNodes) {
                if (cnode.isConnected !== undefined) {
                    if (!cnode.isConnected)
                        continue;
                }
                else if (!document.contains(cnode))
                    continue;
                if (cnode.nodeType == 1) {
                    var nname = cnode.getAttribute('data-clove_name');
                    if (nname == name)
                    {
                        result = cnode._clove_widget;
                        break;
                    }
                    if (!cnode.getAttribute('data-clove_namescope'))
                    {
                        result = RootNameScope._getByName(name, cnode);
                        if (result)
                            break;
                    }
                }
            }
            if (!result)
            {
                if (childnamescopes)
                    for(let cn of childnamescopes) {
                        result = cn.getByName(name);
                        if (result)
                            break;
                    }
            }
            return result;
        }

        getByName(name) {
            return RootNameScope._getByName(name, this.contentnode?this.contentnode:document.body, this._childnamescopes);
        }

        addChildNameScope(childnamescope) {
            var self = this;
            this._childnamescopes.push(childnamescope);
            clove.utils.addOnDestroyHandler(childnamescope.contentnode, function() {
                self._childnamescopes.splice(self._childnamescopes.indexOf(childnamescope), 1);
            });
        }

    }

    clove.NameScope = NameScope;

    class RootNameScope extends clove.NameScope(Object) {

        constructor(a, b) {
            super(a, b);
        }

        nameScope() {
            return this;
        }

        containingNameScope() {
            return undefined;
        }

    }

    clove.RootNameScope = RootNameScope;

    clove.rootNameScope = new RootNameScope();

    clove.getByName = function(name) { return clove.rootNameScope.getByName(name); };

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          AUXILIARY

    clove.utils = new Object();

    clove.utils.OnMainResized = new clove.Event();

    var _main_parts_resized_flag = false;
    clove.utils.main_parts_resized = function (reason) {
        clove.geometryFingerprint = clove.utils.getGeometryFingerprint();
        if (_main_parts_resized_flag)
            return;
        _main_parts_resized_flag = true;
        setTimeout(function(){
            _main_parts_resized_flag = false;
            clove.utils.OnMainResized.trigger({reason: reason||{}});
        });
    }

    window.onresize = clove.utils.main_parts_resized;

    clove.utils.applyDefaultsToConfig = function(config, defaults) {
        var result = {};
        for (var k in config)
            result[k] = config[k];
        for (var k in defaults)
            if (!result.hasOwnProperty(k))
                result[k] = defaults[k];
        return result;
    };

    $(document).ready(function(){
        var observer = new MutationObserver(function(mutations) {
            for(let mutation of mutations) {
                for (let mn of mutation.removedNodes) {
                    if (mn._ondestroyedhandlers)
                        for (let handler of mn._ondestroyedhandlers) {
                            if (!handler._wascalled) {
                                handler._wascalled = true;
                                handler();
                            }
                        }
                }
            }
        });
        observer.observe(document.body, { childList: true, subtree: true });
    });

    clove.utils.addOnDestroyHandler = function(domnode, handler) {
        var n = domnode;
        while (n) {
            if (!n._ondestroyedhandlers)
                n._ondestroyedhandlers = [];
            n._ondestroyedhandlers.push(handler);
            n = n.parentNode;
        }
    };

    clove.utils.addOnDragHandler = function(domnode, dragbeginhandler, draghandler, dragendhandler) {
        $(domnode).mousedown(function(e){
            var mouseup;
            var mousemove;
            var beginx = e.originalEvent.pageX;
            var beginy = e.originalEvent.pageY;
            mouseup = function(e){
                $(document.body).off("mouseup", mouseup);
                $(document.body).off("mousemove", mousemove);
                if (dragendhandler())
                    e.stopPropagation();
            };
            mousemove = function(e){
                if (draghandler(e.originalEvent.pageX - beginx, e.originalEvent.pageY - beginy))
                    e.stopPropagation();
            };
            $(document.body).mouseup(mouseup);
            $(document.body).mousemove(mousemove);
            if (dragbeginhandler())
                e.preventDefault();
        });
        $(domnode).bind('touchstart', function(e){
            var touchend;
            var touchmove;
            var beginx = e.originalEvent.changedTouches[0].pageX;
            var beginy = e.originalEvent.changedTouches[0].pageY;
            touchend = function(e){
                $(document.body).off("touchend", touchend);
                $(document.body).off("touchcancel", touchend);
                $(document.body).off("touchmove", touchmove);
                if (dragendhandler())
                    e.stopPropagation();
            };
            touchmove = function(e){
                if (draghandler(e.originalEvent.changedTouches[0].pageX - beginx, e.originalEvent.changedTouches[0].pageY - beginy))
                    e.stopPropagation();
            };
            $(document.body).bind("touchend", touchend);
            $(document.body).bind("touchcancel", touchend);
            $(document.body).bind("touchmove", touchmove);
            if (dragbeginhandler())
                e.preventDefault();
        });
    }

    clove.utils.addOnDoubleClickTapHandler = function(domnode, handler) {
        domnode.addEventListener('dblclick', handler);
        handler._handlertouchstart = function(e) {
            handler._touchstart = (new Date()).getTime();
        }
        domnode.addEventListener('touchstart', handler._handlertouchstart);
        handler._handlertouchend = function(e) {
            if ((new Date()).getTime() - handler._touchstart < 300) {
                if (handler._firsttouch && (new Date()).getTime() - handler._firsttouch < 600 && handler._firsttouch_target == e.target)
                    handler(e);
                else {
                    handler._firsttouch = (new Date()).getTime();
                    handler._firsttouch_target = e.target;
                }
            }
        }
        domnode.addEventListener('touchend', handler._handlertouchend);
    }

    clove.utils.removeOnDoubleClickTapHandler = function(domnode, handler) {
        domnode.removeEventListener('dblclick', handler);
        domnode.removeEventListener('touchstart', handler._handlertouchstart);
        domnode.removeEventListener('touchend', handler._handlertouchend);
        handler._handlertouchstart = undefined;
        handler._handlertouchend = undefined;
        handler._firsttouch_target = undefined;
    }

    clove.utils.arraysum = function (x) {
        var result = 0;
        for (let i of x)
            result += i;
        return result;
    };

    clove.utils.isDescendantOf = function(d, p) {
        if (d.parentNode == p)
            return true;
        if (!d.parentNode)
            return false;
        return clove.utils.isDescendantOf(d.parentNode, p);
    };

    clove.utils.findWidgetForDomNode = function(node) {
        while (node) {
            if (node._clove_widget)
                return node._clove_widget;
            node = node.parentNode;
        }
    };

    function _getCorrectComputedDims(st, node) {
        // this is because getComputedStyle does not always return useful information
        // it those cases, we have to fiddle the stuff manually
        var result = st;
        if (st.width == "auto") {
            var jnode = $(node);
            result = {width: jnode.width(), height: jnode.height()};
        }
        return result;
    }

    clove.utils.getExtraSize = function(node, withMargin) {
        var w = node.offsetWidth - node.clientWidth;
        var h = node.offsetHeight - node.clientHeight;
        if (withMargin) {
            var st = getComputedStyle(node);
            w += parseFloat(st.marginLeft) + parseFloat(st.marginRight);
            h += parseFloat(st.marginTop) + parseFloat(st.marginBottom);
        }
        return {width: w, height: h};
    };

    clove.utils.getInnerSize = function(node) {
        var st = getComputedStyle(node);
        var sst = _getCorrectComputedDims(st, node);
        return {width: parseFloat(sst.width)-node.offsetWidth+node.clientWidth+parseFloat(st.borderRightWidth)+parseFloat(st.borderLeftWidth), height: parseFloat(sst.height)-node.offsetHeight+node.clientHeight+parseFloat(st.borderTopWidth)+parseFloat(st.borderBottomWidth)};
    };

    clove.utils.getOuterSize = function(node, withMargin) {
        var st = getComputedStyle(node);
        var sst = _getCorrectComputedDims(st, node);
        var w = parseFloat(sst.width);
        var h = parseFloat(sst.height);
        if (st.boxSizing != "border-box") {
            w += parseFloat(st.paddingLeft) + parseFloat(st.paddingRight) + parseFloat(st.borderLeftWidth) + parseFloat(st.borderRightWidth);
            h += parseFloat(st.paddingTop) + parseFloat(st.paddingBottom) + parseFloat(st.borderTopWidth) + parseFloat(st.borderBottomWidth);
        }
        if (withMargin) {
            w += parseFloat(st.marginLeft) + parseFloat(st.marginRight);
            h += parseFloat(st.marginTop) + parseFloat(st.marginBottom);
        }
        return {width: w, height: h};
    };

    var _clove_resizesuspended = undefined;
    var _clove_resizesuspended_counter = [];

    clove.utils.suspendResizing = function() {
        var token = new Object();
        if (!_clove_resizesuspended)
            _clove_resizesuspended = new Set();
        _clove_resizesuspended_counter.push(token);
        return token;
    }

    clove.utils.resumeResizing = function(token) {
        _clove_resizesuspended_counter.splice(_clove_resizesuspended_counter.indexOf(token), 1);
        if (_clove_resizesuspended_counter.length == 0) {
            var slst = _clove_resizesuspended;
            _clove_resizesuspended = undefined;
            for(let x of slst) {
                var force = x._force_resize;
                x._force_resize = undefined;
                x.resize(force);
            }
        }
    }

    clove.utils.arrayToDatasource = function(array) {
        var result = new clove.NativeDatasource();
        result.root().addColumn([]);
        for(var i=0; i<array.length; i++)
            result.root().addRow([array[i]]);
        return result;
    }

    clove.utils.animateNode = function(node, animationname, animationduration, animationendedfct) {
        if (node.style.animationName == animationname) {
            node.style.animationName = '';
            setTimeout(function(){
                clove.utils.animateNode(node, animationname, animationduration, animationendedfct);
            });
        }
        else {
            function _animationended() {
                node.removeEventListener('animationend', _animationended);
                node.style.animationName = '';
                if (animationendedfct)
                    animationendedfct();
            }
            node.addEventListener('animationend', _animationended, false);
            node.style.animationDuration = (animationduration/1000)+"s";
            node.style.animationFillMode = 'forwards';
            node.style.animationName = animationname;
        }
    }

    clove.utils.setStyleClassAssigned = function(node, clss, assigned) {
        var jn = $(node);
        if (!assigned)
            jn.removeClass(clss);
        else if (!jn.hasClass(clss))
            jn.addClass(clss);
    }

    var _geometryfingerprinthelper;

    clove.utils.getGeometryFingerprint = function() {
        if (!_geometryfingerprinthelper) {
            _geometryfingerprinthelper = new Array();
            _geometryfingerprinthelper.push($("<span style='display: inline-block; visibility:hidden; font-family:serif;font-size:2em;'>abcdef49<br/></span>")[0]);
            _geometryfingerprinthelper.push($("<span style='display: inline-block; visibility:hidden; font-family:sans-serif;'>ghijkl5-<br/></span>")[0]);
            _geometryfingerprinthelper.push($("<span style='display: inline-block; visibility:hidden; font-size:1.2em;font-weight:bold'>mnopqr6.<br/></span>")[0]);
            _geometryfingerprinthelper.push($("<span style='display: inline-block; visibility:hidden; font-family:monospace;'>stuvwx7:<br/></span>")[0]);
            _geometryfingerprinthelper.push($("<span style='display: inline-block; visibility:hidden; font-size:0.2em;'>yz01238,<br/></span>")[0]);
            for(var i=0; i<_geometryfingerprinthelper.length; i++)
                document.body.appendChild(_geometryfingerprinthelper[i]);
        }
        var result = "";
        for(var i=0; i<_geometryfingerprinthelper.length; i++) {
            var s = clove.utils.getOuterSize(_geometryfingerprinthelper[i]);
            result += "#" + s.width + "#" + s.height;
        }
        return result;
    }

    clove.geometryFingerprint = "#";

    clove._ajaxprovider = $.ajax;

    var _loaddeferby = [];
    var _loadedhandlers = [];

    clove.utils.deferLoading = function(eload) {
        _loaddeferby.push(eload);
        eload.addHandlerOnce(function(e){
            _loaddeferby.splice(_loaddeferby.indexOf(eload), 1);
            $(document).ready(function(){
                if (!_loaddeferby.length) {
                    if (_loadedhandlers) {
                        for(let loadedhandler of _loadedhandlers)
                            loadedhandler();
                        _loadedhandlers = undefined;
                    }
                }
            });
        });
    };

    clove.utils.runOnLoaded = function(fct) {
        $(document).ready(function(){
            if (_loaddeferby.length && _loadedhandlers)
                _loadedhandlers.push(fct);
            else
                setTimeout(fct);
        });
    };

    clove.utils.getFullPathForLoadedResource = function(tgtname, elementtype, elementsrcattrname) {
        tgtname = tgtname || "clove.js";
        elementtype = elementtype || "script";
        elementsrcattrname = elementsrcattrname || ((elementtype == "style") ? "href" : "src");
        var xe1 = $(elementtype + '['+elementsrcattrname+'="'+tgtname+'"]');
        var xe;
        if (xe1.length)
            xe = xe1[0];
        var xe2 = $(elementtype + '['+elementsrcattrname+'$="/'+tgtname+'"]');
        if (xe2.length)
            xe = xe2[0];
        if (xe)
            return xe[elementsrcattrname];
    }

    function _tryLoadHelper(tgtpath, config, elementtype, elementsrcattrname, addnodefct) {
        var isl = tgtpath.lastIndexOf("/");
        var tgtname = (isl == -1) ? tgtpath : tgtpath.substring(isl+1);
        var xe = clove.utils.getFullPathForLoadedResource(tgtname, elementtype, elementsrcattrname);
        if (xe)
            return xe;
        config = clove.utils.applyDefaultsToConfig(config);
        var fullpath;
        if (tgtpath.startsWith('/'))
            fullpath = tgtpath;
        else {
            var fpath = clove.utils.getFullPathForLoadedResource(config.rootrefscript);
            if (fpath) {
                var isla = fpath.lastIndexOf('/');
                var dirpath = (isla == -1) ? '' : fpath.substring(0, isla+1);
                fullpath = dirpath + tgtpath;
            }
        }
        if (fullpath) {
            var node = document.createElement(elementtype);
            var loadedevent = new clove.Event();
            clove.utils.deferLoading(loadedevent);
            $(node).one('load', function() {
                loadedevent.trigger({});
                if (config.onload)
                    config.onload();
            });
            node[elementsrcattrname] = fullpath;
            addnodefct(node);
            return fullpath;
        }
    }

    function _tryLoadHelper_addnode_style(node) {
        node.rel = 'stylesheet';
        node.type = 'text/css';
        document.head.appendChild(node);
    }

    var _tryLoadHelper_addnode_script_firstscript;
    function _tryLoadHelper_addnode_script(node) {
        if (!_tryLoadHelper_addnode_script_firstscript)
            _tryLoadHelper_addnode_script_firstscript = $('script:first')[0];
        node.type = 'text/javascript';
        _tryLoadHelper_addnode_script_firstscript.parentNode.insertBefore(node, _tryLoadHelper_addnode_script_firstscript);
    }

    clove.utils.tryLoadScript = function(scriptpath, config) {
        return _tryLoadHelper(scriptpath, config, 'script', 'src', _tryLoadHelper_addnode_script);
    }

    clove.utils.tryLoadStyle = function(stylepath, config) {
        return _tryLoadHelper(stylepath, config, 'link', 'href', _tryLoadHelper_addnode_style);
    }

    clove.utils.tryLoadStyle("clove.css");

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          INFRASTRUCTURE

    clove.build = function(config, buildconfig) {
        var xxx = clove.utils.suspendResizing();
        try {
            if (!buildconfig)
                buildconfig = {};
            if (!buildconfig.domnode && !buildconfig.domparentnode)
                buildconfig.domparentnode = document.body;
            if (buildconfig.domparentnode) {
                buildconfig.domnode = document.createElement("div");
                buildconfig.domnode.style.position = "absolute";
                buildconfig.domnode.style.top = "0";
                buildconfig.domnode.style.left = "0";
                buildconfig.domnode.style.bottom = "0";
                buildconfig.domnode.style.right = "0";
                buildconfig.domparentnode.appendChild(buildconfig.domnode);
            }
            buildconfig.domnode.style.zIndex = buildconfig.zIndex || 0;
            if ("view" in config)
                var sstype = config.view;
            else if ("rows" in config)
                var sstype = "VerticalStack";
            else if ("cols" in config)
                var sstype = "HorizontalStack";
            else if ("width" in config)
                var sstype = "Label";
            else if ("height" in config)
                var sstype = "Label";
            else if ("label" in config)
                var sstype = "Label";
            else
                throw new Error("Invalid or incomplete ui config: " + config);
            var cnstr = eval(sstype);
            if (config.newNameScope)
                cnstr = clove.NameScope(cnstr);
            var result = new cnstr(config, buildconfig.domnode);
            if (buildconfig.parent)
                result._parent = buildconfig.parent;
            result.init(buildconfig.nameScope);
            if (!buildconfig.parent)
                clove.utils.OnMainResized.addHandler(function(){ result.resize(); }, result);
            if (buildconfig.animationName)
                clove.utils.animateNode(result.topnode, buildconfig.animationName, buildconfig.animationDuration||150);
        }
        finally {
            clove.utils.resumeResizing(xxx);
        }
        return result;
    };

    var _clove_geomeasurediv = undefined;

    $(document).ready(function(){
        _clove_geomeasurediv = document.createElement("div");
        _clove_geomeasurediv.style.display = "none";
        document.body.appendChild(_clove_geomeasurediv);
        clove.geometryFingerprint = clove.utils.getGeometryFingerprint();
    });

    clove.utils.cssLengthToPixels = function(v, axis) {
        if (v === undefined)
            return undefined;
        else if (v.endsWith("px"))
            return parseFloat(v);
        var n = axis=="y" ? "height" : "width";
        _clove_geomeasurediv.style[n] = v;
        var result = parseFloat(getComputedStyle(_clove_geomeasurediv)[n]);
        _clove_geomeasurediv.style[n] = "";
        return result;
    };

    clove.utils.insertToArray = function(array, i, v) {
        if (array.length == i)
            array.push(v);
        else
            array.splice(i, 0, v);
    }

    var _uiframes = [{base:true}];

    function _currentUiFrame() {
        for (var i=_uiframes.length-1; i>=0; i--) {
            var x = _uiframes[i];
            if (x.widget && !x.widget.isAlive())
                _uiframes.splice(i, 1);
            else
                break;
        }
        return _uiframes[i];
    }

    clove.utils.popup = function(config, showconfig) {
        var dlgnamescope = new clove.RootNameScope();
        showconfig = clove.utils.applyDefaultsToConfig(showconfig, {
            autocenter: true,
            creationAnimationName: "cloveanimation_fadein",
            creationAnimationDuration: 150,
            removalAnimationName: "cloveanimation_fadeout",
            removalAnimationDuration: 150,
            modalPanelCreationAnimationName: "cloveanimation_fadein",
            modalPanelCreationAnimationDuration: 150,
            modalPanelRemovalAnimationName: "cloveanimation_fadeout",
            modalPanelRemovalAnimationDuration: 150,
        });
        var modalpanel;
        if (showconfig.modal) {
            var fullytransparent = (showconfig.fullyTransparent!==undefined) ? showconfig.fullyTransparent : true;
            modalpanel = clove.build({
                view: 'ModalPanel',
                defaultRemovalAnimationName: fullytransparent ? '': showconfig.modalPanelRemovalAnimationName,
                defaultRemovalAnimationDuration: showconfig.modalPanelRemovalAnimationDuration,
                fullyTransparent: fullytransparent,
            }, {
                animationName: showconfig.modalPanelCreationAnimationName,
                animationDuration: showconfig.modalPanelCreationAnimationDuration,
                domparentnode: showconfig.domparentnode, // not set in most cases, though
            });
        }
        var wdiv = document.createElement("div");
        (showconfig.domparentnode || document.body).appendChild(wdiv);
        var w = clove.build(clove.utils.applyDefaultsToConfig(config, {
                doStandaloneResizing: true,
                doStandaloneResizing_clipToParent: true,
                width: showconfig.width,
                height: showconfig.height,
                defaultRemovalAnimationName: showconfig.removalAnimationName,
                defaultRemovalAnimationDuration: showconfig.removalAnimationDuration,
            }), {
                nameScope: dlgnamescope,
                domnode: wdiv,
                animationName: showconfig.creationAnimationName,
                animationDuration: showconfig.creationAnimationDuration,
            });
        var uiframe = {root: w};
        _uiframes.push(uiframe);
        if (showconfig.modal) {
            if (document.activeElement)
                document.activeElement.blur();
            var ondocumentfocus = function(e) {
                if (uiframe != _currentUiFrame())
                    return;
                if (!w.isAlive())
                    return;
                if (!clove.utils.isDescendantOf(e.target, w.topnode)) {
                    e.preventDefault();
                    e.stopPropagation();
                    var dlgchlds = $(w.topnode).find("*");
                    for(var idlgchlds=0; idlgchlds<dlgchlds.length; idlgchlds++) {
                        var dlgchld = dlgchlds[idlgchlds];
                        dlgchld.focus();
                        if (document.activeElement == dlgchld)
                            return;
                    }
                    if (document.activeElement)
                        document.activeElement.blur();
                }
            };
            $(document.body).on('focusin', ondocumentfocus);
            w.OnDestroyed.addHandler(function(){
                modalpanel.remove();
                $(document.body).off('focusin', ondocumentfocus);
            });
            w.OnVisibilityChanged.addHandler(function(){
                modalpanel.setVisibility(_config.visibility());
            });
        }
        w.OnDestroyed.addHandler(function(){
            var ifr = _uiframes.indexOf(uiframe);
            if (ifr > -1)
                _uiframes.splice(ifr, 1);
        });
        if (showconfig.alignTo) {
            var alignnode = showconfig.alignTo;
            var alignwidget;
            if (alignnode instanceof clove.Widget) {
                alignwidget = alignnode;
                alignnode = alignwidget.topnode;
            }
            var alignToAxis = showconfig.alignToAxis || "vertical";
            var alignToInside = showconfig.alignToInside || false;
            var alignSyncDimension = showconfig.alignSyncDimension || false;
            var _align = function() {
                var pr = alignnode.getBoundingClientRect();
                if (alignToInside) {
                    w.topnode.style.left = pr.left + "px";
                    w.topnode.style.top = pr.top + "px";
                    w.topnode.style.width = pr.width + "px";
                    w.topnode.style.height = pr.height + "px";
                } else {
                    var roomt = pr.top;
                    var roomb = window.innerHeight - pr.bottom;
                    var rooml = pr.left;
                    var roomr = window.innerWidth - pr.right;
                    var horz1 = (alignToAxis == "horizontal") ? pr.left : pr.right;
                    var horz2 = (alignToAxis == "horizontal") ? pr.right : pr.left;
                    var vert1 = (alignToAxis == "horizontal") ? pr.bottom : pr.top;
                    var vert2 = (alignToAxis == "horizontal") ? pr.top : pr.bottom;
                    var tnos = clove.utils.getOuterSize(w.topnode);
                    if (rooml > roomr)
                        var npl = horz1 - tnos.width;
                    else
                        var npl = horz2;
                    npl = Math.max(0, Math.min(npl, window.innerWidth - tnos.width));
                    w.topnode.style.left = npl + "px";
                    if (roomt > roomb)
                        var npt = vert1 - tnos.height;
                    else
                        var npt = vert2;
                    npt = Math.max(0, Math.min(npt, window.innerHeight - tnos.height));
                    w.topnode.style.top = npt + "px";
                    if (alignSyncDimension) {
                        if (alignToAxis == "horizontal")
                            w.setHeight(pr.height + "px");
                        else
                            w.setWidth(pr.width + "px");
                    }
                }
            }
            w.OnResized.addHandler(_align);
            if (alignwidget)
                alignwidget.OnResized.addHandler(_align, w);
            else {
                var observer = new MutationObserver(_align);
                observer.observe(alignnode, {
                    attributes: true,
                    childList: true,
                    characterData: true,
                    subtree: true
                });
                w.OnDestroyed.addHandler(function(){
                    observer.disconnect();
                });
            }
            clove.utils.OnMainResized.addHandler(_align, w);
            _align();
        }
        w.topnode.style.position = "absolute";
        if (showconfig.x)
            w.topnode.style.left = showconfig.x;
        if (showconfig.y)
            w.topnode.style.top = showconfig.y;
        if (showconfig.left)
            w.topnode.style.left = showconfig.left;
        if (showconfig.top)
            w.topnode.style.top = showconfig.top;
        if (showconfig.bottom)
            w.topnode.style.bottom = showconfig.bottom;
        if (showconfig.right)
            w.topnode.style.right = showconfig.right;
        if (showconfig.width)
            w.topnode.style.width = showconfig.width;
        if (showconfig.height)
            w.topnode.style.height = showconfig.height;
        if (showconfig.modal && showconfig.panelClickClosesPopup) {
            modalpanel.OnClicked.addHandler(function(){
                w.remove();
            }, w);
        }
        if (showconfig.x || showconfig.y || showconfig.top || showconfig.left || showconfig.bottom || showconfig.right || showconfig.alignTo)
            showconfig.autocenter = false;
        if (showconfig.autocenter) {
            var _center = function() {
                var wr = w.topnode.getBoundingClientRect();
                var x = (window.innerWidth - wr.width) / 2;
                var y = (window.innerHeight - wr.height) / 2;
                w.topnode.style.left = x + "px";
                w.topnode.style.top = y + "px";
            }
            w.OnResized.addHandler(_center);
            clove.utils.OnMainResized.addHandler(_center, w);
            _center();
        }
        return { namescope: dlgnamescope, widget: w, modalpanel: modalpanel };
    };

    clove.populateUI = function(initfct, populateconfig) {
        populateconfig = clove.utils.applyDefaultsToConfig(populateconfig, {
            showLoadingIndicator: true,
        });
        $(document).ready(function(){
            if (populateconfig.showLoadingIndicator)
                var ww = clove.build({view: 'Widget', busy: true});
            clove.utils.runOnLoaded(function(){
                var xxx = clove.utils.suspendResizing();
                try {
                    initfct();
                }
                finally {
                    clove.utils.resumeResizing(xxx);
                    if (populateconfig.showLoadingIndicator)
                        ww.remove({animationName: "cloveanimation_fadeout", animationDuration: 500});
                }
            });
        });
    };

    clove.utils.connectDatasourceToWidget = function(widget, datasource, dspropname, rootptrpropname, inserthandlername, removehandlername, updatehandlername, afterdisconnectedfct, beforeconnectedfct, afterconnectedfct) {
        var wlastds = widget[dspropname];
        var inserttempobjname = dspropname + "_hodi";
        var removetempobjname = dspropname + "_hodr";
        var updatetempobjname = dspropname + "_hodu";
        var wdestroytempobjname = dspropname + "_wdestroy";
        function _disconnectfct(_wlastds) {
            if (_wlastds) {
                _wlastds.OnDataInsert.removeHandler(widget[inserttempobjname]);
                _wlastds.OnDataRemove.removeHandler(widget[removetempobjname]);
                _wlastds.OnDataUpdate.removeHandler(widget[updatetempobjname]);
            }
            if (afterdisconnectedfct)
                afterdisconnectedfct(_wlastds);
            widget[dspropname] = undefined;
        }
        _disconnectfct(wlastds);
        if (datasource) {
            if (beforeconnectedfct)
                beforeconnectedfct();
            var actionsparent = rootptrpropname ? widget.getProperty(rootptrpropname, undefined) : undefined;
            widget[dspropname] = datasource;
            widget[inserttempobjname] = function(e){widget[inserthandlername](e);};
            widget[removetempobjname] = function(e){widget[removehandlername](e);};
            widget[updatetempobjname] = function(e){widget[updatehandlername](e);};
            datasource.OnDataInsert.addHandler(widget[inserttempobjname]);
            datasource.OnDataRemove.addHandler(widget[removetempobjname]);
            datasource.OnDataUpdate.addHandler(widget[updatetempobjname]);
            if (!afterconnectedfct || afterconnectedfct(actionsparent))
                widget[inserthandlername]({r1:0, c1:0, r2:datasource.rowCount(actionsparent)-1, c2:datasource.columnCount(actionsparent)-1, parent: actionsparent});
            if (!widget[wdestroytempobjname]) {
                widget[wdestroytempobjname] = true;
                widget.OnDestroyed.addHandler(function(){
                    _disconnectfct(widget[dspropname]);
                });
            }
        }
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          BROWSER

    clove.browser = new Object();

    clove.browser.webkit = navigator.userAgent.toLowerCase().indexOf("webkit") > -1;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          DATA

    class DatasourceValuePointer {

        constructor(irow, icol, backendObject, datasource) {
            this.irow = irow;
            this.icol = icol;
            this.backendObject = backendObject;
            this.datasource = datasource;
        }

        parent() {
            return this.datasource.parent(this);
        }

        sibling(irow, icol) {
            return this.datasource.valuePointer(irow, icol, this);
        }

        rowCount() {
            return this.datasource.rowCount(this);
        }

        columnCount() {
            return this.datasource.columnCount(this);
        }

        static toString(ptr) {
            if (!ptr)
                return "";
            else
                return DatasourceValuePointer.toString(ptr.parent()) + "/" + ptr.irow + "," + ptr.icol;
        }

        static equals(ptr1, ptr2) {
            if (ptr1 === undefined && ptr2 === undefined)
                return true;
            if (ptr1 === undefined || ptr2 === undefined)
                return false;
            if (ptr1.irow != ptr2.irow)
                return false;
            if (ptr1.icol != ptr2.icol)
                return false;
            return DatasourceValuePointer.equals(ptr1.parent(), ptr2.parent());
        }

        getValue() {
            return this.datasource.getValue(this);
        }

        static toPath(ptr) {
            if (ptr) {
                var result = DatasourceValuePointer.toPath(ptr.parent());
                result.push([ptr.irow, ptr.icol]);
                return result;
            }
            else
                return [];
        }

        static fromPath(datasource, path) {
            var result;
            for (var i=0; i<path.length; i++)
                result = datasource.valuePointer(path[i][0], path[i][1], result);
            return result;
        }

    }

    clove.DatasourceValuePointer = DatasourceValuePointer;

    let Datasource = (superclass) => class extends superclass {

        constructor() {
            super(...arguments);
            this.OnDataInsert = new clove.Event();
            this.OnDataRemove = new clove.Event();
            this.OnDataUpdate = new clove.Event();
            this._ui_hideExpanderOnChildlessNodes = false;
        }

        getValue(ptr) {
        }

        getMetadata(ptr) {
        }

        changeValue(ptr, value) {
        }

        rowCount(parent) {
        }

        columnCount(parent) {
        }

        valuePointer(irow, icol, parent) {
        }

        parent(ptr) {
        }

        valuePointerNavigateInDepth(ptr, direction, mayexpandfct) {
            if (!mayexpandfct)
                mayexpandfct = function(){return true;}
            if (direction == +1) {
                if (this.rowCount(ptr) > 0 && this.columnCount(ptr) > 0 && mayexpandfct(ptr)) {
                    return this.valuePointer(0, 0, ptr);
                } else {
                    while (ptr) {
                        var ptrparent = this.parent(ptr);
                        if (this.rowCount(ptrparent) > (ptr.irow + 1))
                            return this.valuePointer(ptr.irow + 1, 0, ptrparent);
                        ptr = ptrparent;
                    }
                }
            } else if (direction == -1) {
                if (ptr.irow == 0)
                    return this.parent(ptr);
                else {
                    var newparent = this.parent(ptr);
                    var newrow = ptr.irow - 1;
                    var pivotptr = this.valuePointer(newrow, 0, newparent);
                    var descent = true;
                    while (true) {
                        var rowcnt = this.rowCount(pivotptr);
                        if (rowcnt > 0 && this.columnCount(pivotptr) > 0 && mayexpandfct(pivotptr))
                            pivotptr = this.valuePointer(rowcnt - 1, 0, pivotptr);
                        else
                            break;
                    }
                    return pivotptr;
                }
            } else
                throw new Error("Not a valid direction: " + direction);
        }

    }

    clove.Datasource = Datasource;

    let Headersource = (superclass) => class extends superclass {

        constructor() {
            super(...arguments);
            this.OnHeaderDataInsert = new clove.Event();
            this.OnHeaderDataRemove = new clove.Event();
            this.OnHeaderDataUpdate = new clove.Event();
            this.OnHeaderVisibilityUpdated = new clove.Event();
        }

        getRowHeader(irow, parent) {
        }

        getColumnHeader(icol, parent) {
        }

        rowHeadersVisible(parent) {
        }

        columnHeadersVisible(parent) {
        }

    }

    clove.Headersource = Headersource;

    class NativeDatasourceNode {

        constructor(datasource, ir, ic, parent) {
            this.value = undefined;
            this.metadata = new Object();
            this.datasource = datasource;
            this.parent = parent;
            this.childs = [];
            this.rowcnt = 0;
            this.colcnt = 0;
            this.ir = ir;
            this.ic = ic;
            this.rowheaders = [];
            this.colheaders = [];
        }

        rowCount() {
            return this.rowcnt;
        }

        columnCount() {
            return this.colcnt;
        }

        toRow(ir) {
            return this.datasource._getNode(ir, this.ic, this.parent.valuePointer());
        }

        toColumn(ic) {
            return this.datasource._getNode(this.ir, ic, this.parent.valuePointer());
        }

        valuePointer() {
            if (this == this.datasource.root())
                return;
            return this.datasource.valuePointer(this.ir, this.ic, this.parent.valuePointer());
        }

        getValue() {
            return this.value;
        }

        getMetadata() {
            return this.metadata;
        }

        setValue(v) {
            this.datasource.setValue(this.valuePointer(), v);
        }

        setMetadata(k, v) {
            this.datasource.setMetadata(this.valuePointer(), k, v);
        }

        removeRow(ir) {
            this.datasource.removeRow(ir, this.valuePointer());
        }

        removeColumn(ic) {
            this.datasource.removeColumn(ic, this.valuePointer());
        }

        addRow(vals) {
            return this.insertRow(this.datasource.rowCount(this.valuePointer()), vals);
        }

        addColumn(vals) {
            return this.insertColumn(this.datasource.columnCount(this.valuePointer()), vals);
        }

        getChild(irow, icol) {
            return this.childs[irow][icol];
        }

        insertRow(i, vals) {
            var xxx = clove.utils.suspendResizing();
            try {
                if (vals === undefined)
                    vals = [];
                var mvalptr = this.valuePointer();
                var origcnt = this.datasource.columnCount(mvalptr);
                this.datasource.insertRow(i, mvalptr);
                for(var iv=0;iv<vals.length;iv++) {
                    if (iv>=origcnt)
                        this.datasource.insertColumn(iv, mvalptr);
                    this.datasource.setValue(this.datasource.valuePointer(i, iv, this.valuePointer()), vals[iv]);
                }
                return this.childs[i][0];
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        insertColumn(i, vals) {
            var xxx = clove.utils.suspendResizing();
            try {
                if (vals === undefined)
                    vals = [];
                var mvalptr = this.valuePointer();
                var origcnt = this.datasource.rowCount(mvalptr);
                this.datasource.insertColumn(i, mvalptr);
                for(var iv=0;iv<vals.length;iv++) {
                    if (iv>=origcnt)
                        this.datasource.insertRow(iv, mvalptr);
                    this.datasource.setValue(this.datasource.valuePointer(iv, i, this.valuePointer()), vals[iv]);
                }
                var firstrow = this.childs[0];
                if (firstrow)
                    return firstrow[i];
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

    }

    class NativeDatasource extends clove.Headersource(clove.Datasource(Object)) {

        constructor(config) {
            super();
            this.config = clove.utils.applyDefaultsToConfig(config, {
                readonly: false,
                rowHeadersVisible: true,
                columnHeadersVisible: true,
            });
            this._rootnode = new NativeDatasourceNode(this);
        }

        root() {
            return this._rootnode;
        }

        valuePointerToNativeNode(ptr) {
            return ptr ? ptr.backendObject : this._rootnode;
        }

        insertRow(ir, parent) {
            var xxx = clove.utils.suspendResizing();
            try {
                var parentbo = this.valuePointerToNativeNode(parent);
                if (ir > parentbo.rowcnt)
                    throw new Error("Unable to insert row " + ir + " in a structure with " + parentbo.rowcnt + " rows.");
                clove.utils.insertToArray(parentbo.rowheaders, ir, undefined);
                if (ir == parentbo.rowcnt) {
                    parentbo.childs.push([]);
                } else {
                    parentbo.childs.splice(ir, 0, []);
                    for(var iir=ir+1;iir<parentbo.childs.length;iir++)
                        for(var iic=0;iic<parentbo.childs[iir].length;iic++)
                            parentbo.childs[iir][iic].ir = iir;
                }
                parentbo.rowcnt++;
                this.OnDataInsert.trigger({r1:ir, r2:ir, parent:parent});
                this.OnHeaderDataInsert.trigger({irow:ir, parent:parent, headersource:this});
                this.OnHeaderVisibilityUpdated.trigger({for_row:true, parent:parent});
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        insertColumn(ic, parent) {
            var xxx = clove.utils.suspendResizing();
            try {
                var parentbo = this.valuePointerToNativeNode(parent);
                if (ic > parentbo.colcnt)
                    throw new Error("Unable to insert column " + ic + " in a structure with " + parentbo.colcnt + " columns.");
                clove.utils.insertToArray(parentbo.colheaders, ic, undefined);
                if (ic < parentbo.colcnt) {
                    for(var iir=0;iir<parentbo.childs.length;iir++) {
                        var collist = parentbo.childs[iir];
                        collist.splice(ic, 0, undefined);
                        for(var iic=ic+1;iic<collist.length;iic++)
                            collist[iic].ic = iic;
                    }
                }
                parentbo.colcnt++;
                this.OnDataInsert.trigger({c1:ic, c2:ic, parent:parent});
                this.OnHeaderDataInsert.trigger({icol:ic, parent:parent, headersource:this});
                this.OnHeaderVisibilityUpdated.trigger({for_column:true, parent:parent});
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        appendRow(parent) {
            this.insertRow(this.rowCount(parent), parent);
        }

        appendColumn(parent) {
            this.insertColumn(this.columnCount(parent), parent);
        }

        removeRow(ir, parent) {
            var xxx = clove.utils.suspendResizing();
            try {
                var parentbo = this.valuePointerToNativeNode(parent);
                if (ir >= parentbo.rowcnt)
                    throw new Error("Unable to remove row " + ir + " in a structure with " + parentbo.rowcnt + " rows.");
                parentbo.childs.splice(ir, 1);
                parentbo.rowheaders.splice(ir, 1);
                for(var iir=ir;iir<parentbo.childs.length;iir++)
                    for(var iic=0;iic<parentbo.childs[iir].length;iic++)
                        parentbo.childs[iir][iic].ir = iir;
                parentbo.rowcnt--;
                this.OnDataRemove.trigger({r1:ir, r2:ir, parent:parent});
                this.OnHeaderDataRemove.trigger({irow:ir, parent:parent, headersource:this});
                this.OnHeaderVisibilityUpdated.trigger({for_row:true, parent:parent});
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        removeColumn(ic, parent) {
            var xxx = clove.utils.suspendResizing();
            try {
                var parentbo = this.valuePointerToNativeNode(parent);
                if (ic >= parentbo.colcnt)
                    throw new Error("Unable to remove column " + ic + " in a structure with " + parentbo.colcnt + " columns.");
                parentbo.colheaders.splice(ic, 1);
                for(var iir=0;iir<parentbo.childs.length;iir++) {
                    var cols = parentbo.childs[iir];
                    cols.splice(ic, 1);
                    for(var iic=ic;iic<cols.length;iic++)
                        cols[iic].ic = iic;
                }
                parentbo.colcnt--;
                this.OnDataRemove.trigger({c1:ic, c2:ic, parent:parent});
                this.OnHeaderDataRemove.trigger({icol:ic, parent:parent, headersource:this});
                this.OnHeaderVisibilityUpdated.trigger({for_column:true, parent:parent});
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        _getNode(ir, ic, parent) {
            var parentbo = this.valuePointerToNativeNode(parent);
            if (ir >= parentbo.rowcnt)
                throw new Error("Bad arguments.");
            if (ic >= parentbo.colcnt)
                throw new Error("Bad arguments.");
            var cc = parentbo.childs[ir];
            while (ic >= cc.length)
                cc.push(undefined);
            var node = cc[ic];
            if (!node) {
                node = new NativeDatasourceNode(this, ir, ic, parentbo);
                cc[ic] = node;
            }
            return node;
        }

        getValue(ptr) {
            return this.valuePointerToNativeNode(ptr).value;
        }

        getMetadata(ptr) {
            return this.valuePointerToNativeNode(ptr).metadata;
        }

        changeValue(ptr, value) {
            if (!this.config.readonly)
                this.setValue(ptr, value);
        }

        setValue(ptr, value) {
            var node = this.valuePointerToNativeNode(ptr);
            node.value = value;
            this.OnDataUpdate.trigger(ptr ? {r1: ptr.irow, r2: ptr.irow, c1: ptr.icol, c2: ptr.icol, parent: ptr.parent()} : {rootnode: true});
        }

        setRootValue(value) {
            this.setValue(undefined, value);
        }

        setMetadata(ptr, key, value) {
            var node = this.valuePointerToNativeNode(ptr);
            node.metadata[key] = value;
            this.OnDataUpdate.trigger(ptr ? {r1: ptr.irow, r2: ptr.irow, c1: ptr.icol, c2: ptr.icol, parent: ptr.parent()} : {rootnode: true});
        }

        rowCount(parent) {
            var parentbo = this.valuePointerToNativeNode(parent);
            return parentbo.rowcnt;
        }

        columnCount(parent) {
            var parentbo = this.valuePointerToNativeNode(parent);
            return parentbo.colcnt;
        }

        valuePointer(irow, icol, parent) {
            var backendObject = this._getNode(irow, icol, parent);
            return new clove.DatasourceValuePointer(irow, icol, backendObject, this);
        }

        parent(ptr) {
            var ptrbo = ptr ? ptr.backendObject : this._rootnode;
            if (ptrbo.parent && ptrbo.parent.ir !== undefined)
                return new clove.DatasourceValuePointer(ptrbo.parent.ir, ptrbo.parent.ic, ptrbo.parent, this);
        }

        setRowHeader(irow, parent, headercfg) {
            var parentbo = this.valuePointerToNativeNode(parent);
            var _visibilitychanged = this._hasrowheaders();
            parentbo.rowheaders[irow] = headercfg;
            var visibilitychanged = (this._hasrowheaders()) != _visibilitychanged;
            this.OnHeaderDataUpdate.trigger({irow:irow, parent:parent, headersource:this});
            if (visibilitychanged)
                this.OnHeaderVisibilityUpdated.trigger({for_row:true, parent:parent});
        }

        setColumnHeader(icol, parent, headercfg) {
            var parentbo = this.valuePointerToNativeNode(parent);
            var _visibilitychanged = this._hascolheaders();
            parentbo.colheaders[icol] = headercfg;
            var visibilitychanged = (this._hascolheaders()) != _visibilitychanged;
            this.OnHeaderDataUpdate.trigger({icol:icol, parent:parent, headersource:this});
            if (visibilitychanged)
                this.OnHeaderVisibilityUpdated.trigger({for_column:true, parent:parent});
        }

        getRowHeader(irow, parent) {
            var parentbo = this.valuePointerToNativeNode(parent);
            return parentbo.rowheaders[irow] || {};
        }

        getColumnHeader(icol, parent) {
            var parentbo = this.valuePointerToNativeNode(parent);
            return parentbo.colheaders[icol] || {};
        }

        _hasrowheaders(parent) {
            if (!this.config.rowHeadersVisible)
                return false;
            var parentbo = this.valuePointerToNativeNode(parent);
            for(var i=0;i<parentbo.rowheaders.length;i++)
                if (parentbo.rowheaders[i] !== undefined)
                    return true;
            return false;
        }

        _hascolheaders(parent) {
            if (!this.config.columnHeadersVisible)
                return false;
            var parentbo = this.valuePointerToNativeNode(parent);
            for(var i=0;i<parentbo.colheaders.length;i++)
                if (parentbo.colheaders[i] !== undefined)
                    return true;
            return false;
        }

        rowHeadersVisible(parent) {
            return this._hasrowheaders(parent);
        }

        columnHeadersVisible(parent) {
            return this._hascolheaders(parent);
        }

    }

    clove.NativeDatasource = NativeDatasource;

    class AsyncDatasource extends NativeDatasource {

        constructor(config) {
            super(clove.utils.applyDefaultsToConfig(config, {
                errorhandler: function(e){
                    throw e;
                },
                cacheAge: 10,
            }));
            this.async_pull(undefined, {
                reason: {id:'AsyncDatasource', sub:'constructor'},
            });
            this._cache_era = 1;
        }

        clearCache() {
            this._cache_era++;
        }

        do_async_pull(ptr, requestConfig) {
        }

        do_async_push(ptr, v, requestConfig) {
        }

        rowCount(ptr, skipFetching) {
            if (!skipFetching)
                this.async_pull(ptr, {
                    reason: {id:'AsyncDatasource', sub:'rowCount'},
                    cached: true,
                });
            return super.rowCount(ptr);
        }

        columnCount(ptr, skipFetching) {
            if (!skipFetching)
                this.async_pull(ptr, {
                    reason: {id:'AsyncDatasource', sub:'columnCount'},
                    cached: true,
                });
            return super.columnCount(ptr);
        }

        _applyrequestconfigdefaults(requestConfig) {
            var self = this;
            return clove.utils.applyDefaultsToConfig(requestConfig, {
                onsuccess: function(){},
                onfailed: function(e){self.config.errorhandler(e);},
                reason: {},
            });
        }

        _request_cached(ptr, requestConfig) {
            var result = false;
            var intstruct = this.valuePointerToNativeNode(ptr);
            if (requestConfig.cached)
                result = (intstruct._async_last_pulled_era == this._cache_era) && (intstruct._async_last_pulled && ((new Date())-intstruct._async_last_pulled)<(this.config.cacheAge * 1000));
            if (!result) {
                intstruct._async_last_pulled = new Date();
                intstruct._async_last_pulled_era = this._cache_era;
            }
            return result;
        }

        async_pull(ptr, requestConfig) {
            var self = this;
            requestConfig = this._applyrequestconfigdefaults(requestConfig);
            if (this._request_cached(ptr, requestConfig))
                return;
            setTimeout(function(){
                self.do_async_pull(ptr, requestConfig);
            });
        }

        async_push(ptr, v, requestConfig) {
            var self = this;
            requestConfig = this._applyrequestconfigdefaults(requestConfig);
            setTimeout(function(){
                self.do_async_push(ptr, v, requestConfig);
            });
        }

        changeValue(ptr, value) {
            var self = this;
            if (!this.config.readonly)
                this.async_push(ptr, value, {
                    onsuccess: function(){self.setValue(ptr, value);},
                    onfailed: function(e){self.async_pull(ptr, {reason: {id:'changeValue', sub:'onfailed', error: e}}); self.config.errorhandler(e);},
                    reason: {id:'changeValue'},
                });
        }

    }

    clove.AsyncDatasource = AsyncDatasource;

    class AjaxAsyncDatasource extends AsyncDatasource {

        constructor(config) {
            super(clove.utils.applyDefaultsToConfig(config, {
                data_pullUrl: "get",
                data_pushUrl: undefined,
                data_pushParams: {},
                data_pullParams: {},
                data_params: {},
                data_mayHaveChildren: true,
                data_autoPullCharily: false,
                params: {},
                rootType: "data",
                ajaxAdditionalArgs: {},
                answerIdKeyName: "id",
                requestIdKeyName: "id",
                headerConfigs: {},
            }));
            this.OnDataArrived = new clove.Event();
        }

        _ajax(ajaxcfg, paramsPost) {
            if (paramsPost) {
                ajaxcfg.method = "POST";
                ajaxcfg.url += "?" + Object.entries(ajaxcfg.data).map(([k, v]) => `${k}=${encodeURIComponent(v)}`).join('&');
                ajaxcfg.data = JSON.stringify(paramsPost);
                ajaxcfg.contentType = "application/json";
            }
            clove._ajaxprovider(ajaxcfg);
        }

        _ajax_helper(url, ptr, params1, params2, params3, paramsPost, ferror, fsuccess) {
            var self = this;
            var pparams = clove.utils.applyDefaultsToConfig(clove.utils.applyDefaultsToConfig(params3, params2), params1);
            if (ptr) {
                var ptrbo = this.valuePointerToNativeNode(ptr);
                pparams[this.config.requestIdKeyName] = ptrbo.ajaxid;
            }
            var ajaxcfg = clove.utils.applyDefaultsToConfig({
                url: url,
                data: pparams,
                success: fsuccess,
                error: ferror,
            }, this.config.ajaxAdditionalArgs);
            this._ajax(ajaxcfg, paramsPost);
        }

        _coldata(ptr, data) {
            var ptrbo = this.valuePointerToNativeNode(ptr);
            var xmlatt2colidx = new Object();
            var lcols = [];
            var idcolname;
            if (ptrbo.colheaders.length == 0) {
                var datachilds = this._datachildren(data);
                for(var i=0; i<datachilds.length; i++) {
                    var xnode = datachilds[i];
                    if (typeof xnode === "string" || xnode instanceof String)
                        xnode = {id: xnode, value: xnode};
                    var xnodeatts = this._dataattributes(xnode);
                    for(var di=0; di<xnodeatts.length; di++) {
                        var att = xnodeatts[di];
                        if (lcols.indexOf(att.name) == -1) {
                            if (att.name == this.config.answerIdKeyName)
                                idcolname = att.name;
                            else {
                                xmlatt2colidx[att.name] = lcols.length;
                                lcols.push(att.name);
                            }
                        }
                    }
                }
                if (idcolname) {
                    xmlatt2colidx[idcolname] = lcols.length;
                    lcols.push(idcolname);
                }
            }
            else {
                for(var i=0; i<ptrbo.colheaders.length; i++) {
                    var aa = ptrbo.colheaders[i];
                    if (aa && aa.xmlattribute) {
                        xmlatt2colidx[aa.xmlattribute] = i;
                        lcols.push(aa.xmlattribute);
                        if (aa.xmlidcolumn)
                            idcolname = aa.xmlattribute;
                    }
                }
            }
            return {xmlatt2colidx: xmlatt2colidx, cols: lcols, idcolname: idcolname};
        }

        _datachildren(data) {
            return (data instanceof Node) ? data.children : data;
        }

        _dataattributes(data) {
            if (data instanceof Node)
                return data.attributes;
            else {
                var res = [];
                for(var x in data)
                    if (data.hasOwnProperty(x))
                        res.push({name: x, value: data[x]});
                return res;
            }
        }

        _dataroot(data) {
            return (data instanceof Node) ? data.children[0] : data;
        }

        _datatype(data) {
            if (data instanceof Node)
                return data.tagName;
            else
                return (data.type) ? data.type : this.rootType;
        }

        _interpretdata(ptr, data) {
            var xxx = clove.utils.suspendResizing();
            try {
                var ptrbo = this.valuePointerToNativeNode(ptr);
                var datachilds = this._datachildren(data);
                var rawMode = (!datachilds.length) && data.hasOwnProperty("value");
                var oldcolcount = this.columnCount(ptr);
                var oldrowcount = this.rowCount(ptr);
                if (rawMode) {
                    var bo = this.valuePointerToNativeNode(ptr);
                    bo.setValue(data.value);
                    bo.ajaxid = data.id;
                    var colcnt = (data.childs.length > 0) ? data.childs[0].length : 0;
                    for(var ddi=oldcolcount-1; ddi>=colcnt; ddi--)
                        this.removeColumn(ddi, ptr);
                    for(var dci=oldcolcount; dci<colcnt; dci++)
                        this.insertColumn(dci, ptr);
                    for(var irow=0; irow<data.childs.length; irow++) {
                        var rowi = data.childs[irow];
                        if (irow >= oldrowcount)
                            this.insertRow(irow, ptr);
                        for(var icol=0; icol<rowi.length; icol++) {
                            var cptr = this.valuePointer(irow, icol, ptr);
                            var cbo = this.valuePointerToNativeNode(cptr);
                            var jobj = rowi[icol];
                            cbo.ajaxid = jobj[0];
                            cbo.ajaxtypename = this.rootType;
                            cbo.setValue(jobj[1]);
                        }
                    }
                    for(var i=oldrowcount-1; i>=data.childs.length; i--)
                        this.removeRow(i, ptr);
                }
                else {
                    var coldata = this._coldata(ptr, data);
                    for(var dci=0; dci<coldata.cols.length; dci++) {
                        if (dci >= oldcolcount) {
                            this.insertColumn(dci, ptr);
                            var kname = coldata.cols[dci];
                            var xmlidcolumn = kname==coldata.idcolname;
                            var chparams = {label: kname, xmlattribute: kname, xmlidcolumn: xmlidcolumn, invisible: xmlidcolumn || kname.startsWith("_headerConfig_")};
                            var achparams = this.config.headerConfigs[kname];
                            if (achparams)
                                for(var iachparams in achparams)
                                    chparams[iachparams] = achparams[iachparams];
                            this.setColumnHeader(dci, ptr, chparams);
                        }
                    }
                    for(var ddi=oldcolcount-1; ddi>=coldata.cols.length; ddi--)
                        this.removeColumn(ddi, ptr);
                    for(var i=0; i<datachilds.length; i++) {
                        var rhparams = {};
                        if (i >= oldrowcount)
                            this.insertRow(i, ptr);
                        var xnode = datachilds[i];
                        if (typeof xnode === "string" || xnode instanceof String)
                            xnode = {id: xnode, value: xnode};
                        var xnodeatts = this._dataattributes(xnode);
                        var colid;
                        var cols = [];
                        for(var dci=0; dci<coldata.cols.length; dci++)
                            cols.push("");
                        for(var di=0; di<xnodeatts.length; di++) {
                            var att = xnodeatts[di];
                            var iix = coldata.xmlatt2colidx[att.name];
                            if (iix !== undefined)
                                cols[iix] = att.value;
                            if (att.name == coldata.idcolname)
                                colid = att.value;
                        }
                        for(var dci=0; dci<cols.length; dci++) {
                            var cptr = this.valuePointer(i, dci, ptr);
                            var cbo = this.valuePointerToNativeNode(cptr);
                            cbo.ajaxid = colid;
                            cbo.ajaxtypename = this._datatype(xnode);
                            var value = cols[dci];
                            this.setValue(cptr, value);
                            if (dci < coldata.cols.length) {
                                var cname = coldata.cols[dci];
                                if (cname.startsWith("_headerConfig_")) {
                                    var ccname = cname.substring(14);
                                    rhparams[ccname] = value;
                                }
                            }
                        }
                        this.setRowHeader(i, ptr, rhparams);
                    }
                    for(var i=oldrowcount-1; i>=datachilds.length; i--)
                        this.removeRow(i, ptr);
                }
                this.OnDataArrived.trigger({datasource: this, parent: ptr});
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        getAjaxId(ptr) {
            return this.valuePointerToNativeNode(ptr).ajaxid;
        }

        do_async_pull(ptr, requestConfig) {
            var self = this;
            var ptrbo = this.valuePointerToNativeNode(ptr);
            var prefix = (ptrbo.ajaxtypename || this.config.rootType) + "_";
            var mayHaveChildren = this.config[prefix+"mayHaveChildren"];
            if (ptr !== undefined && !mayHaveChildren)
                return;
            var params1 = this.config["params"];
            var params2 = this.config[prefix+"params"];
            var params3 = this.config[prefix+"pullParams"];
            var charily = this.config[prefix+"autoPullCharily"] || this.config["autoPullCharily"];
            if (!charily || requestConfig.reason.id != "AsyncDatasource" || requestConfig.reason.sub == "constructor") {
                this._ajax_helper(this.config[prefix+"pullUrl"], ptr, params1, params2, params3, undefined, requestConfig.onfailed, function(data){
                    self._interpretdata(ptr, self._dataroot(data));
                });
            }
        }

        do_async_push(ptr, v, requestConfig) {
            var ptrbo = this.valuePointerToNativeNode(ptr);
            var prefix = (ptrbo.ajaxtypename || this.config.rootType) + "_";
            var params1 = this.config["params"];
            var params2 = this.config[prefix+"params"];
            var params3 = this.config[prefix+"pushParams"];
            this._ajax_helper(this.config[prefix+"pushUrl"], ptr, params1, params2, params3, {value:v}, requestConfig.onfailed, function(data){});
        }

    }

    clove.AjaxAsyncDatasource = AjaxAsyncDatasource;

    class ProxyDatasource extends NativeDatasource {

        constructor() {
            super();
            this._rootnode.native2proxyrows = [];
            this._rootnode.native2proxycols = [];
            this._rootnode.proxy2nativerows = [];
            this._rootnode.proxy2nativecols = [];
        }

        setDatasource(datasource) {
            if (this.datasource) {
                this.datasource.OnDataInsert.removeHandler(this.hodi);
                this.datasource.OnDataRemove.removeHandler(this.hodr);
                this.datasource.OnDataUpdate.removeHandler(this.hodu);
                this.datasource.OnHeaderDataUpdate.removeHandler(this.hohdu);
                this.datasource.OnHeaderVisibilityUpdated.removeHandler(this.hohvu);
                this._ondataremovehandler({r1:0, c1:0, r2:this.datasource.rowCount()-1, c2:this.datasource.columnCount()-1});
            }
            this.datasource = datasource;
            if (datasource) {
                var self = this;
                this.hodi = function(e){self._ondatainserthandler(e);};
                this.hodr = function(e){self._ondataremovehandler(e);};
                this.hodu = function(e){self._ondataupdatehandler(e);};
                this.hohdu = function(e){self._onheaderdataupdatehandler(e);};
                this.hohvu = function(e){self._onheadervisibilityupdatehandler(e);};
                datasource.OnDataInsert.addHandler(this.hodi);
                datasource.OnDataRemove.addHandler(this.hodr);
                datasource.OnDataUpdate.addHandler(this.hodu);
                datasource.OnHeaderDataUpdate.addHandler(this.hohdu);
                datasource.OnHeaderVisibilityUpdated.addHandler(this.hohvu);
                this._ondatainserthandler({r1:0, c1:0, r2:datasource.rowCount()-1, c2:datasource.columnCount()-1});
            }
        }

        refresh() {
            this.setDatasource(this.datasource);
        }

        proxyPointerToNativePointer(proxyptr) {
            if (!proxyptr)
                return undefined;
            var popo = proxyptr.backendObject;
            var nirow = popo.parent.proxy2nativerows[popo.ir];
            var nicol = popo.parent.proxy2nativecols[popo.ic];
            if (nirow === undefined || nicol === undefined)
                throw new Error("Bad pointer: " + proxyptr);
            return this.datasource.valuePointer(nirow, nicol, this.proxyPointerToNativePointer(popo.parent.valuePointer()));
        }

        nativePointerToProxyPointer(nativeptr) {
            if (!nativeptr)
                return undefined;
            var pparent = this.nativePointerToProxyPointer(nativeptr.parent());
            if (pparent === false)
                return false;
            var pnode = this.valuePointerToNativeNode(pparent);
            var nirow = pnode.native2proxyrows[nativeptr.irow];
            var nicol = pnode.native2proxycols[nativeptr.icol];
            if (nirow === undefined || nicol === undefined)
                return false;
            return this.valuePointer(nirow, nicol, pparent);
        }

        _getNode(ir, ic, parent) {
            var result = super._getNode(ir, ic, parent);
            if (!result.native2proxyrows) {
                result.native2proxyrows = [];
                result.native2proxycols = [];
                result.proxy2nativerows = [];
                result.proxy2nativecols = [];
            }
            return result;
        }

        _ondatainserthandler(e) {
            if (e.c1 !== undefined)
                this._ondatainserthandler_helper(false, e.parent, e.c1, e.c2);
            if (e.r1 !== undefined)
                this._ondatainserthandler_helper(true, e.parent, e.r1, e.r2);
        }

        _ondataremovehandler(e) {
            if (e.c1 !== undefined)
                this._ondataremovehandler_helper(false, e.parent, e.c1, e.c2);
            if (e.r1 !== undefined)
                this._ondataremovehandler_helper(true, e.parent, e.r1, e.r2);
        }

        _ondataupdatehandler(e) {
            var pparent = this.nativePointerToProxyPointer(e.parent);
            if (pparent === false)
                return;
            var updaterows = [];
            var updatecols = [];
            this._ondataupdatehandler_helper(false, e, e.c1, e.c2, updatecols);
            this._ondataupdatehandler_helper(true, e, e.r1, e.r2, updaterows);
            for(var iir=0; iir<updaterows.length; iir++)
                for(var iic=0; iic<updatecols.length; iic++)
                    this.setValue(this.valuePointer(updaterows[iir], updatecols[iic], pparent));
        }

        _onheaderdataupdatehandler(e) {
            var self = this;
            var pparent = this.nativePointerToProxyPointer(e.parent);
            if (pparent === false)
                return;
            var pnode = this.valuePointerToNativeNode(pparent);
            if (e.irow !== undefined)
                this._ondata_applyfilter(true, e.parent, pnode, pparent, e.irow, false, function(idx){ self.OnHeaderDataUpdate.trigger({irow:idx, parent:pparent, headersource: self}); });
            if (e.icol !== undefined)
                this._ondata_applyfilter(false, e.parent, pnode, pparent, e.icol, false, function(idx){ self.OnHeaderDataUpdate.trigger({icol:idx, parent:pparent, headersource: self}); });
        }

        _onheadervisibilityupdatehandler(e) {
            var pparent = this.nativePointerToProxyPointer(e.parent);
            if (pparent === false)
                return;
            var pnode = this.valuePointerToNativeNode(pparent);
            var eprms = clove.utils.applyDefaultsToConfig(e);
            eprms.parent = pparent;
            this.OnHeaderVisibilityUpdated.trigger(eprms);
        }

        _getcelltarget(nativeds, axisisrow, iaxis, oidx, nparent, pnode) {
            return iaxis;
        }

        _ondata_applyfilter(axisisrow, nparent, pnode, pparent, iaxis, forcereject, updatefct) {
            var insertfctname = axisisrow ? "insertRow" : "insertColumn";
            var removefctname = axisisrow ? "removeRow" : "removeColumn";
            var native2proxyaxisname = axisisrow ? "native2proxyrows" : "native2proxycols";
            var proxy2nativeaxisname = axisisrow ? "proxy2nativerows" : "proxy2nativecols";
            var proxy2nativeorthoname = axisisrow ? "proxy2nativecols" : "proxy2nativerows";
            var oidx = pnode[native2proxyaxisname][iaxis];
            var nidx = (forcereject) ? undefined : this._getcelltarget(this.datasource, axisisrow, iaxis, oidx, nparent, pnode);
            var justupdate = true;
            var _tempnidx;
            if (nidx !== undefined && oidx !== undefined && nidx != oidx) {
                _tempnidx = (nidx > oidx) ? nidx-1 : nidx;
                if (_tempnidx == oidx) {
                    _tempnidx = undefined;
                    nidx = oidx;
                }
                else
                    nidx = undefined;
            }
            if (nidx === undefined && oidx !== undefined) {
                pnode[proxy2nativeaxisname].splice(oidx, 1);
                pnode[native2proxyaxisname][iaxis] = undefined;
                for(var icor=0; icor<pnode[native2proxyaxisname].length; icor++) {
                    if (pnode[native2proxyaxisname][icor] > oidx)
                        pnode[native2proxyaxisname][icor]--;
                }
                this[removefctname](oidx, pparent);
                justupdate = false;
            }
            if (_tempnidx !== undefined) {
                oidx = undefined;
                nidx = _tempnidx;
            }
            if (nidx !== undefined && oidx === undefined) {
                for(var icor=0; icor<pnode[native2proxyaxisname].length; icor++) {
                    if (pnode[native2proxyaxisname][icor] >= nidx)
                        pnode[native2proxyaxisname][icor]++;
                }
                pnode[native2proxyaxisname][iaxis] = nidx;
                clove.utils.insertToArray(pnode[proxy2nativeaxisname], nidx, iaxis);
                this[insertfctname](nidx, pparent);
                for(var iiortho=0;iiortho<pnode[proxy2nativeorthoname].length;iiortho++) {
                    var iortho = pnode[proxy2nativeorthoname][iiortho];
                    var _x = axisisrow ? iaxis : iortho;
                    var _y = axisisrow ? iortho : iaxis;
                    var nptr = this.datasource.valuePointer(_x, _y, nparent);
                    var rcnt = this.datasource.rowCount(nptr);
                    var ccnt = this.datasource.columnCount(nptr);
                    if (rcnt > 0 && ccnt > 0)
                        this._ondatainserthandler({r1:0, r2:rcnt-1, c1:0, c2:ccnt-1, parent: nptr});
                }
                justupdate = false;
            }
            if (justupdate && oidx !== undefined && updatefct)
                updatefct(oidx);
        }

        _ondatainserthandler_helper(axisisrow, nparent, axis1, axis2) {
            var native2proxyaxisname = axisisrow ? "native2proxyrows" : "native2proxycols";
            var proxy2nativeaxisname = axisisrow ? "proxy2nativerows" : "proxy2nativecols";
            var pparent = this.nativePointerToProxyPointer(nparent);
            if (pparent === false)
                return;
            var pnode = this.valuePointerToNativeNode(pparent);
            for(var iaxis=axis1; iaxis<=axis2; iaxis++) {
                clove.utils.insertToArray(pnode[native2proxyaxisname], iaxis, undefined);
                for(var icor=pnode[proxy2nativeaxisname].length-1;icor>=0;icor--) {
                    var idxx = pnode[proxy2nativeaxisname][icor];
                    if (idxx >= iaxis)
                        pnode[proxy2nativeaxisname][icor] = idxx + 1;
                    else if (pnode.proxyHasSameOrderAsNative)
                        break;
                }
                this._ondata_applyfilter(axisisrow, nparent, pnode, pparent, iaxis, false, undefined);
            }
        }

        _ondataremovehandler_helper(axisisrow, nparent, axis1, axis2) {
            var native2proxyaxisname = axisisrow ? "native2proxyrows" : "native2proxycols";
            var proxy2nativeaxisname = axisisrow ? "proxy2nativerows" : "proxy2nativecols";
            var pparent = this.nativePointerToProxyPointer(nparent);
            if (pparent === false)
                return;
            var pnode = this.valuePointerToNativeNode(pparent);
            for(var iaxis=axis2; iaxis>=axis1; iaxis--) {
                this._ondata_applyfilter(axisisrow, nparent, pnode, pparent, iaxis, true, undefined);
                pnode[native2proxyaxisname].splice(iaxis, 1);
                for(var icor=pnode[proxy2nativeaxisname].length-1;icor>=0;icor--) {
                    var idxx = pnode[proxy2nativeaxisname][icor];
                    if (idxx >= iaxis)
                        pnode[proxy2nativeaxisname][icor] = idxx - 1;
                    else if (pnode.proxyHasSameOrderAsNative)
                        break;
                }
            }
        }

        _ondataupdatehandler_helper(axisisrow, e, axis1, axis2, updatelist) {
            var pparent = this.nativePointerToProxyPointer(e.parent);
            if (pparent === false)
                return;
            var pnode = this.valuePointerToNativeNode(pparent);
            for(var iaxis=axis1; iaxis<=axis2; iaxis++)
                this._ondata_applyfilter(axisisrow, e.parent, pnode, pparent, iaxis, false, function(idx){ updatelist.push(idx); });
        }

        getValue(ptr) {
            return this.datasource.getValue(this.proxyPointerToNativePointer(ptr));
        }

        changeValue(ptr, value) {
            this.datasource.changeValue(this.proxyPointerToNativePointer(ptr), value);
        }

        getMetadata(ptr) {
            return this.datasource.getMetadata(this.proxyPointerToNativePointer(ptr));
        }

        setValue(ptr, value) {
            this.OnDataUpdate.trigger(ptr ? {r1: ptr.irow, r2: ptr.irow, c1: ptr.icol, c2: ptr.icol, parent: ptr.parent()} : {rootnode: true});
        }

        getRowHeader(irow, parent) {
            var nparent = this.proxyPointerToNativePointer(parent);
            var pnode = this.valuePointerToNativeNode(parent);
            return this.datasource.getRowHeader(pnode.proxy2nativerows[irow], nparent);
        }

        getColumnHeader(icol, parent) {
            var nparent = this.proxyPointerToNativePointer(parent);
            var pnode = this.valuePointerToNativeNode(parent);
            return this.datasource.getColumnHeader(pnode.proxy2nativecols[icol], nparent);
        }

        rowHeadersVisible(parent) {
            return this.datasource.rowHeadersVisible(this.proxyPointerToNativePointer(parent));
        }

        columnHeadersVisible(parent) {
            return this.datasource.columnHeadersVisible(this.proxyPointerToNativePointer(parent));
        }

    }

    clove.ProxyDatasource = ProxyDatasource;

    class FilterProxyDatasource extends ProxyDatasource {

        constructor(datasource, rowfilterfct, colfilterfct) {
            super();
            this.setRowFilter(rowfilterfct);
            this.setColumnFilter(colfilterfct);
            this.setDatasource(datasource);
        }

        setRowFilter(rowfilterfct) {
            this.rowfilterfct = rowfilterfct;
            this.refresh();
        }

        setColumnFilter(colfilterfct) {
            this.colfilterfct = colfilterfct;
            this.refresh();
        }

        _getNode(ir, ic, parent) {
            var result = super._getNode(ir, ic, parent);
            result.proxyHasSameOrderAsNative = true;
            return result;
        }

        _getcelltarget(nativeds, axisisrow, iaxis, oidx, nparent, pnode) {
            var native2proxyaxisname = axisisrow ? "native2proxyrows" : "native2proxycols";
            var filterfctname = axisisrow ? "rowfilterfct" : "colfilterfct";
            var filterfct = this[filterfctname];
            if (!filterfct || filterfct(nativeds, iaxis, nparent)) {
                if (oidx !== undefined)
                    return oidx;
                var insc = iaxis - 1;
                while (insc >= 0 && pnode[native2proxyaxisname][insc] === undefined)
                    insc--;
                if (insc == -1)
                    return 0;
                else
                    return pnode[native2proxyaxisname][insc] + 1;
            }
        }

    }

    clove.FilterProxyDatasource = FilterProxyDatasource;

    class SortProxyDatasource extends ProxyDatasource {

        constructor(datasource, rowcomparefct, colcomparefct) {
            super();
            this.setRowComparator(rowcomparefct);
            this.setColumnComparator(colcomparefct);
            this.setDatasource(datasource);
        }

        setRowComparator(rowcomparefct) {
            this.rowcomparefct = rowcomparefct;
            this.refresh();
        }

        setColumnComparator(colcomparefct) {
            this.colcomparefct = colcomparefct;
            this.refresh();
        }

        _defaultcomparator(nativeds, iaxis, jaxis, nparent, pnode, axisisrow) {
            var cellcountpropname = axisisrow ? "colcnt" : "rowcnt";
            for(var icell=0; icell<pnode[cellcountpropname]; icell++) {
                var iptr = nativeds.valuePointer(axisisrow ? iaxis : icell, axisisrow ? icell : iaxis, nparent);
                var jptr = nativeds.valuePointer(axisisrow ? jaxis : icell, axisisrow ? icell : jaxis, nparent);
                var ival = nativeds.getValue(iptr);
                var jval = nativeds.getValue(jptr);
                if (ival == jval)
                    return 0;
                else if (ival < jval)
                    return -1;
                else
                    return 1;
            }
        }

        _getcelltarget(nativeds, axisisrow, iaxis, oidx, nparent, pnode) {
            var proxy2nativeaxisname = axisisrow ? "proxy2nativerows" : "proxy2nativecols";
            var elementcountpropname = axisisrow ? "rowcnt" : "colcnt";
            var comparefctname = axisisrow ? "rowcomparefct" : "colcomparefct";
            var comparefct = this[comparefctname] || this._defaultcomparator;
            for(var jaxis=0; jaxis<pnode[elementcountpropname]; jaxis++) {
                var jcmp = comparefct(nativeds, iaxis, pnode[proxy2nativeaxisname][jaxis], nparent, pnode, axisisrow);
                if (jcmp < 0)
                    return jaxis;
            }
            return jaxis;
        }

    }

    clove.SortProxyDatasource = SortProxyDatasource;

    var _databindings_in_setproperty = false;

    class DataBinding {

        constructor(widget, propertyName, datasource, ptr, dataDirection, valueComparator, valueConverter) {
            valueComparator = valueComparator || function(a, b) {
                if (a && a.equals)
                    return a.equals(b);
                else if (b && b.equals)
                    return b.equals(a);
                else
                    return a === b;
            };
            this.widget = widget;
            this.propertyName = propertyName;
            this.datasource = datasource;
            this.ptr = ptr;
            this.bound = false;
            var self = this;
            if (!dataDirection)
                dataDirection = "bidirectional";
            var dataDirection_todatasource = (dataDirection == "todatasource") || (dataDirection == "bidirectional");
            var dataDirection_towidget = (dataDirection == "towidget") || (dataDirection == "bidirectional");
            this._hodu = function(e){
                if (!dataDirection_towidget)
                    return;
                if (e===undefined || (e.rootnode && self.ptr===undefined) || (!e.rootnode && self.ptr && DatasourceValuePointer.equals(e.parent, self.ptr.parent()) && e.r1 <= self.ptr.irow && e.r2 >= self.ptr.irow && e.c1 <= self.ptr.icol && e.c2 >= self.ptr.icol)) {
                    var vval = self.datasource.getValue(self.ptr);
                    if (valueConverter)
                        vval = valueConverter.convertTo(vval);
                    if (!valueComparator(vval, self.widget.config[self.propertyName])) {
                        _databindings_in_setproperty = true;
                        try {
                            self.widget.setProperty(self.propertyName, vval);
                        }
                        finally {
                            _databindings_in_setproperty = false;
                        }
                    }
                }
            }
            this._hudo = function(e){
                if (!_databindings_in_setproperty && dataDirection_todatasource && e.propertyName == self.propertyName) {
                    var propertyValue = e.getPropertyValue();
                    if (valueConverter)
                        propertyValue = valueConverter.convertFrom(propertyValue);
                    if (!valueComparator(e.oldPropertyValue, propertyValue))
                        self.datasource.changeValue(self.ptr, propertyValue);
                }
            }
        }

        bind() {
            if (this.bound)
                return;
            this.bound = true;
            this.datasource.OnDataUpdate.addHandler(this._hodu, this.widget);
            this.datasource.OnDataInsert.addHandler(this._hodu, this.widget);
            this.widget.OnPropertyChanged.addHandler(this._hudo, this.widget);
            this._hodu();
            if (this.datasource.async_pull)
                this.datasource.async_pull(this.ptr, {
                    reason: {id: 'DataBinding', sub: 'bind'},
                });
        }

        unbind() {
            if (!this.bound)
                return;
            this.bound = false;
            this.datasource.OnDataUpdate.removeHandler(this._hodu);
            this.datasource.OnDataInsert.removeHandler(this._hodu);
            this.widget.OnPropertyChanged.removeHandler(this._hudo);
        }

    }

    clove.DataBinding = DataBinding;

    class DataBindingConverter {

        convertTo(x) {
            return x;
        }

        convertFrom(y) {
            return y;
        }

    }

    clove.DataBindingConverter = DataBindingConverter;

    clove.databind = function(config) {
        return new DataBinding(undefined, undefined, config.datasource, config.ptr, config.dataDirection, config.valueComparator, config.valueConverter);
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          WIDGET

    class Widget {

        constructor(config, domnode) {
            this.isInitialized = false;
            this._properties = [];
            this._layout_invalidated = false;
            this._cache_mw = undefined;
            this._cache_pw = undefined;
            this._cache_mh = undefined;
            this._cache_ph = undefined;
            this._cache_mh_w = undefined;
            this._cache_ph_w = undefined;
            this._sizecache_timestamp = 0;
            this._sizecache_topnode_extra_ts = undefined;
            this._sizecache_topnode_extra = undefined;
            this._enabled_effective = true;
            this._visibility_effective = clove.InvisibleCollapsed;
            this.config = config;
            this.topnode = domnode;
            this._removing = false;
            this.contentnode = document.createElement("div");
            this.contentnode.className = "clove_widget_contentnode";
            this.topnode.appendChild(this.contentnode);
            $(this.topnode).addClass("clove_widget");
            this.topnode._clove_widget = this;
            this.busynode = undefined;
            this._busytokens = undefined;
            this.OnDestroyed = new clove.Event();
            this.OnResized = new clove.Event();
            this.OnVisibilityChanged = new clove.Event();
            this.OnPropertyChanged = new clove.Event();
            this.OnKeyDown = new clove.Event();
            this.OnKeyUp = new clove.Event();
            this.OnKeyPress = new clove.Event();
            var self = this;
            clove.utils.addOnDestroyHandler(this.topnode, function(){ self.OnDestroyed.trigger({widget: self}); });
            this.declareProperty("name");
            this.declareProperty("busy", false);
            this.declareProperty("enabled", true);
            this.declareProperty("width");
            this.declareProperty("height");
            this.declareProperty("hstretch");  // shorter aliases for the ones below
            this.declareProperty("vstretch");
            this.declareProperty("horizontalStretchAffinity", undefined);
            this.declareProperty("verticalStretchAffinity", undefined);
            this.declareProperty("strictHorizontalSizing", false);
            this.declareProperty("strictVerticalSizing", false);
            this.declareProperty("styleClass", "");
            this.declareProperty("style");
            this.declareProperty("visibility", clove.Visible);
            this.declareProperty("doStandaloneResizing", false);
            this.declareProperty("doStandaloneResizing_clipToParent", false); // this property is used internally only
            this.declareProperty("mayFocus", false);
            this._onkeydown_handler = function(e){ self.OnKeyDown.trigger({widget: self, jqEvent: e, originalEvent: e.originalEvent, keyCode: e.keyCode }) };
            this._onkeyup_handler = function(e){ self.OnKeyUp.trigger({widget: self, jqEvent: e, originalEvent: e.originalEvent, keyCode: e.keyCode }) };
            this._onkeypress_handler = function(e){ self.OnKeyPress.trigger({widget: self, jqEvent: e, originalEvent: e.originalEvent, keyCode: e.keyCode }) };
            $(this.topnode).on("focusin", function(e){
                if (!self._enabled_effective) {
                    var focusnext = self.topnode;
                    while (focusnext) {
                        while(focusnext && !focusnext.nextSibling)
                            focusnext = focusnext.parentNode;
                        if (focusnext) {
                            focusnext = focusnext.nextSibling;
                            var afocusnext = $(focusnext).find("*").addBack();
                            for(var iaf=0; iaf<afocusnext.length; iaf++) {
                                var af = afocusnext[iaf];
                                af.focus();
                                if (!document.activeElement || !clove.utils.isDescendantOf(document.activeElement, self.topnode))
                                    return;
                            }
                        }
                    }
                    if (document.activeElement)
                        document.activeElement.blur();
                }
            });
        }

        mayFocus_setter(v) {
            var jtn = $(this.topnode);
            jtn.off("keydown", this._onkeydown_handler);
            jtn.off("keyup", this._onkeyup_handler);
            jtn.off("keypress", this._onkeypress_handler);
            if (v) {
                $(this.topnode).attr('tabindex', '0');
                jtn.on("keydown", this._onkeydown_handler);
                jtn.on("keyup", this._onkeyup_handler);
                jtn.on("keypress", this._onkeypress_handler);
            }
            else
                $(this.topnode).removeAttr('tabindex');
        }

        hasFocus() {
            var v = document.activeElement;
            if (!document.hasFocus())
                return false;
            while (v && v != document.body) {
                if (v == this.topnode)
                    return true;
                v = v.parentNode;
            }
            return false;
        }

        isAlive() {
            if (this.topnode.isConnected !== undefined)
                return this.topnode.isConnected;
            return document.contains(this.topnode);
        }

        innerSize() {
            return clove.utils.getInnerSize(this.topnode);
        }

        outerSize() {
            return clove.utils.getOuterSize(this.topnode, true);
        }

        hstretch_setter(v) {
            if (v !== undefined)
                this.setHorizontalStretchAffinity(v);
        }

        vstretch_setter(v) {
            if (v !== undefined)
                this.setVerticalStretchAffinity(v);
        }

        hstretch_getter() {
            return this.horizontalStretchAffinity_getter();
        }

        vstretch_getter() {
            return this.verticalStretchAffinity_getter();
        }

        horizontalStretchAffinity_getter() {
            return this.config.horizontalStretchAffinity || 0;
        }

        verticalStretchAffinity_getter() {
            return this.config.verticalStretchAffinity || 0;
        }

        declareProperty(k, defaultV) {
            this._properties.push([k, defaultV]);
        }

        getProperty(k) {
            var f = this[k+"_getter"];
            if (f)
                return f.call(this);
            else if (k in this.config)
                return this.config[k];
            else
                return (arguments.length > 1) ? arguments[1] : undefined;
        }

        setProperty(k, v) {
            var self = this;
            var oldvalue = this.config[k];
            this.config[k] = v;
            var f = this[k+"_setter"];
            if (f)
                f.call(this, v);
            this.OnPropertyChanged.trigger({widget: this, propertyName: k, oldPropertyValue: oldvalue, getPropertyValue: function(){return self.getProperty(k);}});
        }

        _compute_enabled_effective() {
            var parent = this.parentWidget();
            this._enabled_effective = (parent ? parent._enabled_effective : true) && this.config.enabled && !this.config.busy;
            for (var chld of this.childrenWidgets())
                chld._compute_enabled_effective();
        }

        _compute_visibility_effective() {
            this._visibility_effective = this.config.visibility;
            var parent = this.parentWidget();
            if (parent) {
                if (this._visibility_effective == clove.Visible && parent._visibility_effective == clove.Invisible)
                    this._visibility_effective = clove.Invisible;
                else if (parent._visibility_effective == clove.InvisibleCollapsed)
                    this._visibility_effective = clove.InvisibleCollapsed;
            }
            for (var chld of this.childrenWidgets())
                chld._compute_visibility_effective();
        }

        effectivelyEnabled() {
            return this._enabled_effective;
        }

        effectiveVisibility() {
            return this._visibility_effective;
        }

        childrenWidgets() {
            return [];
        }

        parentWidget() {
            return this._parent;
        }

        enabled_getter() {
            return this.config.enabled;
        }

        enabled_setter(v, _forremoving) {
            if (this._removing_disabled)
                return;
            if (_forremoving)
                this._removing_disabled = true;
            if (this._disabledpanel) {
                this.removeStyleClass("clove_disabled", true);
                if (v) {
                    this._disabledpanel.remove();
                    this._disabledpanel = undefined;
                }
            }
            if (!v) {
                this.addStyleClass("clove_disabled", true);
                if (!this._disabledpanel) {
                    this._disabledpanel = $("<div class='clove_disabled_panel' style='position:absolute; top:0; bottom:0; left:0; right:0;'></div>");
                    $(this.topnode).append(this._disabledpanel);
                }
                if (document.activeElement && clove.utils.isDescendantOf(document.activeElement, this.topnode))
                    document.activeElement.blur();
            }
            this._compute_enabled_effective();
        }

        _createBusyPanel() {
            var n = document.createElement('div');
            n.style.position = 'absolute';
            n.style.top = 0;
            n.style.bottom = 0;
            n.style.left = 0;
            n.style.right = 0;
            n.style.opacity = 0;
            n.style.animationFillMode = 'forwards';
            n.style.animationDuration = '0.5s';
            n.style.animationName = 'cloveanimation_busypanel_fadein';
            return n;
        }

        _createBusyAnimation() {
            var pnl = this._createBusyPanel();
            pnl.className += " clove_busy_panel"
            for (var bpnl of ['clove_busy_panel_1', 'clove_busy_panel_2', 'clove_busy_panel_3']) {
                var n = document.createElement('div');
                n.className = bpnl;
                n.style = "position:absolute; top:0; bottom:0; left:0; right:0;";
                pnl.appendChild(n);
            }
            var nin = document.createElement('div');
            nin.className = 'clove_busy_panel_inner';
            pnl.appendChild(nin);
            for (var bis of ['clove_busy_panel_innerspacing_1', 'clove_busy_panel_innerspacing_2', 'clove_busy_panel_innerspacing_3', 'clove_busy_panel_innerspacing_4', 'clove_busy_panel_innerspacing_5', 'clove_busy_panel_innerspacing_6', 'clove_busy_panel_innerspacing_7', 'clove_busy_panel_innerspacing_8']) {
                var nsp = document.createElement('span');
                nsp.innerHTML = "&nbsp;";
                nsp.className = bis;
                nin.appendChild(nsp);
                nin.appendChild(document.createTextNode("•"));
            }
            return pnl;
        }

        busy_setter(v) {
            var self = this;
            if (v == (this.busynode === undefined)) {
                if (v) {
                    this.addStyleClass("clove_busy", true);
                    if (this._pending_busynode) {
                        this.busynode = this._pending_busynode;
                        this._pending_busynode = undefined;
                    }
                    else {
                        this.busynode = this._createBusyAnimation();
                        $(this.topnode).append(this.busynode);
                    }
                    this.busynode.style.opacity = window.getComputedStyle(this.busynode).opacity;
                    this.busynode.style.animationName = "cloveanimation_busypanel_fadein";
                }
                else {
                    this.removeStyleClass("clove_busy", true);
                    if (this.busynode !== undefined) {
                        var pending_busynode = this._pending_busynode = this.busynode;
                        this.busynode = undefined;
                        pending_busynode.style.opacity = window.getComputedStyle(pending_busynode).opacity;
                        clove.utils.animateNode(pending_busynode, "cloveanimation_fadeout", 500, function(){
                            if (self._pending_busynode == pending_busynode)
                                pending_busynode.remove();
                                self._pending_busynode = undefined;
                        });
                    }
                }
                this._compute_enabled_effective();
            }

        }

        registerBusy() {
            var self = this;
            var toks;
            if (this._busytokens)
                toks = this._busytokens;
            else {
                toks = this._busytokens = new Set();
                this.setBusy(true);
            }
            var token = {_token: true};
            toks.add(token);
            return token;
        }

        unregisterBusy(token) {
            if (this._busytokens) {
                if (this._busytokens.delete(token) && this._busytokens.size == 0) {
                    this._busytokens = undefined;
                    this.setBusy(false);
                    return true;
                }
            }
            return false;
        }

        name_setter(v) {
            if (v)
                this.topnode.dataset.clove_name = v;
            else
                this.topnode.dataset.clove_name = "";
        }

        width_setter(v) {
            this.relayout();
        }

        height_setter(v) {
            this.relayout();
        }

        styleClass_setter(sc) {
            this.removeStyleClass(this._lastsc, true);
            this.addStyleClass(sc);
            this._lastsc = sc;
        }

        style_setter(s) {
            if (s)
                this.contentnode.setAttribute('style', s);
        }

        visibility_getter() {
            if (this.isStyleClass(clove.Invisible))
                return clove.Invisible;
            if (this.isStyleClass(clove.InvisibleCollapsed))
                return clove.InvisibleCollapsed;
            return clove.Visible;
        }

        visibility_setter(v) {
            var veffbef = this._visibility_effective;
            this._compute_visibility_effective();
            var veffaft = this._visibility_effective;
            this.removeStyleClass(clove.Visible, true);
            this.removeStyleClass(clove.Invisible, true);
            this.removeStyleClass(clove.InvisibleCollapsed, true);
            this.addStyleClass(v, true);
            this.OnVisibilityChanged.trigger({widget: this});
            if (veffbef != veffaft)
                this.relayout();
        }

        bindProperty(k, vb) {
            var bindname = "_bind_" + k;
            var oldbinding = this[bindname];
            if (oldbinding)
                oldbinding.unbind();
            this[bindname] = vb;
            if (vb) {
                vb.widget = this;
                vb.propertyName = k;
                vb.bind();
            }
        }

        init(rootNameScope) {
            function _setProperty(k, v, bindings) {
                var bnd = bindings[k];
                if (bnd)
                    this.bindProperty(k, bnd);
                else
                    this.setProperty(k, v);
            }
            this._rootNameScope = rootNameScope || clove.rootNameScope;
            var self = this;
            var _wasset = new Set();
            var _propsbyname = new Object();
            for (var ip in this._properties) {
                var p = this._properties[ip];
                var pk = p[0];
                var pdefaultV = p[1];
                var getname = pk;
                var setname = "set" + pk.substring(0, 1).toUpperCase() + pk.substring(1);
                this[getname] = (function(_k){ return function(){return self.getProperty(_k);} })(pk);
                this[setname] = (function(_k){ return function(v){return self.setProperty(_k, v);} })(pk);
                if (!this.config.hasOwnProperty(pk))
                    this.config[pk] = pdefaultV;
                _propsbyname[pk] = p;
            }
            this._compute_enabled_effective();
            this._compute_visibility_effective();
            var _databindings = new Object();
            for(var k in this.config) {
                var p = _propsbyname[k];
                var v = this.config[k];
                if (v instanceof DataBinding)
                {
                    _databindings[k] = v;
                    this.config[k] = p ? p[1] : undefined;
                }
            }
            this.doinitEarly();
            for (var ip in this._properties) {
                var p = this._properties[ip];
                var pk = p[0];
                if (!_wasset.has(pk)) {
                    _setProperty.call(this, pk, this.config[pk], _databindings);
                    _wasset.add(pk);
                }
            }
            for(var k in this.config) {
                if (k.startsWith("On"))
                    this[k].addHandler(this.config[k]);
                else
                    if (!_wasset.has(k))
                        _setProperty.call(this, k, this.config[k], _databindings);
            }
            this.doinit();
            this.isInitialized = true;
            this.resize(true);
        }

        doinit() {
        }

        doinitEarly() {
        }

        resize(_force) {
            var self = this;
            if (_clove_resizesuspended) {
                _clove_resizesuspended.add(this);
                if (_force)
                    this._force_resize = true;
            }
            else {
                var chldw = this.topnode.style.width;
                var chldh = this.topnode.style.height;
                if (this._layout_invalidated || _force || !chldw || !chldh || chldw == "" || chldh == "" || this._layout_forwidth != chldw || this._layout_forheight != chldh) {
                    var lafowi = this._layout_forwidth;
                    var lafohe = this._layout_forheight;
                    this._layout_forwidth = chldw;
                    this._layout_forheight = chldh;
                    this.doresize();
                    this._layout_invalidated = false;
                    var nchldw = this.topnode.style.width;
                    var nchldh = this.topnode.style.height;
                    if ((chldw != lafowi || chldh != lafohe || !lafowi || !lafohe) && this.OnResized.hasHandlers())
                        setTimeout(function(){ self.OnResized.trigger({ widget: self }); });
                }
            }
        }

        doresize() {
        }

        doStandaloneResizing_clipToParent_setter(v) {
            var self = this;
            if (v) {
                clove.utils.OnMainResized.addHandler(function(){
                    self.relayout();
                }, this);
            }
        }

        doStandaloneResizing_setter(v) {
            this.setStyleClassAssigned("clove_widget_standaloneresizing", v);
        }

        _topnode_extrasizewithmargin() {
            if (this._sizecache_topnode_extra_ts === this._sizecache_timestamp)
                return this._sizecache_topnode_extra;
            this._sizecache_topnode_extra = clove.utils.getExtraSize(this.topnode, true);
            this._sizecache_topnode_extra_ts = this._sizecache_timestamp;
            return this._sizecache_topnode_extra;
        }

        relayout() {
            if (!this.isAlive())
                return;
            this._sizecache_timestamp++;
            this._layout_invalidated = true;
            this._cache_mw = undefined;
            this._cache_pw = undefined;
            this._cache_mh = undefined;
            this._cache_ph = undefined;
            this._cache_mh_w = undefined;
            this._cache_ph_w = undefined;
            if (this.parentWidget())
                this.parentWidget().relayout();
            if (this.config.doStandaloneResizing) {
                var _clipto = (this.config.doStandaloneResizing_clipToParent) ?
                                    (this.topnode.parentNode == document.body ? ({width:window.innerWidth, height:window.innerHeight}) : clove.utils.getInnerSize(this.topnode.parentNode)) :
                                    undefined;
                if (this.config.width === undefined) {
                    var w = this.getPreferredWidth();
                    if (_clipto)
                        w = Math.min(w, _clipto.width);
                    this.topnode.style.width = w + "px";
                } else {
                    this.topnode.style.width = this.config.width;
                    var w = clove.utils.getInnerSize(this.topnode).width;
                }
                if (this.config.height === undefined) {
                    var h = this.getPreferredHeightForWidth(w);
                    if (_clipto)
                        h = Math.min(h, _clipto.height);
                    this.topnode.style.height = h + "px";
                }
                else
                    this.topnode.style.height = this.config.height;
                this.resize();
            }
        }

        focus() {
            if (this.config.mayFocus)
                this.topnode.focus();
        }

        _calculate_preferred_width() {
            var oclss = this.contentnode.className;
            this.contentnode.className += " clove_widget_contentnode_measure";
            var result = clove.utils.getInnerSize(this.contentnode).width;
            this.contentnode.className = oclss;
            return result;
        }

        _calculate_preferred_height_for_width(w) {
            var oclss = this.contentnode.className;
            this.contentnode.className += " clove_widget_contentnode_measure";
            this.contentnode.style.width = (w+clove.utils.getExtraSize(this.topnode).width) + "px";
            var result = clove.utils.getInnerSize(this.contentnode).height;
            this.contentnode.className = oclss;
            this.contentnode.style.width = "";
            return result;
        }

        computeMinimalWidth() {
            return this._calculate_preferred_width();
        }

        computePreferredWidth() {
            return this._calculate_preferred_width();
        }

        computeMinimalHeightForWidth(w) {
            return this._calculate_preferred_height_for_width(w);
        }

        computePreferredHeightForWidth(w) {
            return this._calculate_preferred_height_for_width(w);
        }

        getMinimalWidth() {
            if (this.effectiveVisibility() == clove.InvisibleCollapsed)
                return 0;
            if (this._cache_mw !== undefined && this._cache_geometryfingerprint_mw == clove.geometryFingerprint)
                return this._cache_mw;
            this._cache_geometryfingerprint_mw = clove.geometryFingerprint;
            var result = this.computeMinimalWidth() + this._topnode_extrasizewithmargin().width;
            this._cache_mw = result;
            return result;
        }

        getPreferredWidth() {
            if (this.effectiveVisibility() == clove.InvisibleCollapsed)
                return 0;
            if (this._cache_pw !== undefined && this._cache_geometryfingerprint_pw == clove.geometryFingerprint)
                return this._cache_pw;
            this._cache_geometryfingerprint_pw = clove.geometryFingerprint;
            var result = this.computePreferredWidth() + this._topnode_extrasizewithmargin().width;
            this._cache_pw = result;
            return result;
        }

        getMinimalHeightForWidth(w) {
            if (this.effectiveVisibility() == clove.InvisibleCollapsed)
                return 0;
            if (this._cache_mh !== undefined && this._cache_geometryfingerprint_mh == clove.geometryFingerprint && this._cache_mh_w == w)
                return this._cache_mh;
            this._cache_geometryfingerprint_mh = clove.geometryFingerprint;
            var extrasize = this._topnode_extrasizewithmargin();
            var result = this.computeMinimalHeightForWidth(Math.max(0, w - extrasize.width)) + extrasize.height;
            this._cache_mh = result;
            this._cache_mh_w = w;
            return result;
        }

        getPreferredHeightForWidth(w) {
            if (this.effectiveVisibility() == clove.InvisibleCollapsed)
                return 0;
            if (this._cache_ph !== undefined && this._cache_geometryfingerprint_ph == clove.geometryFingerprint && this._cache_ph_w == w)
                return this._cache_ph;
            this._cache_geometryfingerprint_ph = clove.geometryFingerprint;
            var extrasize = this._topnode_extrasizewithmargin();
            var result = this.computePreferredHeightForWidth(Math.max(0, w - extrasize.width)) + extrasize.height;
            this._cache_ph = result;
            this._cache_ph_w = w;
            return result;
        }

        addStyleClass(clss, _skiprelayout) {
            $(this.topnode).addClass(clss);
            if (this.isInitialized && !_skiprelayout)
                this.relayout();
        }

        removeStyleClass(clss, _skiprelayout) {
            $(this.topnode).removeClass(clss);
            if (this.isInitialized && !_skiprelayout)
                this.relayout();
        }

        isStyleClass(clss) {
            return $(this.topnode).hasClass(clss);
        }

        setStyleClassAssigned(clss, assigned, _skiprelayout) {
            clove.utils.setStyleClassAssigned(this.topnode, clss, assigned);
            if (this.isInitialized && !_skiprelayout)
                this.relayout();
        }

        _getNameScope(n) {
            var t = n.closest("[data-clove_namescope='1']");
            if (t.length > 0)
                return t[0]['_clove_widget'];
            else
                return this._rootNameScope;
        }

        containingNameScope() {
            return this._getNameScope($(this.topnode).parent());
        }

        nameScope() {
            return this._getNameScope($(this.topnode));
        }

        remove(removeconfig) {
            var self = this;
            this._removing = true;
            removeconfig = removeconfig || new Object();
            var animationname = removeconfig.animationName;
            if (animationname === undefined)
                animationname = this.getProperty("defaultRemovalAnimationName");
            var animationduration = removeconfig.animationDuration;
            if (animationduration === undefined)
                animationduration = this.getProperty("defaultRemovalAnimationDuration") || 150;
            if (animationname && animationname != "" /* yes, the latter part is redundant */) {
                this.addStyleClass("clove_widget_removing", true);
                this.enabled_setter(false, true);
                clove.utils.animateNode(this.topnode, animationname, animationduration, function(){
                    self.remove({animationName: false});
                });
            }
            else
                this.topnode.remove();
        }

    }

    clove.Widget = Widget;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          LABEL

    class Label extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            var self = this;
            this._noop = new Object();
            this.declareProperty("label", this._noop);
            this.declareProperty("htmlContent", this._noop);
            this.declareProperty("focusBuddy");
            $(this.topnode).addClass("clove_label");
            $(this.topnode).click(function(){
                if (self.config.focusBuddy) {
                    var w = self.config.focusBuddy;
                    if (!w.focus) {
                        var ns;
                        var _ns = self;
                        while (_ns) {
                            ns = _ns;
                            _ns = _ns.containingNameScope();
                        }
                        w = ns.getByName(w);
                    }
                    if (w)
                        w.focus();
                }
            });
        }

        label_getter() {
            return this.contentnode.textContent;
        }

        label_setter(text) {
            if (text == this._noop)
                return;
            this.contentnode.textContent = text;
            this.relayout();
        }

        htmlContent_getter() {
            return this.contentnode.innerHTML;
        }

        htmlContent_setter(text) {
            if (text == this._noop)
                return;
            this.contentnode.innerHTML = text;
            this.relayout();
        }

        computeMinimalWidth() {
            return 0;
        }

    }

    clove.Label = Label;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          HTMLVIEW

    class HtmlView extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("contentRoot");
            $(this.topnode).addClass("clove_htmlview");
        }

        contentRoot_getter() {
            return this.contentnode.childNodes[0];
        }

        contentRoot_setter(rootnode) {
            this.contentnode.innerHTML = "";
            if (rootnode)
                this.contentnode.appendChild(rootnode);
            this.relayout();
        }

    }

    clove.HtmlView = HtmlView;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          SPACER

    class Spacer extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            $(this.topnode).addClass("clove_spacer");
        }

    }

    clove.Spacer = Spacer;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          LAYOUT

    let Layout = (superclass) => class extends superclass {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("children", []);
            this._children = [];
        }

        childrenWidgets() {
            return this._children;
        }

        children_setter(childs) {
            var xxx = clove.utils.suspendResizing();
            try {
                this.clearChilds();
                for(var i=0; i<childs.length; i++)
                    this.addChild(childs[i], true);
                this.relayout();
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        clearChilds() {
            this._children = [];
            this.contentnode.innerHTML = "";
            this.relayout();
        }

        addChild(config, _skiprelayout) {
            var self = this;
            var indiv1 = document.createElement("div");
            this.contentnode.appendChild(indiv1);
            var child = clove.build(config, {domnode: indiv1, parent: this});
            this._children.push(child);
            child.OnDestroyed.addHandler(function(){
                var io = self._children.indexOf(child);
                if (io > -1) {
                    self._children.splice(io, 1);
                    self.relayout();
                }
            });
            if (!_skiprelayout)
                this.relayout();
            return child;
        }

        doresize() {
            var chlds = this.childrenWidgets();
            for(var i=0; i<chlds.length; i++) {
                var chld = chlds[i];
                chld.resize();
            }
            super.doresize();
        }

        relayout() {
            super.relayout();
            this.resize();
        }

    };

    clove.Layout = Layout;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          GRID

    let GridLayout = (superclass) => class extends Layout(superclass) {

        constructor(config, domnode) {
            super(config, domnode);
        }

        _getDims(dim, mode, widths) {
            if (dim == "w") {
                var partname = "col";
                var dimname = "width";
                var iscol = true;
                var oraclename = (mode=="p") ? "getPreferredWidth" : "getMinimalWidth";
                var axisname = "x";
            } else {
                var partname = "row";
                var dimname = "height";
                var iscol = false;
                var oraclename = (mode=="p") ? "getPreferredHeightForWidth" : "getMinimalHeightForWidth";
                var axisname = "y";
            }
            var cnt = 0;
            var chlds = this.childrenWidgets();
            for(var ichld in chlds)
                cnt = Math.max(cnt, chlds[ichld].getProperty(partname, 0) + 1);
            var dims = [];
            var rawdims = [];
            for(var ic=0; ic<cnt; ic++)
                dims.push(0);
            for(var ichld in chlds)
            {
                var chld = chlds[ichld];
                var col = chld.getProperty("col", 0);
                var row = chld.getProperty("row", 0);
                var fixwi = clove.utils.cssLengthToPixels(chld[dimname](), axisname);
                var i = iscol?col:row;
                if (dim == "w")
                    var oracleparam = undefined;
                else
                    var oracleparam = widths[col];
                if (chld.visibility() == clove.InvisibleCollapsed)
                    var dval = 0;
                else if (fixwi === undefined)
                    var dval = chld[oraclename](oracleparam) + 0.1 /* this avoids trouble with rounding */;
                else
                    var dval = fixwi + 0.1 /* this avoids trouble with rounding */;
                dims[i] = Math.max(dims[i], dval);
                while (rawdims.length <= row)
                    rawdims.push([]);
                while (rawdims[row].length <= col)
                    rawdims[row].push([]);
                rawdims[row][col] = dval;
            }
            return [dims, rawdims];
        }

        _getFullWidths(mywidth) {
            var chlds = this.childrenWidgets();
            var rows = 0;
            var cols = 0;
            for(var ichld in chlds) {
                var chld = chlds[ichld];
                var row = chld.getProperty("row", 0);
                var col = chld.getProperty("col", 0);
                rows = Math.max(rows, row + 1);
                cols = Math.max(cols, col + 1);
            }
            var acolstretch = [];
            var arowstretch = [];
            for(var ir=0; ir<rows; ir++)
                arowstretch.push(0);
            for(var ic=0; ic<cols; ic++)
                acolstretch.push(0);
            for(var ichld in chlds) {
                var chld = chlds[ichld];
                if (chld.visibility() != clove.InvisibleCollapsed) {
                    var col = chld.getProperty("col", 0);
                    var row = chld.getProperty("row", 0);
                    acolstretch[col] = Math.max(chld.horizontalStretchAffinity(), acolstretch[col]);
                    arowstretch[row] = Math.max(chld.verticalStretchAffinity(), arowstretch[row]);
                }
            }
            var minwidths = this._getDims("w", "m")[0];
            var _prefwidths = this._getDims("w", "p");
            var prefwidths = _prefwidths[0];
            var rawprefwidths = _prefwidths[1];
            var minwidthsum = clove.utils.arraysum(minwidths);
            var widths = minwidths;
            var widthsum = minwidthsum;
            if (minwidthsum < mywidth) {
                var prefwidthsum = clove.utils.arraysum(prefwidths);
                if (prefwidthsum <= mywidth) {
                    var widths = prefwidths;
                    var widthsum = prefwidthsum;
                }
                else
                    acolstretch = this._getMissingToPreferred(widths, prefwidths);
                var over = mywidth - widthsum;
                if (over > 0) {
                    var wstrsum = clove.utils.arraysum(acolstretch);
                    if (wstrsum)
                        for(var ic=0; ic<cols;ic++)
                            widths[ic] += over * (acolstretch[ic] / wstrsum);
                }
            }
            return [widths, cols, rows, arowstretch, rawprefwidths];
        }

        _getMissingToPreferred(tgtsizes, prefsizes) {
            var result = [];
            for(var i in tgtsizes)
                result.push(Math.max(0, prefsizes[i] - tgtsizes[i]));
            return result;
        }

        _getFullDims(mywidth, myheight) {
            var chlds = this.childrenWidgets();
            var fullwidths = this._getFullWidths(mywidth);
            var widths = fullwidths[0];
            var cols = fullwidths[1];
            var rows = fullwidths[2];
            var arowstretch = fullwidths[3];
            var rawprefwidths = fullwidths[4];
            var minheights = this._getDims("h", "m", widths)[0];
            var _prefheights = this._getDims("h", "p", widths);
            var prefheights = _prefheights[0];
            var rawprefheights = _prefheights[1];
            var minheightsum = clove.utils.arraysum(minheights);
            var heights = minheights;
            var heightsum = minheightsum;
            if (minheightsum < myheight) {
                var prefheightsum = clove.utils.arraysum(prefheights);
                if (prefheightsum <= myheight) {
                    var heights = prefheights;
                    var heightsum = prefheightsum;
                }
                else
                    arowstretch = this._getMissingToPreferred(heights, prefheights);
                var over = myheight - heightsum;
                if (over > 0) {
                    var wstrsum = clove.utils.arraysum(arowstretch);
                    if (wstrsum)
                        for(var ir=0; ir<rows;ir++)
                            heights[ir] += (over) * (arowstretch[ir] / wstrsum);
                }
            }
            var xcoords = [];
            for (var ic=0; ic<cols; ic++)
                xcoords.push(widths[ic] + (ic>0 ? xcoords[ic-1] : 0));
            var ycoords = [];
            for (var ir=0; ir<rows; ir++)
                ycoords.push(heights[ir] + (ir>0 ? ycoords[ir-1] : 0));
            return [xcoords, ycoords, widths, heights, cols, rows, rawprefwidths, rawprefheights];
        }

        computeMinimalWidth() {
            return clove.utils.arraysum(this._getDims("w", "m")[0]);
        }

        computePreferredWidth() {
            return clove.utils.arraysum(this._getDims("w", "p")[0]);
        }

        computeMinimalHeightForWidth(w) {
            var fullwidths = this._getFullWidths(w)[0];
            return clove.utils.arraysum(this._getDims("h", "m", fullwidths)[0]);
        }

        computePreferredHeightForWidth(w) {
            var fullwidths = this._getFullWidths(w)[0];
            return clove.utils.arraysum(this._getDims("h", "p", fullwidths)[0]);
        }

        doresize() {
            var chlds = this.childrenWidgets();
            var myisize = clove.utils.getInnerSize(this.topnode);
            var _x = this._getFullDims(myisize.width, myisize.height);
            var xcoords = _x[0];
            var ycoords = _x[1];
            var colwidths = _x[2];
            var rowheights = _x[3];
            var rawprefwidths = _x[6];
            var rawprefheights = _x[7];
            for(var ichld in chlds)
            {
                var chld = chlds[ichld];
                var row = chld.getProperty("row", 0);
                var col = chld.getProperty("col", 0);
                var strictHorizontalSizing = chld.strictHorizontalSizing();
                var strictVerticalSizing = chld.strictVerticalSizing();
                var tn = chld.topnode;
                var x = xcoords[col-1]||0;
                var y = ycoords[row-1]||0;
                var w = colwidths[col];
                var h = rowheights[row];
                if (rawprefwidths) {
                    var pw = rawprefwidths[row][col];
                    var ph = rawprefheights[row][col];
                    var diffw = w - pw;
                    var diffh = h - ph;
                    if (strictHorizontalSizing && diffw > 0)
                        w = pw;
                    if (strictVerticalSizing && diffh > 0)
                        h = ph;
                }
                tn.style.position = "absolute";
                tn.style.left = x + "px";
                tn.style.top = y + "px";
                var st = getComputedStyle(tn);
                var hmarg = parseFloat(st.marginLeft) + parseFloat(st.marginRight);
                var vmarg = parseFloat(st.marginTop) + parseFloat(st.marginBottom);
                tn.style.width = (w - hmarg) + "px";
                tn.style.height = (h - vmarg) + "px";
            }
            super.doresize();
        }

        _computeStretchAffinity(sumaxis, fctname) {
            var parts = [];
            var chlds = this.childrenWidgets();
            for(var ichld in chlds) {
                var chld = chlds[ichld];
                var ipos = chld.getProperty(sumaxis);
                while(parts.length <= ipos)
                    parts.push(0);
                parts[ipos] = Math.max(parts[ipos], chld[fctname]());
            }
            return clove.utils.arraysum(parts);
        }

        horizontalStretchAffinity_getter() {
            if (this.config.horizontalStretchAffinity === undefined)
                return this._computeStretchAffinity("col", "horizontalStretchAffinity");
            else
                return this.config.horizontalStretchAffinity || 0;
        }

        verticalStretchAffinity_getter() {
            if (this.config.verticalStretchAffinity === undefined)
                return this._computeStretchAffinity("row", "verticalStretchAffinity");
            else
                return this.config.verticalStretchAffinity || 0;
        }

        insertRow(i) {
            var lcw = this.childrenWidgets();
            for(var icw=0; icw<lcw.length; icw++) {
                var cw = lcw[icw];
                var cwrow = cw.getProperty("row", 0);
                if (cwrow >= i)
                    cw.setProperty("row", cwrow+1);
            }
            this.relayout();
        }

        insertColumn(i) {
            var lcw = this.childrenWidgets();
            for(var icw=0; icw<lcw.length; icw++) {
                var cw = lcw[icw];
                var cwcol = cw.getProperty("col", 0);
                if (cwcol >= i)
                    cw.setProperty("col", cwcol+1);
            }
            this.relayout();
        }

        removeRow(i) {
            var lcw = this.childrenWidgets();
            for(var icw=0; icw<lcw.length; icw++) {
                var cw = lcw[icw];
                var cwrow = cw.getProperty("row", 0);
                if (cwrow > i)
                    cw.setProperty("row", cwrow-1);
                else if (cwrow == i)
                    cw.topnode.remove();
            }
        }

        removeColumn(i) {
            var lcw = this.childrenWidgets();
            for(var icw=0; icw<lcw.length; icw++) {
                var cw = lcw[icw];
                var cwcol = cw.getProperty("col", 0);
                if (cwcol > i)
                    cw.setProperty("col", cwcol-1);
                else if (cwcol == i)
                    cw.topnode.remove();
            }
        }

        addChild(config, _skiprelayout) {
            var irow = config["row"]||0;
            var icol = config["col"]||0;
            var lcw = this.childrenWidgets();
            for(var icw=0; icw<lcw.length; icw++) {
                var cw = lcw[icw];
                var cwcol = cw.getProperty("col", 0);
                var cwrow = cw.getProperty("row", 0);
                if (cwcol == icol && cwrow == irow)
                    cw.remove({animationName: ""});
            }
            return super.addChild(config, _skiprelayout);
        }

    };

    clove.GridLayout = GridLayout;

    class Grid extends clove.GridLayout(clove.Widget) {

        constructor(config, domnode) {
            super(config, domnode);
        }

    }

    clove.Grid = Grid;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          STACK

    let StackLayout = (superclass) => class extends GridLayout(superclass) {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("orientation", "vertical");
        }

        orientation() {
            return this.config.orientation;
        }

        addChild(config) {
            var pref = (this.orientation() == 'horizontal') ? 'col' : 'row';
            var dflts = {};
            dflts[pref] = this.childrenWidgets().length;
            return super.addChild(clove.utils.applyDefaultsToConfig(config, dflts));
        }

    }

    clove.StackLayout = StackLayout;

    class HorizontalStack extends clove.StackLayout(clove.Widget) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'horizontal',
            }), domnode);
            this.declareProperty("cols", []);
        }

        cols_setter(v) {
            this.setChildren(v);
        }

    }

    clove.HorizontalStack = HorizontalStack;

    class VerticalStack extends clove.StackLayout(clove.Widget) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
            }), domnode);
            this.declareProperty("rows", []);
        }

        rows_setter(v) {
            this.setChildren(v);
        }

    }

    clove.VerticalStack = VerticalStack;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          WRAP

    let WrapLayout = (superclass) => class extends Layout(superclass) {

        constructor(config, domnode) {
            super(config, domnode);
        }

        computeMinimalWidth() {
            return 100;
        }

        computePreferredWidth() {
            return 100;
        }

        computeMinimalHeightForWidth(w) {
            return this._calcgeometry(w - 0.1 /* this avoids trouble with rounding */).height;
        }

        computePreferredHeightForWidth(w) {
            return this._calcgeometry(w - 0.1 /* this avoids trouble with rounding */).height;
        }

        _calcgeometry(w) {
            var chlds = this.childrenWidgets();
            var ycur = 0;
            var xcur = 0;
            var hmax = 0;
            var childgeos = [];
            for (var chld of chlds) {
                var tn = chld.topnode;
                var hval = 0, wval = 0;
                tn.style.position = "absolute";
                if (chld.visibility() != clove.InvisibleCollapsed) {
                    var fixwi = clove.utils.cssLengthToPixels(chld.width(), 'x');
                    wval = ((fixwi === undefined) ? chld.getPreferredWidth() : fixwi) + 0.1 /* this avoids trouble with rounding */;
                    var fixhe = clove.utils.cssLengthToPixels(chld.height(), 'y');
                    hval = ((fixhe === undefined) ? chld.getPreferredHeightForWidth(wval) : fixhe) + 0.1 /* this avoids trouble with rounding */;
                    hmax = Math.max(hmax, hval);
                }
                if (xcur + wval > w) {
                    xcur = 0;
                    ycur += hmax;
                    hmax = hval;
                }
                var st = getComputedStyle(tn);
                var hmarg = parseFloat(st.marginLeft) + parseFloat(st.marginRight);
                var vmarg = parseFloat(st.marginTop) + parseFloat(st.marginBottom);
                childgeos.push({tn: tn, x: xcur, y: ycur, w: wval-hmarg, h: hval-vmarg});
                xcur += wval;
            }
            return {childgeos: childgeos, height: ycur+hmax};
        }

        doresize() {
            for (var childgeo of this._calcgeometry(clove.utils.getInnerSize(this.topnode).width).childgeos) {
                childgeo.tn.style.left = childgeo.x + "px";
                childgeo.tn.style.top = childgeo.y + "px";
                childgeo.tn.style.width = childgeo.w + "px";
                childgeo.tn.style.height = childgeo.h + "px";
            }
            super.doresize();
        }

    };

    clove.WrapLayout = WrapLayout;

    class Wrap extends clove.WrapLayout(clove.Widget) {

        constructor(config, domnode) {
            super(config, domnode);
        }

    }

    clove.Wrap = Wrap;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          BUTTON

    class Button extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("icon");
            this.declareProperty("checkable", false);
            this.declareProperty("checked", false);
            this.declareProperty("label", undefined);
            $(this.topnode).addClass("clove_button");
            var self = this;
            this._a = document.createElement("a");
            this._a.style.whiteSpace = "pre-wrap";
            this._a_icon = undefined;
            this.contentnode.appendChild(this._a);
            this._a_checked = document.createElement("span");
            this._a.appendChild(this._a_checked);
            this._a_checked.className = "clove_button_checkflag";
            this._a_label = document.createElement("span");
            this._a_label.className = "clove_button_label";
            this._a.appendChild(this._a_label);
            this.OnClicked = new clove.Event();
            $(this._a).click(function(e){
                e.preventDefault();
                self.OnClicked.trigger({widget: self});
            });
        }

        _check_helper() {
            if (this.config.checkable) {
                this._a_checked.style.visibility = "inherit";
                if (this.config.checked) {
                    this._a_checked.textContent = clove.symbols.CircleFilled;
                    this.setStyleClassAssigned("clove_button_checked", true);
                    this.setStyleClassAssigned("clove_button_unchecked", false);
                }
                else {
                    this._a_checked.textContent = clove.symbols.Circle;
                    this.setStyleClassAssigned("clove_button_checked", false);
                    this.setStyleClassAssigned("clove_button_unchecked", true);
                }
            }
            else {
                this._a_checked.textContent = clove.symbols.Circle;
                this._a_checked.style.visibility = "hidden";
                this.setStyleClassAssigned("clove_button_checked", false);
                this.setStyleClassAssigned("clove_button_unchecked", false);
            }
        }

        checkable_setter(v) {
            this._check_helper();
        }

        checked_setter(v) {
            this._check_helper();
        }

        label_setter(text) {
            this._a_label.textContent = text;
            this.relayout();
        }

        icon_setter(icon) {
            if (this._a_icon)
                this._a_icon.remove();
            if (icon) {
                this._a_icon = icon.createHtml(clove.utils.cssLengthToPixels(getComputedStyle(this._a_label).lineHeight, "y"), true);
                if (this._a_icon) {
                    this._a.insertBefore(this._a_icon, this._a_label);
                    this._a_icon.style.display = "inline-block";
                }
            }
            this.relayout();
        }

        focus() {
            this._a.focus();
        }

    }

    clove.Button = Button;

    class PopupMenuButton extends clove.Button {

        constructor(config, domnode) {
            super(config, domnode);
            var self = this;
            this.declareProperty("actions", []);
            this._a_popupsymbol = document.createElement("span");
            this._a_popupsymbol.className = "clove_button_popupmarker";
            this._a_popupsymbol.textContent = clove.symbols.TriangleDown;
            $(this._a_checked).before(this._a_popupsymbol);
            this.OnActionTriggered = new clove.Event();
            this.OnClicked.addHandler(function(){
                var popupmenu = clove.PopupMenu.show({view:'PopupMenu', actions:self.actions()}, {alignTo: self}).widget;
                popupmenu.OnActionTriggered.addHandler(function(e){
                    self.OnActionTriggered.trigger({widget: self, action: e.action, originalwidget: self});
                });
            });
        }

    }

    clove.PopupMenuButton = PopupMenuButton;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          MENU

    clove.MenuSeparator = { separator: true };

    class AbstractMenu extends clove.StackLayout(clove.Widget) {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("actions", []);
            this.OnActionTriggered = new clove.Event();
            this.OnBeforeSubactionsExpanded = new clove.Event();
            this._alignToAxis = "vertical";
            this._insertFctName = "insertColumn";
            this._removeFctName = "removeColumn";
            this._idxName = "col";
        }

        remove(removeconfig) {
            super.remove(removeconfig);
        }

        actions_setter(v) {
            if (v && !v.valuePointer) // not a datasource
                return this.setActions(clove.utils.arrayToDatasource(v));
            var xxx = clove.utils.suspendResizing();
            try {
                this._getItemsContainer().setChildren([]);
                this._clearAllActions();
                clove.utils.connectDatasourceToWidget(this, v, "_lastds", "actionsParent",
                                                      "_ondatainserthandler", "_ondataremovehandler", "_ondataupdatehandler");
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        _actionlabel(i, actionconfig) {
            return (actionconfig.label === undefined) ? actionconfig.name : actionconfig.label;
        }

        _expanderIndicatorDirection(btn) {
            return (btn.topnode.getBoundingClientRect().left < (window.innerWidth/2)) ? 'right' : 'left';
        }

        _ondatainserthandler(ee) {
            var xxx = clove.utils.suspendResizing();
            try {
                var actionsparent = this.getProperty("actionsParent", undefined);
                if (DatasourceValuePointer.equals(ee.parent, actionsparent)) {
                    var _inserted = false;
                    if (ee.c1 !== undefined) {
                        if (ee.c1<=0 && ee.c2>=0) {
                            this._clearAllActions();
                            var rowcount = this.config.actions.rowCount(actionsparent);
                            for(var ir=0;ir<rowcount;ir++)
                                this._insertAction(ir, this.config.actions.getValue(this.config.actions.valuePointer(ir, 0, actionsparent)));
                            _inserted = true;
                        }
                    }
                    if (!_inserted && ee.r1 !== undefined) {
                        var colcount = this.config.actions.columnCount(actionsparent);
                        if (colcount > 0) {
                            for(var ir=ee.r1;ir<=ee.r2;ir++)
                                this._insertAction(ir, this.config.actions.getValue(this.config.actions.valuePointer(ir, 0, actionsparent)));
                        }
                    }
                    this.relayout();
                }
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        _ondataremovehandler(ee) {
            var xxx = clove.utils.suspendResizing();
            try {
                var actionsparent = this.getProperty("actionsParent", undefined);
                if (DatasourceValuePointer.equals(ee.parent, actionsparent)) {
                    if (ee.c1 !== undefined) {
                        var columncount = this.config.actions.columnCount(actionsparent);
                        if (columncount == 0)
                            this._clearAllActions();
                    }
                    if (ee.r1 !== undefined) {
                        for(var ir=ee.r2;ir>=ee.r1;ir--)
                            this._removeAction(ir);
                    }
                    this.relayout();
                }
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        _ondataupdatehandler(ee) {
            var xxx = clove.utils.suspendResizing();
            try {
                var actionsparent = this.getProperty("actionsParent", undefined);
                if (DatasourceValuePointer.equals(ee.parent, actionsparent)) {
                    for(var ir=ee.r1; ir<=ee.r2;ir++)
                        for(var ic=ee.c1; ic<=ee.c2;ic++) {
                            if (ic == 0)
                                this._createMenuActionButton(ir, this.config.actions.getValue(this.config.actions.valuePointer(ir, ic, actionsparent)));
                        }
                    this.relayout();
                }
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        _getItemsContainer() {
            return this;
        }

        _getSubactions(i, actionconfig) {
            if (this.config.actions && this.config.actions.valuePointer) {
                if (this.config.actions.rowCount(this.config.actionsParent)>i && this.config.actions.columnCount(this.config.actionsParent)>0) {
                    var ptr = this.config.actions.valuePointer(i, 0, this.config.actionsParent);
                    if (this.config.actions.rowCount(ptr)>0 && this.config.actions.columnCount(ptr)>0)
                        return [this.config.actions, ptr];
                }
            }
            if (actionconfig.subactions)
                return [actionconfig.subactions];
        }

        _actionicon(i, actionconfig) {
            return actionconfig.icon;
        }

        _createMenuActionButton(i, actionconfig) {
            var self = this;
            var itemscontainer = this._getItemsContainer();
            actionconfig = clove.utils.applyDefaultsToConfig(actionconfig, { label: undefined, name: '' });
            var actlabel = this._actionlabel(i, actionconfig);
            var acticon = this._actionicon(i, actionconfig);
            var actvis = actionconfig.invisible ? clove.InvisibleCollapsed : clove.Visible;
            if (actionconfig.separator) {
                var btncfg = { view: 'Widget', styleClass: 'clove_menu_separator', visibility: actvis };
                btncfg[this._idxName] = i;
                var btn = itemscontainer.addChild(btncfg);
            }
            else if (actionconfig.header) {
                var btncfg = {cols: [{view: 'IconView', icon: acticon, size: '1em'}, {view: 'Label', label: actlabel}], visibility: actvis, styleClass: 'clove_menu_header', hstretch: 1};
                btncfg[this._idxName] = i;
                var btn = itemscontainer.addChild(btncfg);
            }
            else {
                var btncfg = { view: 'Button', label: actlabel, icon: acticon, checkable: actionconfig.checkable, checked: actionconfig.checked, style: actionconfig.style, visibility: actvis, disabled: actionconfig.disabled, styleClass: (actionconfig.disabled?"clove_menu_disableditem":"") };
                btncfg[this._idxName] = i;
                var btn = itemscontainer.addChild(btncfg);
                this._getSubactions(i, actionconfig);  // this helps pre-fetching stuff in an async datasource
                btn.OnClicked.addHandler(function(e){
                    if (btn.getProperty("disabled"))
                        return;
                    var myi = btn.config[self._idxName];
                    self.OnBeforeSubactionsExpanded.trigger({widget: self, action: actionconfig, valuePointer: (self.config.actions && self.config.actions.valuePointer) ? (self.config.actions.valuePointer(myi, 0, self.config.actionsParent)) : undefined});
                    var subactions = self._getSubactions(myi, actionconfig);
                    if (subactions) {
                        var expanderIndicatorDirection = self._expanderIndicatorDirection(btn);
                        var popupmenu = clove.PopupMenu.show({
                            view: "PopupMenu",
                            actions: subactions[0],
                            actionsParent: subactions[1],
                            parentMenu: self,
                            expanderIndicatorDirection: expanderIndicatorDirection,
                        }, {alignTo: btn, alignToAxis: self._alignToAxis}).widget;
                        popupmenu.OnActionTriggered.addHandler(function(e){
                            self.OnActionTriggered.trigger(e);
                        });
                        popupmenu.OnBeforeSubactionsExpanded.addHandler(function(e){
                            e.widget = self;
                            self.OnBeforeSubactionsExpanded.trigger(e);
                        });
                        self.OnDestroyed.addHandler(function(){
                            try {
                                popupmenu.remove();
                            }
                            catch(e) {}
                        });
                        popupmenu.topnode.style.textAlign = (expanderIndicatorDirection == 'left') ? 'right' : 'left';
                    }
                    else
                        self.OnActionTriggered.trigger({ action: actionconfig.name, widget: self, originalwidget: btn });
                });
            }
            this.relayout();
            return btn;
        }

        _insertAction(i, actionconfig) {
            this._getItemsContainer()[this._insertFctName](i);
            return this._createMenuActionButton(i, actionconfig);
        }

        _removeAction(i) {
            this._getItemsContainer()[this._removeFctName](i);
        }

        _clearAllActions() {
            this._getItemsContainer().clearChilds();
        }

    }

    class Menubar extends AbstractMenu {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'horizontal',
            }), domnode);
            $(this.topnode).addClass("clove_menubar");
        }

    }

    clove.Menubar = Menubar;

    class PopupMenu extends clove.NameScope(AbstractMenu) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [{view: 'ScrollView', body: {rows:[], name: 'mainbody'}}],
            }), domnode);
            this.declareProperty("expanderIndicatorDirection", "right");
            $(this.topnode).addClass("clove_popupmenu");
            this._alignToAxis = "horizontal";
            this._insertFctName = "insertRow";
            this._removeFctName = "removeRow";
            this._idxName = "row";
        }

        _getItemsContainer() {
            var r = this._itemscontainer = this._itemscontainer || this.getByName("mainbody");
            return r;
        }

        _actionlabel(i, actionconfig) {
            var actlabel = (actionconfig.label === undefined) ? actionconfig.name : actionconfig.label;
            var subactions = this._getSubactions(i, actionconfig);
            if (subactions) {
                if (this.config.expanderIndicatorDirection == 'left')
                    actlabel = '🢐 ' + actlabel;
                else
                    actlabel = actlabel + ' 🢒';
            }
            return actlabel;
        }

        _actionicon(i, actionconfig) {
            return actionconfig.icon||clove.Icon.bySymbol(' ');
        }

        _expanderIndicatorDirection(btn) {
            return this.config.expanderIndicatorDirection;
        }

        static show(menu, showconfig) {
            var result = clove.utils.popup(menu, clove.utils.applyDefaultsToConfig(showconfig, {
                modal: true,
            }));
            result.modalpanel.OnClicked.addHandler(function(e){
                result.widget.remove();
                var pm = menu.parentMenu;
                var clx = e.originalEvent.clientX;
                var cly = e.originalEvent.clientY;
                while (pm) {
                    var _pmr = clove.utils.getOuterSize(pm.topnode);
                    var _pmo = $(pm.topnode).offset();
                    var pmx = _pmo.left;
                    var pmy = _pmo.top;
                    if (pm instanceof PopupMenu && (clx < pmx || clx > (pmx+_pmr.width) || cly < pmy || cly > (pmy+_pmr.height)))
                        pm.remove();
                    else
                        break;
                    pm = pm.config.parentMenu;
                }
            });
            result.widget.OnActionTriggered.addHandler(function(){
                result.widget.remove();
            });
            return result;
        }

    }

    clove.PopupMenu = PopupMenu;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          TOOLBAR

    class Toolbar extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            var children = [
                {cols:[
                    {cols:[
                        {view: 'IconView', name: 'icon', strictVerticalSizing: true, styleClass: 'clove_toolbar_icon'},
                        {rows: [
                            { view: 'Label', name: 'head1', styleClass: 'clove_toolbar_head1' },
                            { view: 'Label', name: 'head2', styleClass: 'clove_toolbar_head2' },
                            { cols: [], name: 'head2alt', styleClass: 'clove_toolbar_head2', newNameScope: true },
                        ]},
                    ], name: 'leftpart'},
                    {width: '1px', horizontalStretchAffinity: 1},
                    {rows: [
                        { label: '', height: '0px', verticalStretchAffinity: 1 },
                        {cols: [
                            { view: 'Menubar', name: 'menubar' },
                            { view: 'PopupMenuButton', name: 'popupbtn', styleClass: 'clove_toolbar_menubutton', icon: clove.Icon.bySymbol(clove.symbols.Actions), label: ''},
                        ]},
                    ]},
                ]},
                {rows: [], horizontalStretchAffinity: 1, name: 'headcontrol', newNameScope: true},
            ];
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: children,
            }), domnode);
            var self = this;
            this.declareProperty("actions", []);
            this.declareProperty("head1");
            this.declareProperty("head2");
            this.declareProperty("headControl");
            this.declareProperty("icon");
            $(this.topnode).addClass("clove_toolbar");
            this.OnActionTriggered = new clove.Event();
        }

        doresize() {
            var _oval = this._smallmode;
            var menubar = this.getByName("menubar");
            if (this.config.actions) {
                var menubarwidth = menubar.getPreferredWidth();
                var leftpartwidth = this.getByName("leftpart").outerSize().width;
                var mywidth = this.innerSize().width;
            }
            this._smallmode =  this.config.actions && (mywidth < (menubarwidth + leftpartwidth + 20));
            if (_oval != this._smallmode) {
                var popupbtn = this.getByName("popupbtn");
                menubar.setWidth(this._smallmode?"1px":undefined);
                popupbtn.setWidth(this._smallmode?undefined:"1px");
                menubar.setHeight(this._smallmode?"1px":undefined);
                popupbtn.setHeight(this._smallmode?undefined:"1px");
                menubar.setVisibility(this._smallmode?clove.Invisible:clove.Visible);
                popupbtn.setVisibility(this._smallmode?clove.Visible:clove.Invisible);
            }
            super.doresize();
        }

        actions_getter() {
            return this.getByName("menubar").actions();
        }

        actions_setter(v) {
            this.getByName("menubar").setActions(v);
            this.getByName("popupbtn").setActions(v);
            this.doresize();
        }

        head1_setter(v) {
            this.getByName("head1").setLabel(v);
        }

        head2_setter(v) {
            var h2 = this.getByName("head2");
            var h2a = this.getByName("head2alt");
            if (v && v.view) {
                h2a.setChildren([v]);
                h2a.setVisibility(clove.Visible);
                h2.setVisibility(clove.InvisibleCollapsed);
            }
            else {
                h2.setLabel(v);
                h2.setVisibility(clove.Visible);
                h2a.setVisibility(clove.InvisibleCollapsed);
            }
        }

        headControlWidget() {
            var chl = this.getByName("headcontrol").childrenWidgets();
            if (chl.length > 0)
                return chl[0];
        }

        headControl_setter(v) {
            var cl = [];
            if (v)
                cl.push(v);
            this.getByName("headcontrol").setChildren(cl);
        }

        icon_setter(v) {
            var wicon = this.getByName("icon");
            wicon.setVisibility(v ? clove.Visible : clove.InvisibleCollapsed);
            wicon.setIcon(v);
        }

        children_setter(v) {
            super.children_setter(v);
            this.containingNameScope().addChildNameScope(this.getByName('head2alt'));
            this.containingNameScope().addChildNameScope(this.getByName('headcontrol'));
        }

        doinit() {
            super.doinit();
            var self = this;
            var _trigger = function(e) {
                self.OnActionTriggered.trigger({widget: self, action: e.action, originalwidget : e.originalwidget});
            }
            this.getByName("menubar").OnActionTriggered.addHandler(_trigger);
            this.getByName("popupbtn").OnActionTriggered.addHandler(_trigger);
        }

    }

    clove.Toolbar = Toolbar;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          BORDER

    class Border extends clove.StackLayout(clove.Widget) {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("body");
            $(this.topnode).addClass("clove_border");
        }

        body_setter(v) {
            this.setChildren([v]);
        }

    }

    clove.Border = Border;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          FORM

    class Form extends clove.GridLayout(clove.Widget) {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("sections", []);
            $(this.topnode).addClass("clove_form");
            this._labels = [];
            this._widgets = [];
            this._smallmode = undefined;
        }

        doresize() {
            this._align();
            super.doresize();
        }

        _align() {
            var itemswidth = 0;
            for(var i=0; i<this._widgets.length; i++)
                itemswidth = Math.max(itemswidth, this._widgets[i].getPreferredWidth());
            var labelswidth = 0;
            for(var i=0; i<this._labels.length; i++)
                labelswidth = Math.max(labelswidth, this._labels[i].getPreferredWidth());
            var smallmode = this.innerSize().width < (itemswidth+labelswidth);
            if (this._smallmode === undefined || this._smallmode != smallmode) {
                this._smallmode = smallmode;
                var rowoff = smallmode?0:1;
                var coloff = smallmode?1:0;
                for(var i=0; i<this._labels.length; i++) {
                    var ll = this._labels[i];
                    ll.setProperty('row', i*2 + rowoff);
                    ll.setProperty('col', coloff);
                }
                this.relayout();
            }
        }

        sections_setter(v) {
            var childs = [];
            if (v) {
                for(var i=0; i<v.length; i++) {
                    childs.push({view: 'Label', row:i*2, styleClass: 'clovestyle_text_label clove_form_sectionlabel'});
                    var dcfg = clove.utils.applyDefaultsToConfig(v[i]);
                    dcfg.col = 1;
                    dcfg.row = 1 + i*2;
                    childs.push(dcfg);
                }
            }
            this.setChildren(childs);
            var childwidgets = this.childrenWidgets();
            this._labels = [];
            this._widgets = [];
            for(var i=0; i<v.length; i++) {
                this._labels[i] = childwidgets[i*2];
                this._widgets[i] = childwidgets[1 + i*2];
                this._labels[i].setLabel(this._widgets[i].getProperty("formSectionLabel", ""));
            }
            this._align();
        }

    }

    clove.Form = Form;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          SCROLLVIEW

    class ScrollView extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'horizontal',
                children: [{rows: [], name: 'mainpanel', newNameScope: true, horizontalStretchAffinity: 1, verticalStretchAffinity: 1}],
                horizontalStretchAffinity: 1,
                verticalStretchAffinity: 1,
            }), domnode);
            this.declareProperty("body", {rows: []});
            this.declareProperty("verticalScrollPosition");
            this.declareProperty("horizontalScrollPosition");
            $(this.topnode).addClass("clove_scrollview");
            this.topnode.style.overflow = "auto";
        }

        verticalScrollPosition_getter() {
            return $(this.topnode).scrollTop();
        }

        verticalScrollPosition_setter(v) {
            if (v !== undefined)
                $(this.topnode).scrollTop(v);
        }

        horizontalScrollPosition_getter() {
            return $(this.topnode).scrollLeft();
        }

        horizontalScrollPosition_setter(v) {
            if (v !== undefined)
                $(this.topnode).scrollLeft(v);
        }

        body_setter(v) {
            this.getByName("mainpanel").setChildren([v,]);
        }

        children_setter(v) {
            super.children_setter(v);
            this.containingNameScope().addChildNameScope(this.getByName('mainpanel'));
        }

        bodySize() {
            return this.getByName("mainpanel").outerSize();
        }

        bodyWidget() {
            return this.getByName("mainpanel").childrenWidgets()[0];
        }

        computeMinimalWidth() {
            return 40;
        }

        computeMinimalHeightForWidth(w) {
            return 40;
        }

        doresize() {
            var self = this;
            var ovpad = this.topnode.offsetHeight - this.topnode.clientHeight;
            var ohpad = this.topnode.offsetWidth - this.topnode.clientWidth;
            super.doresize();
            if (clove.browser.webkit) { // workaround for issues with chrome scrollbars appearing for no reason
                this.topnode.style.overflow = "hidden"; this.topnode.offsetHeight; this.topnode.style.overflow = "auto";
            }
            var vpad = this._vpad;
            var hpad = this._hpad;
            this._vpad = this.topnode.offsetHeight - this.topnode.clientHeight;
            this._hpad = this.topnode.offsetWidth - this.topnode.clientWidth;
            if ((vpad != this._vpad) || (hpad != this._hpad) || (ovpad != this._vpad) || (ohpad != this._hpad))
                setTimeout(function(){ self.relayout(); });
        }

    }

    clove.ScrollView = ScrollView;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          MAINVIEW

    class MainViewMenuHelperDatasource extends clove.Datasource(Object) {

        constructor(innerdatasource) {
            super();
            var self = this;
            this._lastv = false;
            this.innerdatasource = innerdatasource;
            innerdatasource.OnDataInsert.addHandler(function(e){self.OnDataInsert.trigger({parent: e.parent, r1: e.r1, r2: e.r2, c1: e.c1, c2: e.c2});});
            innerdatasource.OnDataRemove.addHandler(function(e){self.OnDataRemove.trigger({parent: e.parent, r1: e.r1, r2: e.r2, c1: e.c1, c2: e.c2});});
            innerdatasource.OnDataUpdate.addHandler(function(e){self.OnDataUpdate.trigger({parent: e.parent, r1: e.r1, r2: e.r2, c1: e.c1, c2: e.c2});});
        }

        setAdditionalActionEnabled(v) {
            if (this._lastv != v) {
                var _rc = this.innerdatasource.rowCount();
                this._lastv = v;
                if (v)
                    this.OnDataInsert.trigger({parent: undefined, r1: _rc, r2: _rc});
                else
                    this.OnDataRemove.trigger({parent: undefined, r1: _rc, r2: _rc});
            }
        }

        setAdditionalActionName(v) {
            this._additionalaction = v;
            if (this._lastv) {
                var _ir = this.rowCount()-1;
                if (this.columnCount() > 0)
                    this.OnDataUpdate.trigger({parent: undefined, r1: _ir, r2: _ir, c1: 0, c2: 0});
            }
        }

        _ptrisadditionalaction(ptr) {
            if (!ptr)
                return false;
            return this._lastv && (ptr.backendObject == this);
        }

        getValue(ptr) {
            if (this._ptrisadditionalaction(ptr))
                return { name: "_clove_mainview_additionalaction_", label: (this._additionalaction ? this._additionalaction : clove.i18n.SwitchView)};
            return this.innerdatasource.getValue(ptr);
        }

        changeValue(ptr, value) {
        }

        rowCount(parent) {
            if (this._ptrisadditionalaction(parent))
                return 0;
            return this.innerdatasource.rowCount(parent) + ((this._lastv && !parent) ? 1 : 0);
        }

        columnCount(parent) {
            if (this._ptrisadditionalaction(parent))
                return 0;
            return Math.max(1, this.innerdatasource.columnCount(parent));
        }

        valuePointer(irow, icol, parent) {
            if (this._lastv && !parent && (irow == (this.rowCount()-1)))
                return new clove.DatasourceValuePointer(irow, icol, this, this);
            try {
                return this.innerdatasource.valuePointer(irow, icol, parent);
            } catch(e)
            { // it might happen that we access an incorrect address when the real column count is 0.
                return new clove.DatasourceValuePointer(irow, icol, this, this);
            }
        }

        parent(ptr) {
            if (this._ptrisadditionalaction(ptr))
                return undefined;
            return this.innerdatasource.parent(ptr);
        }

    }

    class MainView extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [
                    { view: 'Toolbar', name: 'toolbar', width: "1px", horizontalStretchAffinity: 1 },
                    { cols: [
                        { name: 'msidebarbtn', styleClass: 'clove_mainview_sidebarbutton', view: 'Button', label: '', verticalStretchAffinity: 1,
                            OnClicked: function(e) {var w = e.widget.containingNameScope().getByName("msidebar"); var wv=(w.visibility()==clove.Visible); e.widget.setIcon(clove.Icon.bySymbol(wv?'❱':'❰')); w.setVisibility(wv?clove.InvisibleCollapsed:clove.Visible); }
                        },
                        { name: 'msidebar', view: 'ScrollView', horizontalStretchAffinity: 0, styleClass: 'clove_mainview_sidebar', body:
                            {rows: [], name: 'mainsidebar', newNameScope: true, verticalStretchAffinity: 1},
                        },
                        { view: 'ScrollView', body:
                            {
                                view:'ResizeSplitter', name: 'splitter',
                                verticalStretchAffinity: 1, horizontalStretchAffinity: 1, body: [
                                    {rows: [], name: 'mainbodyleft', newNameScope: true, verticalStretchAffinity: 1, horizontalStretchAffinity: 1},
                                    {rows: [], name: 'mainbodyright', newNameScope: true, verticalStretchAffinity: 1, horizontalStretchAffinity: 1},
                                ],
                            }, verticalStretchAffinity: 1, horizontalStretchAffinity: 1,
                        },
                    ], verticalStretchAffinity: 1},
                ],
                horizontalStretchAffinity: 1,
                verticalStretchAffinity: 1,
                mayFocus: true,
            }), domnode);
            var self = this;
            var bodyshadow = document.createElement("div");
            bodyshadow.style.position = "absolute";
            bodyshadow.style.left = "0";
            bodyshadow.style.right = "0";
            bodyshadow.className = "clove_mainview_bodyshadow";
            function _alignBodyShadow() {
                bodyshadow.style.top = self.getByName("toolbar").outerSize().height + "px";
            }
            setTimeout(_alignBodyShadow);
            this.topnode.appendChild(bodyshadow);
            var self = this;
            this.declareProperty("splitterPosition", 0.5);
            this.declareProperty("actions", []);
            this.declareProperty("sidebar");
            this.declareProperty("head1");
            this.declareProperty("head2");
            this.declareProperty("headControl");
            this.declareProperty("icon");
            this.declareProperty("bodyLeftViewActionLabel");
            this.declareProperty("bodyRightViewActionLabel");
            this.declareProperty("bodyLeft");
            this.declareProperty("bodyRight");
            this.declareProperty("showOnly", undefined);
            $(this.topnode).addClass("clove_mainview");
            this.OnResized.addHandler(function(){
                self._handleInnerViewMode(true);
                _alignBodyShadow();
            });
            this.OnActionTriggered = new clove.Event();
        }

        _handleInnerViewMode(noforce) {
            var wleft = this.getByName("mainbodyleft");
            var wright = this.getByName("mainbodyright");
            var msidebar = this.getByName("msidebar");
            var msidebarbtn = this.getByName("msidebarbtn");
            var wsplitter = this.getByName("splitter");
            var _oval = this._smallmode;
            this._smallmode = (clove.utils.getOuterSize(this.topnode).width < 800);
            if (!noforce || (_oval != this._smallmode)) {
                if (this._smallmode || (!this.config.bodyLeft) || (!this.config.bodyRight) || (this.config.showOnly !== undefined)) {
                    if (this.config.showOnly !== undefined)
                        this._smallmoderightwidget = this.config.showOnly === 1;
                    if (!this.config.bodyLeft)
                        this._smallmoderightwidget = true;
                    if (!this.config.bodyRight)
                        this._smallmoderightwidget = false;
                    var isrightwidget = this._smallmoderightwidget === true;
                    wleft.setVisibility(isrightwidget?clove.InvisibleCollapsed:clove.Visible);
                    wright.setVisibility(isrightwidget?clove.Visible:clove.InvisibleCollapsed);
                    wsplitter.setShowOnly(isrightwidget?1:0);
                    this._actionsds.setAdditionalActionName(isrightwidget ? this.bodyLeftViewActionLabel() : this.bodyRightViewActionLabel());
                }
                else {
                    wleft.setVisibility(clove.Visible);
                    wright.setVisibility(clove.Visible);
                    wsplitter.setShowOnly(undefined);
                }
                msidebar.setVisibility((this._smallmode && this.config.sidebar) ? clove.InvisibleCollapsed : clove.Visible);
                msidebarbtn.setVisibility((this._smallmode && this.config.sidebar) ? clove.Visible : clove.InvisibleCollapsed);
                msidebarbtn.setIcon(clove.Icon.bySymbol('❱'));
                wsplitter.relayout();
                this._actionsds.setAdditionalActionEnabled(this._smallmode && this.config.bodyLeft && this.config.bodyRight && (this.config.showOnly === undefined));
            }
        }

        showOnly_setter(v) {
            this._handleInnerViewMode();
        }

        splitterPosition_getter() {
            return this.getByName("splitter").sizeFractions()[0];
        }

        splitterPosition_setter(v) {
            var sp = this.getByName("splitter");
            var lsf = sp.sizeFractions();
            var nlsf = [v];
            for(var i=0; i<lsf.length; i++)
                nlsf.push((lsf[i] / v) * (1 - v));
            sp.setSizeFractions(nlsf);
        }

        icon_getter() {
            return this.getByName("toolbar").icon();
        }

        icon_setter(v) {
            this.getByName("toolbar").setIcon(v);
        }

        head1_getter() {
            return this.getByName("toolbar").head1();
        }

        head1_setter(v) {
            this.getByName("toolbar").setHead1(v);
        }

        head2_getter() {
            return this.getByName("toolbar").head2();
        }

        head2_setter(v) {
            this.getByName("toolbar").setHead2(v);
        }

        headControlWidget() {
            return this.getByName("toolbar").headControlWidget();
        }

        headControl_getter() {
            return this.getByName("toolbar").headControl();
        }

        headControl_setter(v) {
            this.getByName("toolbar").setHeadControl(v);
        }

        bodyLeftViewActionLabel_setter(v) {
            if (this._smallmode && this._smallmoderightwidget)
                this._actionsds.setAdditionalActionName(v);
        }

        bodyRightViewActionLabel_setter(v) {
            if (this._smallmode && !this._smallmoderightwidget)
                this._actionsds.setAdditionalActionName(v);
        }

        bodyLeft_setter(v) {
            this.getByName("mainbodyleft").setChildren(v?[v,]:[]);
            this._handleInnerViewMode();
        }

        bodyRight_setter(v) {
            this.getByName("mainbodyright").setChildren(v?[v,]:[]);
            this._handleInnerViewMode();
        }

        switchToRightBody() {
            this._smallmoderightwidget = true;
            if (this._smallmode)
                this._handleInnerViewMode();
        }

        switchToLeftBody() {
            this._smallmoderightwidget = false;
            if (this._smallmode)
                this._handleInnerViewMode();
        }

        sidebar_setter(v) {
            this.getByName("mainsidebar").setChildren(v?[v,]:[]);
            this._handleInnerViewMode();
        }

        actions_setter(v) {
            var ads = v;
            if (!ads)
                ads = [];
            if (!ads.valuePointer) {
                ads = new clove.NativeDatasource();
                for(var ii=0;ii<v.length;ii++)
                    ads.root().addRow([v[ii]]);
            }
            this.getByName("toolbar").setActions(this._actionsds = new MainViewMenuHelperDatasource(ads));
            this._handleInnerViewMode();
        }

        children_setter(v) {
            super.children_setter(v);
            this.containingNameScope().addChildNameScope(this.getByName('mainbodyleft'));
            this.containingNameScope().addChildNameScope(this.getByName('mainbodyright'));
            this.containingNameScope().addChildNameScope(this.getByName('mainsidebar'));
        }

        doinit() {
            super.doinit();
            var self = this;
            this.getByName("toolbar").OnActionTriggered.addHandler(function(e){
                if (e.action == "_clove_mainview_additionalaction_") {
                    self._smallmoderightwidget = !self._smallmoderightwidget;
                    self._handleInnerViewMode();
                }
                else
                    self.OnActionTriggered.trigger(e);
            });
        }

    }

    clove.MainView = MainView;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          MODALPANEL

    class ModalPanel extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("fullyTransparent", false);
            var self = this;
            $(this.topnode).addClass("clove_modalpanel");
            this.OnClicked = new clove.Event();
            $(this.topnode).click(function(e){
                self.OnClicked.trigger({ widget: self, jqEvent: e, originalEvent: e.originalEvent });
            });
        }

        fullyTransparent_setter(v) {
            let ct = "clove_modalpanel_transparent";
            let cst = "clove_modalpanel_semitransparent";
            this.removeStyleClass(v?cst:ct);
            this.addStyleClass(v?ct:cst);
        }

    }

    clove.ModalPanel = ModalPanel;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          DIALOG

    class Dialog extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [
                    { cols: [
                        { view: 'IconView', name: 'titleicon', size: "40pt", strictVerticalSizing: true },
                        { view: 'Label', name: 'titlelabel', horizontalStretchAffinity: 1, width: "1px" },
                    ], horizontalStretchAffinity: 1, styleClass: "clove_dialog_titlebar" },
                    { view:'ScrollView', body: { rows: [], name: 'body', newNameScope: true, horizontalStretchAffinity: 1, verticalStretchAffinity: 1}, },
                ],
            }), domnode);
            this.declareProperty("body", {rows: []});
            this.declareProperty("titleicon");
            this.declareProperty("title", "");
            $(this.topnode).addClass("clove_dialog");
        }

        children_setter(v) {
            super.children_setter(v);
            this.containingNameScope().addChildNameScope(this.getByName('body'));
        }

        body_setter(v) {
            this.getByName("body").setChildren([v,]);
        }

        title_setter(v) {
            this.getByName("titlelabel").setLabel(v);
        }

        titleicon_setter(v) {
            var titleicon = this.getByName("titleicon");
            titleicon.setVisibility(v ? clove.Visible : clove.InvisibleCollapsed);
            titleicon.setIcon(v);
        }

        close() {
            this.remove({animationName: 'cloveanimation_fadeout'});
        }

        static show(dlg, showconfig) {
            var newshowconfig = $.extend({}, showconfig);
            if (newshowconfig.fullyTransparent === undefined)
                newshowconfig.fullyTransparent = false;
            return clove.utils.popup(dlg, newshowconfig);
        }

    }

    clove.Dialog = Dialog;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          DATAVIEW

    class DataView extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("hideExpanders", false);
            this.declareProperty("alwaysAllocateExpanderSpace", false);
            this.declareProperty("showOnlyFirstColumn", false);
            this.declareProperty("allowSelection", false);
            this.declareProperty("allowChecking", false);
            this.declareProperty("granularity", "cell");
            this.declareProperty("dataViewProcessor", undefined);
            this.declareProperty("datasource");
            this.declareProperty("headersource");
            this.declareProperty("gridVisible", true);
            this.declareProperty("nodeActivationNeedsDoubleClick", false);
            this.declareProperty("cellGenerator");
            this.declareProperty("showChangeMenu", false);
            this.declareProperty("editOnGesture", false);
            this.declareProperty("rowsResizable", false);
            this.declareProperty("columnsResizable", false);
            this.OnSelectionChanged = new clove.Event();
            this.OnCheckedNodesChanged = new clove.Event();
            this.OnNodeActivated = new clove.Event();
            $(this.topnode).addClass("clove_dataview");
            this.table = document.createElement("table");
            this.contentnode.appendChild(this.table);
            this.colheaderline = $("<tr><th class='clove_dataview_rowheader clove_dataview_colheader'></th></tr>")[0];
            this.table.appendChild(this.colheaderline);
            var self = this;
            var _getPtrForClickEvent = function(e) {
                var cellnode = e.target;
                while (cellnode && cellnode.tagName != "TD")
                    cellnode = cellnode.parentElement;
                if (cellnode) {
                    var cellidx = (self.config.granularity == 'row') ? undefined : cellnode.cellIndex;
                    return self._rowIdxToPtr(cellnode.parentElement.rowIndex, cellidx);
                }
            };
            var _clickhandler = function(e, fordoubleclick) {
                var ptr = _getPtrForClickEvent(e);
                if (ptr && self.config.allowSelection) {
                    self.selectCell(ptr);
                    if (self.config.nodeActivationNeedsDoubleClick == fordoubleclick)
                        self.OnNodeActivated.trigger({widget: self, node: ptr});
                }
            };
            this.table.addEventListener("dblclick", function(e){
                _clickhandler(e, true);
            });
            this.table.addEventListener("click", function(e){
                _clickhandler(e, false);
            });
            this._lineHeightCache = undefined;
            this.OnResized.addHandler(function(e){
                self._lineHeightCache = undefined;
            });
            this.OnKeyDown.addHandler(function(e){
                var s = self.selection();
                var ds = self.config.datasource;
                var xdirection = 0;
                var ydirection = 0;
                if (e.keyCode == 38)
                    ydirection = -1;
                else if (e.keyCode == 40)
                    ydirection = +1;
                else if (e.keyCode == 37)
                    xdirection = -1;
                else if (e.keyCode == 39)
                    xdirection = +1;
                else if ((e.keyCode == 10 || e.keyCode == 13) && s)
                    self.OnNodeActivated.trigger({widget: self, node: s});
                else if ((e.keyCode == 171 || e.keyCode == 173 || e.keyCode == 187 || e.keyCode == 189) && s && !self.config.hideExpanders) {
                    self.isCellExpanded(s) ? self.collapseCell(s) : self.expandCell(s);
                }
                else if (e.keyCode == 32 && s && self.config.allowChecking) {
                    self.setCellChecked(s, !self.isCellChecked(s));
                }
                if (ds && (xdirection||ydirection)) {
                    if (s) {
                        var rowi = self._ptrToIntStruct(s).domrow.rowIndex;
                        var coli = s.icol + 1;
                        if (xdirection)
                            coli = Math.min(Math.max(1, coli+xdirection), ds.columnCount());
                        if (ydirection) {
                            var _rowi = rowi;
                            while(true) {
                                _rowi = _rowi + ydirection;
                                if (_rowi < 1)
                                    break;
                                if (_rowi > self.table.childNodes.length-1)
                                    break;
                                if (self.table.childNodes[_rowi].style.visibility != 'collapse') {
                                    rowi = _rowi;
                                    break;
                                }
                            }
                        }
                        self.selectCell(self._rowIdxToPtr(rowi, coli));
                        var pw = self;
                        while (pw && !(pw.setVerticalScrollPosition && pw.setHorizontalScrollPosition))
                            pw = pw.parentWidget();
                        if (pw) {
                            var pwc = pw.contentnode;
                            var sp = $(self.table.childNodes[rowi]).position();
                            var tp = $(pwc).position();
                            var x = sp.left + tp.left;
                            var y = sp.top + tp.top;
                            var pws = pw.innerSize();
                            var nx = Math.min(Math.max(x, 0.3 * pws.width), 0.7 * pws.width);
                            var ny = Math.min(Math.max(y, 0.3 * pws.height), 0.7 * pws.height);
                            var dx = nx - x;
                            var dy = ny - y;
                            if (dx)
                                pw.setHorizontalScrollPosition(pw.horizontalScrollPosition() - dx);
                            if (dy)
                                pw.setVerticalScrollPosition(pw.verticalScrollPosition() - dy);
                        }
                    }
                    else if (ds.rowCount() > 0 && ds.columnCount() > 0) {
                        self.selectCell(ds.valuePointer(0, 0));
                    }
                    e.originalEvent.preventDefault();
                }
            });
        }

        _getLineHeight() {
            if (this._lineHeightCache)
                return this._lineHeightCache;
            var res = undefined;
            if (this.table.childNodes.length >= 2) {
                var ppr = this.table.childNodes[1];
                if (ppr.childNodes.length >= 2) {
                    var cell = ppr.childNodes[1];
                    var cellinner = cell._cellinner;
                    if (cellinner)
                        res = this._lineHeightCache = parseInt(getComputedStyle(cellinner).lineHeight);
                }
            }
            return res;
        }

        rowsResizable_setter(v) {
            var mfct = v ? '_add_row_resizer' : '_remove_row_resizer';
            for (var irow=1; irow<this.table.childNodes.length; irow++)
                this[mfct](irow);
        }

        columnsResizable_setter(v) {
            var mfct = v ? '_add_column_resizer' : '_remove_column_resizer';
            for (var icol=1; icol<this.table.childNodes[0].childNodes.length; icol++)
                this[mfct](icol);
        }

        _add_row_resizer(irow) {
            this._add_resizer(this.table.childNodes[irow].childNodes[0], true);
        }

        _remove_row_resizer(irow) {
            this._remove_resizer(this.table.childNodes[irow].childNodes[0]);
        }

        _add_column_resizer(icol) {
            this._add_resizer(this.table.childNodes[0].childNodes[icol], false);
        }

        _remove_column_resizer(icol) {
            this._remove_resizer(this.table.childNodes[0].childNodes[icol]);
        }

        _add_resizer(h, isrow) {
            if (!h._bandresizer) {
                var rdiv = h._bandresizer = document.createElement('div');
                rdiv.className = 'clove_dataview_bandresizer ' + (isrow ? 'clove_dataview_bandresizer_row' : 'clove_dataview_bandresizer_column');
                h.odiv.appendChild(rdiv);
                clove.utils.addOnDragHandler(rdiv, function dragbegin(){
                    var rgdiv = h._bandresizer_ghost = document.createElement('div');
                    rgdiv.className = 'clove_dataview_bandresizer_ghost';
                    var brcs = window.getComputedStyle(h._bandresizer);
                    rgdiv.style.top = brcs.top;
                    rgdiv.style.left = brcs.left;
                    rgdiv.style.width = brcs.width;
                    rgdiv.style.height = brcs.height;
                    h.odiv.appendChild(rgdiv);
                    h._bandresizer_ghost_startdim = clove.utils.getInnerSize(h)[isrow?'height':'width'];
                    h._bandresizer_ghost_minval = -h._bandresizer_ghost_startdim + 15;
                    h._bandresizer_ghost_val = 0;
                    h._bandresizer_ghost_offset = parseInt(brcs[isrow?'top':'left']);
                }, function drag(dx, dy){
                    h._bandresizer_ghost_val = Math.max(isrow ? dy : dx, h._bandresizer_ghost_minval);
                    h._bandresizer_ghost.style[isrow?'top':'left'] = (h._bandresizer_ghost_offset + h._bandresizer_ghost_val) + 'px';
                }, function dragend(){
                    h._bandresizer_ghost.remove();
                    h._bandresizer_ghost = undefined;
                    h.style[isrow?'height':'width'] = h.style[isrow?'minHeight':'minWidth'] = h.style[isrow?'maxHeight':'maxWidth'] = (h._bandresizer_ghost_startdim + h._bandresizer_ghost_val) + 'px';
                });
                clove.utils.addOnDoubleClickTapHandler(rdiv, function() {
                    h.style[isrow?'height':'width'] = h.style[isrow?'minHeight':'minWidth'] = h.style[isrow?'maxHeight':'maxWidth'] = 'initial';
                });
            }
        }

        _remove_resizer(h) {
            if (h._bandresizer) {
                h._bandresizer.remove();
                h._bandresizer = undefined;
            }
        }

        editOnGesture_setter(v) {
            var self = this;
            if (this._editongesture_handlerdblclick) {
                clove.utils.removeOnDoubleClickTapHandler(this.table, this._editongesture_handlerdblclick);
                this._editongesture_handlerdblclick = undefined;
            }
            if (v) {
                this._editongesture_handlerdblclick = function(e) {
                    var tgt = e.target;
                    while (tgt != document.body && tgt.tagName != "TD")
                        tgt = tgt.parentNode;
                    var _c = tgt;
                    while (_c != document.body && _c != self.table)
                        _c = _c.parentNode;
                    if (_c == document.body)
                        return;
                    var ptr = self._rowIdxToPtr(tgt.parentElement.rowIndex, tgt.cellIndex);
                    self.editCell(ptr);
                }
                clove.utils.addOnDoubleClickTapHandler(this.table, this._editongesture_handlerdblclick);
            }
        }

        showChangeMenu_setter(v) {
            var self = this;
            function _getcurrentcell(widget) {
                var s = widget.selection();
                if (s)
                    return s;
                var a = widget.checkedCells();
                if (a.length > 0)
                    return a[0];
            }
            function _remove_helper(widget, datasource, iaxis, getheaderfct, strconfirm1, strconfirmn, removefct) {
                var cc = widget.checkedCells();
                if (cc.length == 0 && widget.selection())
                    cc = [widget.selection()];
                if (cc.length == 0)
                    return;
                var lst = [];
                for(var i=0; i<cc.length; i++) {
                    var v = [cc[i][iaxis], cc[i].parent()];
                    var exists = false;
                    for (var j=0; j<lst.length; j++) {
                        var lstj = lst[j];
                        if (lstj[0] == v[0] && clove.DatasourceValuePointer.equals(lstj[1], v[1])) {
                            exists = true;
                            break;
                        }
                    }
                    if (!exists)
                        lst.push(v);
                }
                var istr = "";
                for (var i=0; i<lst.length; i++) {
                    var lsti = lst[i];
                    if (istr != "")
                        istr += ", ";
                    if (self.config.headersource) {
                        var lbl = self.config.headersource[getheaderfct](lsti[0], lsti[1]).label;
                        if (lbl) {
                            istr += lbl;
                            continue;
                        }
                    }
                    istr += "#" + (lsti[0]+1);
                }
                var dlgmsg = ((lst.length > 1) ? clove.i18n[strconfirmn] : clove.i18n[strconfirm1]).replace("{items}", istr);
                clove.messageDialog({message: dlgmsg, buttons: [clove.i18n.No, clove.i18n.Yes]}).addHandler(function(e){
                    if (e.button === 1) {
                        var paths = [];
                        for (var i=0; i<lst.length; i++)
                            paths.push([clove.DatasourceValuePointer.toPath(lst[i][1]), lst[i][0]]);
                        paths.sort();
                        paths.reverse();
                        for (var i=0; i<paths.length; i++)
                            datasource[removefct](paths[i][1], clove.DatasourceValuePointer.fromPath(datasource, paths[i][0]));
                    }
                });
            }
            if (this._changeMenu)
                this._changeMenu.remove();
            if (v) {
                this.config.showChangeMenu = v = clove.utils.applyDefaultsToConfig(v, {
                    item_addrow_label: function(widget, datasource) { return clove.i18n.DataView_InsertRow; },
                    item_addrow_icon: function(widget, datasource) { return clove.Icon.bySymbol("⮱"); },
                    item_addrow_isvisible: function(widget, datasource) {
                        if (datasource)
                            return true;
                    },
                    item_addrow_action: function(widget, datasource) {
                        var cc = _getcurrentcell(widget);
                        var irow = 0;
                        var parent;
                        if (cc) {
                            irow = cc.irow + 1;
                            parent = cc.parent();
                        }
                        datasource.insertRow(irow, parent);
                    },
                    item_addrowbefore_label: function(widget, datasource) { return clove.i18n.DataView_InsertRowBefore; },
                    item_addrowbefore_icon: function(widget, datasource) { return clove.Icon.bySymbol("⮳"); },
                    item_addrowbefore_isvisible: function(widget, datasource) {
                        if (datasource) {
                            var s = widget.selection();
                            if (s && s.irow == 0)
                                return true;
                            var c = widget.checkedCells();
                            if (c.length == 1 && c[0].icol == 0)
                                return true;
                        }
                    },
                    item_addrowbefore_action: function(widget, datasource) {
                        var cc = _getcurrentcell(widget);
                        if (cc)
                            datasource.insertRow(cc.irow, cc.parent());
                    },
                    item_addcolumn_label: function(widget, datasource) { return clove.i18n.DataView_InsertColumn; },
                    item_addcolumn_icon: function(widget, datasource) { return clove.Icon.bySymbol("⮷"); },
                    item_addcolumn_isvisible: function(widget, datasource) {
                        return datasource && self.config.granularity == 'cell';
                    },
                    item_addcolumn_action: function(widget, datasource) {
                        var cc = _getcurrentcell(widget);
                        var icol = 0;
                        var parent;
                        if (cc) {
                            icol = cc.icol + 1;
                            parent = cc.parent();
                        }
                        datasource.insertColumn(icol, parent);
                    },
                    item_addcolumnbefore_label: function(widget, datasource) { return clove.i18n.DataView_InsertColumnBefore; },
                    item_addcolumnbefore_icon: function(widget, datasource) { return clove.Icon.bySymbol("⮶"); },
                    item_addcolumnbefore_isvisible: function(widget, datasource) {
                        if (datasource && self.config.granularity == 'cell') {
                            var s = widget.selection();
                            if (s && s.icol == 0)
                                return true;
                            var c = widget.checkedCells();
                            if (c.length == 1 && c[0].icol == 0)
                                return true;
                        }
                    },
                    item_addcolumnbefore_action: function(widget, datasource) {
                        var cc = _getcurrentcell(widget);
                        if (cc)
                            datasource.insertColumn(cc.icol, cc.parent());
                    },
                    item_addchild_label: function(widget, datasource) { return clove.i18n.DataView_InsertChild; },
                    item_addchild_icon: function(widget, datasource) { return clove.Icon.bySymbol("▸"); },
                    item_addchild_isvisible: function(widget, datasource) {
                        return datasource && !widget.hideExpanders() && (widget.selection() || widget.checkedCells().length == 1);
                    },
                    item_addchild_action: function(widget, datasource) {
                        var cc = _getcurrentcell(widget);
                        if (cc) {
                            if (cc.columnCount() == 0)
                                datasource.insertColumn(0, cc);
                            datasource.appendRow(cc);
                            widget.expandCell(cc);
                        }
                    },
                    item_removerow_label: function(widget, datasource) { return clove.i18n.DataView_RemoveRow; },
                    item_removerow_icon: function(widget, datasource) { return clove.Icon.bySymbol(clove.symbols.Remove); },
                    item_removerow_isvisible: function(widget, datasource) {
                        return datasource && (widget.selection() || widget.checkedCells().length > 0);
                    },
                    item_removerow_action: function(widget, datasource) {
                        _remove_helper(widget, datasource, "irow", "getRowHeader", "strDataView_ConfirmRemoveRow", "strDataView_ConfirmRemoveRows", "removeRow");
                    },
                    item_removecolumn_label: function(widget, datasource) { return clove.i18n.DataView_RemoveColumn; },
                    item_removecolumn_icon: function(widget, datasource) { return clove.Icon.bySymbol(clove.symbols.Remove); },
                    item_removecolumn_isvisible: function(widget, datasource) {
                        return datasource && self.config.granularity == 'cell' && (widget.selection() || widget.checkedCells().length == 1);
                    },
                    item_removecolumn_action: function(widget, datasource) {
                        _remove_helper(widget, datasource, "icol", "getColumnHeader", "strDataView_ConfirmRemoveColumn", "strDataView_ConfirmRemoveColumns", "removeColumn");
                    },
                    item_edit_label: function(widget, datasource) { return clove.i18n.DataView_Edit; },
                    item_edit_icon: function(widget, datasource) { return clove.Icon.bySymbol(clove.symbols.Change); },
                    item_edit_isvisible: function(widget, datasource) {
                        return datasource && (widget.selection() || widget.checkedCells().length == 1);
                    },
                    item_edit_action: function(widget, datasource) {
                        var cc = _getcurrentcell(widget);
                        if (cc)
                            widget.editCell(cc);
                    },
                });
                var items = {};
                for(var k in v) {
                    if (k.startsWith("item_")) {
                        var ni = k.indexOf("_", 6);
                        if (ni > -1) {
                            var itemname = k.substring(5, ni);
                            if (!items[itemname]) {
                                var itemprefix = k.substring(0, ni + 1);
                                items[itemname] = {action: v[itemprefix+"action"] || function(){},
                                                   label: v[itemprefix+"label"] || function(){return "";},
                                                   icon: v[itemprefix+"icon"] || function(){},
                                                   isvisible: v[itemprefix+"isvisible"] || function(){return true;}};
                            }
                        }
                    }
                }
                this._changeMenu = document.createElement("div");
                this._changeMenu.style.position = "absolute";
                this.topnode.appendChild(this._changeMenu);
                var ds = new clove.NativeDatasource();
                var actionroot = ds.root().addRow([{icon: clove.Icon.bySymbol(clove.symbols.Actions)}]);
                var menubar = clove.build({view: 'Menubar', actions: ds, doStandaloneResizing: true, styleClass: 'clove_dataview_changemenubar'}, {domnode: this._changeMenu});
                var cntnr = this._changeMenu;
                while (cntnr != document.body) {
                    var cst = window.getComputedStyle(cntnr);
                    if ((cst.overflowX != "hidden" || cst.overflowY != "hidden") && !$(cntnr).hasClass("clove_widget_contentnode"))
                        break;
                    cntnr = cntnr.parentNode;
                }
                function _positionmenu() {
                    var r1 = self.topnode.getBoundingClientRect();
                    var r2 = cntnr.getBoundingClientRect();
                    self._changeMenu.style.top = Math.max(0, r2.top - r1.top) + "px";
                    self._changeMenu.style.right = Math.max(0, r1.right - r2.right) + "px";
                }
                $(cntnr).scroll(_positionmenu);
                this.OnResized.addHandler(_positionmenu, this);
                var nwcntnr = cntnr;
                while (nwcntnr && !nwcntnr._clove_widget)
                    nwcntnr = nwcntnr.offsetParent;
                if (nwcntnr)
                    nwcntnr._clove_widget.OnResized.addHandler(_positionmenu, this);
                setTimeout(_positionmenu);
                menubar.OnBeforeSubactionsExpanded.addHandler(function(){
                    while (actionroot.rowCount() > 0)
                        actionroot.removeRow(0);
                    for(var kitem in items) {
                        var item = items[kitem];
                        if (item.isvisible(self, self.config.datasource))
                            actionroot.addRow([{name: kitem, label: item.label(self, self.config.datasource), icon: item.icon(self, self.config.datasource)}]);
                    }
                }, menubar);
                menubar.OnActionTriggered.addHandler(function(e){
                    var item = items[e.action];
                    if (item)
                        item.action(self, self.config.datasource);
                }, menubar);
            }
        }

        datasource_setter(ds) {
            var self = this;
            var xxx = clove.utils.suspendResizing();
            try {
                this._hideexpanderonchildlessnodes = (ds && ds._ui_hideExpanderOnChildlessNodes);
                clove.utils.connectDatasourceToWidget(this, ds, "_lastds", undefined,
                        "_ondatainserthandler", "_ondataremovehandler", "_ondataupdatehandler",
                        function(lastds){ // afterdisconnectedfct
                            if (self._lasths) {
                                self.config.datasource = lastds;
                                self.setHeadersource();
                                self.config.datasource = ds;
                            }
                            self.selectCell(undefined);
                            if (lastds)
                                self._ondataremovehandler({r1:0, c1:0, r2:lastds.rowCount()-1, c2:lastds.columnCount()-1});
                            self._internalroot = { domrow: undefined, childs: [], parent: undefined, expanded: true };
                        },
                        function(){ // beforeconnectedfct
                            if (ds.getColumnHeader)
                                self.setHeadersource(ds);
                        },
                        function(){ // afterconnectedfct
                            if (ds.async_pull)
                                ds.async_pull(undefined, {reason: 'DataView', sub: 'datasource_setter', cached: true});
                            return true;
                        });
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        _selection_modes_helper() {
            var n = $(this.contentnode);
            n.removeClass("clove_dataview_selectnone");
            n.removeClass("clove_dataview_selectrows");
            n.removeClass("clove_dataview_selectcells");
            if (!this.config.allowSelection)
                n.addClass("clove_dataview_selectnone");
            else if (this.config.granularity == 'cell')
                n.addClass("clove_dataview_selectcells");
            else if (this.config.granularity == 'row')
                n.addClass("clove_dataview_selectrows");
            this.setMayFocus(this.config.allowSelection || this.config.allowChecking);
        }

        allowSelection_setter(v) {
            this._selection_modes_helper();
            if (!v)
                this.selectCell(undefined);
        }

        allowChecking_setter(v) {
            if (this._lastds)
                this.setDatasource(this.config.datasource);
            if (!v)
                this.setCheckedCells([]);
        }

        granularity_setter() {
            this._selection_modes_helper();
            this.selectCell(undefined);
        }

        selectCell(ptr) {
            if (this._lastselection)
                $(this._lastselection).removeClass('clove_dataview_selected');
            if (this.config.allowSelection) {
                var cell;
                if (ptr) {
                    var row = this._ptrToRow(ptr);
                    if (this.config.granularity == 'row')
                        cell = row;
                    else
                        cell = row.childNodes[ptr.icol+1];
                    $(cell).addClass('clove_dataview_selected');
                }
                this._lastselection = cell;
            } else
                this._lastselection = undefined;
            this.OnSelectionChanged.trigger({widget: this});
        }

        isCellSelected(ptr) {
            var rows = this.table.childNodes;
            var row = this._ptrToRow(ptr);
            if (this.config.granularity == 'row')
                return row.hasClass('clove_dataview_selected');
            else
                return $(row.children()[ptr.icol+1]).hasClass('clove_dataview_selected');
        }

        selection() {
            if (this._lastselection && document.contains(this._lastselection)) {
                if (this.config.granularity == 'row')
                    return this._rowIdxToPtr(this._lastselection.rowIndex);
                else
                    return this._rowIdxToPtr(this._lastselection.parentNode.rowIndex, this._lastselection.cellIndex);
            }
        }

        checkedCells() {
            var ds = this.config.datasource;
            var result = [];
            var rows = this.table.childNodes;
            for(var rowidx=1; rowidx<rows.length; rowidx++) {
                var row = $(rows[rowidx]);
                var cols = row.children();
                var colcnt = (this.config.granularity == 'row' && cols.length >= 2) ? 2 : cols.length;
                for (var colidx=1;colidx<colcnt;colidx++) {
                    var cell = cols[colidx];
                    if (cell._cellchecked)
                        result.push(this._rowIdxToPtr(rowidx, colidx));
                }
            }
            return result;
        }

        setCheckedCells(ptrs) {
            var rows = this.table.childNodes;
            for(var rowidx=1; rowidx<rows.length; rowidx++) {
                var row = $(rows[rowidx]);
                var cols = row.children();
                for (var colidx=1;colidx<cols.length;colidx++) {
                    var cell = cols[colidx];
                    var cptr = this._rowIdxToPtr(rowidx, colidx);
                    var found = false;
                    for (var i=0;i<ptrs.length;i++) {
                        var ptr = ptrs[i];
                        if (ptr) {
                            var drow = this._ptrToIntStruct(ptr).domrow;
                            if (this.config.granularity == 'row') {
                                if (ptr.irow == cptr.irow && clove.DatasourceValuePointer.equals(ptr.parent(), cptr.parent()) && colidx == 1) {
                                    found = true;
                                    ptrs.splice(i, 1);
                                    break;
                                }
                            }
                            else {
                                if (clove.DatasourceValuePointer.equals(ptr, cptr)) {
                                    found = true;
                                    ptrs.splice(i, 1);
                                    break;
                                }
                            }
                        }
                    }
                    cell._cellchecked = found;
                    this._updatecellcheck(cptr);
                }
            }
        }

        isCellChecked(ptr) {
            if (!ptr)
                return false;
            if (this.config.granularity == 'row')
                ptr = this.config.datasource.valuePointer(ptr.irow, 0, ptr.parent());
            var row = this._ptrToRow(ptr);
            var cell = row.childNodes[ptr.icol+1];
            return cell._cellchecked == true;
        }

        setCellChecked(ptr, val) {
            if (!ptr)
                return;
            if (this.config.granularity == 'row')
                ptr = this.config.datasource.valuePointer(ptr.irow, 0, ptr.parent());
            var row = this._ptrToRow(ptr);
            var cell = row.childNodes[ptr.icol+1];
            cell._cellchecked = val;
            this._updatecellcheck(ptr);
        }

        _updatecellcheck(ptr, skipchangedhandler) {
            if (this.config.allowChecking) {
                var cell = this._ptrToRow(ptr).childNodes[ptr.icol+1];
                if (cell._cellcheck) {
                    cell._cellcheck.innerHTML = cell._cellchecked ? clove.symbols.CircleFilled : clove.symbols.Circle;
                    if (!skipchangedhandler)
                        this.OnCheckedNodesChanged.trigger({widget: this, cell: ptr});
                }
            }
        }

        _ondatainserthandler(ee) {
            if (ee.c1 !== undefined) {
                for(var ic=ee.c1;ic<=ee.c2;ic++)
                    this._addcolumn(ic, ee.parent);
            }
            if (ee.r1 !== undefined) {
                for(var ir=ee.r1;ir<=ee.r2;ir++)
                    this._addrow(ir, ee.parent);
            }
            this.relayout();
        }

        _ondataremovehandler(ee) {
            if (ee.c1 !== undefined) {
                for(var ic=ee.c2;ic>=ee.c1;ic--)
                    this._deletecolumn(ic, ee.parent);
            }
            if (ee.r1 !== undefined) {
                for(var ir=ee.r2;ir>=ee.r1;ir--)
                    this._deleterow(ir, ee.parent);
            }
            this.relayout();
        }

        _ondataupdatehandler(ee) {
            for(var ir=ee.r1; ir<=ee.r2;ir++)
                for(var ic=ee.c1; ic<=ee.c2;ic++)
                    this._updatecell(this.config.datasource.valuePointer(ir, ic, ee.parent));
            this.relayout();
        }

        editCell(ptr) {
            var cell = this._getcelldom(ptr);
            var rct = cell.getBoundingClientRect();
            var popupres = clove.utils.popup({view: "EditBox", styleClass: "clove_dataview_editcell"}, {alignTo: cell, alignToInside: true, modal: true, panelClickClosesPopup: true });
            var binding = new clove.DataBinding(popupres.widget, "text", this.datasource(), ptr);
            binding.bind();
            popupres.widget.focus();
            var _keydown = function(e){
                if (e.keyCode == 9)
                    e.preventDefault();
                else if (e.keyCode == 10 || e.keyCode == 13 || e.keyCode == 27) {
                    if (e.keyCode == 27)
                        binding.unbind();
                    popupres.widget.remove();
                }
            };
            $(document.body).keydown(_keydown);
            popupres.widget.OnDestroyed.addHandler(function(){
                $(document.body).off("keydown", _keydown);
            });
        }

        _ptrToIntStruct(ptr) {
            var ds = this.config.datasource;
            if (ptr) {
                var parint = this._ptrToIntStruct(ds.parent(ptr));
                if (!parint)
                    return;
                return parint.childs[ptr.irow];
            }
            else
                return this._internalroot;
        }

        _ptrToRow(ptr) {
            return this._ptrToIntStruct(ptr).domrow;
        }

        _rowIdxToPtr(i, cellidx) {
            if (i<1)
                return false;
            return this._intStructToPtr(this._rowIdxToIntStruct(i), cellidx);
        }

        _rowIdxToIntStruct(i) {
            if (i<1)
                return false;
            return this.table.childNodes[i]._internalstruct;
        }

        _intStructToRow(st) {
            return st.domrow;
        }

        _intStructToPtr(st, cellidx) {
            if ((cellidx !== undefined) && (cellidx < 1))
                return false;
            var ds = this.config.datasource;
            var npath = [];
            var nst = st;
            while (nst) {
                npath.push(nst);
                nst = nst.parent;
            }
            var result = undefined;
            for(var inpath=npath.length-2; inpath>=0; inpath--) {
                var nnst = npath[inpath];
                var ccellidx = 0;
                if (cellidx !== undefined && inpath==0)
                    ccellidx = cellidx-1;
                result = ds.valuePointer(nnst.idx, ccellidx, result);
            }
            return result;
        }

        gridVisible_setter(v) {
            $(this.table).removeClass("clove_dataview_gridded");
            if (v)
                $(this.table).addClass("clove_dataview_gridded");
            this.relayout();
        }

        headersource_setter(hs) {
            if (this._lasths) {
                this._lasths.OnHeaderDataUpdate.removeHandler(this.hohdu);
                this._lasths.OnHeaderVisibilityUpdated.removeHandler(this.hohvu);
            }
            this._lasths = hs;
            this._colheaders = [];
            if (hs) {
                var self = this;
                this.hohdu = function(e){self._onheaderdataupdated(e);};
                this.hohvu = function(e){self._onheadervisibilityupdated(e);};
                hs.OnHeaderDataUpdate.addHandler(this.hohdu);
                hs.OnHeaderVisibilityUpdated.addHandler(this.hohvu);
            }
            var rows = this.table.childNodes;
            var lrows = rows.length;
            var lcols = rows[0].childNodes.length;
            for(var ir=1;ir<lrows;ir++) {
                var irptr = this._rowIdxToPtr(ir);
                this._onheaderdataupdated({irow: irptr.irow, parent: irptr.parent()});
            }
            for(var ic=1;ic<lcols;ic++)
                this._onheaderdataupdated({icol: ic-1});
            this._onheadervisibilityupdated({for_row:true, for_column:true});
        }

        _onheadervisibilityupdated(e) {
            var jtable = $(this.table);
            if (e.for_row) {
                var v = this.config.headersource ? this.config.headersource.rowHeadersVisible() : false;
                var v1 = v ? 'clove_dataview_rowheadersvisible' : 'clove_dataview_rowheadershidden';
                var v2 = v ? 'clove_dataview_rowheadershidden' : 'clove_dataview_rowheadersvisible';
                jtable.addClass(v1);
                jtable.removeClass(v2);
            }
            if (e.for_column) {
                var v = this.config.headersource ? this.config.headersource.columnHeadersVisible() : false;
                var v1 = v ? 'clove_dataview_colheadersvisible' : 'clove_dataview_colheadershidden';
                var v2 = v ? 'clove_dataview_colheadershidden' : 'clove_dataview_colheadersvisible';
                jtable.addClass(v1);
                jtable.removeClass(v2);
            }
            this.relayout();
        }

        showOnlyFirstColumn_setter(v) {
            this.setStyleClassAssigned("clove_dataview_showonlyfirstcolumn", v);
        }

        _getcelldom(ptr) {
            if (ptr.irow == -1)
                var ppr = this.table.childNodes[0];
            else {
                var parint = this._ptrToIntStruct(ptr.parent());
                if (!parint)
                    return;
                var pchl = parint.childs[ptr.irow];
                if (!pchl)
                    return;
                var ppr = pchl.domrow;
            }
            return ppr.childNodes[ptr.icol+1];
        }

        _onheaderdataupdated(e) {
            var ds = this.config.datasource;
            var rows = this.table.childNodes;
            if (e.irow !== undefined)
                this._updaterowheader(this._getcelldom({irow: e.irow, icol: -1, parent: function(){return e.parent;}}), e.irow, e.parent);
            if (e.icol !== undefined)
                this._updatecolheader(this._getcelldom({irow: -1, icol: e.icol, parent: function(){return e.parent;}}), e.icol);
        }

        _updaterowheader(thnode, i, parent) {
            var hcfg = this.config.headersource ? this.config.headersource.getRowHeader(i, parent) : undefined;
            thnode.idiv.textContent = hcfg ? hcfg.label : "";
            if (thnode.parentNode._hideexpander ^ (hcfg && hcfg.hideExpander)) {
                thnode.parentNode._hideexpander = hcfg && hcfg.hideExpander;
                var intstruct = this._rowIdxToIntStruct(thnode.parentNode.rowIndex);
                this._updatecellexpander(intstruct);
            }
            this.relayout();
        }

        _updatecolheader(thnode, i) {
            var ccfg = this.config.headersource ? this.config.headersource.getColumnHeader(i) : {label:""};
            this._colheaders[i] = ccfg;
            thnode.idiv.textContent = ccfg.label;
            this._setcolumnvisibility(i, !ccfg.invisible);
            this.relayout();
        }

        _setcolumnvisibility(i, val) {
            var v = val ? "" : "none";
            var rows = this.table.childNodes;
            for(var rowi=0; rowi<rows.length; rowi++) {
                var row = rows[rowi];
                if (row._istoplevel || rowi == 0)
                    row.childNodes[i+1].style.display = v;
            }
        }

        _createcell(pintstruct, withcheck) {
            var self = this;
            var cell = document.createElement("td");
            var cellinner = document.createElement("span");
            cellinner.style.position = 'relative';
            cell._cellinner = cellinner;
            if (pintstruct) {
                var cellexpander = document.createElement("span");
                cellexpander.className = 'clove_dataview_expander';
                cell._cellexpander = cellexpander;
                cell.appendChild(cellexpander);
                var ind = -2;
                var x = pintstruct;
                while (x) {
                    ind++;
                    x = x.parent;
                }
                var pd = ind * 20;
                cellexpander.style.paddingLeft = pd + 'px';
                cellexpander.addEventListener("click", function(e){
                    var sptr = self._intStructToPtr(pintstruct);
                    pintstruct.expanded ? self.collapseCell(sptr) : self.expandCell(sptr);
                });
            }
            if (this.config.allowChecking && withcheck) {
                var cellcheck = document.createElement("span");
                cellcheck.className = 'clove_dataview_check';
                cellcheck.addEventListener("click", function(e){
                    var cellnode = e.target;
                    while (cellnode && cellnode.tagName != "TD")
                        cellnode = cellnode.parentElement;
                    if (cellnode) {
                        var cptr = self._rowIdxToPtr(cellnode.parentNode.rowIndex, cellnode.cellIndex);
                        self.setCellChecked(cptr, !self.isCellChecked(cptr));
                    }
                });
                cell._cellcheck = cellcheck;
                cell._cellchecked = false;
                cell.appendChild(cellcheck);
            }
            cell.appendChild(cellinner);
            return cell;
        }

        _createcell2(cell, cptr) {
            if (this.config.cellGenerator) {
                var x = this.config.cellGenerator(this, cptr);
                if (x) {
                    var ww = cell._cellinnercomplex = clove.build(clove.utils.applyDefaultsToConfig(x, {newNameScope: true}), {domnode: cell._cellinner});
                    ww.setDoStandaloneResizing(true);
                    ww.topnode.style.display = 'inline-block';
                    ww.relayout();
                }
            }
            this._updatecell(cptr);
            this._updatecellcheck(cptr, true);
            if (this._colheaders && this._colheaders.length > cptr.icol && this._colheaders[cptr.icol].invisible)
                cell.style.display = "none";
        }

        isCellExpanded(ptr) {
            return this._ptrToIntStruct(ptr).expanded;
        }

        _setexpandnode(pintstruct, val) {
            pintstruct.expanded = val;
            this._updatecellexpander(pintstruct);
            if (pintstruct.childs.length > 0) {
                var ifirst = pintstruct.childs[0].domrow.rowIndex;
                var x = pintstruct;
                while (x.childs.length > 0)
                    x = x.childs[x.childs.length-1];
                var ilast = x.domrow.rowIndex;
                var rows = this.table.childNodes;
                for(var rowi=ifirst;rowi<=ilast;rowi++) {
                    var visible = val;
                    var _pintstruct = this._rowIdxToIntStruct(rowi).parent;
                    while(_pintstruct && visible) {
                        visible = visible && _pintstruct.expanded;
                        _pintstruct = _pintstruct.parent;
                    }
                    var rrow = rows[rowi];
                    rrow.style.visibility = visible ? 'inherit' : 'collapse';
                    if (clove.browser.webkit)  // workaround for webkit ignoring the styling from above (also used in another method)
                        rrow.style.display = visible ? 'table-row' : 'none';
                    rrow.className = (visible ? '' : 'clove_dataview_collapsedrow') + ((rrow == this._lastselection) ? " clove_dataview_selected" : "");
                }
            }
            this.relayout();
        }

        expandCell(ptr) {
            var intstruct = this._ptrToIntStruct(ptr);
            this._setexpandnode(intstruct, true);
            if (this.config.datasource.async_pull)
                this.config.datasource.async_pull(ptr, {reason: 'DataView', sub: 'expandCell'});
        }

        expandCellRecursive(ptr) {
            while (ptr) {
                this.expandCell(this._ptrToIntStruct(ptr));
                ptr = ptr.parent();
            }
        }

        collapseCell(ptr) {
            this._setexpandnode(this._ptrToIntStruct(ptr), false);
        }

        _updatecellexpander(pintstruct) {
            var row = this._intStructToRow(pintstruct);
            if (row) {
                if (row.childNodes.length > 1) {
                    var expander = $(row.childNodes[1]._cellexpander);
                    if (this.config.hideExpanders ||
                        (pintstruct.childs.length == 0 && this._hideexpanderonchildlessnodes) ||
                        row._hideexpander)
                    {
                        expander.html(this.config.alwaysAllocateExpanderSpace ? "▸" : "");
                        expander.addClass('clove_dataview_expander_hidden');
                    }
                    else {
                        expander.html(pintstruct.expanded ? "▾" : "▸");
                        expander.removeClass('clove_dataview_expander_hidden');
                    }
                }
            }
        }

        hideExpanders_setter() {
            if (this._lastds)
                this.setDatasource(this.config.datasource);
        }

        alwaysAllocateExpanderSpace_setter() {
            if (this._lastds)
                this.setDatasource(this.config.datasource);
        }

        _addrow(i, parent) {
            var parint = this._ptrToIntStruct(parent);
            var cols = this.config.datasource.columnCount(parent);
            var row = document.createElement('tr');
            row._istoplevel = (!parent);
            var headcell = document.createElement('th');
            headcell.className = 'clove_dataview_rowheader';
            row.appendChild(headcell);
            var odiv = headcell.odiv = document.createElement('div');
            var idiv = headcell.idiv = document.createElement('div');
            odiv.style.position = 'relative';
            odiv.style.height = '100%';
            headcell.appendChild(odiv);
            odiv.appendChild(idiv);
            var x = parint;
            var visible = true;
            while (x && visible) {
                visible = x.expanded && visible;
                x = x.parent;
            }
            if (!visible) {
                row.className = "clove_dataview_collapsedrow";
                row.style.visibility = 'collapse';
                if (clove.browser.webkit)  // workaround for webkit ignoring the styling from above (also used in another method)
                    row.style.display = 'none';
            }
            if (i == parint.childs.length) {
                if (i == 0) {
                    var rowi = (parint.domrow ? parint.domrow.rowIndex : 0 )+ 1;
                }
                else {
                    var lstc = parint.childs[i-1];
                    while (lstc.childs.length>0)
                        lstc = lstc.childs[lstc.childs.length-1];
                    var rowi = lstc.domrow.rowIndex+1;
                }
            }
            else
                var rowi = parint.childs[i].domrow.rowIndex;
            clove.utils.insertToArray(parint.childs, i, row._internalstruct = {domrow: row, childs: [], parent: parint, idx: i, expanded: false});
            for (var xi=i+1;xi<parint.childs.length;xi++)
                parint.childs[xi].idx = xi;
            this._updaterowheader(headcell, i, parent);
            if (rowi == this.table.childNodes.length)
                this.table.appendChild(row);
            else
                this.table.insertBefore(row, this.table.childNodes[rowi]);
            if (this.config.rowsResizable)
                this._add_row_resizer(rowi);
            for(var ic=0;ic<cols;ic++) {
                var cell = this._createcell((ic==0) ? row._internalstruct : undefined, (this.config.granularity != 'row' || ic == 0));
                row.appendChild(cell);
                var cptr = this.config.datasource.valuePointer(i, ic, parent);
                this._createcell2(cell, cptr);
                if (visible && ic == 0) {
                    var rc = this.config.datasource.rowCount(cptr, true);  // we must skip fetching new stuff by that call in async datasources, otherwise it will loop deeply
                    var cc = this.config.datasource.columnCount(cptr, true);
                    if (cc)
                        this._ondatainserthandler({c1: 0, c2: cc-1, parent: cptr});
                    if (rc)
                        this._ondatainserthandler({r1: 0, r2: rc-1, parent: cptr});
                }
            }
            this._updatecellexpander(row._internalstruct);
            this._updatecellexpander(row._internalstruct.parent);
            this.relayout();
        }

        _addcolumn(i, parent) {
            var parint = this._ptrToIntStruct(parent);
            for(var iis=0; iis<parint.childs.length; iis++) {
                var prrow = parint.childs[iis].domrow;
                var cell = this._createcell((i==0) ? parint.childs[iis] : undefined, (this.config.granularity != 'row' || i == 0));
                var cptr = this._rowIdxToPtr(prrow.rowIndex, i+1);
                var cols = prrow.childNodes;
                if (i == cols.length-1)
                    prrow.appendChild(cell);
                else
                    prrow.insertBefore(cell, cols[i+1]);
                this._createcell2(cell, cptr);
                if (i==0) {
                    this._updatecellexpander(parint.childs[iis]);
                    this._updatecellexpander(parint.childs[iis].parent);
                }
            }
            var cell = document.createElement('th');
            cell.className = 'clove_dataview_colheader';
            var odiv = cell.odiv = document.createElement('div');
            var idiv = cell.idiv = document.createElement('div');
            odiv.style.position = 'relative';
            odiv.style.height = '100%';
            cell.appendChild(odiv);
            odiv.appendChild(idiv);
            if (!parent) {
                var colheaders = this.colheaderline.childNodes;
                if (i == colheaders.length-1)
                    this.colheaderline.appendChild(cell);
                else
                    this.colheaderline.insertBefore(cell, colheaders[i+1]);
                this._updatecolheader(cell, i);
            }
            if (this.config.columnsResizable)
                this._add_column_resizer(i+1);
            this.relayout();
        }

        _deleterow(i, parent) {
            var parint = this._ptrToIntStruct(parent);
            var x = parint.childs[i];
            var ifirst = this._intStructToRow(x).rowIndex;
            while (x.childs.length > 0)
                x = x.childs[x.childs.length-1];
            var ilast = this._intStructToRow(x).rowIndex;
            for(var rowi=ilast; rowi>=ifirst; rowi--)
                this.table.childNodes[rowi].remove();
            parint.childs.splice(i, 1);
            for (var xi=i;xi<parint.childs.length;xi++)
                parint.childs[xi].idx = xi;
            this.relayout();
        }

        _deletecolumn(i, parent) {
            var parint = this._ptrToIntStruct(parent);
            var rows = this.table.childNodes;
            this.colheaderline.childNodes[i+1].remove();
            for(var ipst=0; ipst<parint.childs.length; ipst++)
                parint.childs[ipst].domrow.childNodes[i+1].remove();
            this.relayout();
        }

        _updatecell(ptr) {
            var cell = this._getcelldom(ptr);
            if (!cell)
                return;
            var celltext = this.config.datasource.getValue(ptr);
            if (cell._cellinnercomplex) {
                if (cell._cellinnercomplex.config.update)
                    cell._cellinnercomplex.config.update(cell._cellinnercomplex, ptr, celltext);
            }
            else {
                if (this.config.dataViewProcessor)
                    celltext = this.config.dataViewProcessor(ptr, celltext);
                var cellinner = cell._cellinner;
                cellinner.innerHTML = "";
                var metadata = this.config.datasource.getMetadata(ptr);
                if (metadata.icon)
                    cellinner.appendChild(metadata.icon.createHtml(this._getLineHeight(), true));
                cell.className = (metadata.styleClass || "") + ((cell == this._lastselection) ? " clove_dataview_selected" : "");
                var lbltxtspan = document.createElement("span");
                lbltxtspan.textContent = celltext;
                cellinner.appendChild(lbltxtspan);
            }
            this.relayout();
        }

    }

    clove.DataView = DataView;

    class TableView extends clove.DataView {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                gridVisible: true,
            }), domnode);
        }

    }

    clove.TableView = TableView;

    class ListView extends clove.DataView {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                allowSelection: true,
                granularity: 'row',
                gridVisible: false,
                hideExpanders: true,
            }), domnode);
        }

    }

    clove.ListView = ListView;

    class TreeView extends clove.DataView {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                gridVisible: false,
                alwaysAllocateExpanderSpace: true,
            }), domnode);
        }

    }

    clove.TreeView = TreeView;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          CHECKBUTTON

    class CheckButton extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("label", "");
            this.declareProperty("checked", false);
            var self = this;
            $(this.topnode).addClass("clove_checkbutton");
            $(this.contentnode).append($("<label><input type='checkbox'/><span></span></label>"));
            this.spannode = $(this.contentnode).find("span")[0];
            this.inputnode = $(this.contentnode).find("input")[0];
            this.OnChanged = this.OnClicked = new clove.Event();
            $(this.inputnode).click(function(e){
                self.OnClicked.trigger({widget: self});
                self.setChecked(self.checked());
            });
        }

        label_getter() {
            return this.spannode.textContent;
        }

        label_setter(text) {
            this.spannode.textContent = text;
            this.relayout();
        }

        checked_getter() {
            return this.inputnode.checked;
        }

        checked_setter(v) {
            this.inputnode.checked = v;
        }

        focus() {
            this.inputnode.focus();
        }

    }

    clove.CheckButton = CheckButton;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          RADIOBUTTON

    var _nextRadioGroupId = 0;
    var _publicRadioGroups = {};

    class RadioGroup {

        constructor(_groupname) {
            this.groupname = _groupname || ("clovegroup_" + _nextRadioGroupId++);
            this.OnChanged = new clove.Event();
            this._childs = [];
        }

        static getGroupByPublicName(name) {
            if (!_publicRadioGroups.hasOwnProperty(name))
                _publicRadioGroups[name] = new RadioGroup(name);
            return _publicRadioGroups[name];
        }

        _addChild(w) {
            this._childs.push(w);
            var self = this;
            w.OnChanged.addHandler(function(){
                self.OnChanged.trigger({widget: w, radiogroup: self});
            });
        }

        _selected_helper() {
            for(var i=0; i<this._childs.length; i++) {
                var w = this._childs[i];
                if (w.checked())
                    return [i, w, w.valueDef()];
            }
            return [-1, undefined, undefined];
        }

        selected() {
            return this._selected_helper()[1];
        }

        selectedIndex() {
            return this._selected_helper()[0];
        }

        selectedValue() {
            var ht = this._selected_helper();
            return (ht[2] !== undefined) ? ht[2] : ht[0];
        }

        select(w) {
            if (!w)
                return this.unselectAll();
            w.setChecked(true);
        }

        selectByIndex(i) {
            this.select(this._childs[i]);
        }

        selectByValue(v) {
            var w;
            for (var w of this._childs) {
                if (w.valueDef() === v)
                    return this.select(w);
            }
            this.selectByIndex(v);
        }

        unselectAll() {
            var s = this.selected();
            if (s)
                s.setChecked(false);
        }

    }

    clove.RadioGroup = RadioGroup;

    class RadioButton extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("label", "");
            this.declareProperty("group");
            this.declareProperty("checked", false);
            this.declareProperty("groupValue");
            this.declareProperty("valueDef", undefined);
            var self = this;
            $(this.topnode).addClass("clove_radiobutton");
            $(this.contentnode).append($("<label><input type='radio'/><span></span></label>"));
            this.spannode = $(this.contentnode).find("span")[0];
            this.inputnode = $(this.contentnode).find("input")[0];
            this.OnChanged = this.OnClicked = new clove.Event();
            $(this.inputnode).click(function(e){
                self.OnClicked.trigger({widget: self});
                self.setGroupValue(self.groupValue());
            });
        }

        groupValue_getter() {
            if (this.config.group)
                return this.config.group.selectedValue();
        }

        groupValue_setter(v) {
            if (this.config.group)
                this.config.group.selectByValue(v);
        }

        label_getter() {
            return this.spannode.textContent;
        }

        label_setter(text) {
            this.spannode.textContent = text;
            this.relayout();
        }

        group_setter(group) {
            if (!group)
                return this.setGroup(new RadioGroup());
            if (typeof group === 'string')
                return this.setGroup(RadioGroup.getGroupByPublicName(group));
            this.inputnode.name = group.groupname;
            group._addChild(this);
        }

        checked_getter() {
            return this.inputnode.checked;
        }

        checked_setter(v) {
            this.inputnode.checked = v;
        }

        focus() {
            this.inputnode.focus();
        }

    }

    clove.RadioButton = RadioButton;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          EDITBOX

    class EditBox extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            var self = this;
            $(this.topnode).addClass("clove_editbox");
            this.declareProperty("readOnly", false);
            this.declareProperty("text", "");
            this.declareProperty("hintText");
            this.declareProperty("autocompletionFilter", 'startswith');
            this.declareProperty("autocompletionOpenForNoText", false);
            this.declareProperty("autocompletionItems");
            this.inputnode = this._createinput();
            this.autocompletionItems_effective = new clove.FilterProxyDatasource(undefined, function(){return self._autocompletionItems_rowfilter(...arguments);});
            $(this.contentnode).append(this.inputnode);
            this.OnChanged = new clove.Event();
            this.OnPopupTextSelected = new clove.Event();
            this.inputnode.bind('input', function(e){
                var oval = self.config.text;
                self.config.text = self.inputnode.val();
                self.OnChanged.trigger({widget: self});
                self.OnPropertyChanged.trigger({widget: self, propertyName: "text", oldPropertyValue: oval, getPropertyValue: self.text});
                if (self.config.autocompletionItems)
                    self.autocompletionItems_effective.setDatasource(self.autocompletionItems_effective.datasource);
            });
            var _onblurfct = function(e) {
                if (self.config.autocompletionItems)
                    setTimeout(function(){
                        if (!self.hasFocus() && self._openedpopup && !self._openedpopup.widget.hasFocus())
                            self._closepopup();
                    });
            }
            this.inputnode.focus(function(e){
                if (self.config.autocompletionItems) {
                    self._openpopup_textlist(self.autocompletionItems_effective);
                    $(self._openedpopup.widget.topnode).blur(_onblurfct);
                }
            });
            this.inputnode.keydown(function(e){
                if (self.config.autocompletionItems && e.keyCode == 40 && /*should be redundant>*/ self._openedpopup) {
                    var ads = self.autocompletionItems_effective;
                    if (ads.rowCount() > 0 && ads.columnCount() > 0)
                        self._openedpopup.widget.selectCell(ads.valuePointer(0, 0));
                    e.stopPropagation();
                    e.preventDefault();
                    self._openedpopup.widget.focus();
                }
            });
            this.inputnode.on("keydown", this._onkeydown_handler);
            this.inputnode.on("keyup", this._onkeyup_handler);
            this.inputnode.on("keypress", this._onkeypress_handler);
            this.inputnode.blur(_onblurfct);
        }

        readOnly_setter(v) {
            if (v)
                this.inputnode.attr("readonly", "readonly");
            else
                this.inputnode.removeAttr("readonly");
        }

        _autocompletionItems_rowfilter(datasource, irow, parent) {
            if (datasource.columnCount(parent) == 0)
                return false;
            var txt = datasource.getValue(datasource.valuePointer(irow, 0, parent));
            var currtxt = this.text() || "";
            return (this.config.autocompletionOpenForNoText || (currtxt.length > 0)) && this.autocompletionFilter_fct(txt, currtxt);
        }

        autocompletionItems_setter(v) {
            this.autocompletionItems_effective.setDatasource(v);
        }

        autocompletionOpenForNoText_setter(v) {
            this.autocompletionItems_effective.setDatasource(this.autocompletionItems_effective.datasource);
        }

        autocompletionFilter_setter(v) {
            if (v == "startswith")
                this.autocompletionFilter_fct = function(s, t){return s.toLowerCase().startsWith(t.toLowerCase());};
            else if (v == "contains")
                this.autocompletionFilter_fct = function(s, t){return s.toLowerCase().indexOf(t.toLowerCase()) > -1;};
            else if (v === undefined)
                this.autocompletionFilter_fct = function(){return true;};
            else
                this.autocompletionFilter_fct = v;
            this.autocompletionItems_effective.setDatasource(this.autocompletionItems_effective.datasource);
        }

        _createinput() {
            return $("<input type='text'/>");
        }

        text_getter() {
            return this.inputnode.val();
        }

        text_setter(v) {
            this.inputnode.val(v);
        }

        hintText_setter(v) {
            this.inputnode.attr('placeholder', v);
        }

        focus() {
            this.inputnode.focus();
        }

        remove(removeconfig) {
            this.inputnode.blur();
            super.remove(removeconfig);
        }

        _openpopup(config, showconfig) {
            var self = this;
            if (this._openedpopup)
                this._closepopup();
            this._openedpopup = clove.utils.popup(config, clove.utils.applyDefaultsToConfig(showconfig, {
                alignTo: this,
                alignToAxis: "vertical",
                alignSyncDimension: true,
            }));
            this.OnDestroyed.addHandler(function(){self._closepopup();}, this._openedpopup.widget);
        }

        _openpopup_textlist(datasource, showconfig) {
            var self = this;
            this._openpopup({view: "ListView", datasource: datasource, styleClass: "clove_editbox_textlistpopup"}, showconfig);
            this._openedpopup.widget.OnNodeActivated.addHandler(function(e){
                var text = datasource.getValue(e.node);
                self.setText(text);
                self.OnPopupTextSelected.trigger({widget: self, text: text, ptr: e.node});
                self._closepopup();
            });
            this._openedpopup.widget.OnKeyDown.addHandler(function(e){
                self.OnKeyDown.trigger({widget: self, keyCode: e.keyCode, jqEvent: e.jqEvent, originalEvent: e.originalEvent});
            });
            this._openedpopup.widget.OnKeyUp.addHandler(function(e){
                self.OnKeyUp.trigger({widget: self, keyCode: e.keyCode, jqEvent: e.jqEvent, originalEvent: e.originalEvent});
            });
            this._openedpopup.widget.OnKeyPress.addHandler(function(e){
                self.OnKeyPress.trigger({widget: self, keyCode: e.keyCode, jqEvent: e.jqEvent, originalEvent: e.originalEvent});
            });
        }

        _closepopup() {
            if (!this._openedpopup)
                return;
            try {
                this._openedpopup.widget.remove();
            }
            catch(e) {
            }
            this._openedpopup = undefined;
        }

        setTextSelection(ifrom, ito) {
            this.inputnode[0].setSelectionRange(ifrom, ito);
        }

        textSelection() {
            return {from: this.inputnode[0].selectionStart, to: this.inputnode[0].selectionEnd};
        }

    }

    clove.EditBox = EditBox;

    class EditComboBox extends clove.EditBox {

        constructor(config, domnode) {
            super(config, domnode);
            var self = this;
            var dropdownhandle = document.createElement("div");
            dropdownhandle.className = "clove_editcombobox_dropdownhandle";
            $(this.topnode).addClass("clove_editcombobox");
            this.declareProperty("popupItems");
            this.contentnode.appendChild(dropdownhandle);
            dropdownhandle.innerHTML = "▼";
            var closepopup = function(e, force) {
                if (force || (self._openedpopup && !clove.utils.isDescendantOf(e.target, self._openedpopup.widget.contentnode))) {
                    self._closepopup();
                    $(document.body).off('click', closepopup);
                    if (e) {
                        e.preventDefault();
                        e.stopPropagation();
                    }
                }
            };
            var openpopup = function(e) {
                if (self._openedpopup)
                    return;
                self._openpopup_textlist(self.config.popupItems);
                var jotn = $(self._openedpopup.widget.topnode);
                jotn.blur(closepopup);
                jotn.focus();
                e.stopPropagation();
                e.preventDefault();
            }
            $(dropdownhandle).click(openpopup);
            this.inputnode.keydown(function(e) {
                if (e.keyCode == 40)
                    openpopup(e);
            });
        }

    }

    clove.EditComboBox = EditComboBox;

    class DropdownBox extends clove.EditComboBox {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                readOnly: true,
            }), domnode);
        }

    }

    clove.DropdownBox = DropdownBox;

    class NumericEditBox extends clove.EditBox {

        constructor(config, domnode) {
            super(config, domnode);
            var self = this;
            var dropdownhandleup = document.createElement("div");
            dropdownhandleup.className = "clove_numericeditbox_dropdownhandle_up";
            dropdownhandleup.innerHTML = "▲";
            this.contentnode.appendChild(dropdownhandleup);
            var dropdownhandledown = document.createElement("div");
            dropdownhandledown.className = "clove_numericeditbox_dropdownhandle_down";
            dropdownhandledown.innerHTML = "▼";
            this.contentnode.appendChild(dropdownhandledown);
            $(this.topnode).addClass("clove_editcombobox");
            this.declareProperty("min", 0);
            this.declareProperty("max", 100);
            this.declareProperty("stepsize", 1);
            this.declareProperty("value", 50);
            this.inputnode.keydown(function(e) {
                if (e.keyCode == 38)
                    self._inc();
                else if (e.keyCode == 40)
                    self._dec();
            });
            this.inputnode.focusout(function(e) {
                self._text_to_value();
            });
            $(dropdownhandleup).click(function(e){
                self._inc();
            });
            $(dropdownhandledown).click(function(e){
                self._dec();
            });
        }

        text_setter(v) {
            super.text_setter(v);
            this._text_to_value();
        }

        _inc() {
            this._text_to_value();
            this.setValue(this.value() + this.stepsize());
        }

        _dec() {
            this._text_to_value();
            this.setValue(this.value() - this.stepsize());
        }

        _checkvalues() {
            var vmin = this.min();
            var vmax = this.max();
            var vstepsize = this.stepsize();
            var vv = this.value();
            var vvv = Math.floor((vv-vmin) / vstepsize) * vstepsize + vmin;
            if (isNaN(vmin))
                this.setMin(0);
            else if (isNaN(vmax))
                this.setMax(0);
            else if (isNaN(vstepsize) || vstepsize == 0)
                this.setStepsize(0.1);
            else if (isNaN(vv))
                this.setValue(0);
            else if (vmax < vmin)
                this.setMax(vmin);
            else if (vmin > vmax)
                this.setMin(vmax);
            else if (vvv > vmax)
                this.setValue(vmax);
            else if (vvv < vmin)
                this.setValue(vmin);
            else if (vvv != vv)
                this.setValue(vvv);
            else
                this.inputnode.val(vv);
        }

        min_setter(v) {
            this._checkvalues();
        }

        max_setter(v) {
            this._checkvalues();
        }

        stepsize_setter(v) {
            this._checkvalues();
        }

        value_setter(v) {
            this._checkvalues();
        }

        min_getter() {
            return parseFloat(this.config.min);
        }

        max_getter() {
            return parseFloat(this.config.max);
        }

        stepsize_getter() {
            return parseFloat(this.config.stepsize);
        }

        _text_to_value() {
            var r = parseFloat(this.text());
            this.setValue(isNaN(r) ? this.min() : r);
        }

    }

    clove.NumericEditBox = NumericEditBox;

    class PasswordEditBox extends clove.EditBox {

        constructor(config, domnode) {
            super(config, domnode);
            $(this.topnode).addClass("clove_passwordeditbox");
        }

        _createinput() {
            var result = super._createinput();
            result.attr('type', 'password');
            return result;
        }

    }

    clove.PasswordEditBox = PasswordEditBox;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          MULTILINEEDITBOX

    class MultilineEditBox extends clove.EditBox {

        constructor(config, domnode) {
            super(config, domnode);
            $(this.topnode).addClass("clove_multilineeditbox");
        }

        _createinput() {
            return $("<textarea style='resize:none;'></textarea>");
        }

    }

    clove.MultilineEditBox = MultilineEditBox;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          RICHEDIT

    class RichEditBox extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            $(this.topnode).addClass("clove_richeditbox");
            this.contentnode.setAttribute("contenteditable", true);
            this.declareProperty('htmlContent');
            this.declareProperty('selection');
        }

        focus() {
            this.contentnode.focus();
        }

        htmlContent_getter() {
            return this.contentnode.innerHTML;
        }

        htmlContent_setter(v) {
            this.contentnode.innerHTML = v||"";
        }

        selection_getter() {
            var sel = window.getSelection();
            for(var i=0; i<sel.rangeCount; i++) {
                var r = sel.getRangeAt(i);
                if (r.startContainer == this.contentnode || clove.utils.isDescendantOf(r.startContainer, this.contentnode))
                    return r;
            }
        }

        selection_setter(v) {
            var sel = window.getSelection();
            for(var i=sel.rangeCount-1; i>=0; i--) {
                var r = sel.getRangeAt(i);
                if (r.startContainer == this.contentnode || clove.utils.isDescendantOf(r.startContainer, this.contentnode))
                    sel.removeRange(r);
            }
            if (v)
                sel.addRange(v);
        }

        contentHtmlNode() {
            return this.contentnode;
        }

        computeMinimalWidth() {
            return clove.utils.cssLengthToPixels("40pt", "x");
        }

        computePreferredWidth() {
            return clove.utils.cssLengthToPixels("160pt", "x");
        }

        computeMinimalHeightForWidth() {
            return clove.utils.cssLengthToPixels("40pt", "y");
        }

        computePreferredHeightForWidth() {
            return clove.utils.cssLengthToPixels("80pt", "y");
        }

        toggleSelectionBold() {
            document.execCommand('bold', false, null);
        }

        toggleSelectionItalic() {
            document.execCommand('italic', false, null);
        }

        toggleSelectionUnderline() {
            document.execCommand('underline', false, null);
        }

        selectionIncreaseFontSize() {
            document.execCommand('increaseFontSize', false, null);
        }

        selectionDecreaseFontSize() {
            document.execCommand('decreaseFontSize', false, null);
        }

        selectionSetForegroundColor(c) {
            document.execCommand('foreColor', false, c);
        }

        selectionSetBackgroundColor(c) {
            document.execCommand('backColor', false, c);
        }

        selectionInsertList(config) {
            var config = clove.utils.applyDefaultsToConfig(config);
            if (config.ordered)
                document.execCommand('insertOrderedList', false, null);
            else
                document.execCommand('insertUnorderedList', false, null);
        }

    }

    clove.RichEditBox = RichEditBox;

    class RichEdit extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [
                    {view: 'Menubar', name: 'menubar', horizontalStretchAffinity: 1, styleClass: 'clove_richedit_formattingbar'},
                    {view: 'RichEditBox', name: 'richedit', horizontalStretchAffinity: 1, verticalStretchAffinity: 1}
                ],
                horizontalStretchAffinity: 1,
                verticalStretchAffinity: 1,
            }), domnode);
            $(this.topnode).addClass("clove_richedit");
            this.declareProperty('htmlContent');
        }

        htmlContent_getter() {
            return this.getByName("richedit").htmlContent();
        }

        htmlContent_setter(v) {
            this.getByName("richedit").setHtmlContent(v);
        }

        richeditbox() {
            return this.getByName("richedit");
        }

        _coloractions(prefix) {
            var colors = ["ffffff", "c0c0c0", "808080", "000000",
                          "ff0000", "800000", "ffff00", "808000",
                          "00ff00", "008000", "00ffff", "008080",
                          "0000ff", "000080", "ff00ff", "800080"];
            var result = [];
            for(var i=0; i<colors.length; i++)
                result.push({name: prefix+'_'+colors[i], label: '', style: 'background: #'+colors[i]+';'});
            return result;
        }

        doinit() {
            super.doinit();
            var menubar = this.getByName("menubar");
            var richedit = this.getByName("richedit");
            menubar.setActions([{name: 'bold', icon: clove.Icon.bySymbol("𝐁"), label: ''},
                                {name: 'italic', icon: clove.Icon.bySymbol("𝐼"), label: ''},
                                {name: 'underline', icon: clove.Icon.bySymbol("⎁"), label: ''},
                                clove.MenuSeparator,
                                {name: 'incsize', icon: clove.Icon.bySymbol("🗚"), label: ''},
                                {name: 'decsize', icon: clove.Icon.bySymbol("🗛"), label: ''},
                                clove.MenuSeparator,
                                {name: 'forecolor', icon: clove.Icon.bySymbol("🄰"), label: '', subactions: this._coloractions("forecolor")},
                                {name: 'backcolor', icon: clove.Icon.bySymbol("🅰"), label: '', subactions: this._coloractions("backcolor")},
                                clove.MenuSeparator,
                                {name: 'orderedlist', icon: clove.Icon.bySymbol("⑴"), label: ''},
                                {name: 'unorderedlist', icon: clove.Icon.bySymbol("•"), label: ''},
                               ]);
            menubar.OnActionTriggered.addHandler(function(e){
                if (e.action == 'bold')
                    richedit.toggleSelectionBold();
                else if (e.action == 'italic')
                    richedit.toggleSelectionItalic();
                else if (e.action == 'underline')
                    richedit.toggleSelectionUnderline();
                else if (e.action == 'incsize')
                    richedit.selectionIncreaseFontSize();
                else if (e.action == 'decsize')
                    richedit.selectionDecreaseFontSize();
                else if (e.action.startsWith('forecolor_'))
                    richedit.selectionSetForegroundColor("#"+e.action.substring(10));
                else if (e.action.startsWith('backcolor_'))
                    richedit.selectionSetBackgroundColor("#"+e.action.substring(10));
                else if (e.action == 'orderedlist')
                    richedit.selectionInsertList({ordered: true});
                else if (e.action == 'unorderedlist')
                    richedit.selectionInsertList({ordered: false});
                richedit.focus();
            });
        }

    }

    clove.RichEdit = RichEdit;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          IMAGEVIEW

    class ImageView extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("zoom", 1.0);
            this.declareProperty("keepAspectRatio", true);
            this.declareProperty("source", "");
            var self = this;
            this.OnLoaded = new clove.Event();
            $(this.topnode).addClass("clove_imageview");
            this.imgnode = document.createElement("img");
            this.imgnode.style.position = "absolute";
            this.contentnode.appendChild(this.imgnode);
            $(this.imgnode).on('load', function(){
                self._refreshimage();
                self.OnLoaded.trigger({widget: self});
            });
            $(this.imgnode).on('error', function(){
                self.setStyleClassAssigned("clove_imageview_notloaded", true);
                self.OnLoaded.trigger({widget: self, error : true});
            });
            this.setStyleClassAssigned("clove_imageview_notloaded", true);
        }

        _refreshimage(skipLayout) {
            this.imgnode.style.top = "0";
            this.imgnode.style.left = "0";
            if (this.config.zoom == "auto") {
                if (this.config.keepAspectRatio) {
                    var myr = clove.utils.getInnerSize(this.contentnode);
                    var imgAspectRatio = this.imgnode.naturalWidth / this.imgnode.naturalHeight;
                    var myAspectRatio = myr.width / myr.height;
                    if (imgAspectRatio > myAspectRatio) {
                        var ah = (myr.width / imgAspectRatio);
                        this.imgnode.style.top = (myr.height-ah)/2 + "px";
                        this.imgnode.style.left = "0";
                        this.imgnode.style.width = "100%";
                        this.imgnode.style.height = ah + "px";
                    } else {
                        var aw = (myr.height * imgAspectRatio);
                        this.imgnode.style.top = "0";
                        this.imgnode.style.left = (myr.width-aw)/2 + "px";
                        this.imgnode.style.width = aw + "px";
                        this.imgnode.style.height = "100%";
                    }
                } else {
                    this.imgnode.style.width = "100%";
                    this.imgnode.style.height = "100%";
                }
            } else {
                this.imgnode.style.width = (this.imgnode.naturalWidth * this.config.zoom) + "px";
                this.imgnode.style.height = (this.imgnode.naturalHeight * this.config.zoom) + "px";
            }
            if (!skipLayout)
                this.relayout();
        }

        doresize() {
            super.doresize();
            this._refreshimage(true);
        }

        zoom_setter(v) {
            this._refreshimage();
        }

        keepAspectRatio_setter(v) {
            this._refreshimage();
        }

        source_setter(url) {
            this.imgnode.src = url;
            if (this.imgnode.complete)
                this.OnLoaded.trigger({widget: this, error: this.imgnode.naturalWidth !== 0});
            this.setStyleClassAssigned("clove_imageview_notloaded", !url);
            this._refreshimage();
        }

        computeMinimalWidth() {
            if (this.config.zoom == "auto")
                return 0;
            return this.computePreferredWidth();
        }

        computePreferredWidth() {
            var _zoom = this.config.zoom;
            if (_zoom == "auto")
                _zoom = 1.0;
            return this.imgnode.naturalWidth * _zoom;
        }

        computeMinimalHeightForWidth(w) {
            if (this.config.zoom == "auto")
                return 0;
            return this.computePreferredHeightForWidth(w);
        }

        computePreferredHeightForWidth(w) {
            var _zoom = this.config.zoom;
            if (_zoom == "auto")
                _zoom = (w / this.imgnode.naturalWidth);
            return this.imgnode.naturalHeight * _zoom;
        }

    }

    clove.ImageView = ImageView;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          ICON

    class Icon {

        constructor(src, chr) {
            this.src = src;
            this.chr = chr;
        }

        static byUrl(url) {
            return new Icon(url);
        }

        static bySymbol(chr) {
            return new Icon(undefined, chr);
        }

        createHtml(height, unpadded) {
            var inner;
            if (this.src) {
                inner = document.createElement("img");
                inner.src = this.src;
            }
            else if (this.chr) {
                var _p2pt = unpadded ? 0 : clove.utils.cssLengthToPixels("2pt", "y");
                inner = document.createElement("span");
                if (!unpadded) {
                    inner.style.paddingLeft = "2pt";
                    inner.style.paddingBottom = "2pt";
                }
                inner.style.fontSize = (height-_p2pt) + "px";
                inner.textContent = this.chr;
            }
            if (inner)
                inner.style.width = inner.style.height = height + "px";
            return inner;
        }

    }

    clove.Icon = Icon;

    class IconView extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("size", "40pt");
            this.declareProperty("icon");
            $(this.topnode).addClass("clove_iconview");
        }

        icon_setter(v) {
            this.contentnode.innerHTML = "";
            if (v) {
                var inner = v.createHtml(this._pixelheight);
                if (inner) {
                    inner.style.height = "100%";
                    this.contentnode.appendChild(inner);
                }
            }
        }

        size_setter(v) {
            this._pixelwidth = clove.utils.cssLengthToPixels(v, "x");
            this._pixelheight = clove.utils.cssLengthToPixels(v, "y");
            this.relayout();
        }

        computeMinimalWidth() {
            return this._pixelwidth;
        }

        computePreferredWidth() {
            return this._pixelwidth;
        }

        computeMinimalHeightForWidth(w) {
            return this._pixelheight;
        }

        computePreferredHeightForWidth(w) {
            return this._pixelheight;
        }

    }

    clove.IconView = IconView;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          MEDIA

    class MediaPlayer extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            var self = this;
            $(this.topnode).addClass("clove_mediaplayer");
            this.playernode = this._createPlayerNode();
            this.contentnode.appendChild(this.playernode);
            this.playernode.style.position = "absolute";
            this.playernode.style.top = "0";
            this.playernode.style.left = "0";
            this.declareProperty('autoPlay', false);
            this.declareProperty('showControls', true);
            this.declareProperty('currentMediaPosition');
            this.declareProperty('loop', false);
            this.declareProperty('muted', false);
            this.declareProperty('preload', true);
            this.declareProperty('volume', 1.0);
            this.declareProperty('source');
            this.OnLoadingAborted = new clove.Event();
            this.OnReadyForPlayback = new clove.Event();
            this.OnDurationChanged = new clove.Event();
            this.OnHasEnded = new clove.Event();
            this.OnIsPaused = new clove.Event();
            this.OnIsPlaying = new clove.Event();
            this.playernode.addEventListener('abort', function(e){
                self.OnLoadingAborted.trigger({widget: self, error: false});
            });
            this.playernode.addEventListener('canplay', function(e){
                self.OnReadyForPlayback.trigger({widget: self, fully: false});
            });
            this.playernode.addEventListener('canplaythrough', function(e){
                self.OnReadyForPlayback.trigger({widget: self, fully: true});
            });
            this.playernode.addEventListener('durationchange', function(e){
                self.OnDurationChanged.trigger({widget: self});
            });
            this.playernode.addEventListener('ended', function(e){
                self.OnHasEnded.trigger({widget: self});
            });
            this.playernode.addEventListener('error', function(e){
                self.OnLoadingAborted.trigger({widget: self, error: e});
            });
            this.playernode.addEventListener('pause', function(e){
                self.OnIsPaused.trigger({widget: self});
            });
            this.playernode.addEventListener('play', function(e){
                self.OnIsPlaying.trigger({widget: self});
            });
            this.playernode.addEventListener('loadedmetadata', function(e){
                self.relayout();
            });
            this.playernode.addEventListener('volumechange', function(e){
                var val = self.config.volume;
                var mval = self.config.muted;
                self.config.volume = self.volume();
                self.config.muted = self.muted();
                self.OnPropertyChanged.trigger({widget: self, propertyName: "volume", oldPropertyValue: val, getPropertyValue: self.volume});
                self.OnPropertyChanged.trigger({widget: self, propertyName: "muted", oldPropertyValue: mval, getPropertyValue: self.muted});
            });
        }

        duration() {
            return this.playernode.duration;
        }

        hasEnded() {
            return this.playernode.ended;
        }

        isPaused() {
            return this.playernode.paused;
        }

        autoPlay_getter(){
            return this.playernode.autoplay;
        }

        autoPlay_setter(v){
            this.playernode.autoplay = v;
        }

        showControls_getter(){
            return this.playernode.controls;
        }

        showControls_setter(v){
            this.playernode.controls = v;
            this.relayout();
        }

        currentMediaPosition_getter(){
            return this.playernode.currentTime;
        }

        currentMediaPosition_setter(v){
            if (v !== undefined)
                this.playernode.currentTime = v;
        }

        loop_getter(){
            return this.playernode.loop;
        }

        loop_setter(v){
            this.playernode.loop = v;
        }

        muted_getter(){
            return this.playernode.muted;
        }

        muted_setter(v){
            this.playernode.muted = v;
        }

        preload_getter(){
            return this.playernode.preload;
        }

        preload_setter(v){
            this.playernode.preload = v;
        }

        volume_getter(){
            return this.playernode.volume;
        }

        volume_setter(v){
            this.playernode.volume = v;
        }

        source_getter(){
            return this.playernode.src;
        }

        source_setter(v){
            this.playernode.src = v;
        }

        nativeNode() {
            return this.playernode;
        }

        focus() {
            this.playernode.focus();
        }

        play() {
            this.playernode.play();
        }

        pause() {
            this.playernode.pause();
        }

        computePreferredWidth() {
            return this.computeMinimalWidth();
        }

        computePreferredHeightForWidth(w) {
            return this.computeMinimalHeightForWidth(w);
        }

    }

    clove.MediaPlayer = MediaPlayer;

    class AudioPlayer extends clove.MediaPlayer {

        constructor(config, domnode) {
            super(config, domnode);
        }

        _createPlayerNode() {
            var playernode = document.createElement('audio');
            playernode.style.width = "100%";
            return playernode;
        }

        computeMinimalWidth() {
            return clove.utils.cssLengthToPixels("150pt");
        }

        computeMinimalHeightForWidth() {
            return clove.utils.getOuterSize(this.playernode).height;
        }

        static canPlayType(t) {
            return document.createElement('audio').canPlayType(t);
        }

    }

    clove.AudioPlayer = AudioPlayer;

    class VideoPlayer extends clove.MediaPlayer {

        constructor(config, domnode) {
            super(config, domnode);
        }

        _createPlayerNode() {
            var playernode = document.createElement('video');
            playernode.style.width = "100%";
            playernode.style.height = "100%";
            return playernode;
        }

        computeMinimalWidth() {
            return this.playernode.videoWidth||50;
        }

        computeMinimalHeightForWidth(w) {
            return this.playernode.videoHeight||50;
        }

        static canPlayType(t) {
            return document.createElement('video').canPlayType(t);
        }

    }

    clove.VideoPlayer = VideoPlayer;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          FLATVIEW

    let FlatLayout = (superclass) => class extends Layout(superclass) {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("switchInvisibleAnimationName", undefined);
            this.declareProperty("switchVisibleAnimationName", undefined);
            this.declareProperty("switchInvisibleAnimationDuration", 300);
            this.declareProperty("switchVisibleAnimationDuration", 300);
            this.declareProperty("collapseHidden", false);
            this.declareProperty("currentView", -1);
        }

        currentView_setter(v) {
            var self = this;
            var lwidg = this.childrenWidgets();
            v = Math.min(v, lwidg.length-1);
            var _hidden = this.config.collapseHidden ? clove.InvisibleCollapsed : clove.Invisible;
            for (var i=0;i<lwidg.length;i++) {
                var widg = lwidg[i];
                if (widg.visibility() == clove.Visible && i != v) {
                    if (this.config.switchInvisibleAnimationName) {
                        widg.topnode.style.animationName = "";  // this is a hack which avoids async animation (animateNode might do so for unchanged animation; this would break the visible animation below)
                        clove.utils.animateNode(widg.topnode, this.config.switchInvisibleAnimationName, this.config.switchInvisibleAnimationDuration, (function(_widg, _i){
                            return function() {
                                if (self.currentView() != _i)
                                    _widg.setVisibility(_hidden);
                            };
                        })(widg, i));
                    }
                    else
                        widg.setVisibility(_hidden);
                }
            }
            if (v >= 0) {
                var widg = lwidg[v];
                if (this.config.switchVisibleAnimationName)
                    clove.utils.animateNode(widg.topnode, this.config.switchVisibleAnimationName, this.config.switchVisibleAnimationDuration, function(){});
                widg.setVisibility(clove.Visible);
            }
        }

        _getMaxDims(dim, mode, width) {
            if (dim == "w") {
                var dimname = "width";
                var iscol = true;
                var oraclename = (mode=="p") ? "getPreferredWidth" : "getMinimalWidth";
                var axisname = "x";
            } else {
                var dimname = "height";
                var iscol = false;
                var oraclename = (mode=="p") ? "getPreferredHeightForWidth" : "getMinimalHeightForWidth";
                var axisname = "y";
            }
            var chlds = this.childrenWidgets();
            var result = 0;
            for(var ichld in chlds)
            {
                var chld = chlds[ichld];
                var fixwi = clove.utils.cssLengthToPixels(chld[dimname](), axisname);
                if (chld.visibility() == clove.InvisibleCollapsed)
                    var dval = 0;
                else if (fixwi === undefined)
                    var dval = chld[oraclename](width) + 0.1 /* this avoids trouble with rounding */;
                else
                    var dval = fixwi + 0.1 /* this avoids trouble with rounding */;
                result = Math.max(result, dval);
            }
            return result;
        }

        computeMinimalWidth() {
            return this._getMaxDims("w", "m");
        }

        computePreferredWidth() {
            return this._getMaxDims("w", "p");
        }

        computeMinimalHeightForWidth(w) {
            return this._getMaxDims("h", "m", w);
        }

        computePreferredHeightForWidth(w) {
            return this._getMaxDims("h", "p", w);
        }

        doresize() {
            var chlds = this.childrenWidgets();
            var myisize = clove.utils.getInnerSize(this.topnode);
            for(var ichld in chlds)
            {
                var chld = chlds[ichld];
                var tn = chld.topnode;
                tn.style.position = "absolute";
                var st = getComputedStyle(tn);
                var hmarg = parseFloat(st.marginLeft) + parseFloat(st.marginRight);
                var vmarg = parseFloat(st.marginTop) + parseFloat(st.marginBottom);
                tn.style.left = "0px";
                tn.style.top = "0px";
                tn.style.width = (myisize.width - hmarg) + "px";
                tn.style.height = (myisize.height - vmarg) + "px";
            }
            super.doresize();
        }

    }

    clove.FlatLayout = FlatLayout;

    class FlatView extends clove.FlatLayout(clove.Widget) {

        constructor(config, domnode) {
            super(config, domnode);
        }

    }

    clove.FlatView = FlatView;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          TABVIEW

    class TabView extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [
                    { cols: [
                        { view: 'Button', label: '', name: 'btnleft', styleClass: 'clove_tabview_tabbarctrlbutton' },
                        { cols: [], name: 'tabs', width: '1px', height: '1px', verticalStretchAffinity: 1, horizontalStretchAffinity: 1},
                        { view: 'Button', label: '', name: 'btnright', styleClass: 'clove_tabview_tabbarctrlbutton' },
                        { view: 'Button', label: '', icon: clove.Icon.bySymbol(clove.symbols.Add), name: 'btnadd', styleClass: 'clove_tabview_tabbarctrlbutton' },
                    ], name: 'tabbar'}, // relayouted in tabBarLocation_setter
                    { view: 'FlatView', name: 'body', verticalStretchAffinity: 1, horizontalStretchAffinity: 1, styleClass: 'clove_tabview_body', newNameScope: true},
                ],
                verticalStretchAffinity: 1,
                horizontalStretchAffinity: 1,
            }), domnode);
            this.declareProperty("tabBarLocation", 'top');
            this.declareProperty("tabs", []);
            this.declareProperty("switchInvisibleAnimationName", undefined);
            this.declareProperty("switchVisibleAnimationName", undefined);
            this.declareProperty("switchInvisibleAnimationDuration", 300);
            this.declareProperty("switchVisibleAnimationDuration", 300);
            this.declareProperty("currentTab", -1);
            this.declareProperty("userMayAddTabs", false);
            $(this.topnode).addClass("clove_tabview");
            this._tabs = [];
            this._tabbarposition = 0;
            this._tabbarlocation_vertical = undefined;
            this.OnTabCreationRequested = new clove.Event();
        }

        switchInvisibleAnimationName_setter(v) {
            this.getByName("body").setSwitchInvisibleAnimationName(v);
        }

        switchVisibleAnimationName_setter(v) {
            this.getByName("body").setSwitchVisibleAnimationName(v);
        }

        switchInvisibleAnimationDuration_setter(v) {
            this.getByName("body").setSwitchInvisibleAnimationDuration(v);
        }

        switchVisibleAnimationDuration_setter(v) {
            this.getByName("body").setSwitchVisibleAnimationDuration(v);
        }

        tabBarLocation_setter(v) {
            var xxx = clove.utils.suspendResizing();
            try {
                var tabbar = this.getByName('tabbar');
                var tabs = this.getByName('tabs');
                var body = this.getByName('body');
                var vertical = false;
                if (v == 'left' || v == 'right') {
                    vertical = true;
                    var isleft = v == 'left';
                    tabbar.setProperty('col', isleft?0:1);
                    body.setProperty('col', isleft?1:0);
                    tabbar.setProperty('row', 0);
                    body.setProperty('row', 0);
                }
                else {
                    if (v != 'bottom')
                        v = 'top';
                    var istop = v == 'top';
                    tabbar.setProperty('col', 0);
                    body.setProperty('col', 0);
                    tabbar.setProperty('row', istop?0:1);
                    body.setProperty('row', istop?1:0);
                }
                this.setStyleClassAssigned('clove_tabview_tabstop', v == 'top');
                this.setStyleClassAssigned('clove_tabview_tabsleft', v == 'left');
                this.setStyleClassAssigned('clove_tabview_tabsbottom', v == 'bottom');
                this.setStyleClassAssigned('clove_tabview_tabsright', v == 'right');
                tabbar.setOrientation(vertical?'vertical':'horizontal');
                tabs.setOrientation(vertical?'vertical':'horizontal');
                tabbar.setHorizontalStretchAffinity(vertical?0:1);
                tabbar.setVerticalStretchAffinity(vertical?1:0);
                if (vertical !== this._tabbarlocation_vertical) {
                    for (let c of tabbar.childrenWidgets()) {
                        c.setProperty(vertical?'row':'col', c.getProperty('col')||c.getProperty('row')||0);
                        c.setProperty(vertical?'col':'row', 0);
                    }
                    tabbar.relayout();
                    for (let c of tabs.childrenWidgets()) {
                        c.setProperty(vertical?'row':'col', c.getProperty('col')||c.getProperty('row')||0);
                        c.setProperty(vertical?'col':'row', 0);
                        c.setProperty('width', vertical?'1px':undefined);
                        c.setProperty('horizontalStretchAffinity', vertical?1:0);
                    }
                    tabs.relayout();
                    this._tabbarlocation_vertical = vertical;
                    this.getByName("btnleft").setIcon(clove.Icon.bySymbol(vertical?clove.symbols.ArrowUp:clove.symbols.ArrowLeft));
                    this.getByName("btnright").setIcon(clove.Icon.bySymbol(vertical?clove.symbols.ArrowDown:clove.symbols.ArrowRight));
                }
                this._handletabbar();
                this.relayout();
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        userMayAddTabs_setter(v) {
            this.getByName("btnadd").setVisibility(v?clove.Visible:clove.InvisibleCollapsed);
        }

        tabs_getter() {
            return this._tabs;
        }

        tabs_setter(tabs) {
            this._tabs = [];
            this.config.currentTab = -1;
            this.getByName("body").setChildren([]);
            this.getByName("tabs").setChildren([]);
            for(var tab of tabs)
                this.addTab(tab);
            this._handletabbar();
        }

        _handletabbar() {
            if (!this.isInitialized)
                return;
            var tabs = this.getByName("tabs");
            var barwidth = this._tabbarlocation_vertical ? tabs.getPreferredHeightForWidth(parseFloat((tabs).innerSize().width)) : tabs.getPreferredWidth();
            var availwidth = clove.utils.getInnerSize(tabs.topnode)[this._tabbarlocation_vertical?'height':'width'];
            var maxpos = Math.max(0, barwidth - availwidth);
            if (this._tabbarposition < 0)
                this._tabbarposition = 0;
            else if (this._tabbarposition > maxpos)
                this._tabbarposition = maxpos;
            this.getByName("tabs").contentnode.style.transform = (this._tabbarlocation_vertical?"translate(0, -{}px)":"translate(-{}px, 0)").replace("{}", this._tabbarposition);
            this.getByName("btnleft").topnode.style.visibility = (this._tabbarposition == 0) ? "hidden" : "inherit";
            this.getByName("btnright").topnode.style.visibility = (this._tabbarposition == maxpos) ? "hidden" : "inherit";
        }

        doresize() {
            super.doresize();
            this._handletabbar();
        }

        currentTab_setter(v) {
            var self = this;
            var bttabs = this.getByName("tabs").contentnode.childNodes;
            var body = this.getByName("body");
            body.setCurrentView(v);
            for (var i=0;i<bttabs.length;i++)
                bttabs[i]._clove_widget.removeStyleClass("clove_tabview_tabbutton_activetab");
            v = Math.min(v, bttabs.length-1);
            if (v >= 0)
                bttabs[v]._clove_widget.addStyleClass("clove_tabview_tabbutton_activetab");
        }

        addTab(config) {
            var self = this;
            var child = this.getByName("body").addChild(config);
            var btncfg = { view: 'Button', label: config.tabLabel, icon: config.tabIcon, styleClass: 'clove_tabview_tabbutton' };
            if (this._tabbarlocation_vertical) {
                btncfg['width'] = '1px';
                btncfg['horizontalStretchAffinity'] = 1;
            }
            var tabbtn = this.getByName("tabs").addChild(btncfg);
            tabbtn.OnClicked.addHandler(function(){
                self.setCurrentTab($(tabbtn.topnode).index());
            });
            child.setVisibility(clove.InvisibleCollapsed);
            this._tabs.push({widget: child});
            if (config.tabMayBeClosedByUser) {
                var closebtn = $("<span class='clove_tabview_closebutton'>" + clove.symbols.Remove + "</span>");
                $(tabbtn._a).append(closebtn);
                closebtn.click(function(e){
                    e.preventDefault();
                    e.stopPropagation();
                    child.remove();
                });
                tabbtn.relayout();
            }
            this.relayout();
            child.OnDestroyed.addHandler(function(){
                tabbtn.remove();
                if (self.isAlive())
                    self.setCurrentTab(self.currentTab());
            });
            if (this.config.currentTab == -1)
                this.setCurrentTab(0);
            return child;
        }

        _movetabbar(v) {
            this._tabbarposition += v;
            this._handletabbar();
        }

        children_setter(v) {
            super.children_setter(v);
            this.containingNameScope().addChildNameScope(this.getByName('body'));
        }

        doinit() {
            super.doinit();
            var self = this;
            this.getByName('btnleft').OnClicked.addHandler(function(){self._movetabbar(-15);});
            this.getByName('btnright').OnClicked.addHandler(function(){self._movetabbar(15);});
            this.getByName('btnadd').OnClicked.addHandler(function(){self.OnTabCreationRequested.trigger({widget: self});});
        }

    }

    clove.TabView = TabView;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          CAROUSEL

    class Carousel extends clove.TabView {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                tabBarLocation: 'bottom',
                switchVisibleAnimationName: 'cloveanimation_fadein',
                switchVisibleAnimationDuration: 1000,
                switchInvisibleAnimationName: 'cloveanimation_fadeout',
                switchInvisibleAnimationDuration: 1000,
            }), domnode);
            this.declareProperty("interval", 4000);
            $(this.topnode).addClass("clove_carousel");
        }

        interval_setter() {
            if (this._carousel_playid !== undefined) {
                this.pause();
                this.play();
            }
        }

        doinit() {
            var self = this;
            super.doinit();
            var tabbar = this.getByName('tabbar');
            var tabs = this.getByName('tabs');
            tabbar.addChild({view: 'Button', name: 'btnplay', icon: clove.Icon.bySymbol(clove.symbols.TriangleRight), styleClass: 'clove_tabview_tabbarctrlbutton'});
            this.getByName('btnplay').OnClicked.addHandler(function(){
                self.next();
                var playing = self.isPlaying();
                self.pause();
                if (!playing)
                    self.play();
            });
            tabs.topnode.addEventListener('click', function() {
                self.pause();
            });
            this.play();
        }

        isPlaying() {
            return this._carousel_playid !== undefined;
        }

        play() {
            var self = this;
            if (this._carousel_playid === undefined)
                this._carousel_playid = setInterval(function(){self.next();}, this.config.interval);
        }

        next() {
            var tabcount = this.tabs().length;
            if (tabcount > 0)
                this.setCurrentTab((this.currentTab() + 1) % tabcount);
        }

        pause() {
            if (this._carousel_playid !== undefined) {
                clearInterval(this._carousel_playid);
                this._carousel_playid = undefined;
            }
        }

    }

    clove.Carousel = Carousel;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          DATETIMEBOX

    class DateBox extends clove.EditBox {

        constructor(config, domnode) {
            super(config, domnode);
            $(this.topnode).addClass("clove_datebox");
            this.declareProperty("date");
        }

        date_setter(v) {
            this.inputnode[0].valueAsDate = v;
        }

        date_getter() {
            return this.inputnode[0].valueAsDate;
        }

        _createinput() {
            return $("<input type='date'/>");
        }

    }

    clove.DateBox = DateBox;

    class TimeBox extends clove.EditBox {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("time");
            $(this.topnode).addClass("clove_timebox");
        }

        time_setter(v) {
            this.inputnode[0].valueAsDate = v;
        }

        time_getter() {
            return this.inputnode[0].valueAsDate;
        }

        _createinput() {
            return $("<input type='time'/>");
        }

    }

    clove.TimeBox = TimeBox;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          SLIDER

    class Slider extends clove.Widget {

        constructor(config, domnode) {
            var isvertical = config && config.orientation == "vertical";
            super(clove.utils.applyDefaultsToConfig(config, {
                width: isvertical?"25pt":"200pt",
                height: isvertical?"200pt":"25pt",
                mayFocus: true,
            }), domnode);
            this.declareProperty("orientation", "horizontal");
            this.declareProperty("min", 0);
            this.declareProperty("max", 10);
            this.declareProperty("stepsize", 1);
            this.declareProperty("value", 5);
            var self = this;
            $(this.topnode).addClass("clove_slider");
            this.sliderbar = $("<div class='clove_slider_bar' style='position:absolute;'></div>");
            this.slider = $("<div class='clove_slider_slider' style='position:absolute;'></div>");
            $(this.contentnode).append(this.sliderbar);
            this.sliderbar.append(this.slider);
            this.OnChanged = new clove.Event();
            this.sliderbar.click(function(e){
                self._click(e);
            });
            this.slider.mousedown(function(e){
                var mouseup;
                var mousemove;
                mouseup = function(e){
                    $(document.body).off("mouseup", mouseup);
                    $(document.body).off("mousemove", mousemove);
                    e.stopPropagation();
                };
                mousemove = function(e){
                    self._click(e);
                    e.stopPropagation();
                };
                $(document.body).mouseup(mouseup);
                $(document.body).mousemove(mousemove);
            });
            $(this.topnode).keydown(function(e){
                var val;
                if (e.keyCode == 37 || e.keyCode == 40)
                    val = self.value() - self.stepsize();
                else if (e.keyCode == 39 || e.keyCode == 38)
                    val = self.value() + self.stepsize();
                if (val !== undefined) {
                    self.setValue(val);
                    self.OnChanged.trigger({widget: self});
                }
            });
            this.sliderbar.bind('wheel', function(e){
                var val;
                if (e.originalEvent.deltaY > 0)
                    val = self.value() - self.stepsize();
                else
                    val = self.value() + self.stepsize();
                self.setValue(val);
                self.OnChanged.trigger({widget: self});
                e.preventDefault();
            });
            this.sliderbar.bind('touchmove', function(e){
                self._click({originalEvent: e.originalEvent.changedTouches[0]});
            });
        }

        _click(e) {
            var sdim = this.sliderbar[this._dimname]();
            var w = sdim - this.slider[this._dimname]();
            var x = e.originalEvent[this._mousecoordname] - $(this.sliderbar).offset()[this._axisbeginname];
            var rel = this._reverseaxisvalues ? ((sdim-x)/w) : (x/w);
            this.setValue(this._relativetoabsolute(rel));
            this.OnChanged.trigger({widget: this});
        }

        _relativetoabsolute(rel) {
            var vmin = this.min();
            var vmax = this.max();
            return vmin + (vmax - vmin) * rel;
        }

        _absolutetorelative(abs) {
            var vmin = this.min();
            var vmax = this.max();
            return (abs - vmin) / ((vmax - vmin));
        }

        _updateslidersize() {
            var vmin = this.min();
            var vmax = this.max();
            var vstepsize = this.stepsize();
            var vv = this.value();
            this.slider.css(this._dimname, (100 / (((vmax-vmin)/vstepsize)+1)) + "%");
            var axispos = ((vv-vmin)/vstepsize) / (((vmax-vmin)/vstepsize)+1);
            this.slider.css(this._axispositioningname, (100 * axispos) + "%");
        }

        _checkvalues() {
            var vmin = this.min();
            var vmax = this.max();
            var vstepsize = this.stepsize();
            var vv = this.value();
            var vvv = Math.floor((vv-vmin) / vstepsize) * vstepsize + vmin;
            if (isNaN(vmin))
                this.setMin(0);
            else if (isNaN(vmax))
                this.setMax(0);
            else if (isNaN(vstepsize) || vstepsize == 0)
                this.setStepsize(0.1);
            else if (isNaN(vv))
                this.setValue(0);
            else if (vmax < vmin)
                this.setMax(vmin);
            else if (vmin > vmax)
                this.setMin(vmax);
            else if (vvv > vmax)
                this.setValue(vmax);
            else if (vvv < vmin)
                this.setValue(vmin);
            else if (vvv != vv)
                this.setValue(vvv);
        }

        orientation_setter(v) {
            var vertical = (v == "vertical");
            this.setStyleClassAssigned("clove_slider_vertical", vertical);
            this.setStyleClassAssigned("clove_slider_horizontal", !vertical);
            this._dimname = vertical ? "height" : "width";
            this._axisbeginname = vertical ? "top" : "left";
            this._axispositioningname = vertical ? "bottom" : "left";
            this._mousecoordname = vertical ? "pageY" : "pageX";
            this._reverseaxisvalues = vertical;
        }

        min_setter(v) {
            this._checkvalues();
            this._updateslidersize();
        }

        max_setter(v) {
            this._checkvalues();
            this._updateslidersize();
        }

        stepsize_setter(v) {
            this._checkvalues();
            this._updateslidersize();
        }

        value_setter(v) {
            this._checkvalues();
            this._updateslidersize();
        }

        min_getter() {
            return parseFloat(this.config.min);
        }

        max_getter() {
            return parseFloat(this.config.max);
        }

        stepsize_getter() {
            return parseFloat(this.config.stepsize);
        }

        value_getter() {
            return parseFloat(this.config.value);
        }

    }

    clove.Slider = Slider;

    class Expander extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
                children: [
                    { view: 'Button', name: 'btnexpand', horizontalStretchAffinity: 1},
                    { rows: [], name: 'body', verticalStretchAffinity: 1, horizontalStretchAffinity: 1, newNameScope: true},
                ],
            }), domnode);
            this.declareProperty("isExpanded", false);
            this.declareProperty("collapsedIcon", clove.Icon.bySymbol(clove.symbols.TriangleRight));
            this.declareProperty("collapsedLabel");
            this.declareProperty("expandedIcon", clove.Icon.bySymbol(clove.symbols.TriangleDown));
            this.declareProperty("expandedLabel");
            this.declareProperty("body");
            $(this.topnode).addClass("clove_expander");
        }

        _handle_expanded() {
            var btnexpand = this.getByName("btnexpand");
            btnexpand.setIcon(this.config.isExpanded ? this.config.expandedIcon : this.config.collapsedIcon);
            btnexpand.setLabel(this.config.isExpanded ? this.config.expandedLabel : this.config.collapsedLabel);
            btnexpand.setStyleClassAssigned('clove_expander_button_expanded', this.config.isExpanded);
            btnexpand.setStyleClassAssigned('clove_expander_button_collapsed', !this.config.isExpanded);
        }

        icon_setter() {
            this._handle_expanded();
        }

        label_setter() {
            this._handle_expanded();
        }

        isExpanded_setter(v) {
            this._handle_expanded();
            this.getByName("body").setVisibility(v?clove.Visible:clove.InvisibleCollapsed);
            this.relayout();
        }

        body_setter(v) {
            this.getByName("body").setChildren([v]);
        }

        children_setter(v) {
            super.children_setter(v);
            this.containingNameScope().addChildNameScope(this.getByName('body'));
        }

        doinit() {
            super.doinit();
            var self = this;
            this.getByName("btnexpand").OnClicked.addHandler(function(){
                self.setIsExpanded(!self.isExpanded());
            });
        }

    }

    clove.Expander = Expander;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          RESIZESPLITTER

    class RawResizeSplitter extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            $(this.topnode).addClass("clove_rawresizesplitter");
            this.OnMoveBegin = new clove.Event();
            this.OnMove = new clove.Event();
            this.OnMoveEnd = new clove.Event();
            var self = this;
            clove.utils.addOnDragHandler(this.topnode, function dragbegin(){
                self.OnMoveBegin.trigger({widget: self});
                return true;
            }, function drag(dx, dy){
                self.OnMove.trigger({widget: self, dx: dx, dy: dy});
                return true;
            }, function dragend(){
                self.OnMoveEnd.trigger({widget: self});
                return true;
            });
        }

    }

    clove.RawResizeSplitter = RawResizeSplitter;

    class ResizeSplitter extends clove.GridLayout(clove.Widget) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'horizontal',
            }), domnode);
            this.declareProperty("splitterWidth", "6pt");
            this.declareProperty("body", []);
            this.declareProperty("sizeFractions", []);
            this.declareProperty("showOnly", undefined);
            this.declareProperty("splitterVisibility", clove.Visible);
            $(this.topnode).addClass("clove_resizesplitter");
            this.ghostsplitter = $("<div style='position:absolute;display:none;' class='clove_resizesplitter_ghost'></div>");
            this._visiblebodywidgets = new Set();
            this._fullclientssize = 0;
            this._fullclientssizes = [];
        }

        sizeFractions_setter(v) {
            var lbody = this.config.body.length;
            var lsum = 0;
            var lcnt = 0;
            var vvv = [];
            for (var i=0; i<lbody; i++) {
                var vv = v[i];
                if (vv < 0)
                    vv = 0;
                vvv.push(vv);
                if (Number.isFinite(vv)) {
                    lsum += vv;
                    lcnt += 1;
                }
            }
            var lavg = lcnt ? lsum / lcnt : 1;
            var lfullsum = lavg * lbody;
            var res = [];
            for (var i=0; i<lbody; i++) {
                var vv = vvv[i];
                if (Number.isFinite(vv))
                    vv = vv / lfullsum;
                else
                    vv = 1 / lbody;
                res.push(vv);
            }
            this.config.sizeFractions = res;
            this.relayout();
        }

        body_setter(v) {
            var xxx = clove.utils.suspendResizing();
            try {
                this.setChildren([]);
                var oldsf = this.config.sizeFractions;
                this.config.body = [];
                for (var vw of v.slice())
                    this.addBodyWidget(vw);
                this.setSizeFractions(oldsf);
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
        }

        _childwidgets(skippayload, skipsplitters) {
            var rres = new Object();
            var vo = this.config.orientation == "vertical" ? "row" : "col";
            var chiwi = this.childrenWidgets();
            for (var w of chiwi)
                rres[w.config[vo]] = w;
            var res = [];
            for(var i=0; i<chiwi.length; i++) {
                if (skippayload && (i % 2) == 0)
                    continue;
                if (skipsplitters && (i % 2) == 1)
                    continue;
                res.push(rres[i]);
            }
            return res;
        }

        bodyWidgets() {
            return this._childwidgets(false, true);
        }

        _observebodywidget(ww) {
            var self = this;
            ww.OnPropertyChanged.addHandler(function(e){
                if (e.propertyName == "visibility") {
                    self._handleSplitters();
                    self.relayout();
                }
            });
            ww.OnDestroyed.addHandler(function() {
                var mdim = self.config.orientation == "vertical" ? "row" : "col";
                var lcs = self._childwidgets(true);
                var mpos = ww.config[mdim];
                var irs = Math.floor(mpos / 2);
                if (irs >= lcs.length)
                    irs = lcs.length -1;
                var chiwi = self.childrenWidgets();
                for(var cw of chiwi) {
                    if (cw.config[mdim] > mpos)
                        cw.config[mdim] -= 2;
                }
                var wic = lcs[irs];
                if (wic) {  // undefined when removing the only widget
                    wic.config[mdim] = -1;
                    wic.remove();
                }
                var idi = Math.floor(mdim/2);
                self.config.sizeFractions.splice(idi, 1);
                self.config.body.splice(idi, 1);
                self._handleSplitters();
                self.relayout();
            });
        }

        setBodyWidget(i, config) {
            var movaxis = (this.config.orientation == "vertical") ? "row" : "col";
            var nmovaxis = (this.config.orientation == "vertical") ? "col" : "row";
            config[movaxis] = npos;
            config[nmovaxis] = 0;
            var ww = this.addChild(config, true);
            this._observebodywidget(ww);
            this.config.body[i] = config;
            this._handleSplitters();
            this.relayout();
            return ww;
        }

        addBodyWidget(config) {
            return this.insertBodyWidget(-1, config);
        }

        insertBodyWidget(i, config) {
            var self = this;
            var ww;
            var xxx = clove.utils.suspendResizing();
            try {
                var bwl = this.bodyWidgets().length;
                var append = false;
                if (i == -1 || i >= bwl) {
                    i = bwl;
                    append = true;
                }
                var npos = 2 * i;
                var vertical = (this.config.orientation == "vertical");
                var movaxis = vertical ? "row" : "col";
                var nmovaxis = vertical ? "col" : "row";
                var splitter;
                if (append) {
                    if (i > 0) {
                        var sconfig = {view: 'RawResizeSplitter'};
                        sconfig[movaxis] = npos - 1;
                        sconfig[nmovaxis] = 0;
                        splitter = this.addChild(sconfig, true);
                    }
                    config[movaxis] = npos;
                    config[nmovaxis] = 0;
                    ww = this.addChild(config, true);
                }
                else {
                    for(var c of this.childrenWidgets()) {
                        if (c.config[movaxis] >= npos)
                            c.config[movaxis] += 2;
                    }
                    config[movaxis] = npos;
                    config[nmovaxis] = 0;
                    ww = this.addChild(config, true);
                    var sconfig = {view: 'RawResizeSplitter'};
                    sconfig[movaxis] = npos + 1;
                    sconfig[nmovaxis] = 0;
                    splitter = this.addChild(sconfig, true);
                }
                if (splitter) {
                    splitter.OnMoveBegin.addHandler(function(e){ self._onmovebegin(splitter, e); });
                    splitter.OnMove.addHandler(function(e){ self._onmove(splitter, e); });
                    splitter.OnMoveEnd.addHandler(function(e){ self._onmoveend(splitter, e); });
                    if (vertical)
                        splitter.addStyleClass("clove_rawresizesplitter_vertical");
                    else
                        splitter.addStyleClass("clove_rawresizesplitter_horizontal");
                    splitter._clove_resizesplitter = this;
                }
                this._observebodywidget(ww);
                this.config.body.splice(i, 0, config);
                var bol = this.config.body.length;
                var mydim = 1.0 / bol;
                var odimdec = (bol == 1) ? 0 /*then its not needed at all*/ : mydim / (bol - 1);
                var sum = 0;
                var nsf = [];
                if (this.config.sizeFractions.length) {
                    for (var viv of this.config.sizeFractions) {
                        sum += viv;
                        nsf.push(viv);
                    }
                    var avg = sum / this.config.sizeFractions.length;
                }
                else
                    var avg = 1;
                nsf.splice(i, 0, avg);
                this.setSizeFractions(nsf);
                this._handleSplitters();
                this.relayout();
            }
            finally {
                clove.utils.resumeResizing(xxx);
            }
            return ww;
        }

        showOnly_setter(v) {
            this.relayout();
        }

        splitterWidth_setter(v) {
            this._splitterwidth = clove.utils.cssLengthToPixels(v, "x");
            this.relayout();
        }

        splitterVisibility_setter(v) {
            this.relayout();
        }

        _onmovebegin(splitter, e) {
            this.ghostsplitter.css('display', '');
            var stn = $(splitter.topnode);
            if (this.config.orientation == "vertical") {
                this.movebegin = stn.position().top;
                this.ghostsplitter.css('height', this._splitterwidth+'px');
                this.ghostsplitter.css('top', this.movebegin+'px');
                this.ghostsplitter.css('left', '0');
                this.ghostsplitter.css('right', '0');
            } else {
                this.movebegin = stn.position().left;
                this.ghostsplitter.css('width', this._splitterwidth+'px');
                this.ghostsplitter.css('top', '0');
                this.ghostsplitter.css('bottom', '0');
                this.ghostsplitter.css('left', this.movebegin+'px');
            }
            this.movelast = 0;
        }

        _onmove(splitter, e) {
            var tnis = clove.utils.getInnerSize(this.topnode);
            if (this.config.orientation == "vertical") {
                this.movelast = e.dy;
                var v = this.movebegin + this.movelast;
                v = Math.max(0, Math.min(v, tnis.height-this._splitterwidth));
                this.ghostsplitter.css('top', v+'px');
            } else {
                this.movelast = e.dx;
                var v = this.movebegin + this.movelast;
                v = Math.max(0, Math.min(v, tnis.width-this._splitterwidth));
                this.ghostsplitter.css('left', v+'px');
            }
        }

        _onmoveend(splitter, e) {
            var tnis = clove.utils.getInnerSize(this.topnode);
            this.ghostsplitter.css('display', 'none');
            var myaxis = (this.config.orientation == "vertical") ? 'row' : 'col';
            var v = this.movebegin + this.movelast;
            var lip = Math.floor(splitter._clove_resizesplitter_leftwidget.config[myaxis] / 2);
            var rip = Math.floor(splitter._clove_resizesplitter_rightwidget.config[myaxis] / 2);
            var sposs = [];
            var sifr = this.sizeFractions();
            for(var ivv=0; ivv<this._fullclientssizes.length; ivv++) {
                var vv = this._fullclientssizes[ivv];
                if (vv == 0)
                    vv = (sifr[ivv] * this._fullclientssize) || 0;
                sposs.push(vv);
            }
            var amt = this.movelast;
            sposs[lip] += amt;
            sposs[rip] -= amt;
            this.setSizeFractions(sposs);
        }

        orientation_setter(v) {
            for(var w of this._childwidgets(true)) {
                w.removeStyleClass("clove_resizesplitter_horizontal");
                w.removeStyleClass("clove_resizesplitter_vertical");
                if (v == "vertical")
                    w.addStyleClass("clove_rawresizesplitter_vertical");
                else
                    w.addStyleClass("clove_rawresizesplitter_horizontal");
            }
        }

        doinit() {
            super.doinit();
            $(this.contentnode).append(this.ghostsplitter);
        }

        _handleSplitters() {
            var sf = false;
            var openwidget;
            var opensplitter;
            this._visiblebodywidgets = new Set();
            for (var ww of this._childwidgets()) {
                if (ww) {  // can be undefined during (unfortunately asynchronous) widget removal
                    if (sf) {
                        ww._clove_resizesplitter_leftwidget = undefined;
                        ww._clove_resizesplitter_rightwidget = undefined;
                        if (openwidget) {
                            ww._clove_resizesplitter_leftwidget = openwidget;
                            opensplitter = ww;
                            openwidget = undefined;
                        }
                    }
                    else {
                        if (ww.visibility() != clove.InvisibleCollapsed) {
                            this._visiblebodywidgets.add(ww);
                            openwidget = ww;
                            if (opensplitter) {
                                opensplitter._clove_resizesplitter_rightwidget = ww;
                                opensplitter = undefined;
                            }
                        }
                    }
                }
                sf = !sf;
            }
        }

        _getFullDims(mywidth, myheight) {
            var mysize = (this.config.orientation == "vertical") ? myheight : mywidth;
            var chiwi = this._childwidgets();
            var visiblewidgets;
            var visiblesplitters = new Set();
            var sizes = [];
            if (this.config.showOnly !== undefined) {
                visiblewidgets = new Set();
                var sf = false;
                for(var iwc=0; iwc<chiwi.length; iwc++) {
                    var wc = chiwi[iwc];
                    if (!sf) {
                        sizes.push(1);
                        if (this.config.showOnly * 2 == iwc)
                            visiblewidgets.add(wc);
                    }
                    sf = !sf;
                }
                var ressize = this._fullclientssize = mysize;
                var sumsizes = 1;
            }
            else {
                visiblewidgets = this._visiblebodywidgets;
                var sf = false;
                for(var iwc=0; iwc<chiwi.length; iwc++) {
                    var wc = chiwi[iwc];
                    if (sf && wc /* can be undefined during (unfortunately asynchronous) widget removal */ && wc._clove_resizesplitter_leftwidget && wc._clove_resizesplitter_rightwidget)
                        visiblesplitters.add(iwc);
                    sf = !sf;
                }
                var ressize = this._fullclientssize = mysize - visiblesplitters.size * this._splitterwidth;
                var wf = true;
                var pl = this.config.sizeFractions;
                var iwc = 0;
                var sumsizes = 0;
                for(var wc of chiwi) {
                    if (wf) {
                        if (visiblewidgets.has(wc)) {
                            sizes.push(pl[iwc]);
                            sumsizes += pl[iwc];
                        }
                        else
                            sizes.push(0);
                        iwc++;
                    }
                    wf = !wf;
                }
            }
            this._fullclientssizes = [];
            var dims = [];
            for(var iss=0; iss<sizes.length; iss++) {
                if (iss > 0) {
                    if (visiblesplitters.has(iss*2-1))
                        dims.push(this._splitterwidth);
                    else
                        dims.push(0);
                }
                if (visiblewidgets.has(chiwi[iss*2])) {
                    var xv = sizes[iss] * ressize / sumsizes;
                    dims.push(xv);
                    this._fullclientssizes.push(xv);
                }
                else {
                    dims.push(0);
                    this._fullclientssizes.push(0);
                }
            }
            var coords = [];
            var ii = 0;
            for (var ic of dims) {
                ii += ic;
                coords.push(ii);
            }
            if (this.config.orientation == "vertical")
                return [[mywidth], coords, [mywidth], dims, 1, dims.length];
            else
                return [coords, [myheight], dims, [myheight], dims.length, 1];
        }

        computeMinimalWidth() {
            return 1;
        }

        computePreferredWidth() {
            return 1;
        }

        computeMinimalHeightForWidth(w) {
            return 1;
        }

        computePreferredHeightForWidth(w) {
            return 1;
        }

    }

    clove.ResizeSplitter = ResizeSplitter;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          PROGRESSBAR

    class ProgressBar extends clove.Widget {

        constructor(config, domnode) {
            super(config, domnode);
            this.declareProperty("value", 0);
            this.declareProperty("orientation", "horizontal");
            this.declareProperty("label");
            this.declareProperty("labelFunction", function(v, w){
                if (v !== undefined)
                    return (w.config.label?(w.config.label+": "):"")+Math.round(v*100)+"%";
                else
                    return "";
            });
            $(this.topnode).addClass("clove_progressbar");
            this._progressdiv = document.createElement("div");
            this._progressdiv.className = "clove_progressbar_progress";
            this.contentnode.appendChild(this._progressdiv);
            this._labeldiv = document.createElement("div");
            this._labeldiv.className = "clove_progressbar_label";
            this.contentnode.appendChild(this._labeldiv);
        }

        _align(skiplabel) {
            var v = undefined;
            if (this.config.value !== undefined) {
                v = Math.min(Math.max(0, this.config.value), 1) || 0;
                this._progressdiv.style[(this.config.orientation=='vertical')?'height':'width'] = (v*100)+"%"
            }
            if (!skiplabel)
                this._labeldiv.textContent = this.config.labelFunction(v, this);
        }

        orientation_setter(v) {
            this.setStyleClassAssigned("clove_progressbar_vertical", v == "vertical");
            this.setStyleClassAssigned("clove_progressbar_horizontal", v != "vertical");
            this._align(true);
        }

        value_setter() {
            if (this.config.value === undefined)
                $(this._progressdiv).addClass("clove_progressbar_progress_indeterminate");
            else
                $(this._progressdiv).removeClass("clove_progressbar_progress_indeterminate");
            this._align();
        }

        labelFunction_setter() {
            this._align();
        }

        doresize() {
            super.doresize();
            this._align(true);
        }

        computeMinimalWidth() {
            return clove.utils.cssLengthToPixels((this.config.orientation == "vertical") ? "1.5em" : "10pt", "x");
        }

        computePreferredWidth() {
            return clove.utils.cssLengthToPixels((this.config.orientation == "vertical") ? "1.5em" : "50pt", "x");
        }

        computeMinimalHeightForWidth() {
            return clove.utils.cssLengthToPixels((this.config.orientation == "vertical") ? "10pt" : "1.5em", "y");
        }

        computePreferredHeightForWidth() {
            return clove.utils.cssLengthToPixels((this.config.orientation == "vertical") ? "50pt" : "1.5em", "y");
        }

    }

    clove.ProgressBar = ProgressBar;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          NOTIFICATIONS

    class NotificationView extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                children: [
                    { rows: [], name: 'body', newNameScope: true, horizontalStretchAffinity: 1 },
                    { view: 'Button', name: 'btnclose', label: '', icon: clove.Icon.bySymbol(clove.symbols.Remove), styleClass: 'clove_notificationview_closebutton'},
                ],
                orientation: 'horizontal',
                horizontalStretchAffinity: 1,
            }), domnode);
            this.declareProperty("body", {rows: []});
            this.declareProperty("closeable", false);
            $(this.topnode).addClass("clove_notificationview");
            this.OnClose = new clove.Event();
        }

        closeable_setter(v) {
            this.getByName('btnclose').setVisibility(v?clove.Visible:clove.InvisibleCollapsed);
        }

        body_setter(v) {
            this.getByName('body').setChildren([v]);
        }

        children_setter(v) {
            super.children_setter(v);
            this.containingNameScope().addChildNameScope(this.getByName('body'));
        }

        doinit() {
            var self = this;
            super.doinit();
            this.getByName('btnclose').OnClicked.addHandler(function(){
                self.OnClose.trigger({widget: self});
            });
        }

    }

    class NotificationsHost extends clove.StackLayout(clove.Widget) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'vertical',
            }), domnode);
            $(this.topnode).addClass("clove_notificationshost");
            this._idx = 0;
        }

        _align() {
            var wl = this.childrenWidgets();
            var wwl = [];
            for(var i=0; i<wl.length; i++) {
                var w = wl[i];
                wwl.push([w.getProperty('priority'), w.getProperty('notificationindex'), w]);
            }
            wwl.sort(function(a,b){ if (a[0]<b[0]) return 1; if (b[0]<a[0]) return -1; return b[1]-a[1]; });
            for(var i=0; i<wwl.length; i++)
                wwl[i][2].setProperty('row', i);
            this.relayout();
        }

        addNotification(config, notificationConfig) {
            var self = this;
            notificationConfig = clove.utils.applyDefaultsToConfig(notificationConfig, {
                priority: 0,
            });
            var w = this.addChild({rows: [
                {view: 'NotificationView', body: config, closeable: notificationConfig.closeable, styleClass: notificationConfig.styleClass, OnClose: function(){w.remove();}},
            ], newNameScope: true, horizontalStretchAffinity: 1, priority: notificationConfig.priority, notificationindex: this._idx++, styleClass: "clove_notificationview_outer"});
            var nv = w.childrenWidgets()[0];
            w.setProperty("defaultRemovalAnimationName", "cloveanimation_fadeout");
            w.setProperty("defaultRemovalAnimationDuration", 400);
            nv.OnDestroyed.addHandler(function(){
                w.remove();
            });
            w.OnDestroyed.addHandler(function(e){
                if (self.childrenWidgets().length == 0)
                    self.remove();
                else
                    self._align();
            });
            if (notificationConfig.timeout !== undefined)
                setTimeout(function(){
                    w.remove();
                }, notificationConfig.timeout*1000);
            this._align();
            clove.utils.animateNode(w.topnode, "cloveanimation_fadein", 700);
            return w;
        }

    }

    class NotificationController {

        _gethost() {
            var self = this;
            if (!this._host || !this._host.widget.isAlive()) {
                this._host = clove.utils.popup({view: 'NotificationsHost'}, {top: '25pt', right: '25pt', width: '300pt'});
                this._host.widget.OnDestroyed.addHandler(function(){ self._host = undefined; });
            }
            return this._host;
        }

        notify(config, notificationConfig) {
            var host = this._gethost().widget;
            return host.addNotification(config, notificationConfig);
        }

    }

    clove.notifications = new NotificationController();

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///
    ///                                                                          CONVERSATIONDIALOG

    clove.conversationDialog = function(config) {
        var title = config.title || "";
        var body = config.body || {rows: []};
        var buttons = config.buttons || [clove.i18n.OK];
        var _buttons = [{rows:[], width: '1px', horizontalStretchAffinity: 1}];
        var buttonnames = [];
        let nameprefix = "clove_conversationdialog_";
        for(var i in buttons) {
            var btnname = nameprefix + i;
            buttonnames.push(btnname);
            _buttons.push({view: 'Button', label: buttons[i], styleClass: 'clove_conversationdialog_button', name: btnname});
        }
        var dlgres = clove.Dialog.show({
            view: 'Dialog',
            styleClass: 'clove_conversationdialog',
            title: title,
            body: {
                cols: [
                    {view: "IconView", icon: config.icon, visibility: (config.icon ? clove.Visible : clove.InvisibleCollapsed)},
                    {rows: [
                        body,
                        { cols: _buttons, styleClass: 'clove_conversationdialog_buttonpanel' },
                    ]}
                ], styleClass: 'clove_conversationdialog_body'
            },
        }, {
            modal: true,
            x: config.x,
            y: config.y,
            width: config.width,
            height: config.height,
        });
        var dlg = dlgres.widget;
        function close(args) {
            dlg.close();
            OnProceed.trigger(args);
        };
        var OnProceed = new clove.Event();
        for(var i in buttons) {
            var btn = dlgres.namescope.getByName(nameprefix+i);
            btn.OnClicked.addHandler(function(e){
                close({button: parseInt(e.widget.name().substring(nameprefix.length)), universe: dlgres.namescope});
            });
            if (i == 0)
                btn.focus();
        }
        dlgres.buttonnames = buttonnames;
        dlgres.OnProceed = OnProceed;
        return dlgres;
    };

    clove.messageDialog = function(config) {
        var message = config.message || "";
        var newconfig = $.extend({}, config);
        newconfig.body = {view: 'Label', label: message};
        var dlgres = clove.conversationDialog(newconfig);
        dlgres.OnProceed.dlgres = dlgres;
        return dlgres.OnProceed;
    };

    clove.inputDialog = function(config) {
        config = clove.utils.applyDefaultsToConfig(config, {
            defaultAnswer: "",
            question: "",
            allowEmpty: true,
            contentWidgetConfig: {view: 'EditBox'},
            withKeyboardControl: true,
        });
        config.buttons = [clove.i18n.OK, clove.i18n.Cancel];
        var contcfg = clove.utils.applyDefaultsToConfig({text: config.defaultAnswer, name: 'editbox'}, config.contentWidgetConfig);
        config.body = {rows: [{view: 'Label', label: config.question}, contcfg]};
        var OnProceed = new clove.Event();
        var dlgres = clove.conversationDialog(config);
        var editbox = dlgres.namescope.getByName('editbox');
        dlgres.OnProceed.addHandler(function(e){
            e.answer = editbox.text();
            if (e.button == 0)
                OnProceed.trigger(e);
            else
                OnProceed.trigger({answer: undefined});
        });
        if (!config.allowEmpty) {
            function _endisableok() {
                dlgres.namescope.getByName(dlgres.buttonnames[0]).setEnabled(editbox.text().length>0);
            }
            _endisableok();
            editbox.OnChanged.addHandler(_endisableok);
        }
        editbox.focus();
        if (config.withKeyboardControl)
            editbox.OnKeyDown.addHandler(function(e) {
                if (e.keyCode == 10 || e.keyCode == 13) {
                    var text = editbox.text();
                    if (text.length > 0 || config.allowEmpty) {
                        dlgres.widget.close();
                        OnProceed.trigger({answer: text});
                    }
                }
                else if (e.keyCode == 27) {
                    dlgres.widget.close();
                    OnProceed.trigger({answer: undefined});
                }
            });
        OnProceed.dlgres = dlgres;
        return OnProceed;
    };

    clove.selectionDialog = function(config) {
        config = clove.utils.applyDefaultsToConfig(config, {
            defaultAnswer: 0,
            question: "",
            contentWidgetConfig: {view: 'DataView', hideExpanders: true, gridVisible: false, horizontalStretchAffinity: 1, allowSelection: true},
            scrollWidgetConfig: {view: 'ScrollView', height: '200pt'},
            choices: [],
            withKeyboardControl: true,
        });
        config.buttons = [clove.i18n.OK, clove.i18n.Cancel];
        var c = (config.choices.valuePointer) ? config.choices : clove.utils.arrayToDatasource(config.choices);
        var contcfg = clove.utils.applyDefaultsToConfig({name: 'dataview', datasource: c}, config.contentWidgetConfig);
        var scrollcfg = clove.utils.applyDefaultsToConfig({name: 'scrollview', body: contcfg}, config.scrollWidgetConfig);
        config.body = {rows: [{view: 'Label', label: config.question}, scrollcfg]};
        var OnProceed = new clove.Event();
        var dlgres = clove.conversationDialog(config);
        var dataview = dlgres.namescope.getByName('dataview');
        if (c.rowCount() > config.defaultAnswer && c.columnCount() > 0)
            dataview.selectCell(c.valuePointer(config.defaultAnswer, 0));
        dlgres.OnProceed.addHandler(function(e){
            var _sel = dataview.selection();
            e.answer = _sel ? _sel.irow : undefined;
            if (e.button == 0)
                OnProceed.trigger(e);
            else
                OnProceed.trigger({answer: undefined});
        });
        dataview.focus();
        if (config.withKeyboardControl)
            dataview.OnKeyDown.addHandler(function(e) {
                if (e.keyCode == 10 || e.keyCode == 13) {
                    dlgres.widget.close();
                    var _sel = dataview.selection();
                    OnProceed.trigger({answer: _sel ? _sel.irow : undefined});
                }
                else if (e.keyCode == 27) {
                    dlgres.widget.close();
                    OnProceed.trigger({answer: undefined});
                }
            });
        OnProceed.dlgres = dlgres;
        return OnProceed;
    };

})();
